<?php

/**
 * Add css and js files on any page
 *
 * @package     PG_Core
 * @subpackage  Helpers
 *
 * @category    helpers
 *
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!function_exists('css')) {

    /**
     * Generate include style code
     *
     * @param string $load_type load extra styles
     *
     * @return string
     */
    function css($load_type = '')
    {
        $CI = &get_instance();
        $load_type_array = explode("|", $load_type);

        if (is_array($load_type_array) && in_array("ui", $load_type_array)) {
            $CI->pg_theme->add_css('application/js/jquery-ui/jquery-ui.custom.css');
        }

        // preview mode
        if ($CI->input->get('template_preview_mode')) {
            $preview_theme = $_SESSION["preview_theme"];
            $preview_scheme = $_SESSION["preview_scheme"];
        } else {
            $preview_theme = '';
            $preview_scheme = '';
        }

        $css_html = $CI->pg_theme->get_include_css_code($preview_theme, $preview_scheme);
        echo $css_html;
    }
}

if (!function_exists('js')) {

    /**
     * Generate include javascript code
     *
     * @param string $load_type load extra javascript
     *
     * @return string
     */
    function js($load_type = '')
    {
        $CI = &get_instance();
        $load_type_array = explode("|", $load_type);

        $CI->pg_theme->add_js('jquery.js');
        $CI->pg_theme->add_js('functions.js');
        $CI->pg_theme->add_js('errors.js');
        $CI->pg_theme->add_js('loading.js');

        if (is_array($load_type_array) && in_array("ui", $load_type_array)) {
            $CI->pg_theme->add_js('jquery-ui.custom.min.js');
            // Dateppicker langs
            $lang = $CI->pg_language->get_lang_by_id($CI->pg_language->current_lang_id);
            $CI->pg_theme->add_js("datepicker-langs/jquery.ui.datepicker-{$lang['code']}.js");
        }
        if (is_array($load_type_array) && in_array("editable", $load_type_array)) {
            $CI->pg_theme->add_js('jquery.jeditable.mini.js');
        }

        $css_html = $CI->pg_theme->get_include_js_code();

        $template_preview_mode = $CI->input->get('template_preview_mode');
        if ($template_preview_mode) {
            $css_html .= "
    <script>
        $(function(){
            $('a[href]').attr('href', function() {
                var href = $(this).attr('href')
                if(href.indexOf('?') != -1){
                    return href+'&template_preview_mode=1';
                }else{
                    return href+'?template_preview_mode=1';
                }
            });
        });
    </script>";
        }

        echo $css_html;
    }
}
