<?php

/**
 * Upload file
 *
 * @package 	PG_Core
 * @subpackage 	Helpers
 *
 * @category	helpers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

/**
 * Upload file
 *
 * @param string $file_elem_name upload guid
 * @param string $upload_path upload path
 * @param array $config upload configuration
 *
 * @return array
 */
function upload_file($file_elem_name, $upload_path, $config = array())
{
    $CI = &get_instance();
    $error = $msg = '';

    // do upload
    $config["upload_path"] = $upload_path;
    $CI->load->library('upload');
    $CI->upload->initialize($config);
    if (!$CI->upload->do_upload($file_elem_name)) {
        $error = $CI->upload->error_msg;
    } else {
        $msg = $CI->upload->data();
    }

    //for security reason, we force to remove all uploaded file
    @unlink($_FILES[$file_elem_name]);

    return array('error' => $error, 'data' => $msg);
}

/**
 * Validate file
 *
 * @param string $file_elem_name upload guid
 * @param array $config upload configuration
 *
 * @return array
 */
function validate_file($file_elem_name, $config = array())
{
    $CI = &get_instance();
    $error = $msg = '';

    // do validate
    $CI->load->library('upload');
    $CI->upload->initialize($config);
    if (!$CI->upload->do_validate($file_elem_name)) {
        $error = $CI->upload->error_msg;
    } else {
        $msg = $CI->upload->data();
    }

    return array('error' => $error, 'data' => $msg);
}

/**
 * Return file extension
 *
 * @param string $file_name file name
 *
 * @return string
 */
function get_extension($file_name)
{
    $path_parts = pathinfo($file_name);

    return $path_parts["extension"];
}

/**
 * Rename file
 *
 * @param string $file_old old file name
 * @param string $file_new new file name
 *
 * @return boolean
 */
function rename_file($file_old, $file_new)
{
    if (copy($file_old, $file_new)) {
        @unlink($file_old);

        return true;
    }

    return false;
}
