(function(root, factory) {

  if (typeof define === 'function' && define.amd) {
    define(['moment', 'jquery', 'exports'], function(momentjs, $, exports) {
      root.daterangepicker = factory(root, exports, momentjs, $);
    });

  } else if (typeof exports !== 'undefined') {
      var momentjs = require('moment');
      var jQuery = (typeof window != 'undefined') ? window.jQuery : undefined;  //isomorphic issue
      if (!jQuery) {
          try {
              jQuery = require('jquery');
              if (!jQuery.fn) jQuery.fn = {}; //isomorphic issue
          } catch (err) {
              if (!jQuery) throw new Error('jQuery dependency not found');
          }
      }

    factory(root, exports, momentjs, jQuery);

  // Finally, as a browser global.
  } else {
    root.daterangepicker = factory(root, {}, root.moment || moment, (root.jQuery || root.Zepto || root.ender || root.$));
  }

}(this || {}, function(root, daterangepicker, moment, $) { // 'this' doesn't exist on a server

    var DateRangePicker = function(element, options) {

        //default settings for options

        this.parentEl = 'body';
        this.element = $(element);
        //fix
        this.startId = false;
        this.endId = false;
        this.startDate = moment().startOf('day');
        this.endDate = moment().endOf('day');
        this.minDate = false;
        this.maxDate = false;
        this.dateLimit = false;
        this.autoApply = false;
        this.singleDatePicker = false;
        this.showDropdowns = false;
        this.showWeekNumbers = false;
        this.timePicker = false;
        this.timePicker24Hour = false;
        this.timePickerIncrement = 1;
        this.timePickerSeconds = false;
        this.linkedCalendars = true;
        this.autoUpdateInput = true;
        this.ranges = {};
        //fix
        this.listing_periods = null;
        this.noDaysOfWeek = false;
        this.singlBlock = false;
        this.no_highlight_periods = false;
        this.clickedStart = false;
        this.clickedEnd = false;
        this.get_special_prices = false;
        this.set_special_prices = false;
        this.listingData = null;
        this.special_prices = null;
        this.siteUrl = '';
        this.show_prices_for_booking = false;
        this.urlSpecialPrices = 'listings_booking/ajax_get_periods';
        this.daterangepickerId = null;
        this.inputStartId = false;
        this.inputEndId = false;
        this.parent_popup = null;
        this.parent_popup_height_val = null;
        this.pricesBlockId = 'exclusive_price';
        this.pricesContentId = 'prices_content';
        this.pricesTableId = 'table_content';
        this.priceDetailsId = 'price_details';
        this.detailsBlockId = 'details_block';
        this.opens = 'right';
        this.isShowPriceDetails = false;
        this.pricesBlockHeight = 40;
        this.drops = 'down';
        if (this.element.hasClass('dropup'))
            this.drops = 'up';

        this.buttonClasses = 'btn btn-sm';
        this.applyClass = 'btn-success';
        this.cancelClass = 'btn-default';

        this.locale = {
            format: 'MM/DD/YYYY',
            formatView: 'MM/DD/YYYY',
            separator: ' - ',
            applyLabel: 'Apply',
            cancelLabel: 'Cancel',
            weekLabel: 'W',
            customRangeLabel: 'Custom Range',
            daysOfWeek: moment.weekdaysMin(),
            monthNames: moment.monthsShort(),
            firstDay: moment.localeData().firstDayOfWeek()
        };

        //some state information
        this.isShowing = false;
        this.leftCalendar = {};
        this.rightCalendar = {};

        //custom options from user
        if (typeof options !== 'object' || options === null)
            options = {};

        //allow setting options with data attributes
        //data-api options will be overwritten with custom javascript options
        options = $.extend(this.element.data(), options);

        //html template for the picker UI
        //fix
        if (typeof options.template !== 'string')
            options.template = this.createCalendar();
        
        this.parentEl = (options.parentEl && $(options.parentEl).length) ? $(options.parentEl) : $(this.parentEl);
        this.container = $(options.template).appendTo(this.parentEl);
        //fix
        var _self = this;

        if (options.inputStartId) {
            this.inputStartId = options.inputStartId;
        }
        if (options.inputEndId) {
            this.inputEndId = options.inputEndId;
        }

        if (options.startId) {
            this.startId = options.startId;
            $("#" + this.startId).click(function(){
                _self.clickedStart = true;
                _self.clickedEnd = false;
                _self.updateView();
                $("#" + _self.endId).removeClass('range_focus');
                $("#" + _self.startId).focus();
                $("#" + _self.startId).addClass('range_focus');
            });

            $('span[for-input="'+_self.startId+'"]').click(function(){
                $("#" + _self.startId).trigger('click');
            });
        }

        if (options.endId) {
            this.endId = options.endId;
            $("#" + this.endId).click(function(){
                _self.clickedStart = false;
                _self.clickedEnd = true;
                _self.show();
                $("#" + _self.startId).removeClass('range_focus')
                $("#" + _self.endId).focus();
                $("#" + _self.endId).addClass('range_focus');
            });

            $('span[for-input="'+_self.endId+'"]').click(function(){
                $("#" + _self.endId).trigger('click');
            });
        }

        this.show_prices_for_booking = options.showPricesBlock;

        if (options.noRightBlock == true) {
            $(".right").remove();
        }
        if (options.noInputBlock == true) {
            $(".daterangepicker_input").remove();
        }
        //
        // handle all the possible options overriding defaults
        //
        if (typeof options.locale === 'object') {

            if (typeof options.locale.format === 'string')
                this.locale.format = options.locale.format;
            
            if (typeof options.locale.formatView === 'string')
                this.locale.formatView = options.locale.formatView;

            if (typeof options.locale.separator === 'string')
                this.locale.separator = options.locale.separator;

            if (typeof options.locale.daysOfWeek === 'object')
                this.locale.daysOfWeek = options.locale.daysOfWeek.slice();

            if (typeof options.locale.monthNames === 'object')
              this.locale.monthNames = options.locale.monthNames.slice();

            if (typeof options.locale.firstDay === 'number')
              this.locale.firstDay = options.locale.firstDay;

            if (typeof options.locale.applyLabel === 'string')
              this.locale.applyLabel = options.locale.applyLabel;

            if (typeof options.locale.cancelLabel === 'string')
              this.locale.cancelLabel = options.locale.cancelLabel;

            if (typeof options.locale.weekLabel === 'string')
              this.locale.weekLabel = options.locale.weekLabel;

            if (typeof options.locale.customRangeLabel === 'string')
              this.locale.customRangeLabel = options.locale.customRangeLabel;
        }

        if (typeof options.startDate === 'string')
            this.startDate = moment(options.startDate, this.locale.format);

        if (typeof options.endDate === 'string')
            this.endDate = moment(options.endDate, this.locale.format);

        if (typeof options.minDate === 'string')
            this.minDate = moment(options.minDate, this.locale.format);

        if (typeof options.maxDate === 'string')
            this.maxDate = moment(options.maxDate, this.locale.format);

        if (typeof options.startDate === 'object')
            this.startDate = moment(options.startDate);

        if (typeof options.endDate === 'object')
            this.endDate = moment(options.endDate);

        if (typeof options.minDate === 'object')
            this.minDate = moment(options.minDate);

        if (typeof options.maxDate === 'object')
            this.maxDate = moment(options.maxDate);

        // sanity check for bad options
        if (this.minDate && this.startDate.isBefore(this.minDate))
            this.startDate = this.minDate.clone();

        // sanity check for bad options
        if (this.maxDate && this.endDate.isAfter(this.maxDate))
            this.endDate = this.maxDate.clone();

        if (typeof options.applyClass === 'string')
            this.applyClass = options.applyClass;

        if (typeof options.cancelClass === 'string')
            this.cancelClass = options.cancelClass;

        if (typeof options.dateLimit === 'object')
            this.dateLimit = options.dateLimit;

        if (typeof options.opens === 'string')
            this.opens = options.opens;

        if (typeof options.drops === 'string')
            this.drops = options.drops;

        if (typeof options.showWeekNumbers === 'boolean')
            this.showWeekNumbers = options.showWeekNumbers;

        if (typeof options.buttonClasses === 'string')
            this.buttonClasses = options.buttonClasses;

        if (typeof options.buttonClasses === 'object')
            this.buttonClasses = options.buttonClasses.join(' ');

        if (typeof options.showDropdowns === 'boolean')
            this.showDropdowns = options.showDropdowns;

        if (typeof options.singleDatePicker === 'boolean') {
            this.singleDatePicker = options.singleDatePicker;
            if (this.singleDatePicker)
                this.endDate = this.startDate.clone();
        }

        if (typeof options.timePicker === 'boolean')
            this.timePicker = options.timePicker;

        if (typeof options.timePickerSeconds === 'boolean')
            this.timePickerSeconds = options.timePickerSeconds;

        if (typeof options.timePickerIncrement === 'number')
            this.timePickerIncrement = options.timePickerIncrement;

        if (typeof options.timePicker24Hour === 'boolean')
            this.timePicker24Hour = options.timePicker24Hour;

        if (typeof options.autoApply === 'boolean')
            this.autoApply = options.autoApply;

        if (typeof options.autoUpdateInput === 'boolean')
            this.autoUpdateInput = options.autoUpdateInput;

        if (typeof options.linkedCalendars === 'boolean')
            this.linkedCalendars = options.linkedCalendars;

        if (typeof options.isInvalidDate === 'function')
            this.isInvalidDate = options.isInvalidDate;

        // update day names order to firstDay
        if (this.locale.firstDay != 0) {
            var iterator = this.locale.firstDay;
            while (iterator > 0) {
                this.locale.daysOfWeek.push(this.locale.daysOfWeek.shift());
                iterator--;
            }
        }

        var start, end, range;

        this.validStart(options.startDate);  
        this.validEnd(options.endDate);

        //fix
        if (options.getSpecialPrice == true) {  
            this.get_special_prices = true;
        }
        if (options.setSpecialPrice == true) {  
            this.set_special_prices = true;
        }
       
        if (options.listingData) {
            this.listingData = options.listingData;
        }

        if (options.curCurrency) {
            this.curCurrency = options.curCurrency;
        }

        if (options.siteUrl) {
            this.siteUrl = options.siteUrl;
        }

        //fix
        if (options.singlBlock == true) {  
            this.singlBlock = true;
        }

        //fix
        if (options.includePeriods == true) {
            this.listing_periods = this.getListingPeriods();
        }

        if (this.singlBlock) {
            this.container.addClass('opens');
        }
 
        if (options.noDaysOfWeek == true) {
            this.noDaysOfWeek = true;
        }

        if (options.idRangepicker) {
            this.daterangepickerId = options.idRangepicker;
            this.container.attr('id', options.idRangepicker);
        }

        this.pricesBlockId = 'exclusive_price_' + this.daterangepickerId;
        this.pricesContentId = 'prices_content_' + this.daterangepickerId;
        this.pricesTableId = 'table_content_' + this.daterangepickerId;
        this.priceDetailsId = 'price_details_' + this.daterangepickerId;
        this.detailsBlockId = 'details_block_' + this.daterangepickerId;

        this.container.find('.calendar')
            .on('click.daterangepicker', '.prev', $.proxy(this.clickPrev, this))
            .on('click.daterangepicker', '.next', $.proxy(this.clickNext, this))
            .on('click.daterangepicker', 'td.available', $.proxy(this.clickDate, this))
            .on('mouseenter.daterangepicker', 'td.available', $.proxy(this.hoverDate, this))
            .on('mouseleave.daterangepicker', 'td.available', $.proxy(this.updateFormInputs, this));

        this.container.find('.ranges')
            .on('click.daterangepicker', 'button.applyBtn', $.proxy(this.clickApply, this))
            .on('click.daterangepicker', 'button.cancelBtn', $.proxy(this.clickCancel, this))
            .on('click.daterangepicker', 'li', $.proxy(this.clickRange, this))
            .on('mouseenter.daterangepicker', 'li', $.proxy(this.hoverRange, this))
            .on('mouseleave.daterangepicker', 'li', $.proxy(this.updateFormInputs, this));

        //fix
        var _this = this;
        this.parent_popup_bg = $('#' + this.startId).closest('div[id^="user_load_content_bg"]');
        this.parent_popup_bg.scroll(function() {
            _this.move();
        });

        this.parent_popup = $('#' + this.startId).closest('div[id^="user_load_content"]');

        $('#' + this.endId).keyup(function(e){
            _this.endElementChanged(e);
        })

        if (this.element.is('input')) {
            this.element.on({
                'click.daterangepicker': $.proxy(this.show, this),
                'focus.daterangepicker': $.proxy(this.show, this),
                'keyup.daterangepicker': $.proxy(this.elementChanged, this),
                'keydown.daterangepicker': $.proxy(this.keydown, this)
            });
        } else {
            this.element.on('click.daterangepicker', $.proxy(this.toggle, this));
        }
        //
        // if attached to a text input, set the initial value
        //
        if (this.element.is('input') && !this.singleDatePicker && this.autoUpdateInput) {
            this.element.val(this.startDate.format(this.locale.format) + this.locale.separator + this.endDate.format(this.locale.format));
            this.element.trigger('change');
        } else if (this.element.is('input') && this.autoUpdateInput) {
            this.element.val(this.startDate.format(this.locale.format));
            this.element.trigger('change');
        }
        
        if (this.singleDatePicker) {
            this.minDate = moment(0);
            if (!options.startDate || !Date.parse(this.startDate)) {
                this.startDate = moment().startOf('day');
            }
        } else {
            if(!options.minDate) {
                this.minDate = moment().startOf('day');
            }
        }

        this.parent_popup_height_val = this.parent_popup.height();
    };

    DateRangePicker.prototype = {

        constructor: DateRangePicker,

        completeShow: function() {
            this.parent_popup.height(this.container.height() + this.parent_popup_height_val);
        },

        callback: function() {
            if (!$('#' + this.startId).html()) {
                this.startDate = null;
            }
            $('#' + this.inputStartId).val(this.startDate);
            if (this.inputEndId) {
                if (!$('#' + this.endId).html()) {
                    this.endDate = null;
                }
                $('#' + this.inputEndId).val(this.endDate);
            }
        },

        //fix
        createCalendar: function() {
            var template = '<div class="daterangepicker dropdown-menu">' +
                                '<div class="calendar left">' +           
                                    '<div class="calendar-table"></div>' +
                                '</div>' +
                            '</div>';
            return template;                
        },

        //fix
        getListingPeriods: function() {
            return JSON.parse(localStorage.getItem('listing_periods'));
        },

        getSpecialPrices: function(date) {
            _self = this;
            var send_date = date.format('YYYY-MM');
            var id = this.listingData.id;
            console.log(this.listingData);
            var id_price = localStorage.getItem('periodId');
            var days_in_month = date.daysInMonth();
            var date_end = send_date + '-' + days_in_month;
            var date_start = send_date + '-01';
            var url = this.siteUrl + this.urlSpecialPrices;
            var params = {  'where' : 
                            {   'id_listing' : id,  
                                'status' : 'open', 
                                'date_start <=' : date_end,
                                'date_end >': date_start,
                            },
                        };
            var order_by = {'date_start' : 'ASC'};
            var convert_currency = true;
                         
            $.ajax({
                url: url,
                type: 'POST',
                dataType: 'json',
                data: {params : params, order_by : order_by, convert_currency : convert_currency},
                success: function (data) {
                    _self.special_prices = {};
                    console.log(data.periods);
                    if (data.periods) {         
                        for(var i = 0; i < data.periods.length; i++) {
                            data.periods[i].date_start = data.periods[i].date_start_tstamp * 1000;
                            data.periods[i].date_end = data.periods[i].date_end_tstamp * 1000;
                        }
                    }
                    _self.special_prices = data.periods;
                    _self.updateCalendars(); 
                }
            });
        },

        validStart: function(dateStart) {
            if (dateStart) {
                if (moment(dateStart, this.locale.format).format('YYYY-MM-DD') == 'Invalid date') {
                    this.no_highlight_periods = true;
                } else {
                    var start = moment(dateStart, this.locale.format);
                    var now_date = Date.parse(moment().startOf('day').toDate());
                    var start_date = Date.parse(start.toDate());
                    if (!this.singleDatePicker) {
                        if (now_date > start_date) {
                            this.no_highlight_periods = true;
                        }
                    }
                    this.setInputStart(start);
                }
            } else {
                this.no_highlight_periods = true;
            }
        },

        setInputStart: function(dateStart) {
            var formatCut = this.locale.formatView.match(/(.*)(MMMM)(.*)/);
            var start_date = '';
            if (formatCut !== null){
                start_date += dateStart.format(formatCut[1]);
                start_date += this.locale.monthNames[(parseInt(dateStart.format('MM')) - 1)];
                start_date += dateStart.format(formatCut[3]);
            } else {
                start_date += dateStart.format(this.locale.formatView);
            }
//            var start_date = dateStart.format('DD') + ' ' + this.locale.monthNames[(parseInt(dateStart.format('MM')) - 1)] + ' ' + dateStart.format('YYYY');
            $('#' + this.startId).html(start_date);
            $('#' + this.inputStartId).val(dateStart);
        },

        setStartDate: function(startDate) {
            //fix
            this.no_highlight_periods = false;
            if (typeof startDate === 'string')
                this.startDate = moment(startDate, this.locale.format);

            if (typeof startDate === 'object')
                this.startDate = moment(startDate);

            if (!this.timePicker)
                this.startDate = this.startDate.startOf('day');

            if (this.timePicker && this.timePickerIncrement)
                this.startDate.minute(Math.round(this.startDate.minute() / this.timePickerIncrement) * this.timePickerIncrement);

            if (this.minDate && this.startDate.isBefore(this.minDate))
                this.startDate = this.minDate;

            if (this.maxDate && this.startDate.isAfter(this.maxDate))
                this.startDate = this.maxDate;

            if (!this.isShowing)
                this.updateElement();

            this.setInputStart(this.startDate);

            $('#' + this.startId).removeClass('range_focus');
            $('#' + this.endId).focus();
            $('#' + this.endId).addClass('range_focus');

            this.updateView();
        },

        validEnd: function(dateEnd) {
            if (dateEnd) {
                if (moment(dateEnd, this.locale.format).format('YYYY-MM-DD') == 'Invalid date') {
                    this.no_highlight_periods = true;
                } else {
                    this.setInputEnd(moment(dateEnd, this.locale.format));
                }  
            }
        },

        setInputEnd: function(dateEnd) {
            var formatCut = this.locale.formatView.match(/(.*)(MMMM)(.*)/);
            var end_date = '';
            if (formatCut !== null){
                end_date += dateEnd.format(formatCut[1]);
                end_date += this.locale.monthNames[(parseInt(dateEnd.format('MM')) - 1)];
                end_date += dateEnd.format(formatCut[3]);
            } else {
                end_date += dateEnd.format(this.locale.formatView);
            }
//            var end_date = dateEnd.format('DD') + ' ' + this.locale.monthNames[(parseInt(dateEnd.format('MM')) - 1)] + ' ' + dateEnd.format('YYYY');
            $('#' + this.endId).html(end_date);
            $('#' + this.inputEndId).val(dateEnd);
        },

        setEndDate: function(endDate) {
            //fix
            this.no_highlight_periods = false;
            if (typeof endDate === 'string')
                this.endDate = moment(endDate, this.locale.format);

            if (typeof endDate === 'object')
                this.endDate = moment(endDate);

            if (!this.timePicker)
                this.endDate = this.endDate.endOf('day');

            if (this.timePicker && this.timePickerIncrement)
                this.endDate.minute(Math.round(this.endDate.minute() / this.timePickerIncrement) * this.timePickerIncrement);

            if (this.endDate.isBefore(this.startDate))
                this.endDate = this.startDate.clone();

            if (this.maxDate && this.endDate.isAfter(this.maxDate))
                this.endDate = this.maxDate;

            if (this.dateLimit && this.startDate.clone().add(this.dateLimit).isBefore(this.endDate))
                this.endDate = this.startDate.clone().add(this.dateLimit);

            this.previousRightTime = this.endDate.clone();

            if (!this.isShowing)
                this.updateElement();

            this.setInputEnd(this.endDate);   

            this.updateView();
        },

        isInvalidDate: function() {
            return false;
        },

        updateView: function() {
            this.updateMonthsInView();
            if (this.get_special_prices) {
                this.getSpecialPrices(this.leftCalendar.month);
            } else {
                this.updateCalendars(); 
            }
            this.updateFormInputs();
        },

        updateMonthsInView: function() {
            if (!this.singleDatePicker) {
                if (this.startDate) {
                    this.leftCalendar.month = this.startDate.clone().date(2); 
                } else {
                    this.leftCalendar.month = this.minDate.clone().date(2); 
                }
                if (this.endDate) {
                    this.leftCalendar.month = this.endDate.clone().date(2); 
                }
                if (this.clickedEnd) {
                    if (this.endDate) {
                        this.leftCalendar.month = this.endDate.clone().date(2); 
                    }
                }
                if (this.clickedStart) {
                    if (this.startDate) {
                        this.leftCalendar.month = this.startDate.clone().date(2); 
                    }
                }
            } else {
                if (this.startDate) {
                    this.leftCalendar.month = this.startDate.clone().date(2); 
                } else {
                    this.leftCalendar.month = moment().startOf('day').clone().date(2); 
                }
            }
        },

        updateCalendars: function() {
            this.renderCalendar('left');

            //highlight any predefined range matching the current start and end dates
            this.container.find('.ranges li').removeClass('active');
            if (this.endDate == null) return;

            var customRange = true;
            var i = 0;
            for (var range in this.ranges) {
                if (this.timePicker) {
                    if (this.startDate.isSame(this.ranges[range][0]) && this.endDate.isSame(this.ranges[range][1])) {
                        customRange = false;
                        this.chosenLabel = this.container.find('.ranges li:eq(' + i + ')').addClass('active').html();
                        break;
                    }
                } else {
                    //ignore times when comparing dates if time picker is not enabled
                    if (this.startDate.format('YYYY-MM-DD') == this.ranges[range][0].format('YYYY-MM-DD') && this.endDate.format('YYYY-MM-DD') == this.ranges[range][1].format('YYYY-MM-DD')) {
                        customRange = false;
                        this.chosenLabel = this.container.find('.ranges li:eq(' + i + ')').addClass('active').html();
                        break;
                    }
                }
                i++;
            }
            if (customRange) {
                this.chosenLabel = this.container.find('.ranges li:last').addClass('active').html();
                this.showCalendars();
            }
        },

        renderCalendar: function(side) {
            //
            // Build the matrix of dates that will populate the calendar
            //
            var calendar = side == 'left' ? this.leftCalendar : this.rightCalendar;
            var month = calendar.month.month();
            var year = calendar.month.year();
            var hour = 0;
            var minute = 0;
            var second = 0;
            var daysInMonth = moment([year, month]).daysInMonth();
            var firstDay = moment([year, month, 1]);
            var lastDay = moment([year, month, daysInMonth]);
            var lastMonth = moment(firstDay).subtract(1, 'month').month();
            var lastYear = moment(firstDay).subtract(1, 'month').year();
            var daysInLastMonth = moment([lastYear, lastMonth]).daysInMonth();
            var dayOfWeek = firstDay.day();          
            //initialize a 6 rows x 7 columns array for the calendar
            var calendar = [];
            calendar.firstDay = firstDay;
            calendar.lastDay = lastDay;

            for (var i = 0; i < 6; i++) {
                calendar[i] = [];
            }
            //populate the calendar with date objects
            var startDay = daysInLastMonth - dayOfWeek + this.locale.firstDay + 1;
            if (startDay > daysInLastMonth)
                startDay -= 7;
            if (dayOfWeek == this.locale.firstDay)
                startDay = daysInLastMonth - 6;
            var curDate = moment([lastYear, lastMonth, startDay, hour, minute, second]);
            var col, row;
            for (var i = 0, col = 0, row = 0; i < 42; i++, col++, curDate = moment(curDate).add(24, 'hour')) {
                if (i > 0 && col % 7 === 0) {
                    col = 0;
                    row++;
                }
                calendar[row][col] = curDate.clone().hour(hour).minute(minute).second(second);
                if (this.minDate && calendar[row][col].format('YYYY-MM-DD') == this.minDate.format('YYYY-MM-DD') && calendar[row][col].isBefore(this.minDate) && side == 'left') {
                    calendar[row][col] = this.minDate.clone();
                }
                if (this.maxDate && calendar[row][col].format('YYYY-MM-DD') == this.maxDate.format('YYYY-MM-DD') && calendar[row][col].isAfter(this.maxDate) && side == 'right') {
                    calendar[row][col] = this.maxDate.clone();
                }
            }
            //make the calendar object available to hoverDate/clickDate
            
            this.leftCalendar.calendar = calendar;
            //
            // Display the calendar
            //
            var minDate = side == 'left' ? this.minDate : this.startDate;
            var maxDate = this.maxDate;
            var selected = side == 'left' ? this.startDate : this.endDate;
            this.tableCondensedId = 'table_condensed_' + this.daterangepickerId; 
            var html = '<table id="'+ this.tableCondensedId +'" class="table-condensed">';
            html += '<thead>';
            html += '<tr>';
            // add empty cell for week number
            if (this.showWeekNumbers)
                html += '<th></th>';
            if ((!minDate || minDate.isBefore(calendar.firstDay)) && (!this.linkedCalendars || side == 'left')) {
                html += '<th colspan="2" class="prev available"><i class="fa fa-long-arrow-left"></i></th>';
            } else {
                html += '<th colspan="2"></th>';
            }

            var dateHtml = this.locale.monthNames[calendar[1][1].month()] + calendar[1][1].format(" YYYY");

            html += '<th colspan="3" class="month">' + dateHtml + '</th>';
            if ((!maxDate || maxDate.isAfter(calendar.lastDay)) && (!this.linkedCalendars || side == 'right' || this.singleDatePicker)) {
                html += '<th colspan="2" class="next available"><i class="fa fa-long-arrow-right"></i></th>';
            } else {
                html += '<th colspan="2"></th>';
            }

            html += '</tr>';
         
            html += '</thead>';
            html += '<tbody>';

            //adjust maxDate to reflect the dateLimit setting in order to
            //grey out end dates beyond the dateLimit
            if (this.endDate == null && this.dateLimit) {
                var maxLimit = this.startDate.clone().add(this.dateLimit).endOf('day');
                if (!maxDate || maxLimit.isBefore(maxDate)) {
                    maxDate = maxLimit;
                }
            }
            
            for (var row = 0; row < 6; row++) {
                html += '<tr>';

                for (var col = 0; col < 7; col++) {

                    var classes = [];             

                    //highlight today's date
                    if (calendar[row][col].isSame(new Date(), "day"))
                        classes.push('today');

                    //highlight weekends
                    if (calendar[row][col].isoWeekday() > 5)
                        classes.push('weekend');

                    //grey out the dates in other months displayed at beginning and end of this calendar
                    if (calendar[row][col].month() != calendar[1][1].month())
                        classes.push('off');

                    //don't allow selection of dates before the minimum date
                    if (this.minDate && calendar[row][col].isBefore(this.minDate, 'day'))
                        classes.push('disabled');

                    //don't allow selection of dates after the maximum date
                    if (maxDate && calendar[row][col].isAfter(maxDate, 'day'))
                        classes.push('disabled');

                    //don't allow selection of date if a custom function decides it's invalid
                    if (this.isInvalidDate(calendar[row][col]))
                        classes.push('disabled');

                    //highlight the currently selected start date
                    //fix
                    if (!this.no_highlight_periods) {
                        if (this.startDate) {
                            if (calendar[row][col].format('YYYY-MM-DD') == this.startDate.format('YYYY-MM-DD')) {
                                classes.push('active', 'start-date');
                            }
                        }
                    }

                    //highlight the currently selected end date
                    //fix
                    if (!this.no_highlight_periods) {
                        if (this.endDate != null && calendar[row][col].format('YYYY-MM-DD') == this.endDate.format('YYYY-MM-DD'))
                            classes.push('active', 'end-date');
                    }

                    //highlight dates in-between the selected dates
                    if (!this.no_highlight_periods) {
                        if (this.endDate != null && calendar[row][col] > this.startDate && calendar[row][col] < this.endDate) {
                            if (this.startDate) {
                                classes.push('in-range');
                            }
                        }
                    }

                    //fix
                    //if day in periods
                    var periods = this.listing_periods;
                    var day = Date.parse(calendar[row][col]);
                    for(i in periods) {
                        if ((day >= periods[i]['date_start']) && (day <= periods[i]['date_end'])) {
                            classes.push('disabled', 'listing_periods');
                        }               
                    }

                    //fix
                    //if day in special_prices
                    var price_title = '';
                    if (this.get_special_prices) {
                        var special_prices = this.special_prices;              
                        for(i in special_prices) {                  
                            special_prices[i]['date_start'];
                            if ((day >= special_prices[i]['date_start']) && (day <= special_prices[i]['date_end'])) {
                                classes.push('special_price', 'special_price_range');
                                price_title = special_prices[i].price_str;
                            }               
                        }
                    }

                    var cname = '', disabled = false;
                    for (var i = 0; i < classes.length; i++) {
                        cname += classes[i] + ' ';
                        if (classes[i] == 'disabled')
                            disabled = true;
                    }
                    if (!disabled)
                        cname += 'available';


                    html += '<td title="' + price_title + '" class="'  + cname.replace(/^\s+|\s+$/g, '') + '" data-title="' + 'r' + row + 'c' + col + '">' + calendar[row][col].date() + '</td>';
                }
                html += '</tr>';
            }

            html += '</tbody>';
            html += '</table>';

            this.container.find('.calendar.' + side + ' .calendar-table').html(html);

            if (this.show_prices_for_booking) {
                $('#' + this.tableCondensedId).after(this.createSpecialPricesBlock());
                if (this.special_prices) {
                    this.setSpecialPrices();            
                } else {
                    $('#'+this.pricesBlockId).hide();
                }
            }
            _self = this;
            $('#' + this.priceDetailsId).click(function(){
                if (!_self.isShowPriceDetails) {
                    _self.isShowPriceDetails = true;
                    _self.parent_popup.height(_self.parent_popup.height() + _self.pricesBlockHeight);
                    _self.move();
                } else {
                    _self.isShowPriceDetails = false;   
                    _self.parent_popup.height(_self.parent_popup.height() - _self.pricesBlockHeight);
                }
            });

            //fix
            if ( this.container.find('td[data-title="r1c0"]').html() == '1') {
                this.container.find('td[data-title="r1c0"]').parent().parent().children(':first-child').hide();
            }
            var val =  this.container.find('td[data-title="r5c0"]').html();
            if (val != '29' && val != '31' && val != '30') {
                this.container.find('td[data-title="r5c0"]').parent().hide();
            }

        },

        createSpecialPricesBlock: function(){
            var block = "<div id='" + this.detailsBlockId + "'><div id='" + this.pricesContentId + "' class='collapse calendar__prices_content'>" 
            + "<div class='calendar__prices_content__standart_price'>"
            + this.listingData.calendar_special_prices_lang.standart_price + ":" + this.listingData.price_str
            + "</div>"
            + "<div id='" + this.pricesBlockId + "' class='calendar__prices_content__exclusive_price'>"
            + "<label>"
            + this.listingData.calendar_special_prices_lang.exclusive_prices.toUpperCase() 
            + "</label>"    
            + "<table id='" + this.pricesTableId + "' class='table'>"
            + "</table></div></div>"
            + "<div class='calendar__price_details'>"
            + "<a data-toggle='collapse' id='" + this.priceDetailsId + "' href='#" + this.pricesContentId + "'>"
            + this.listingData.calendar_special_prices_lang.calendar_prices_details 
            + "</a>"
            +"</div></div>";
            return block;
        },

        setSpecialPrices: function() {
            var date = this.leftCalendar.month.clone();
            var num_days = this.leftCalendar.month.daysInMonth();
            var start_month = Date.parse(date.format('YYYY-MM') + '-01' + ' 00:00:00');
            var end_month = Date.parse(date.format('YYYY-MM') + '-' + num_days + ' 00:00:00');
            var month_str = this.locale.monthNames[(parseInt(this.leftCalendar.month.format('MM')) - 1)];
            var block = "<tbody>";
            this.block_special_prices = this.special_prices;
            this.pricesBlockHeight = 40;
            for(i in this.special_prices) {
                this.pricesBlockHeight += 40;
                if (start_month >= this.special_prices[i]['date_start']) {
                    this.block_special_prices[i]['date_start'] = start_month;
                } else {
                    this.block_special_prices[i]['date_start'] = this.special_prices[i]['date_start'];
                }
                if (end_month <= this.special_prices[i]['date_end']) {
                    this.block_special_prices[i]['date_end'] = end_month;         
                } else {
                    this.block_special_prices[i]['date_end'] = this.special_prices[i]['date_end']
                }

                var start_day = moment(new Date(this.block_special_prices[i]['date_start'])).format('DD');     
                var end_day = moment(new Date(this.block_special_prices[i]['date_end'])).format('DD'); 
                var date_range = start_day + '-' + end_day;
                if (start_day == end_day) {
                    date_range = start_day;
                }

                block += "<tr><th class='left'>" + date_range +"<span class='month'>" + ' ' + month_str 
                + "</span></th><th class='right'>"
                + this.special_prices[i].price_str
                + "</th></tr>"
            }
            block += "</tbody>";

            $('#'+this.pricesTableId).append(block);

        },

        renderTimePicker: function(side) {

        },

        updateFormInputs: function() {
           
        },

        move: function() {
            var parentOffset = { top: 0, left: 0 },
                containerTop;
            var parentRightEdge = $(window).width();
            if (!this.parentEl.is('body')) {
                parentOffset = {
                    top: this.parentEl.offset().top - this.parentEl.scrollTop(),
                    left: this.parentEl.offset().left - this.parentEl.scrollLeft()
                };
                parentRightEdge = this.parentEl[0].clientWidth + this.parentEl.offset().left;
            }

            if (this.drops == 'up')
                containerTop = this.element.offset().top - this.container.outerHeight() - parentOffset.top;
            else
                containerTop = this.element.offset().top + this.element.outerHeight() - parentOffset.top;
            this.container[this.drops == 'up' ? 'addClass' : 'removeClass']('dropup');

            this.container.css({
                position: 'absolute',
            });

            if (this.opens == 'left') {
                this.container.css({
                    top: containerTop,
                    right: parentRightEdge - this.element.offset().left - this.element.outerWidth(),
                    left: 'auto'
                });
                if (this.container.offset().left < 0) {
                    this.container.css({
                        right: 'auto',
                        left: 9
                    });
                }
            } else if (this.opens == 'center') {
                this.container.css({
                    top: containerTop,
                    left: this.element.offset().left - parentOffset.left + this.element.outerWidth() / 2
                            - this.container.outerWidth() / 2,
                    right: 'auto'
                });
                if (this.container.offset().left < 0) {
                    this.container.css({
                        right: 'auto',
                        left: 9
                    });
                }
            } else {
                this.container.css({
                    top: containerTop,
                    left: this.element.offset().left - parentOffset.left,
                    right: 'auto'
                });
                if (this.container.offset().left + this.container.outerWidth() > $(window).width()) {
                    this.container.css({
                        left: 'auto',
                        right: 0
                    });
                }
            }

            if ($(window).width() <= 344) {
                
                this.container.css({
                    left: 14,
                    right: 14,
                 
                });
                if ($(window).width() < 328) {
                    var cont_marg = $(window).width() - 320;
                    if (cont_marg < 4) {
                        cont_marg = 4;
                    }
                    this.container.find('.calendar').css({
                        'margin-left': cont_marg + 'px',
                        'margin-right': cont_marg + 'px',
                    });
                }
            }
        },

        show: function(e) { 
            this.isShowPriceDetails = false;
            if (this.isShowing) return;
            // Create a click proxy that is private to this instance of datepicker, for unbinding
            this._outsideClickProxy = $.proxy(function(e) { this.outsideClick(e); }, this);
            // Bind global datepicker mousedown for hiding and
            $(document)
              .on('mousedown.daterangepicker', this._outsideClickProxy)
              // also support mobile devices
              .on('touchend.daterangepicker', this._outsideClickProxy)
              // also explicitly play nice with Bootstrap dropdowns, which stopPropagation when clicking them
              .on('click.daterangepicker', '[data-toggle=dropdown]', this._outsideClickProxy)
              // and also close when focus changes to outside the picker (eg. tabbing between controls)
              .on('focusin.daterangepicker', this._outsideClickProxy);

            // Reposition the picker if the window is resized while it's open
            $(window).on('resize.daterangepicker', $.proxy(function(e) { this.move(e); }, this));
            //fix
            if (this.startDate) {
                this.oldStartDate = this.startDate.clone();
            } else {
                if (this.singleDatePicker) {
                    this.no_highlight_periods = true;
                    this.startDate = moment().startOf('day'); 
                }
            }
            if (this.endDate) {
                this.oldEndDate = this.endDate.clone();
            }
            if (this.endDate) {
                this.previousRightTime = this.endDate.clone();
            }
            this.updateView();
            this.container.show();
            this.move();
            this.element.trigger('show.daterangepicker', this);
            this.isShowing = true;
            this.parent_popup.height(this.container.height() + this.parent_popup_height_val);
        },

        hide: function(e) {
            this.callback();
            this.updateElement();
            $(document).off('.daterangepicker');
            $(window).off('.daterangepicker');
            this.container.hide();
            this.element.trigger('hide.daterangepicker', this);
            this.isShowing = false;
            $('#' + this.startId).removeClass('range_focus');
            $('#' + this.endId).removeClass('range_focus');
            this.isShowPriceDetails = false;
            $('#' +  this.pricesContentId).hide();
            this.parent_popup.height(this.parent_popup_height_val);
        },

        toggle: function(e) {
            if (this.isShowing) {              
                this.hide();
            } else {
                this.show();
            }
        },

        outsideClick: function(e) {
            var target = $(e.target);
            // if the page is clicked anywhere except within the daterangerpicker/button
            // itself then call this.hide()
            if (
                // ie modal dialog fix
                e.type == "focusin" ||
                target.closest(this.element).length ||
                target.closest(this.container).length ||
                target.closest('.calendar-table').length
                ) return;
            this.hide();
        },

        showCalendars: function() {                
            this.container.addClass('show-calendar');
            this.move();
            this.element.trigger('showCalendar.daterangepicker', this);
            this.completeShow();
        },

        hideCalendars: function() {
            this.container.removeClass('show-calendar');
            this.element.trigger('hideCalendar.daterangepicker', this);
        },

        hoverRange: function(e) {
            //ignore mouse movements while an above-calendar text input has focus
            
        },

        clickRange: function(e) {
            var label = e.target.innerHTML;
            this.chosenLabel = label;
            if (label == this.locale.customRangeLabel) {
                this.showCalendars();
            } else {
                var dates = this.ranges[label];
                this.startDate = dates[0];
                this.endDate = dates[1];

                if (!this.timePicker) {
                    this.startDate.startOf('day');
                    this.endDate.endOf('day');
                }

                this.hideCalendars();
                this.clickApply();
            }
        },

        clickPrev: function(e) {
            this.leftCalendar.month.subtract(1, 'month');
            if (this.get_special_prices) {
                this.getSpecialPrices(this.leftCalendar.month);
            } else {
                this.updateCalendars(); 
            }
        },

        clickNext: function(e) {         
            this.leftCalendar.month.add(1, 'month');
            if (this.get_special_prices) {
                this.getSpecialPrices(this.leftCalendar.month);
            } else {
                this.updateCalendars(); 
            }
        },

        hoverDate: function(e) {
            //ignore mouse movements while an above-calendar text input has focus
            //ignore dates that can't be selected
            if (!$(e.target).hasClass('available')) return;

            //have the text inputs above calendars reflect the date being hovered over
            var title = $(e.target).attr('data-title');
            var row = title.substr(1, 1);
            var col = title.substr(3, 1);
            var cal = $(e.target).parents('.calendar');
            var date = cal.hasClass('left') ? this.leftCalendar.calendar[row][col] : this.rightCalendar.calendar[row][col];

            //highlight the dates between the start date and the date being hovered as a potential end date
            var leftCalendar = this.leftCalendar;
            var rightCalendar = this.rightCalendar;
            var startDate = this.startDate;
            var endDate = this.endDate;
            //fix
            var _this = this;
            if (this.endDate && !startDate &&  $('#' + this.startId).html() == '') {
                this.container.find('.calendar td').each(function(index, el) {
                    //skip week numbers, only look at dates
                    if ($(el).hasClass('week')) return;
                    var title = $(el).attr('data-title');
                    var row = title.substr(1, 1);
                    var col = title.substr(3, 1);
                    var cal = $(el).parents('.calendar');
                    var dt = cal.hasClass('left') ? leftCalendar.calendar[row][col] : rightCalendar.calendar[row][col];

                    if (dt.isBefore(endDate) && dt.isAfter(date) && !($('#' + _this.endId).attr('class') == 'form-control range_focus') ) {
                        $(el).addClass('in-range');             
                    } else {
                        $(el).removeClass('in-range');
                    }
                });
            }

            if (!this.endDate) {
                this.container.find('.calendar td').each(function(index, el) {
                    //skip week numbers, only look at dates
                    if ($(el).hasClass('week')) return;
                    var title = $(el).attr('data-title');
                    var row = title.substr(1, 1);
                    var col = title.substr(3, 1);
                    var cal = $(el).parents('.calendar');
                    var dt = cal.hasClass('left') ? leftCalendar.calendar[row][col] : rightCalendar.calendar[row][col];
                    if (dt.isAfter(startDate) && dt.isBefore(date)) {
                        //fix
                        if (!($('#' + _this.startId).hasClass('range_focus')) && !($('#' + _this.endId).html() == '') ) {
                            $(el).addClass('in-range');
                        } else if (($('#' + _this.endId).hasClass('range_focus'))) {
                            $(el).addClass('in-range');
                        }
                    } else {
                        $(el).removeClass('in-range');
                    }
                });
            }
            //fix for listing periods
            if (this.listing_periods) {
                this.container.find('.calendar td').each(function(index, el) {
                    var cal = $(el).parents('.calendar');
                    var dt = cal.hasClass('left') ? leftCalendar.calendar[row][col] : rightCalendar.calendar[row][col];
                    var day = Date.parse(dt.toDate());
                    var is_inactive = false;     
                    if (!_this.no_highlight_periods) {       
                        if (($('#' + _this.startId).hasClass('range_focus')) && endDate) {
                            var end =  Date.parse(endDate.toDate());                
                            var periods = _this.listing_periods;              
                            for(i in periods) {
                                if ((day <= periods[i]['date_end']) && (end >= periods[i]['date_end'])) {
                                    is_inactive = true;
                                    $(el).removeClass('in-range');
                                    $(el).addClass('inactive_period');
                                } 
                            }
                            if (!is_inactive) {
                                $(el).removeClass('inactive_period');
                            }                       
                        }
                        if (($('#' + _this.endId).hasClass('range_focus')) && startDate) {
                            var start =  Date.parse(startDate.toDate());
                            var is_inactive = false;
                            var periods = _this.listing_periods;
                            for(i in periods) {
                                if ((day >= periods[i]['date_start']) && (start <= periods[i]['date_start'])) {
                                    is_inactive = true;
                                    $(el).removeClass('in-range');
                                    $(el).addClass('inactive_period');
                                } 
                            }
                            if (!is_inactive) {
                                $(el).removeClass('inactive_period');
                            }                       
                        }
                    }
                });
            }
        },

        clickDate: function(e) {

            if (!$(e.target).hasClass('available')) return;

            var title = $(e.target).attr('data-title');
            var row = title.substr(1, 1);
            var col = title.substr(3, 1);
            var cal = $(e.target).parents('.calendar');
            var date = cal.hasClass('left') ? this.leftCalendar.calendar[row][col] : this.rightCalendar.calendar[row][col];

            //fix
            if (!$(e.target).hasClass("inactive_period")) {
                if (($('#' + this.startId).hasClass('range_focus'))) {  
                    if (date.isAfter(this.endDate)) {
                        this.endDate = null;
                        $('#' + this.endId).html('');
                        $('#' + this.inputEndId).val('');
                    }
                    if ($('#' + this.endId).html() == '') {
                        this.endDate = null;                  
                    }
                    this.setStartDate(date.clone());
                } else {
                    if (date.isBefore(this.startDate, 'day') || $('#' + this.startId).html() == '') {
                        this.startDate = null;
                        $('#' + this.startId).html(''); 
                        $('#' + this.inputStartId).val('');                 
                        $('#' + this.endId).removeClass('range_focus');
                        $('#' + this.startId).addClass('range_focus');
                        $('#' + this.startId).focus();
                    } 
                    this.setEndDate(date.clone());
                    if ($('#' + this.startId).html() != '' &&  $('#' + this.endId).html() != '') {
                        if (this.autoApply) {
                            this.clickApply();
                        }     
                    }
                }
            }
            //fix
            if (this.singleDatePicker) {
                this.setStartDate(date.clone());
                this.setEndDate(this.startDate);
                this.clickApply();
            }
            this.updateView();
        },

        clickApply: function(e) {
            this.hide();
            this.element.trigger('apply.daterangepicker', this);
        },

        clickCancel: function(e) {
            //fix
            this.hide();
            this.element.trigger('cancel.daterangepicker', this);
        },

        monthOrYearChanged: function(e) {
        },

        timeChanged: function(e) {
        },

        formInputsChanged: function(e) {
            if (start.isValid() && end.isValid()) {
                this.setStartDate(start);
                this.setEndDate(end);
            }
            if (this.get_special_prices) {
                this.getSpecialPrices(this.leftCalendar.month);
            } else {
                this.updateCalendars(); 
            }
        },
        //fix
        endElementChanged: function(e) {
            if ((e.keyCode === 9) || (e.keyCode === 13)) {
                this.hide();
            } else {
                $('#' + this.endId).html('');
                this.endDate = null;
                this.updateView();
            }
        },

        elementChanged: function(e) {
            if ((e.keyCode === 9) || (e.keyCode === 13)) {
                this.hide();
            } else {
                $('#' + this.startId).html('');
                this.startDate = null;
                if (this.singleDatePicker) {
                    this.endDate = null;
                }
                this.updateView();
            }
        },

        keydown: function(e) {
            //hide on tab or enter
            if ((e.keyCode === 9) || (e.keyCode === 13)) {
                this.hide();
            }
        },

        updateElement: function() {
            if (this.element.is('input') && !this.singleDatePicker && this.autoUpdateInput) {
                this.element.val(this.startDate.format(this.locale.format) + this.locale.separator + this.endDate.format(this.locale.format));
                this.element.trigger('change');
            } else if (this.element.is('input') && this.autoUpdateInput) {
                this.element.val(this.startDate.format(this.locale.format));
                this.element.trigger('change');
            }
        },

        remove: function() {
            this.container.remove();
            this.element.off('.daterangepicker');
            this.element.removeData();
        }

    };

    $.fn.daterangepicker = function(options) {
        this.each(function() {
            var el = $(this);
            if (el.data('daterangepicker'))
                el.data('daterangepicker').remove();
            el.data('daterangepicker', new DateRangePicker(el, options));
        });
        return this;
    };
    
    return DateRangePicker;

}));
