<?php

namespace Pg\Modules\Contact\Controllers;

/**
 * Contact module
 *
 * @package     PG_RealEstate
 *
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
use Pg\Libraries\View;

/**
 * Contact admin side controller
 *
 * @package     PG_RealEstate
 * @subpackage  Contact
 *
 * @category    controllers
 *
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Admin_Contact extends \Controller
{

    /**
     * Data filters
     *
     * @var array
     */
    protected $filters = array("all", "unread", "need_approve");

    /**
     * Class constructor
     *
     * @return Admin_contact
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model("Menu_model");
        $this->Menu_model->set_menu_active_item("admin_menu", "system-items");
    }

    /**
     * Manage contacts
     *
     * @param string  $filter          filter data
     * @param string  $order           sorting order
     * @param string  $order_direction sorting direction
     * @param integer $page            page of contacts
     *                                 return void
     */
    public function index($filter = "all", $order = "date_add", $order_direction = "DESC", $page = 1)
    {
        $this->load->model("Contact_model");

        $this->Menu_model->set_menu_active_item("admin_contact_menu", "contact_list_item");

        if (!in_array($filter, $this->filters)) {
            $filter = $this->filters[0];
        }

        $current_settings = isset($_SESSION["contacts_list"]) ? $_SESSION["contacts_list"] : array();
        if (!isset($current_settings["order"])) {
            $current_settings["order"] = "date_add";
        }
        if (!isset($current_settings["order_direction"])) {
            $current_settings["order_direction"] = "DESC";
        }
        if (!isset($current_settings["page"])) {
            $current_settings["page"] = 1;
        }

        $current_settings["page"] = $page;
        $current_settings["filter"] = $filter;

        if (!$order) {
            $order = $current_settings["order"];
        }
        $this->view->assign("order", $order);
        $current_settings["order"] = $order;

        if (!$order_direction) {
            $order_direction = $current_settings["order_direction"];
        }
        $this->view->assign("order_direction", $order_direction);
        $current_settings["order_direction"] = $order_direction;

        $contacts_count = $this->Contact_model->{"get_" . $filter . "_contacts_count"}();

        if (!$page) {
            $page = $current_settings["page"];
        }
        $items_on_page = $this->pg_module->get_module_config("start", "admin_items_per_page");

        $this->load->helper("sort_order");
        $page = get_exists_page_number($page, $contacts_count, $items_on_page);
        $current_settings["page"] = $page;

        $_SESSION["contacts_list"] = $current_settings;

        $sort_links = array(
            "date_add" => site_url() . "admin/contact/index/" . $filter . "/date_add/" . (($order != "date_add" xor $order_direction == "DESC") ? "ASC" : "DESC"),
            "sender" => site_url() . "admin/contact/index/" . $filter . "/sender/" . (($order != "sender" xor $order_direction == "DESC") ? "ASC" : "DESC"),
            "message" => site_url() . "admin/contact/index/" . $filter . "/message/" . (($order != "message" xor $order_direction == "DESC") ? "ASC" : "DESC"),
            "status" => site_url() . "admin/contact/index/" . $filter . "/status/" . (($order != "status" xor $order_direction == "DESC") ? "ASC" : "DESC"),
        );
        $this->view->assign("sort_links", $sort_links);

        if ($contacts_count > 0) {
            $contacts = $this->Contact_model->{"get_" . $filter . "_contacts_list"}($page, $items_on_page, array($order => $order_direction));
            $this->view->assign("contacts", $contacts);
        }

        $this->load->helper('navigation');

        $url = site_url() . "admin/contact/index/" . $filter . "/" . $order . "/" . $order_direction . "/";
        $page_data = get_admin_pages_data($url, $contacts_count, $items_on_page, $page, "briefPage");
        $page_data["date_format"] = $this->pg_date->get_format('date_time_literal', 'st');
        $this->view->assign("page_data", $page_data);

        $filter_data["contact_make_copies"] = $this->pg_module->get_module_config("contact", "contact_make_copies");
        if ($filter_data["contact_make_copies"]) {
            $filter_data["contact_need_approve"] = $this->pg_module->get_module_config("contact", "contact_need_approve");
            $filter_data["all"] = $this->Contact_model->get_all_contacts_count();
            $filter_data["unread"] = $this->Contact_model->get_unread_contacts_count();
            if ($filter_data["contact_need_approve"]) {
                $filter_data["need_approve"] = $this->Contact_model->get_need_approve_contacts_count();
            }
        }
        $this->view->assign("filter", $filter);
        $this->view->assign("filter_data", $filter_data);
        $this->view->setHeader(l("admin_header_contacts_list", "contact"));
        $this->view->render("list");
    }

    /**
     * Manage contacts which need approved
     *
     * @param string  $order           sorting order
     * @param string  $order_direction sorting direction
     * @param integer $page            page of results
     *
     * @return void
     */
    public function need_approve($order = "date_add", $order_direction = "DESC", $page = 1)
    {
        $this->_get_contacts("need_approve", $order, $order_direction, $page);
    }

    /**
     * Show contact data
     *
     * @param integer $id contact identifier
     *
     * @return void
     */
    public function show($id)
    {
        if (!$id) {
            show_404();

            return;
        }

        $this->Menu_model->set_menu_active_item("admin_contact_menu", "contact_list_item");

        $this->load->model("Contact_model");

        $this->Contact_model->mark_contact_as_read($id);

        $current_settings = isset($_SESSION["contacts_list"]) ? $_SESSION["contacts_list"] : array("filter" => "all");
        $this->view->assign("filter", $current_settings["filter"]);

        $this->load->helper('navigation');

        $url = site_url() . "admin/contact/index/" . $current_settings['filter'] . "/" . $order . "/" . $order_direction . "/";
        $page_data = get_admin_pages_data($url, $contacts_count, $items_on_page, $page, "briefPage");
        $page_data["date_format"] = $this->pg_date->get_format('date_time_literal', 'st');
        $this->view->assign("page_data", $page_data);

        $data = $this->Contact_model->get_contact_by_id($id);
        $this->view->assign("data", $data);

        $this->view->setHeader(l("admin_header_contact_show", "contact"));
        $this->view->render("view");
    }

    /**
     * Change read contact status
     *
     * @param integer $id     contact idetnfier
     * @param integer $status read status
     *
     * @return void
     */
    public function read($id, $status)
    {
        if (!$id) {
            return;
        }
        $this->load->model("Contact_model");

        if (intval($status)) {
            $this->Contact_model->mark_contact_as_read($id);
        } else {
            $this->Contact_model->mark_contact_as_unread($id);
        }
        redirect(site_url() . "admin/contact/index");
    }

    /**
     * Send or decline message to contact
     *
     * @param integer $status contact status
     * @param integer $id     contact identifier
     *
     * @return void
     */
    public function send($status, $id = null)
    {
        if (!$id) {
            $id = $this->input->post("ids");
        }
        if (!empty($id)) {
            $this->load->model("Contact_model");
            foreach ((array) $id as $i) {
                if (intval($status)) {
                    $this->Contact_model->send_contact($i);
                } else {
                    $this->Contact_model->decline_contact($i);
                }
            }
            $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_contact_send", "contact"));
        }
        $filter = isset($_SESSION["contacts_list"]['filter']) ? $_SESSION["contacts_list"]['filter'] : "all";
        redirect(site_url() . "admin/contact/index/" . $filter);
    }

    /**
     * Remove contacts
     *
     * @param integer/array $ids contact identifiers
     *
     * @return void
     */
    public function delete($ids = null)
    {
        if (!$ids) {
            $ids = $this->input->post("ids");
        }

        if (!empty($ids)) {
            $this->load->model("Contact_model");
            foreach ((array) $ids as $id) {
                $this->Contact_model->delete_contact($id);
            }
            $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_contact_delete", "contact"));
        }
        $filter = isset($_SESSION["contacts_list"]['filter']) ? $_SESSION["contacts_list"]['filter'] : "all";
        redirect(site_url() . "admin/contact/index/" . $filter);
    }

    /**
     * Edit module settings
     *
     * @return void
     */
    public function settings()
    {
        $this->Menu_model->set_menu_active_item("admin_contact_menu", "contact_settings_item");

        if ($this->input->post("btn_save")) {
            $this->load->model("Contact_model");

            $post_data["contact_make_copies"] = $this->input->post("contact_make_copies");
            $post_data["contact_need_approve"] = $this->input->post("contact_need_approve");
            $post_data["contact_send_mail"] = $this->input->post("contact_send_mail");
            $post_data["contact_admin_email"] = $this->input->post("contact_admin_email");

            $validate_data = $this->Contact_model->validate_contact_settings($post_data);
            if (!empty($validate_data["errors"])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
                $data = $post_data;
            } else {
                foreach ($validate_data["data"] as $setting => $value) {
                    $this->pg_module->set_module_config("contact", $setting, $value);
                }
                $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_settings_saved", "contact"));
                $data = $validate_data["data"];
            }
        } else {
            $data["contact_make_copies"] = $this->pg_module->get_module_config("contact", "contact_make_copies");
            $data["contact_need_approve"] = $this->pg_module->get_module_config("contact", "contact_need_approve");
            $data["contact_send_mail"] = $this->pg_module->get_module_config("contact", "contact_send_mail");
            $data["contact_admin_email"] = $this->pg_module->get_module_config("contact", "contact_admin_email");
        }
        $this->view->assign("data", $data);
        $this->view->setHeader(l("admin_header_contacts_list", "contact"));
        $this->view->render("settings");
    }

}
