<?php

namespace Pg\Modules\Contact_us\Controllers;

/**
 * Contact us module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Contact us api controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	Contact us
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Api_Contact_us extends \Controller
{

    /**
     * Class constructor
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model('Contact_us_model');
    }

    /**
     * Manage contact us reasons
     *
     * @return void
     */
    public function get_reasons()
    {
        $reasons = $this->Contact_us_model->get_reason_list();
        $this->set_api_content('data', array('reasons' => $reasons));
    }

    /**
     * Send contact us form
     *
     * @param int    $id_reason  reason identifier
     * @param string $user_name  user name
     * @param string $user_email user email
     * @param string $subject    contact subject
     * @param string $message    contact message
     *
     * @todo add antispam
     *
     * @return void
     */
    public function send_form()
    {
        $post_data = array(
            'id_reason' => $this->input->post('id_reason', true),
            'user_name' => $this->input->post('user_name', true),
            'user_email' => $this->input->post('user_email', true),
            'subject' => $this->input->post('subject', true),
            'message' => $this->input->post('message', true),
        );
        $validate_data = $this->Contact_us_model->validate_contact_form($post_data);

        if (!empty($validate_data['errors'])) {
            $this->set_api_content('errors', $validate_data['errors']);
            $this->set_api_content('data', array('post_data' => $post_data));
        } else {
            $return = $this->Contact_us_model->send_contact_form($validate_data['data']);
            if (!empty($return['errors'])) {
                $this->set_api_content('errors', $return['errors']);
            } else {
                $this->set_api_content('messages', l('success_send_form', 'contact_us'));
            }
        }
    }

}
