<?php

namespace Pg\Modules\Contact_us\Controllers;

/**
 * Contact us module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
use Pg\Libraries\View;

/**
 * Contact us user side controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	Contact us
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Contact_us extends \Controller
{

    /**
     * Class constructor
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model("Contact_us_model");
    }

    /**
     * Contact us form
     *
     * @return void
     */
    public function index()
    {
        $reasons = $this->Contact_us_model->get_reason_list();
        $this->view->assign('reasons', $reasons);

        if ($this->input->post('btn_save')) {
            $post_data = array(
                "id_reason" => $this->input->post('id_reason', true),
                "user_name" => $this->input->post('user_name', true),
                "user_email" => $this->input->post('user_email', true),
                "subject" => $this->input->post('subject', true),
                "message" => $this->input->post('message', true),
                "captcha_code" => $this->input->post('captcha_code', true),
            );
            $validate_data = $this->Contact_us_model->validate_contact_form($post_data);

            if (!empty($validate_data["errors"])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
                $data = $validate_data["data"];
            } else {
                $return = $this->Contact_us_model->send_contact_form($validate_data["data"]);

                if (!empty($return["errors"])) {
                    $this->system_messages->addMessage(View::MSG_ERROR, $return["errors"]);
                } else {
                    $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_send_form', 'contact_us'));
                }
                redirect(site_url() . "contact_us");
            }
        }

        $this->load->plugin('captcha');
        $vals = array(
            'img_path' => TEMPPATH . '/captcha/',
            'img_url' => SITE_VIRTUAL_PATH . 'temp/captcha/',
            'font_path' => BASEPATH . 'fonts/arial.ttf',
            'img_width' => '120',
            'img_height' => '27',
            'expiration' => 7200,
        );

        $cap = create_captcha($vals);
        $data["captcha"] = $cap['image'];
        $_SESSION["captcha_word"] = $cap['word'];
        $this->view->assign('data', $data);

        $this->load->model('Menu_model');
        $this->Menu_model->breadcrumbs_set_active(l('header_contact_us_form', 'contact_us'));

        $this->view->render('form');
    }

}
