<?php

namespace Pg\Modules\Country_identification\Controllers;

/**
 * Country_identification module
 *
 * @package     PG_RealEstate
 *
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Country_identification user side controller
 *
 * @package     PG_RealEstate
 * @subpackage  Country_identification
 *
 * @category    controllers
 *
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Country_identification extends \Controller
{

    /**
     * Class constructor
     *
     * @return Country_identification
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->helper('cookie');
    }

    /**
     * Module management
     *
     * @return void
     */
    public function index()
    {
        $point = $this->input->post('point');
        if (!$point) {
            return;
        }

        $location = explode(';', $point);
        if (empty($location)) {
            return;
        }
        
        $this->load->model('Countries_model');
        
        $latitude = floatval($location[0]);
        $longitude = floatval($location[1]);

        $closest_cities = $this->Countries_model->get_closest_cities($latitude, $longitude, 10000);
        if (!empty($closest_cities)) {
            $city = $closest_cities[0];
        } else {
            $city = array();
        }

        $this->load->helper('country_identification');
        $this->view->output(country_identification_selector(array('city' => $city, 'is_container' => false)));

        $this->view->render();
    }

    /**
     * Get regions
     *
     */
    public function ajax_get_regions()
    {
        $country_code = $this->input->post('country_code', true);

        $this->load->model("Countries_model");

        $regions = $this->Countries_model->get_regions($country_code);

        $this->view->assign('regions', $regions);

        if (!empty($regions)) {
            $cities = $this->Countries_model->get_cities(null, null, array(), array('where' => array(
                'country_code' => $country_code,
                'id_region' => key($regions),
            )));
        } else {
            $cities = [];
        }

        $this->view->assign('cities', $cities);

        $this->view->render();
    }

    /**
     * Get cities
     *
     */
    public function ajax_get_cities()
    {
        $region_id = (int)$this->input->post('region_id', true);

        $this->load->model("Countries_model");
        $this->view->assign('cities', $this->Countries_model->get_cities(
            null, null, array(), array(
                'where' => array(
                    'id_region' => $region_id,
                )
            )
        ));

        $this->view->render();
    }

    public function change_user_default_country()
    {
        $this->load->helper('lang');
        $return = ['success' => l('location_determined', 'country_identification'), 'error' => ''];
        $country_code = $this->input->post('select_country', true);
        $region_id = (int)$this->input->post('select_region', true);
        $city_id = (int)$this->input->post('select_city', true);

        if ($city_id && $region_id && $country_code) {
            $this->load->model("Countries_model");
            $country = $this->Countries_model->get_country($country_code);
            $region = $this->Countries_model->get_region($region_id);
            $city = $this->Countries_model->get_city($city_id);
            if ($country && $region && $city) {
                $this->load->model('Country_identification_model');
                if ($this->session->userdata('auth_type') === 'user') {
                    $this->load->model('Users_model');
                    $user = $this->Users_model->get_user_by_id($this->session->userdata('user_id'));
                    if (!empty($user)) {
                        $this->Users_model->save_user($user['id'], array(
                            'id_region' => $region['id'],
                            'id_city' => $city['id'],
                            'id_country' => $country['code'],
                        ));
                        $this->Country_identification_model->setLocation($city['id'], $region['id'], $country['code']);
                        $this->session->set_userdata('user_no_change_location', '1');
                    }
                } else {
                    $this->Country_identification_model->setLocation($city['id'], $region['id'], $country['code']);
                }
            } else {
                $return['error'] = true;
            }
        } else {
            $return['error'] = true;
        }

        $this->view->assign($return);
        $this->view->render();
    }

    public function ajax_user_dont_change_location()
    {
        if ($this->input->post('dont_change_location')) {
            $this->session->set_userdata('user_no_change_location', '1');
        }
    }

}
