<?php

/**
 * Currencies module
 *
 * @package     PG_RealEstate
 *
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
/**
 * Currencies management
 *
 * @package     PG_RealEstate
 * @subpackage  Currencies
 *
 * @category    helpers
 *
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!function_exists('currency_format')) {

    /**
     * Returns formatted currency string
     *
     * cur_id - currency id
     * cur_gid - currency gid
     * value - amount
     * template - [abbr][value|dec_part:2|dec_sep:.|gr_sep: ])
     *
     * @param array $params block parameters
     *
     * @return string
     */
    function currency_format($params = array())
    {
        $ci = &get_instance();
        $ci->load->model('Currencies_model');
        $pattern_value = '/\[value\|([^]]*)\]/';

        $default_cur = $ci->Currencies_model->default_currency;

        // Get specified or default currency
        if (!empty($params['cur_gid'])) {
            if ($params['cur_gid'] != $default_cur['gid']) {
                $cur = $ci->Currencies_model->get_currency_by_gid($params['cur_gid']);
                if ($cur['per_base'] && (float) $default_cur['per_base'] && empty($params['use_gid'])) {
                    if (!empty($params['value'])) {
                        $params['value'] *= $cur['per_base'] / $default_cur['per_base'];
                    }
                } else {
                    $default_cur = $cur;
                }
            }
        } elseif (!empty($params['cur_id'])) {
            if ($params['cur_id'] != $default_cur['id']) {
                $cur = $ci->Currencies_model->get_currency_by_id($params['cur_id']);
                if ($cur['per_base'] && $default_cur['per_base']) {
                    if (!empty($params['value'])) {
                        $params['value'] *= $cur['per_base'] / $default_cur['per_base'];
                    }
                } else {
                    $default_cur = $cur;
                }
            }
        }

        if (!empty($params['template'])) {
            $template = $params['template'];
        } else {
            $template = $default_cur['template'];
        }
        
        if (empty($params['value'])) {
            $params['value'] = 0;
        }

        $matches = array();
        // Parse the number format
        preg_match($pattern_value, $template, $matches);
        $value_params = explode('|', $matches[1]);
        foreach ($value_params as $param) {
            $param_arr = explode(':', $param);
            $number_arr[$param_arr[0]] = $param_arr[1];
        }
        // Format number
        if ('-' == $number_arr['dec_part'] || '–' == $number_arr['dec_part']) {
            $value = number_format($params['value'], 0, $number_arr['dec_sep'], $number_arr['gr_sep']);
            $value .= $number_arr['dec_sep'] . '–';
        } else {
            $value = number_format($params['value'], (int) $number_arr['dec_part'], $number_arr['dec_sep'], $number_arr['gr_sep']);
        }

        if (!empty($params['disable_abbr'])) {
            $default_cur['abbr'] = '';
            $default_cur['gid'] = '';
        }

        $str = preg_replace(array($pattern_value, '/(\[abbr\])/', '/(\[gid\])/', '/\s/'), array($value, $default_cur['abbr'], $default_cur['gid'], ' '), $template
        );

        if (empty($params['no_tags']) || false == $params['no_tags']) {
            return '<span dir="ltr">' . trim($str) . '</span>';
        } else {
            return trim($str);
        }
    }
}

if (!function_exists('currency')) {

    /**
     * Return formatted currency string
     *
     * cur_id - currency id
     * cur_gid - currency gid
     * value - amount
     * template - [abbr][value|dec_part:2|dec_sep:.|gr_sep: ])
     *
     * @param array $params block parameters
     *
     * @return string
     */
    function currency($params = array())
    {
        $ci = &get_instance();
        $ci->load->model('Currencies_model');
        $pattern_value = '/\[value\|([^]]*)\]/';

        $default_cur = $ci->Currencies_model->default_currency;

        // Get specified or default currency
        if (!empty($params['cur_gid'])) {
            if ($params['cur_gid'] != $default_cur['gid']) {
                $cur = $ci->Currencies_model->get_currency_by_gid($params['cur_gid']);
                if ($cur['per_base'] && (float) $default_cur['per_base'] && empty($params['use_gid'])) {
                    if (!empty($params['value'])) {
                        $params['value'] *= $cur['per_base'] / $default_cur['per_base'];
                    }
                } else {
                    $default_cur = $cur;
                }
            }
        } elseif (!empty($params['cur_id'])) {
            if ($params['cur_id'] != $default_cur['id']) {
                $cur = $ci->Currencies_model->get_currency_by_id($params['cur_id']);
                if ($cur['per_base'] && $default_cur['per_base']) {
                    if (!empty($params['value'])) {
                        $params['value'] *= $cur['per_base'] / $default_cur['per_base'];
                    }
                } else {
                    $default_cur = $cur;
                }
            }
        }

        if (!empty($params['template'])) {
            $template = $params['template'];
        } else {
            $template = $default_cur['template'];
        }

        if (!empty($params['disable_abbr'])) {
            $default_cur['abbr'] = '';
            $default_cur['gid'] = '';
        }

        if (!empty($params['editable']) && !$ci->router->is_api_class) {
            $ci->view->assign('block_data', $params);
            $value = $ci->view->fetch('helper_currency_output_value', null, 'currencies');

            $ci->view->assign('block_abbr', $default_cur['abbr']);
            $abbr = $ci->view->fetch('helper_currency_output_abbr', null, 'currencies');

            $ci->view->assign('block_gid', $default_cur['gid']);
            $gid = $ci->view->fetch('helper_currency_output_gid', null, 'currencies');

            $str = preg_replace(array($pattern_value, '/(\[abbr\])/', '/(\[gid\])/', '/\s/'), array($value, $abbr, $gid, ' '), $template);

            return $str;
        } else {
            $str = preg_replace(array($pattern_value, '/(\[abbr\])/', '/(\[gid\])/', '/\s/'), array(get($params['value']), get($default_cur['abbr']), get($default_cur['gid']), ' '), $template);

            return '<span dir="ltr">' . trim($str) . '</span>';
        }
    }
}

if (!function_exists('currency_format_regexp')) {

    /**
     * Returns formatted currency regexp string
     *
     * @param array $params block parameters
     *
     * @return string
     */
    function currency_format_regexp($params = array())
    {
        $ci = &get_instance();
        $ci->load->model('Currencies_model');
        $pattern_value = '/\[value\|([^]]*)\]/';
        $value = '';

        $default_cur = $ci->Currencies_model->default_currency;

        if (!empty($params['template'])) {
            $template = $params['template'];
        } else {
            $template = $default_cur['template'];
        }

        $matches = array();
        // Parse the number format
        preg_match($pattern_value, $template, $matches);
        $value_params = explode('|', $matches[1]);
        foreach ($value_params as $param) {
            $param_arr = explode(':', $param);
            $number_arr[$param_arr[0]] = $param_arr[1];
        }
        $ci->view->assign('pattern_value', $pattern_value);

        // Format number
        if ('-' == $number_arr['dec_part'] || '–' == $number_arr['dec_part']) {
            $value = 'number_format(value, 0, \'' . $number_arr['dec_sep'] . '\', \'' . $number_arr['gr_sep'] . '\') + \'' . $number_arr['dec_sep'] . '–\'';
        } else {
            $value = 'number_format(value, ' . ((int) $number_arr['dec_part']) . ', \'' . $number_arr['dec_sep'] . '\', \'' . $number_arr['gr_sep'] . '\')';
        }
        $ci->view->assign('value', $value);

        $template = preg_replace(array('/(\[abbr\])/', '/(\[gid\])/', '/\s/'), array($default_cur['abbr'], $default_cur['gid'], ' '), $template);
        $ci->view->assign('template', $template);

        return $ci->view->fetch('helper_currency_regexp', 'user', 'currencies');
    }
}

if (!function_exists('site_currency_select')) {

    /**
     * Get currency selector
     *
     * @return string
     */
    function site_currency_select()
    {
        $ci = &get_instance();
        $ci->load->model('Currencies_model');
        $currencies = $ci->Currencies_model->get_currency_list();
        foreach ($currencies as $cid => $currency_data) {
            if (!$currency_data["status"]) {
                unset($currencies[$cid]);
            }
        }
        if (count($currencies) <= 1) {
            return '';
        }

        $ci->view->assign("currencies", $currencies);

        return $ci->view->fetch("helper_currency_select", "user", "currencies");
    }
}

if (!function_exists('currency_value')) {

    /**
     * Transform value to specific currency
     *
     * @param array $params block parameters
     *
     * @return float
     */
    function currency_value($params = array())
    {
        $ci = &get_instance();
        $ci->load->model('Currencies_model');
        $pattern_value = '/\[value\|([^]]*)\]/';

        if (!empty($params['to_gid'])) {
            $default_cur = $ci->Currencies_model->get_currency_by_gid($params['to_gid']);
        } else {
            $default_cur = $ci->Currencies_model->default_currency;
        }

        // Get specified or default currency
        if (!empty($params['cur_gid'])) {
            if ($params['cur_gid'] != $default_cur['gid']) {
                $cur = $ci->Currencies_model->get_currency_by_gid($params['cur_gid']);
                if ($cur['per_base'] && (float) $default_cur['per_base'] && empty($params['use_gid'])) {
                    if (!empty($params['value'])) {
                        $params['value'] *= $cur['per_base'] / $default_cur['per_base'];
                    }
                }
            }
        } elseif (!empty($params['cur_id'])) {
            if ($params['cur_id'] != $default_cur['id']) {
                $cur = $ci->Currencies_model->get_currency_by_id($params['cur_id']);
                if ($cur['per_base'] && $default_cur['per_base']) {
                    if (!empty($params['value'])) {
                        $params['value'] *= $cur['per_base'] / $default_cur['per_base'];
                    }
                }
            }
        } else {
            $cur = $ci->Currencies_model->default_currency;
            if ($cur['id'] != $default_cur['id']) {
                if ($cur['per_base'] && $default_cur['per_base']) {
                    if (!empty($params['value'])) {
                        $params['value'] *= $cur['per_base'] / $default_cur['per_base'];
                    }
                }
            }
        }

        return $params['value'];
    }
}
