<?php

namespace Pg\Modules\Dynamic_block_map\Models;

use Pg\Libraries\Setup;

/**
 * Google map block module
 *
 * @package 	PG_RealEstate
 * @copyright 	Copyright (c) 2000-2015 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Google map block install model
 *
 * @package 	PG_RealEstate
 * @subpackage
 * @category	models
 * @copyright 	Copyright (c) 2000-2015 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Dynamic_block_map_install_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    private $modules_data = [];

    /**
     * Constructor
     *
     * @return Listings_install_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
        $this->modules_data = Setup::getModuleData('dynamic_block_map', Setup::TYPE_MODULES_DATA);
    }

    /**
     * Install data of dynamic blocks module
     *
     * @return void
     */
    public function install_dynamic_blocks()
    {
        $this->ci->load->model("Dynamic_blocks_model");
        $area_ids = array();
        foreach ((array) $this->modules_data['dynamic_blocks'] as $block_data) {
            $validate_data = $this->ci->Dynamic_blocks_model->validate_block(null, $block_data);
            if (!empty($validate_data["errors"])) {
                continue;
            }
            $id_block = $this->ci->Dynamic_blocks_model->save_block(null, $validate_data["data"]);

            if (!empty($block_data["area"])) {
                if (!isset($area_ids[$block_data["area"]["gid"]])) {
                    $area = $this->ci->Dynamic_blocks_model->get_area_by_gid($block_data["area"]["gid"]);
                    $area_ids[$block_data["area"]["gid"]] = $area["id"];
                }

                // index area
                $block_data["area"]["id_area"] = $area_ids[$block_data["area"]["gid"]];
                $block_data["area"]["id_block"] = $id_block;

                $validate_data = $this->ci->Dynamic_blocks_model->validate_area_block(null, $block_data["area"]);
                if (empty($validate_data["errors"])) {
                    $this->ci->Dynamic_blocks_model->save_area_block(null, $validate_data["data"]);
                }
            }

            if (!empty($block_data["presets"])) {
                foreach ($block_data["presets"] as $preset_data) {
                    $preset = $this->ci->Dynamic_blocks_model->get_preset_by_gid($preset_data['gid_preset']);
                    $preset_data['id_preset'] = $preset['id'];
                    $preset_data['id_block'] = $id_block;
                    $validate_data = $this->ci->Dynamic_blocks_model->validate_preset_block(null, $preset_data);
                    if (!empty($validate_data["errors"])) {
                        continue;
                    }
                    $this->ci->Dynamic_blocks_model->save_preset_block(null, $validate_data["data"]);
                }
            }
        }
    }

    /**
     * Import languages of dynamic blocks module
     *
     * @param array $langs_ids languages identifiers
     * @return boolean
     */
    public function install_dynamic_blocks_lang_update($langs_ids = null)
    {
        if (!is_array($langs_ids)) {
            $langs_ids = (array) $langs_ids;
        }
        if (empty($langs_ids)) {
            return false;
        }

        $langs_file = $this->ci->Install_model->language_file_read("dynamic_block_map", "dynamic_blocks", $langs_ids);
        if (!$langs_file) {
            log_message("info", "Empty dynamic_blocks langs data");
            return false;
        }

        $this->ci->load->model("Dynamic_blocks_model");

        $data = array();
        foreach ((array) $this->modules_data['dynamic_blocks'] as $block_data) {
            $block = $this->ci->Dynamic_blocks_model->get_block_by_gid($block_data["gid"]);
            if (!$block) {
                continue;
            }
            $data[] = $block;
        }

        $this->ci->Dynamic_blocks_model->update_langs($data, $langs_file, $langs_ids);
    }

    /**
     * Export languages of dynamic blocks module
     *
     * @param array $langs_ids languages identifiers
     * @return array
     */
    public function install_dynamic_blocks_lang_export($langs_ids = null)
    {
        if (!is_array($langs_ids)) {
            $langs_ids = (array) $langs_ids;
        }
        $this->ci->load->model("Dynamic_blocks_model");
        $data = array();

        foreach ((array) $this->modules_data['dynamic_blocks'] as $block_data) {
            $block = $this->ci->Dynamic_blocks_model->get_block_by_gid($block_data["gid"]);
            if (!$block) {
                continue;
            }
            $data[] = $block;
        }

        $langs = $this->ci->Dynamic_blocks_model->export_langs($data, $langs_ids);
        return array("dynamic_blocks" => $langs);
    }

    /**
     * Unistall data of dynamic blocks module
     *
     * @return void
     */
    public function deinstall_dynamic_blocks()
    {
        $this->ci->load->model("Dynamic_blocks_model");
        foreach ((array) $this->modules_data['dynamic_blocks'] as $block_data) {
            $this->ci->Dynamic_blocks_model->delete_block_by_gid($block_data["gid"]);
        }
    }

    /**
     * Install data of geomap module
     *
     * @return void
     */
    public function install_geomap()
    {
        // add geomap settings
        $this->ci->load->model("geomap/models/Geomap_settings_model");

        foreach ((array) $this->modules_data['geomap'] as $settings) {
            $map_gid = $settings["map_gid"];
            unset($settings["map_gid"]);

            $id_user = intval($settings["id_user"]);
            unset($settings["id_user"]);

            $id_object = intval($settings["id_object"]);
            unset($settings["id_object"]);

            $gid = $settings["gid"];
            unset($settings["gid"]);

            $validate_data = $this->ci->Geomap_settings_model->validate_settings($settings);
            if (!empty($validate_data["errors"])) {
                log_message('error', '(dynamic_block_map) Wrong settings (' . var_export($settings, true) . ')');
                continue;
            }
            $this->ci->Geomap_settings_model->save_settings($map_gid, $id_user, $id_object, $gid, $validate_data["data"]);
        }
    }

    /**
     * Import languages of geomap module
     *
     * @param array $langs_ids languages identifiers
     * @return boolean
     */
    public function install_geomap_lang_update($langs_ids)
    {
        if (empty($langs_ids)) {
            return false;
        }

        $langs_file = $this->ci->Install_model->language_file_read("dynamic_block_map", "geomap", $langs_ids);
        if (!$langs_file) {
            log_message("info", "Empty geomap langs data");
            return false;
        }

        $this->ci->load->model("geomap/models/Geomap_settings_model");

        $gids = array();
        foreach ((array) $this->modules_data['geomap'] as $geomap_data) {
            $gids[] = 'map_' . $geomap_data['gid'];
        }
        $this->ci->Geomap_settings_model->update_lang($gids, $langs_file, $langs_ids);

        return true;
    }

    /**
     * Export languages of geomap module
     *
     * @param array $langs_ids languages identifiers
     * @return array
     */
    public function install_geomap_lang_export($langs_ids = null)
    {
        $this->ci->load->model("geomap/models/Geomap_settings_model");

        $gids = array();
        foreach ((array) $this->modules_data['geomap'] as $geomap_data) {
            $gids[] = 'map_' . $geomap_data['gid'];
        }
        $langs = $this->ci->Geomap_settings_model->export_lang($gids, $langs_ids);
        return array("geomap" => $langs);
    }

    /**
     * Uninstall data of geomap module
     *
     * @return void
     */
    public function deinstall_geomap()
    {
        // add geomap settings
        $this->ci->load->model("geomap/models/Geomap_settings_model");

        foreach ((array) $this->modules_data['geomap'] as $settings) {
            $map_gid = $settings["map_gid"];
            unset($settings["data"]["map_gid"]);

            $id_user = $settings["id_user"];
            unset($settings["id_user"]);

            $id_object = $settings["id_object"];
            unset($settings["id_object"]);

            $gid = $settings["gid"];
            unset($settings["gid"]);

            $this->ci->Geomap_settings_model->delete_settings($map_gid, $id_user, $id_object, $gid);
        }
    }

}
