<?php

/**
 * Export module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined("BASEPATH")) {
    exit("No direct script access allowed");
}

/**
 * Yandex realty driver model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Export
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Export_yrl_model extends Model
{
    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $CI;

    /**
     * Driver configuration
     *
     * @var array
     */
    public $drivers = array(
        "gid"      => "yrl",
        "link"     => "http://help.yandex.ru/webmaster/?id=1113400",
        "settings" => array(),
        "elements" => array(
            array("name" => "realty-type", "type" => "text", "required" => 0),
            array("name" => "internal-id", "type" => "int", "required" => 1),
            array("name" => "type", "type" => "text", "required" => 1),
            array("name" => "property-type", "type" => "text", "required" => 1),
            array("name" => "category", "type" => "text", "required" => 1),
            array("name" => "url", "type" => "text", "required" => 1),
            array("name" => "creation-date", "type" => "text", "required" => 1),
            array("name" => "last-update-date", "type" => "text", "required" => 0),
            array("name" => "expire-date", "type" => "text", "required" => 0),
            array("name" => "payed-adv", "type" => "int", "required" => 0),
            array("name" => "manually-added", "type" => "int", "required" => 0),
            array("name" => "location.country", "type" => "text", "required" => 1),
            array("name" => "location.region", "type" => "text", "required" => 0),
            array("name" => "location.district", "type" => "text", "required" => 0),
            array("name" => "location.locality-name", "type" => "text", "required" => 0),
            array("name" => "location.sub-locality-name", "type" => "text", "required" => 0),
            array("name" => "location.non-admin-sub-locality", "type" => "text", "required" => 0),
            array("name" => "location.address", "type" => "text", "required" => 0),
            array("name" => "location.direction", "type" => "text", "required" => 0),
            array("name" => "location.distance", "type" => "text", "required" => 0),
            array("name" => "location.latitude", "type" => "text", "required" => 0),
            array("name" => "location.longitude", "type" => "text", "required" => 0),
            array("name" => "location.metro", "type" => "text", "required" => 0),
            array("name" => "location.time-on-transport", "type" => "text", "required" => 0),
            array("name" => "location.time-on-foot", "type" => "text", "required" => 0),
            array("name" => "location.railway-station", "type" => "text", "required" => 0),
            array("name" => "sales-agent.name", "type" => "text", "required" => 0),
            array("name" => "sales-agent.phone", "type" => "text", "required" => 1),
            array("name" => "sales-agent.category", "type" => "text", "required" => 0),
            array("name" => "sales-agent.organization", "type" => "text", "required" => 0),
            array("name" => "sales-agent.agency-id", "type" => "int", "required" => 0),
            array("name" => "sales-agent.url", "type" => "text", "required" => 0),
            array("name" => "sales-agent.email", "type" => "text", "required" => 0),
            array("name" => "sales-agent.partner", "type" => "text", "required" => 0),
            array("name" => "price.value", "type" => "int", "required" => 1),
            array("name" => "price.currency", "type" => "text", "required" => 1),
            array("name" => "price.period", "type" => "text", "required" => 0),
            array("name" => "price.unit", "type" => "text", "required" => 0),
            array("name" => "not-for-agents", "type" => "int", "required" => 0),
            array("name" => "haggle", "type" => "int", "required" => 0),
            array("name" => "mortgage", "type" => "int", "required" => 0),
            array("name" => "prepayment", "type" => "text", "required" => 0),
            array("name" => "rent-pledge", "type" => "int", "required" => 0),
            array("name" => "agent-fee", "type" => "text", "required" => 0),
            array("name" => "with-pets", "type" => "int", "required" => 0),
            array("name" => "with-children", "type" => "int", "required" => 0),
            array("name" => "image", "type" => "url", "required" => 0),
            array("name" => "renovation", "type" => "text", "required" => 0),
            array("name" => "description", "type" => "text", "required" => 0),
            array("name" => "area.value", "type" => "int", "required" => 0),
            array("name" => "area.unit", "type" => "text", "required" => 0),
            array("name" => "living-space.value", "type" => "text", "required" => 0),
            array("name" => "living-space.unit", "type" => "text", "required" => 0),
            array("name" => "kitchen-space.value", "type" => "text", "required" => 0),
            array("name" => "kitchen-space.unit", "type" => "text", "required" => 0),
            array("name" => "lot-area.value", "type" => "text", "required" => 0),
            array("name" => "lot-area.unit", "type" => "text", "required" => 0),
            array("name" => "new-flat", "type" => "int", "required" => 0),
            array("name" => "rooms", "type" => "int", "required" => 1),
            array("name" => "rooms-offered", "type" => "text", "required" => 1),
            array("name" => "open-plan", "type" => "int", "required" => 0),
            array("name" => "rooms-type", "type" => "text", "required" => 0),
            array("name" => "phone", "type" => "int", "required" => 0),
            array("name" => "internet", "type" => "int", "required" => 0),
            array("name" => "room-furniture", "type" => "int", "required" => 0),
            array("name" => "kitchen-furniture", "type" => "int", "required" => 0),
            array("name" => "television", "type" => "int", "required" => 0),
            array("name" => "washing-machine", "type" => "int", "required" => 0),
            array("name" => "refrigerator", "type" => "int", "required" => 0),
            array("name" => "balcony", "type" => "text", "required" => 0),
            array("name" => "bathroom-unit", "type" => "text", "required" => 0),
            array("name" => "floor-covering", "type" => "text", "required" => 0),
            array("name" => "window-view", "type" => "text", "required" => 0),
            array("name" => "floor", "type" => "text", "required" => 0),
            array("name" => "floors-total", "type" => "text", "required" => 0),
            array("name" => "building-name", "type" => "text", "required" => 0),
            array("name" => "building-type", "type" => "text", "required" => 0),
            array("name" => "building-series", "type" => "text", "required" => 0),
            array("name" => "building-state", "type" => "text", "required" => 0),
            array("name" => "built-year", "type" => "text", "required" => 0),
            array("name" => "ready-quarter", "type" => "text", "required" => 0),
            array("name" => "lift", "type" => "int", "required" => 0),
            array("name" => "rubbish-chute", "type" => "int", "required" => 0),
            array("name" => "is-elite", "type" => "int", "required" => 0),
            array("name" => "parking", "type" => "int", "required" => 0),
            array("name" => "alarm", "type" => "int", "required" => 0),
            array("name" => "ceiling-height", "type" => "text", "required" => 0),
            array("name" => "pmg", "type" => "int", "required" => 0),
            array("name" => "toilet", "type" => "text", "required" => 0),
            array("name" => "shower", "type" => "text", "required" => 0),
            array("name" => "kitchen", "type" => "int", "required" => 0),
            array("name" => "pool", "type" => "int", "required" => 0),
            array("name" => "billiard", "type" => "int", "required" => 0),
            array("name" => "sauna", "type" => "int", "required" => 0),
            array("name" => "heating-supply", "type" => "int", "required" => 0),
            array("name" => "water-supply", "type" => "int", "required" => 0),
            array("name" => "sewerage-supply", "type" => "int", "required" => 0),
            array("name" => "electricity-supply", "type" => "int", "required" => 0),
            array("name" => "gas-supply", "type" => "int", "required" => 0),
        ),
        "editable" => 0,
        "status"   => 1,
    );

    /**
     *  Demo data
     *
     *  @return void
     */
    public $demo_data = array(
        'gid_driver'      => 'yrl',
        'id_object'       => '2',
        'module'          => 'listings',
        'name'            => 'Test sample',
        'relations'       => 'a:24:{i:1;a:4:{s:4:"name";s:11:"internal-id";s:4:"type";s:3:"int";s:4:"link";s:2:"id";s:5:"label";s:14:"ID ref # (int)";}i:2;a:4:{s:4:"name";s:4:"type";s:4:"type";s:4:"text";s:4:"link";s:9:"type_code";s:5:"label";s:30:"Type of operation (text, code)";}i:3;a:4:{s:4:"name";s:13:"property-type";s:4:"type";s:4:"text";s:4:"link";s:13:"category_code";s:5:"label";s:21:"Category (text, code)";}i:4;a:4:{s:4:"name";s:8:"category";s:4:"type";s:4:"text";s:4:"link";s:13:"property_type";s:5:"label";s:20:"Property type (text)";}i:5;a:4:{s:4:"name";s:3:"url";s:4:"type";s:4:"text";s:4:"link";s:7:"seolink";s:5:"label";s:15:"View url (text)";}i:6;a:4:{s:4:"name";s:13:"creation-date";s:4:"type";s:4:"text";s:4:"link";s:12:"date_created";s:5:"label";s:14:"Created (text)";}i:7;a:4:{s:4:"name";s:16:"last-update-date";s:4:"type";s:4:"text";s:4:"link";s:13:"date_modified";s:5:"label";s:14:"Updated (text)";}i:8;a:4:{s:4:"name";s:11:"expire-date";s:4:"type";s:4:"text";s:4:"link";s:11:"date_expire";s:5:"label";s:22:"Expiration date (text)";}i:11;a:4:{s:4:"name";s:16:"location.country";s:4:"type";s:4:"text";s:4:"link";s:10:"id_country";s:5:"label";s:14:"Country (text)";}i:12;a:4:{s:4:"name";s:15:"location.region";s:4:"type";s:4:"text";s:4:"link";s:9:"id_region";s:5:"label";s:13:"Region (text)";}i:14;a:4:{s:4:"name";s:22:"location.locality-name";s:4:"type";s:4:"text";s:4:"link";s:7:"id_city";s:5:"label";s:11:"City (text)";}i:17;a:4:{s:4:"name";s:16:"location.address";s:4:"type";s:4:"text";s:4:"link";s:7:"address";s:5:"label";s:14:"Address (text)";}i:26;a:4:{s:4:"name";s:16:"sales-agent.name";s:4:"type";s:4:"text";s:4:"link";s:7:"id_user";s:5:"label";s:26:"Listing provided by (text)";}i:34;a:4:{s:4:"name";s:11:"price.value";s:4:"type";s:3:"int";s:4:"link";s:5:"price";s:5:"label";s:11:"Price (int)";}i:35;a:4:{s:4:"name";s:14:"price.currency";s:4:"type";s:4:"text";s:4:"link";s:12:"gid_currency";s:5:"label";s:15:"Currency (text)";}i:48;a:4:{s:4:"name";s:11:"description";s:4:"type";s:4:"text";s:4:"link";s:13:"fe_comments_1";s:5:"label";s:28:"Comment - Description (text)";}i:49;a:4:{s:4:"name";s:10:"area.value";s:4:"type";s:3:"int";s:4:"link";s:6:"square";s:5:"label";s:19:"Property size (int)";}i:50;a:4:{s:4:"name";s:9:"area.unit";s:4:"type";s:4:"text";s:4:"link";s:11:"square_unit";s:5:"label";s:11:"Unit (text)";}i:58;a:4:{s:4:"name";s:5:"rooms";s:4:"type";s:3:"int";s:4:"link";s:13:"fe_bd_rooms_1";s:5:"label";s:28:"General - Bedrooms (int, id)";}i:59;a:4:{s:4:"name";s:13:"rooms-offered";s:4:"type";s:4:"text";s:4:"link";s:13:"fe_bd_rooms_1";s:5:"label";s:25:"General - Bedrooms (text)";}i:20;a:4:{s:4:"name";s:17:"location.latitude";s:4:"type";s:4:"text";s:4:"link";s:3:"lat";s:5:"label";s:15:"Latitude (text)";}i:21;a:4:{s:4:"name";s:18:"location.longitude";s:4:"type";s:4:"text";s:4:"link";s:3:"lon";s:5:"label";s:16:"Longitude (text)";}i:27;a:4:{s:4:"name";s:17:"sales-agent.phone";s:4:"type";s:4:"text";s:4:"link";s:10:"user_phone";s:5:"label";s:18:"Agent phone (text)";}i:46;a:4:{s:4:"name";s:5:"image";s:4:"type";s:4:"text";s:4:"link";s:7:"uploads";s:5:"label";s:11:"Photo (url)";}}',
        'admin_form_data' => '',
        'output_type'     => 'browser',
        'file_format'     => 'test.yrl',
        'append_date'     => '1',
        'scheduler'       => 'a:1:{s:4:"type";s:6:"manual";}',
        'published'       => '0',
        'editable'        => '0',
        'status'          => '1',
        'date_created'    => '',
    );

    /**
     * Not available symbols
     *
     * @var array
     */
    protected $search = array('&', "'", '"', ">", "<");

    /**
     * Symbols for replace
     *
     * @var array
     */
    protected $replace = array('&amp;', '&apos;', '&quot;', "&gt;", "&lt;");

    /**
     * Content type
     *
     * @var object
     */
    public $content_file = "text/xml";

    /**
     * Realty type options
     *
     * @var array
     */
    protected $realty_type = array("городская", "загородная");

    /**
     * Yrl realty type options
     *
     * @var array
     */
    protected $type = array("продажа", "аренда");

    /**
     * Property type options
     *
     * @var array
     */
    protected $property_type = array('жилая', 'коммерческая', 'земельный участок');

    /**
     * Realty category options
     *
     * @var array
     */
    protected $category = array(
        "комната", "квартира", "дом", "участок", "flat", "room",
        "house", "cottage", "townhouse", "таунхаус", "часть дома",
        "house with lot", "дом с участком", "дача", "lot", "земельный участок",
    );

    /**
     * Currency options
     *
     * @var array
     */
    protected $currency = array("RUR", "RUB", "EUR", "USD", "UAH", "BYR", "KZT");

    /**
     * Rental period options
     *
     * @var array
     */
    protected $period = array("день", "месяц", "day", "month");

    /**
     * Square units options
     *
     * @var array
     */
    protected $unit = array("кв. м", "гектар", "cотка", "sq.m", "hectare");

    /**
     * Building state options
     *
     * @var array
     */
    protected $building_state = array("unfinished", "built", "hand-over");

    /**
     * Ready quarter options
     *
     * @var array
     */
    protected $ready_quarter = array(1, 2, 3, 4);

    /**
     * Constructor
     *
     * @return Export_yrl_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->CI = &get_instance();
    }

    /**
     * Return filename of export data by default
     *
     * @return string
     */
    public function get_filename()
    {
        return "export.xml";
    }

    /**
     * Generate output file
     *
     * @param string $filename output filename
     * @param array  $data     export data
     * @param array  $settings driver settings
     *
     * @return void
     */
    public function generate($filename, $data, $settings)
    {
        $f = fopen($filename, "wb");
        fputs($f, '<?xml version="1.0" encoding="utf-8"?>' . "\n");
        fputs($f, '<realty-feed xmlns="http://webmaster.yandex.ru/schemas/feed/realty/2010-06">' . "\n");
        fputs($f, '	<generation-date>' . date('c') . '</generation-date>' . "\n");
        foreach ($data as $i => $row) {
            if (!$i) {
                continue;
            }
            fputs($f, '	<offer internal-id="' . $row['internal-id'] . '">' . "\n");
            fputs($f, '		<type>' . $row['type'] . '</type>' . "\n");
            fputs($f, '		<property-type>' . $row['property-type'] . '</property-type>' . "\n");
            fputs($f, '		<category>' . $row['category'] . '</category>' . "\n");
            fputs($f, '		<url>' . $row['url'] . '</url>' . "\n");
            fputs($f, '		<creation-date>' . $row['creation-date'] . '</creation-date>' . "\n");

            if (isset($row['last-update-date']) && !empty($row['last-update-date'])) {
                fputs($f, '		<last-update-date>' . $row['last-update-date'] . '</last-update-date>' . "\n");
            }

            if (isset($row['expire-date']) && !empty($row['expire-date'])) {
                fputs($f, '		<expire-date>' . $row['expire-date'] . '</expire-date>' . "\n");
            }

            if (isset($row['payed-adv'])) {
                fputs($f, '		<payed-adv>' . $row['payed-adv'] . '</payed-adv>' . "\n");
            }

            if (isset($row['manually-added'])) {
                fputs($f, '		<manually-added>' . $row['manually-added'] . '</manually-added>' . "\n");
            }

            fputs($f, '		<location>' . "\n");
            fputs($f, '			<country>' . $row['location.country'] . '</country>' . "\n");

            if (isset($row['location.region']) && !empty($row['location.region'])) {
                fputs($f, '			<region>' . $row['location.region'] . '</region>' . "\n");
            }

            if (isset($row['location.district']) && !empty($row['location.district'])) {
                fputs($f, '			<district>' . $row['location.district'] . '</district>' . "\n");
            }

            if (isset($row['location.locality-name']) && !empty($row['location.locality-name'])) {
                fputs($f, '			<locality-name>' . $row['location.locality-name'] . '</locality-name>' . "\n");
            }

            if (isset($row['location.sub-locality-name']) && !empty($row['location.sub-locality-name'])) {
                fputs($f, '			<sub-locality-name>' . $row['location.sub-locality-name'] . '</sub-locality-name>' . "\n");
            }

            if (isset($row['location.non-admin-sub-locality']) && !empty($row['location.non-admin-sub-locality'])) {
                fputs($f, '			<non-admin-sub-locality>' . $row['location.non-admin-sub-locality'] . '</non-admin-sub-locality>' . "\n");
            }

            if (isset($row['location.address']) && !empty($row['location.address'])) {
                fputs($f, '			<address>' . $row['location.address'] . '</address>' . "\n");
            }

            if (isset($row['location.direction']) && !empty($row['location.direction'])) {
                fputs($f, '			<direction>' . $row['location.direction'] . '</direction>' . "\n");
            }

            if (isset($row['location.distance']) && !empty($row['location.distance'])) {
                fputs($f, '			<distance>' . $row['location.distance'] . '</distance>' . "\n");
            }

            if (isset($row['location.latitude']) && !empty($row['location.latitude'])) {
                fputs($f, '			<latitude>' . $row['location.latitude'] . '</latitude>' . "\n");
            }

            if (isset($row['location.longitude']) && !empty($row['location.longitude'])) {
                fputs($f, '			<longitude>' . $row['location.longitude'] . '</longitude>' . "\n");
            }

            if ((isset($row['location.metro']) && !empty($row['location.metro'])) ||
                (isset($row['location.time-on-foot']) && !empty($row['location.time-on-foot'])) ||
                (isset($row['location.time-on-transport']) && !empty($row['location.time-on-transport']))) {
                fputs($f, '			<metro>' . "\n");

                if (isset($row['location.metro']) && !empty($row['location.metro'])) {
                    fputs($f, '				<name>' . $row['location.metro'] . '</name>' . "\n");
                }

                if (isset($row['location.time-on-foot']) && !empty($row['location.time-on-foot'])) {
                    fputs($f, '				<time-on-foot>' . $row['location.time-on-foot'] . '</time-on-foot>' . "\n");
                }

                if (isset($row['location.time-on-transport']) && !empty($row['location.time-on-transport'])) {
                    fputs($f, '				<time-on-transport>' . $row['location.time-on-transport'] . '</time-on-transport>' . "\n");
                }

                fputs($f, '			</metro>' . "\n");
            }

            if (isset($row['location.railway-station']) && !empty($row['location.railway-station'])) {
                fputs($f, '			<railway-station>' . $row['location.railway-station'] . '</railway-station>' . "\n");
            }

            fputs($f, '		</location>' . "\n");
            fputs($f, '		<sales-agent>' . "\n");

            if (isset($row['sales-agent.name']) && !empty($row['sales-agent.name'])) {
                fputs($f, '			<name>' . $row['sales-agent.name'] . '</name>' . "\n");
            }

            fputs($f, '			<phone>' . $row['sales-agent.phone'] . '</phone>' . "\n");

            if (isset($row['sales-agent.category']) && !empty($row['sales-agent.category'])) {
                fputs($f, '			<category>' . $row['sales-agent.category'] . '</category>' . "\n");
            }

            if (isset($row['sales-agent.organization']) && !empty($row['sales-agent.organization'])) {
                fputs($f, '			<organization>' . $row['sales-agent.organization'] . '</organization>' . "\n");
            }

            if (isset($row['sales-agent.agency-id']) && !empty($row['sales-agent.agency-id'])) {
                fputs($f, '			<agency-id>' . $row['sales-agent.agency-id'] . '</agency-id>' . "\n");
            }

            if (isset($row['sales-agent.url']) && !empty($row['sales-agent.url'])) {
                fputs($f, '			<url>' . $row['sales-agent.url'] . '</url>' . "\n");
            }

            if (isset($row['sales-agent.email']) && !empty($row['sales-agent.email'])) {
                fputs($f, '			<email>' . $row['sales-agent.email'] . '</email>' . "\n");
            }

            if (isset($row['sales-agent.partner']) && !empty($row['sales-agent.partner'])) {
                fputs($f, '			<partner>' . $row['sales-agent.partner'] . '</partner>' . "\n");
            }

            fputs($f, '		</sales-agent>' . "\n");

            fputs($f, '		<price>' . "\n");
            fputs($f, '			<value>' . $row['price.value'] . '</value>' . "\n");
            fputs($f, '			<currency>' . $row['price.currency'] . '</currency>' . "\n");

            if (isset($row['price.period']) && !empty($row['price.period'])) {
                fputs($f, '			<period>' . $row['price.period'] . '</period>' . "\n");
            }

            if (isset($row['price.unit']) && !empty($row['price.unit'])) {
                fputs($f, '			<unit>' . $row['price.unit'] . '</unit>' . "\n");
            }

            fputs($f, '		</price>' . "\n");

            if (isset($row['not-for-agents'])) {
                fputs($f, '		<not-for-agents>' . $row['not-for-agents'] . '</not-for-agents>' . "\n");
            }

            if (isset($row['haggle'])) {
                fputs($f, '		<haggle>' . $row['haggle'] . '</haggle>' . "\n");
            }

            if (isset($row['mortgage'])) {
                fputs($f, '		<mortgage>' . $row['mortgage'] . '</mortgage>' . "\n");
            }

            if (isset($row['prepayment'])) {
                fputs($f, '		<prepayment>' . $row['prepayment'] . '</prepayment>' . "\n");
            }

            if (isset($row['rent-pledge'])) {
                fputs($f, '		<rent-pledge>' . $row['rent-pledge'] . '</rent-pledge>' . "\n");
            }

            if (isset($row['rent-pledge'])) {
                fputs($f, '		<agent-fee>' . $row['agent-fee'] . '</agent-fee>' . "\n");
            }

            if (isset($row['with-pets'])) {
                fputs($f, '		<with-pets>' . $row['with-pets'] . '</with-pets>' . "\n");
            }

            if (isset($row['with-children'])) {
                fputs($f, '		<with-children>' . $row['with-children'] . '</with-children>' . "\n");
            }

            if(isset($row['image']) && !empty($row['image'])){
                foreach($row['image'] as $image){
                    if(!empty($image)){
                        fputs($f, '     <image>'.$image.'</image>'."\n");
                    }
                }
            }

            if (isset($row['renovation']) && !empty($row['renovation'])) {
                fputs($f, '		<renovation>' . $row['renovation'] . '</renovation>' . "\n");
            }

            if (isset($row['description']) && !empty($row['description'])) {
                fputs($f, '		<description>' . $row['description'] . '</description>' . "\n");
            }

            if (isset($row['area.value']) && !empty($row['area.value']) && isset($row['area.unit']) && !empty($row['area.unit'])) {
                fputs($f, '		<area>' . "\n");
                fputs($f, '			<value>' . $row['area.value'] . '</value>' . "\n");
                fputs($f, '			<unit>' . $row['area.unit'] . '</unit>' . "\n");
                fputs($f, '		</area>' . "\n");
            }

            if (isset($row['living-space.value']) && !empty($row['living-space.value']) && isset($row['living-space.unit']) && !empty($row['living-space.unit'])) {
                fputs($f, '		<living-space>' . "\n");
                fputs($f, '			<value>' . $row['living-space.value'] . '</value>' . "\n");
                fputs($f, '			<unit>' . $row['living-space.unit'] . '</unit>' . "\n");
                fputs($f, '		</living-space>' . "\n");
            }

            if (isset($row['kitchen-space.value']) && !empty($row['kitchen-space.value']) && isset($row['kitchen-space.unit']) && !empty($row['kitchen-space.unit'])) {
                fputs($f, '		<kitchen-space>' . "\n");
                fputs($f, '			<value>' . $row['kitchen-space.value'] . '</value>' . "\n");
                fputs($f, '			<unit>' . $row['kitchen-space.unit'] . '</unit>' . "\n");
                fputs($f, '		</kitchen-space>' . "\n");
            }

            if (isset($row['lot-area.value']) && !empty($row['lot-area.value']) && isset($row['lot-area.unit']) && !empty($row['lot-area.unit'])) {
                fputs($f, '		<lot-area>' . "\n");
                fputs($f, '			<value>' . $row['lot-area.value'] . '</value>' . "\n");
                fputs($f, '			<unit>' . $row['lot-area.unit'] . '</unit>' . "\n");
                fputs($f, '		</lot-area>' . "\n");
            }

            if (isset($row['lot-type']) && !empty($row['lot-type'])) {
                fputs($f, '		<lot-type>' . $row['lot-type'] . '</lot-type>' . "\n");
            }

            switch ($row['property-type']) {
                case 'жилая':
                    if (isset($row['new-flat']) && !empty($row['new-flat'])) {
                        fputs($f, '		<new-flat>' . $row['new-flat'] . '</new-flat>' . "\n");
                    }

                    fputs($f, '		<rooms>' . $row['rooms'] . '</rooms>' . "\n");
                    fputs($f, '     <rooms-offered>'.$row['rooms'].'</rooms-offered>'."\n");

                    if (isset($row['open-plan'])) {
                        fputs($f, '		<open-plan>' . $row['open-plan'] . '</open-plan>' . "\n");
                    }

                    if (isset($row['rooms-type']) && !empty($row['rooms-type'])) {
                        fputs($f, '		<rooms-type>' . $row['rooms-type'] . '</rooms-type>' . "\n");
                    }

                    if (isset($row['phone'])) {
                        fputs($f, '		<phone>' . $row['phone'] . '</phone>' . "\n");
                    }

                    if (isset($row['internet'])) {
                        fputs($f, '		<internet>' . $row['internet'] . '</internet>' . "\n");
                    }

                    if (isset($row['room-furniture'])) {
                        fputs($f, '		<room-furniture>' . $row['room-furniture'] . '</room-furniture>' . "\n");
                    }

                    if (isset($row['kitchen-furniture'])) {
                        fputs($f, '		<kitchen-furniture>' . $row['kitchen-furniture'] . '</kitchen-furniture>' . "\n");
                    }

                    if (isset($row['television'])) {
                        fputs($f, '		<television>' . $row['television'] . '</television>' . "\n");
                    }

                    if (isset($row['washing-machine'])) {
                        fputs($f, '		<washing-machine>' . $row['washing-machine'] . '</washing-machine>' . "\n");
                    }

                    if (isset($row['refrigerator'])) {
                        fputs($f, '		<refrigerator>' . $row['refrigerator'] . '</refrigerator>' . "\n");
                    }

                    if (isset($row['balcony']) && !empty($row['balcony'])) {
                        fputs($f, '		<balcony>' . $row['balcony'] . '</balcony>' . "\n");
                    }

                    if (isset($row['bathroom-unit']) && !empty($row['bathroom-unit'])) {
                        fputs($f, '		<bathroom-unit>' . $row['bathroom-unit'] . '</bathroom-unit>' . "\n");
                    }

                    if (isset($row['floor-covering']) && !empty($row['floor-covering'])) {
                        fputs($f, '		<floor-covering>' . $row['floor-covering'] . '</floor-covering>' . "\n");
                    }

                    if (isset($row['window-view']) && !empty($row['window-view'])) {
                        fputs($f, '		<window-view>' . $row['window-view'] . '</window-view>' . "\n");
                    }

                    if (isset($row['floor']) && !empty($row['floor'])) {
                        fputs($f, '		<floor>' . $row['floor'] . '</floor>' . "\n");
                    }

                    if (isset($row['floors-total']) && !empty($row['floors-total'])) {
                        fputs($f, '		<floors-total>' . $row['floors-total'] . '</floors-total>' . "\n");
                    }

                    if (isset($row['building-name']) && !empty($row['building-name'])) {
                        fputs($f, '		<building-name>' . $row['building-name'] . '</building-name>' . "\n");
                    }

                    if (isset($row['building-type']) && !empty($row['building-type'])) {
                        fputs($f, '		<building-type>' . $row['building-type'] . '</building-type>' . "\n");
                    }

                    if (isset($row['building-series']) && !empty($row['building-series'])) {
                        fputs($f, '		<building-series>' . $row['building-series'] . '</building-series>' . "\n");
                    }

                    if (isset($row['building-state']) && !empty($row['building-state'])) {
                        fputs($f, '		<building-state>' . $row['building-state'] . '</building-state>' . "\n");
                    }

                    if (isset($row['built-year']) && !empty($row['built-year']) && strlen($row['built-year']) == 4) {
                        fputs($f, '		<built-year>' . $row['built-year'] . '</built-year>' . "\n");
                    }

                    if (isset($row['ready-quarter']) && in_array($row['ready-quarter'], $this->ready_quarter)) {
                        fputs($f, '		<ready-quarter>' . $row['ready-quarter'] . '</ready-quarter>' . "\n");
                    }

                    if (isset($row['lift'])) {
                        fputs($f, '		<lift>' . $row['lift'] . '</lift>' . "\n");
                    }

                    if (isset($row['rubbish-chute'])) {
                        fputs($f, '		<rubbish-chute>' . $row['rubbish-chute'] . '</rubbish-chute>' . "\n");
                    }

                    if (isset($row['is-elite'])) {
                        fputs($f, '		<is-elite>' . $row['is-elite'] . '</is-elite>' . "\n");
                    }

                    if (isset($row['parking'])) {
                        fputs($f, '		<parking>' . $row['parking'] . '</parking>' . "\n");
                    }

                    if (isset($row['alarm'])) {
                        fputs($f, '		<alarm>' . $row['alarm'] . '</alarm>' . "\n");
                    }

                    if (isset($row['ceiling-height']) && !empty($row['ceiling-height'])) {
                        fputs($f, '		<ceiling-height>' . $row['ceiling-height'] . '</ceiling-height>' . "\n");
                    }
                break;
            }

            switch ($row['realty-type']) {
                case 'загородная':
                    if (isset($row['pmg'])) {
                        fputs($f, '		<pmg>' . $row['pmg'] . '</pmg>' . "\n");
                    }

                    if (isset($row['toilet']) && !empty($row['toilet'])) {
                        fputs($f, '		<toilet>' . $row['toilet'] . '</toilet>' . "\n");
                    }

                    if (isset($row['shower']) && !empty($row['shower'])) {
                        fputs($f, '		<shower>' . $row['shower'] . '</shower>' . "\n");
                    }

                    if (isset($row['kitchen'])) {
                        fputs($f, '		<kitchen>' . $row['kitchen'] . '</kitchen>' . "\n");
                    }

                    if (isset($row['pool'])) {
                        fputs($f, '		<pool>' . $row['pool'] . '</pool>' . "\n");
                    }

                    if (isset($row['billiard'])) {
                        fputs($f, '		<billiard>' . $row['billiard'] . '</billiard>' . "\n");
                    }

                    if (isset($row['sauna'])) {
                        fputs($f, '		<sauna>' . $row['sauna'] . '</sauna>' . "\n");
                    }

                    if (isset($row['water-supply'])) {
                        fputs($f, '		<water-supply>' . $row['water-supply'] . '</water-supply>' . "\n");
                    }

                    if (isset($row['electricity-supply'])) {
                        fputs($f, '		<electricity-supply>' . $row['electricity-supply'] . '</electricity-supply>' . "\n");
                    }

                    if (isset($row['sewerage-supply'])) {
                        fputs($f, '		<sewerage-supply>' . $row['sewerage-supply'] . '</sewerage-supply>' . "\n");
                    }

                    if (isset($row['gas-supply'])) {
                        fputs($f, '		<gas-supply>' . $row['gas-supply'] . '</gas-supply>' . "\n");
                    }
                break;
            }
            fputs($f, '	</offer>' . "\n");
        }
        fputs($f, '</realty-feed>' . "\n");
        fclose($f);
    }

    /**
     * Validate export  data
     *
     * @param array $data export data
     *
     * @return array
     */
    public function validate_data($data)
    {
        $return = array('errors' => array(), 'data' => array());

        if (isset($data['realty-type'])) {
            $return['data']['realty-type'] = str_replace($this->search, $this->replace, $data['realty-type']);
            if (!in_array($return['data']['realty-type'], $this->realty_type)) {
                $return['data']['realty-type'] = current($this->realty_type);
            }
        } else {
            $return['data']['realty-type'] = current($this->realty_type);
        }

        if (isset($data['internal-id'])) {
            $return['data']['internal-id'] = intval($data['internal-id']);
            if (empty($return['data']['internal-id'])) {
                $return['errors'][] = 'internal-id';
            }
        } else {
            $return['errors'][] = 'internal-id';
        }

        if (isset($data['type'])) {
            $return['data']['type'] = str_replace($this->search, $this->replace, $data['type']);
            switch ($return['data']['type']) {
                case 'sale':
                    $return['data']['type'] = 'продажа';
                break;
                case 'buy':
                    $return['data']['type'] = 'покупка';
                break;
                case 'rent':
                    $return['data']['type'] = 'аренда';
                break;
            }
            if (!in_array($return['data']['type'], $this->type)) {
                $return['errors'][] = 'type';
            }
        } else {
            $return['errors'][] = 'type';
        }

        if (isset($data['property-type'])) {
            $return['data']['property-type'] = str_replace($this->search, $this->replace, $data['property-type']);
            switch ($return['data']['property-type']) {
                case 'residential':
                    $return['data']['property-type'] = 'жилая';
                break;
                case 'commercial':
                    $return['data']['property-type'] = 'коммерческая';
                break;
                case  'lot_and_land':
                    $return['data']['property-type'] = 'земельный участок';
                break;
            }
            if (!in_array($return['data']['property-type'], $this->property_type)) {
                $return['data']['property-type'] = current($this->property_type);
                //$return['errors'][] = 'property-type';
            }
        } else {
            $return['data']['property-type'] = current($this->property_type);
            //$return['errors'][] = 'property-type';
        }

        if (isset($data['category'])) {
            $return['data']['category'] = str_replace($this->search, $this->replace, $data['category']);
            $return['data']['category'] = mb_convert_case($return['data']['category'], MB_CASE_LOWER, "UTF-8");
            
            if(($key = array_search($return['data']["category"], $this->category)) === FALSE){
                if (isset($data['category2'])) {
                    $return['data']['category'] = str_replace($this->search, $this->replace, $data['category2']);
                    if (!in_array($return['data']["category"], $this->category)) {
                        if (isset($data['category3'])) {
                            $return['data']['category'] = str_replace($this->search, $this->replace, $data['category3']);
                            if (!in_array($return['data']["category"], $this->category)) {
                                if (isset($data['category4'])) {
                                    $return['data']['category'] = str_replace($this->search, $this->replace, $data['category4']);
                                    if (!in_array($return['data']["category"], $this->category)) {
                                        if (isset($data['category5'])) {
                                            $return['data']['category'] = str_replace($this->search, $this->replace, $data['category5']);
                                            if (!in_array($return['data']["category"], $this->category)) {
                                                if (isset($data['category6'])) {
                                                    $return['data']['category'] = str_replace($this->search, $this->replace, $data['category6']);
                                                    if (!in_array($return['data']["category"], $this->category)) {
                                                        $return['data']['category'] = current($this->category);
                                                        //$return['errors'][] = 'category';
                                                    }
                                                } else {
                                                    $return['data']['category'] = current($this->category);
                                                    //$return['errors'][] = 'category';
                                                }
                                            }
                                        } else {
                                            $return['data']['category'] = current($this->category);
                                            //$return['errors'][] = 'category';
                                        }
                                    }
                                } else {
                                    $return['data']['category'] = current($this->category);
                                    //$return['errors'][] = 'category';
                                }
                            }
                        } else {
                            $return['data']['category'] = current($this->category);
                            //$return['errors'][] = 'category';
                        }
                    }
                } else {
                    $return['data']['category'] = current($this->category);
                    //$return['errors'][] = 'category';
                }
            }
        } else {
            $return['data']['category'] = current($this->category);
            //$return['errors'][] = 'category';
        }

        if (isset($data['url'])) {
            $return['data']['url'] = str_replace($this->search, $this->replace, $data['url']);
            if (empty($return['data']['url'])) {
                $return['errors'][] = 'url';
            }
        } else {
            $return['errors'][] = 'url';
        }

        if (isset($data['creation-date'])) {
            $value = strtotime($data['creation-date']);
            if ($value > 0) {
                $return['data']['creation-date'] = date('c', $value);
            } else {
                $return['errors'][] = 'creation-date';
            }
        } else {
            $return['errors'][] = 'creation-date';
        }

        if (isset($data['last-update-date'])) {
            $value = strtotime($data['last-update-date']);
            if ($value > 0) {
                $return['data']['last-update-date'] = date('c', $value);
            }
        }

        if (isset($data['expire-date'])) {
            $value = strtotime($data['expire-date']);
            if ($value > 0) {
                $return['data']['expire-date'] = date('c', $value);
            }
        }

        if (isset($data['payed-adv'])) {
            $return['data']['payed-adv'] = $data['payed-adv'] ? 1 : 0;
        }

        if (isset($data['manually-added'])) {
            $return['data']['manually-added'] = $data['manually-added'] ? 1 : 0;
        }

        if (isset($data['location.country'])) {
            $return['data']['location.country'] = str_replace($this->search, $this->replace, $data['location.country']);
            if (empty($return['data']['location.country'])) {
                $return["errors"][] = 'location.country';
            }
        } else {
            $return['errors'][] = 'location.region';
        }

        if (isset($data['location.region'])) {
            if ($data['location.locality-name'] != 'Москва' || $data['location.locality-name'] != 'Санкт-Петербург') {
                $return['data']['location.region'] = str_replace($this->search, $this->replace, $data['location.region']);
                if (empty($return['data']['location.region'])) {
                    $return['errors'][] = 'location.region';
                }
            }
        } else {
            $return['errors'][] = 'location.region';
        }

        if (isset($data['location.district'])) {
            $return['data']['location.district'] = str_replace($this->search, $this->replace, $data['location.district']);
        }

        if (isset($data['location.locality-name'])) {
            $return['data']['location.locality-name'] = str_replace($this->search, $this->replace, $data['location.locality-name']);
            if (empty($return['data']['location.locality-name']) && $return['data']['realty-type'] == 'городская') {
                $return['errors'][] = 'location.locality-name';
            }
        } elseif ($return['data']['realty-type'] == 'городская') {
            $return['errors'][] = 'location.locality-name';
        }

        if (isset($data['location.sub-locality-name'])) {
            $return['data']['location.sub-locality-name'] = str_replace($this->search, $this->replace, $data['location.sub-locality-name']);
        }

        if (isset($data['location.non-admin-sub-locality'])) {
            $return['data']['location.non-admin-sub-locality'] = str_replace($this->search, $this->replace, $data['location.non-admin-sub-locality']);
        }

        if (isset($data['location.address'])) {
            $return['data']['location.address'] = str_replace($this->search, $this->replace, $data['location.address']);
            if (empty($return['data']['location.address']) && $return['data']['realty-type'] == 'городская') {
                $return['errors'][] = 'location.address';
            }
        } elseif ($return['data']['realty-type'] == 'городская') {
            $return['errors'][] = 'location.address';
        }

        if (isset($data['location.direction'])) {
            if ($return['data']['location.locality-name'] == 'Москва') {
                $return['data']['location.direction'] = str_replace($this->search, $this->replace, $data['location.direction']);
            }
        }

        if (isset($data['location.distance'])) {
            if ($return['data']['location.locality-name'] == 'Москва') {
                $return['data']['location.distance'] = intval($this->search, $this->replace, $data['location.distance']);
            }
        }

        if (isset($data['location.latitude'])) {
            $return['data']['location.latitude'] = floatval($data['location.latitude']);
        }

        if (isset($data['location.longitude'])) {
            $return['data']['location.longitude'] = floatval($data['location.longitude']);
        }

        if (isset($data['location.metro'])) {
            $return['data']['location.metro'] = str_replace($this->search, $this->replace, $data['location.metro']);
        }

        if (isset($data['location.time-on-foot'])) {
            $return['data']['location.time-on-foot'] = intval($data['location.time-on-foot']);
        }

        if (isset($data['location.time-on-transport'])) {
            $return['data']['location.time-on-transport'] = intval($data['location.time-on-transport']);
        }

        if (isset($data['location.railway-station'])) {
            $return['data']['location.railway-station'] = str_replace($this->search, $this->replace, $data['location.railway-station']);
        }

        if (isset($data['sales-agent.name'])) {
            $return['data']['sales-agent.name'] = str_replace($this->search, $this->replace, $data['sales-agent.name']);
        }

        if (isset($data['sales-agent.phone'])) {
            $return['data']['sales-agent.phone'] = str_replace($this->search, $this->replace, $data['sales-agent.phone']);
            if (empty($return['data']['sales-agent.phone'])) {
                $return["errors"][] = "sales-agent.phone";
            }
        } else {
            $return["errors"][] = "sales-agent.phone";
        }

        if (isset($data['sales-agent.category'])) {
            $return['data']['sales-agent.category'] = str_replace($this->search, $this->replace, $data['sales-agent.category']);
        }

        if (isset($data['sales-agent.organization'])) {
            $return['data']['sales-agent.organization'] = str_replace($this->search, $this->replace, $data['sales-agent.organization']);
        }

        if (isset($data['sales-agent.agency-id'])) {
            $return['data']['sales-agent.agency-id'] = intval($data['sales-agent.agency-id']);
        }

        if (isset($data['sales-agent.url'])) {
            $return['data']['sales-agent.url'] = str_replace($this->search, $this->replace, $data['sales-agent.url']);
        }

        if (isset($data['sales-agent.email'])) {
            $return['data']['sales-agent.email'] = str_replace($this->search, $this->replace, $data['sales-agent.email']);
        }

        if (isset($data['sales-agent.partner'])) {
            $return['data']['sales-agent.partner'] = str_replace($this->search, $this->replace, $data['sales-agent.partner']);
        }

        if (isset($data['price.value'])) {
            $return['data']['price.value'] = intval($data['price.value']);
            if (empty($return['data']["price.value"])) {
                $return["errors"][] = "price.value";
            }
        } else {
            $return["errors"][] = "price.value";
        }

        if (isset($data['price.currency'])) {
            $return['data']['price.currency'] = str_replace($this->search, $this->replace, $data['price.currency']);
            if (!in_array($return['data']["price.currency"], $this->currency)) {
                $return["errors"][] = "price.currency";
            }
        } else {
            $return["errors"][] = "price.currency";
        }

        if (isset($data['price.period'])) {
            $return['data']['price.period'] = str_replace($this->search, $this->replace, $data['price.period']);
        }

        if (isset($data['price.unit'])) {
            $return['data']['price.unit'] = str_replace($this->search, $this->replace, $data['price.unit']);
        }

        if (isset($data['not-for-agents'])) {
            $return['data']['not-for-agents'] = $data['not-for-agents'] ? 1 : 0;
        }

        if (isset($data['haggle'])) {
            $return['data']['haggle'] = $data['haggle'] ? 1 : 0;
        }

        if (isset($data['mortgage'])) {
            $return['data']['mortgage'] = $data['mortgage'] ? 1 : 0;
        }

        if (isset($data['prepayment'])) {
            $return['data']['prepayment'] = $data['prepayment'] ? 1 : 0;
        }

        if (isset($data['rent-pledge'])) {
            $return['data']['rent-pledge'] = $data['rent-pledge'] ? 1 : 0;
        }

        if (isset($data['agent-fee'])) {
            $return['data']['agent-fee'] = intval($data['agent-fee']);
            if ($return['data']['agent-fee'] < 0 || $return['data']['agent-fee'] > 100) {
                unset($return['data']['agent-fee']);
            }
        }

        if (isset($data['with-pets'])) {
            $return['data']['with-pets'] = $data['with-pets'] ? 1 : 0;
        }

        if (isset($data['with-children'])) {
            $return['data']['with-children'] = $data['with-children'] ? 1 : 0;
        }

        if (isset($data['image'])) {
            $return['data']['image'] = explode('|', $data['image']);
            foreach ($return['data']['image'] as $key => $image) {
                $return['data']['image'][$key] = str_replace($this->search, $this->replace, $image);
            }
        }

        if (isset($data['renovation'])) {
            $return['data']['renovation'] = str_replace($this->search, $this->replace, $data['renovation']);
        }

        if (isset($data['description'])) {
            $return['data']['description'] = str_replace($this->search, $this->replace, $data['description']);
        }

        if (isset($data['area.value'])) {
            $return['data']['area.value'] = intval($data['area.value']);
        }

        if (isset($data['area.unit'])) {
            $return['data']['area.unit'] = str_replace($this->search, $this->replace, $data['area.unit']);
        }

        if (isset($data['living-space.value'])) {
            $return['data']['living-space.value'] = intval($data['living-space.value']);
        }

        if (isset($data['living-space.unit'])) {
            $return['data']['living-space.unit'] = str_replace($this->search, $this->replace, $data['living-space.unit']);
        }

        if (isset($data['kitchen-space.value'])) {
            $return['data']['kitchen-space.value'] = intval($data['kitchen-space.value']);
        }

        if (isset($data['kitchen-space.unit'])) {
            $return['data']['kitchen-space.unit'] = str_replace($this->search, $this->replace, $data['kitchen-space.unit']);
        }

        if (isset($data['lot-area.value'])) {
            $return['data']['lot-area.value'] = intval($data['lot-area.value']);
        }

        if (isset($data['lot-area.unit'])) {
            $return['data']['lot-area.unit'] = str_replace($this->search, $this->replace, $data['lot-area.unit']);
        }

        if (isset($data['lot-type'])) {
            $return['data']['lot-type'] = str_replace($this->search, $this->replace, $data['lot-type']);
        }

        if (isset($data['new-flat'])) {
            $return['data']['new-flat'] = $data['new-flat'] ? 1 : 0;
        }

        if (isset($data['rooms'])) {
            $return['data']['rooms'] = intval($data['rooms']);
            if ($return['data']['property-type'] == 'жилая' && empty($return['data']['rooms'])) {
                $return["errors"][] = "rooms";
            }
        } elseif ($return['data']['property-type'] == 'жилая') {
            $return["errors"][] = "rooms";
        }

        if (isset($data['rooms-offered'])) {
            $return['data']['rooms-offered'] = str_replace($this->search, $this->replace, $data['rooms-offered']);
            if ($return['data']['property-type'] == 'жилая' && empty($return['data']["rooms-offered"])) {
                $return["errors"][] = "rooms-offered";
            }
        } elseif ($return['data']['property-type'] == 'жилая') {
            $return["errors"][] = "rooms-offered";
        }

        if (isset($data['open-plan'])) {
            $return['data']['open-plan'] = $data['open-plan'] ? 1 : 0;
        }

        if (isset($data['rooms-type'])) {
            $return['data']['rooms-type'] = str_replace($this->search, $this->replace, $data['rooms-type']);
        }

        if (isset($data['phone'])) {
            $return['data']['phone'] = $data['phone'] ? 1 : 0;
        }

        if (isset($data['internet'])) {
            $return['data']['internet'] = $data['internet'] ? 1 : 0;
        }

        if (isset($data['room-furniture'])) {
            $return['data']['room-furniture'] = $data['room-furniture'] ? 1 : 0;
        }

        if (isset($data['kitchen-furniture'])) {
            $return['data']['kitchen-furniture'] = $data['kitchen-furniture'] ? 1 : 0;
        }

        if (isset($data['television'])) {
            $return['data']['television'] = $data['television'] ? 1 : 0;
        }

        if (isset($data['washing-machine'])) {
            $return['data']['washing-machine'] = $data['washing-machine'] ? 1 : 0;
        }

        if (isset($data['refrigerator'])) {
            $return['data']['refrigerator'] = $data['refrigerator'] ? 1 : 0;
        }

        if (isset($data['balcony'])) {
            $return['data']['balcony'] = str_replace($this->search, $this->replace, $data['balcony']);
        }

        if (isset($data['bathroom-unit'])) {
            $return['data']['bathroom-unit'] = str_replace($this->search, $this->replace, $data['bathroom-unit']);
        }

        if (isset($data['floor-covering'])) {
            $return['data']['floor-covering'] = str_replace($this->search, $this->replace, $data['floor-covering']);
        }

        if (isset($data['window-view'])) {
            $return['data']['window-view'] = str_replace($this->search, $this->replace, $data['window-view']);
        }

        if (isset($data['floor'])) {
            $return['data']['floor'] = intval($data['floor']);
        }

        if (isset($data['floors-total'])) {
            $return['data']['floors-total'] = intval($data['floors-total']);
        }

        if (isset($data['building-name'])) {
            $return['data']['building-name'] = str_replace($this->search, $this->replace, $data['building-name']);
        }

        if (isset($data['building-type'])) {
            $return['data']['building-type'] = str_replace($this->search, $this->replace, $data['building-type']);
        }

        if (isset($data['building-series'])) {
            $return['data']['building-series'] = str_replace($this->search, $this->replace, $data['building-series']);
        }

        if (isset($data['building-state'])) {
            $return['data']['building-state'] = str_replace($this->search, $this->replace, $data['building-state']);
        }

        if (isset($data['built-year'])) {
            if (strlen($data['built-year']) != 4) {
                $data['built-year'] = 0;
            }
            $return['data']['built-year'] = intval($data['built-year']);
        }

        if (isset($data['ready-quarter'])) {
            $return['data']['ready-quarter'] = $data['ready-quarter'];
            if (!in_array($return['data']['ready-quarter'], $this->ready_quarter)) {
                unset($return['data']['ready-quarter']);
            }
        }

        if (isset($data['lift'])) {
            $return['data']['lift'] = $data['lift'] ? 1 : 0;
        }

        if (isset($data['rubbish-chute'])) {
            $return['data']['rubbish-chute'] = $data['rubbish-chute'] ? 1 : 0;
        }

        if (isset($data['is-elite'])) {
            $return['data']['is-elite'] = $data['is-elite'] ? 1 : 0;
        }

        if (isset($data['parking'])) {
            $return['data']['parking'] = $data['parking'] ? 1 : 0;
        }

        if (isset($data['alarm'])) {
            $return['data']['alarm'] = $data['alarm'] ? 1 : 0;
        }

        if (isset($data['ceiling-height'])) {
            $return['data']['ceiling-height'] = floatval($data['ceiling-height']);
        }

        if (isset($data['pmg'])) {
            $return['data']['pmg'] = $data['pmg'] ? 1 : 0;
        }

        if (isset($data['toilet'])) {
            $return['data']['toilet'] = str_replace($this->search, $this->replace, $data['toilet']);
        }

        if (isset($data['shower'])) {
            $return['data']['shower'] = str_replace($this->search, $this->replace, $data['shower']);
        }

        if (isset($data['kitchen'])) {
            $return['data']['kitchen'] = $data['kitchen'] ? 1 : 0;
        }

        if (isset($data['pool'])) {
            $return['data']['pool'] = $data['pool'] ? 1 : 0;
        }

        if (isset($data['billiard'])) {
            $return['data']['billiard'] = $data['billiard'] ? 1 : 0;
        }

        if (isset($data['sauna'])) {
            $return['data']['sauna'] = $data['sauna'] ? 1 : 0;
        }

        if (isset($data['water-supply'])) {
            $return['data']['water-supply'] = $data['water-supply'] ? 1 : 0;
        }

        if (isset($data['electricity-supply'])) {
            $return['data']['electricity-supply'] = $data['electricity-supply'] ? 1 : 0;
        }

        if (isset($data['sewerage-supply'])) {
            $return['data']['sewerage-supply'] = $data['sewerage-supply'] ? 1 : 0;
        }

        if (isset($data['gas-supply'])) {
            $return['data']['gas-supply'] = $data['gas-supply'] ? 1 : 0;
        }

        return $return;
    }
}
