<?php

namespace Pg\Modules\Field_editor\Controllers;

/**
 * Field editor module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
use Pg\Libraries\View;

/**
 * Field Editor admin side controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	Field editor
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Admin_Field_editor extends \Controller
{

    /**
     * Allow edit config settings
     *
     * @var boolean
     */
    protected $allow_config_add = false;

    /**
     * Controller
     *
     * @return Admin_Field_editor
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model('Field_editor_model');
        $this->load->model('Menu_model');
        $this->Menu_model->set_menu_active_item('admin_menu', 'system-items');
    }

    /**
     * Manage sections
     * 
     * @param strign $type editor type
     * @param string $mode view mode
     *
     * @return void
     */
    public function sections($type = '', $mode = '')
    {
        if (empty($type)) {
            $type = $this->Field_editor_model->get_default_editor_type(true);
        }
        $this->Field_editor_model->initialize($type);
        $this->view->assign('type', $type);

        $types = $this->Field_editor_model->get_editor_types(true);
        $this->view->assign('types', $types);

        $sections = $this->Field_editor_model->get_section_list();
        $this->view->assign('sections', $sections);
        $sections_count = count($sections);

        $this->load->helper("navigation");
        $page_data = get_admin_pages_data(site_url() . "admin/field_editor/sections/" . $type, $sections_count,
                                          $sections_count, 1, 'briefPage');
        $this->view->assign('page_data', $page_data);

        $this->view->assign('mode', $mode);

        $this->Menu_model->set_menu_active_item('admin_fields_menu', 'sections_list_item');
        $this->view->setHeader(l('admin_header_section_list', 'field_editor'));
        $this->view->render('list_sections');
    }

    /**
     * Edit section data
     *
     * @param string  $type editor type
     * @param integer $id   section identifier
     *
     * @return void
     */
    public function section_edit($type, $id = null)
    {
        if (empty($type)) {
            $type = $this->Field_editor_model->get_default_editor_type(true);
        }
        $this->Field_editor_model->initialize($type);
        $this->view->assign('type', $type);

        if (!empty($id)) {
            $data = $this->Field_editor_model->get_section_by_id($id);
            foreach ($this->pg_language->languages as $lang_id => $lang_data) {
                $validate_lang[$lang_id] = l('section_' . $data["id"], 'field_editor_sections', $lang_id);
            }
        } else {
            $data = array();
        }

        if ($this->input->post('btn_save')) {
            $post_data = array(
                "gid" => $this->input->post("gid", true),
                "editor_type_gid" => $type,
            );
            $langs = $this->input->post("langs", true);
            $validate_data = $this->Field_editor_model->validate_section($id, $post_data, $langs);

            if (!empty($validate_data["errors"])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
                $validate_lang[] = $validate_data["lang"];
            } else {
                if (!$id) {
                    $validate_data["data"]["sorter"] = $this->Field_editor_model->get_section_count($params) + 1;
                }

                $this->Field_editor_model->save_section($id, $validate_data["data"], $validate_data["lang"]);

                $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_update_section_data', 'field_editor'));
                $this->view->setRedirect(site_url() . "admin/field_editor/sections/" . $data['editor_type_gid']);
            }
        }

        // languages
        $this->view->assign('languages', $this->pg_language->languages);
        $this->view->assign('languages_count', count($this->pg_language->languages));
        $this->view->assign('cur_lang', $this->pg_language->current_lang_id);

        if (!empty($validate_lang)) {
            $this->view->assign('validate_lang', $validate_lang);
        }

        $this->view->assign('data', $data);
        $this->Menu_model->set_menu_active_item('admin_fields_menu', 'sections_list_item');
        $this->view->setHeader(l('admin_header_section_list', 'field_editor'));
        $this->view->render('edit_sections');
    }

    /**
     * Remove section data
     *
     * @param string  $type editor type
     * @param integer $id   section identifier
     *
     * @return void
     */
    public function section_delete($type, $id)
    {
        $this->Field_editor_model->initialize($type);
        if (!empty($id)) {
            $this->Field_editor_model->delete_section($id);
            $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_delete_section', 'field_editor'));
        }
        $this->view->setRedirect(site_url() . "admin/field_editor/sections/" . $type);
    }

    /**
     * Resort sections
     *
     * @return boolean
     */
    public function ajax_section_sort()
    {
        $item_data = $this->input->post('sorter');
        $item_data = $item_data["parent_0"];
        if (empty($item_data)) {
            return false;
        }

        foreach ($item_data as $key => $sorter) {
            $section_id = intval(str_replace("item_", "", $key));
            if (empty($section_id)) {
                continue;
            }
            $this->Field_editor_model->set_section_sorter($section_id, $sorter);
        }

        return true;
    }

    /**
     * Manage fields
     *
     * @param string $type    editor type
     * @param string $section section guid
     * @param string $mode    view mode
     *
     * @return void
     */
    public function fields($type = '', $section = '', $mode = '')
    {
        if (!$this->pg_module->is_module_active("users")) {
            $this->sections();
            return false;
        }

        if (empty($type)) {
            $type = $this->Field_editor_model->get_default_editor_type(true);
        }

        $this->Field_editor_model->initialize($type);
        $this->view->assign('type', $type);

        $types = $this->Field_editor_model->get_editor_types(true);
        $this->view->assign('types', $types);

        $sections = $this->Field_editor_model->get_section_list();
        $this->view->assign('sections', $sections);


        if (empty($section)) {
            $t = current($sections);
            $section = $t["gid"];
        }

        $this->view->assign('section', $section);

        $params["where"]["section_gid"] = $section;
        $fields = $this->Field_editor_model->get_fields_list($params);
        $this->view->assign('fields', $fields);
        $fields_count = count($fields);

        $this->load->helper("navigation");
        $page_data = get_admin_pages_data(site_url() . "admin/field_editor/fields/" . $type . "/" . $section,
                                          $fields_count, $fields_count, 1, 'briefPage');
        $this->view->assign('page_data', $page_data);

        $this->view->assign('mode', $mode);

        $this->Menu_model->set_menu_active_item('admin_fields_menu', 'fields_list_item');
        $this->view->setHeader(l('admin_header_fields_list', 'field_editor'));
        $this->view->render('list_fields');
    }

    /**
     * Edit field data
     *
     * @param string  $type    editor type
     * @param string  $section section gid
     * @param integer $id      field identifier
     *
     * @return void
     */
    public function field_edit($type, $section, $id = null)
    {
        if (empty($type)) {
            $type = $this->Field_editor_model->get_default_editor_type();
        }
        $this->Field_editor_model->initialize($type);
        $this->view->assign('type', $type);
        $this->view->assign('type_settings', $this->Field_editor_model->get_settings());
        $validate_data['data'] = array();
        if (!empty($id)) {
            $data = $this->Field_editor_model->get_field_by_id($id);
            foreach ($this->pg_language->languages as $lang_id => $lang_data) {
                $validate_lang[$lang_id] = $this->Field_editor_model->format_field_name($data, $lang_id);
            }
            $section = $data["section_gid"];
        } else {
            $data['gid'] = $this->Field_editor_model->get_field_gid();
            $data["field_type"] = "text";
        }

        $this->view->assign('section', $section);
        $section_data = $this->Field_editor_model->get_section_by_gid($section);
        $this->view->assign('section_data', $section_data);

        if ($this->input->post('btn_save')) {
            if ($id) {
                $flag = "change";
                $field_type = $data["field_type"];
                $post_data = array(
                    "settings_data" => $this->input->post("settings_data", true),
                    "fts" => $this->input->post("fts", true),
                );
            } else {
                $flag = "add";
                $post_data = array(
                    "gid" => $this->input->post("gid", true),
                    "section_gid" => $section,
                    "editor_type_gid" => $type,
                    "field_type" => $this->input->post("field_type", true),
                    "fts" => $this->input->post("fts", true),
                );
                $field_type = $this->input->post("field_type", true);
            }

            $comment = $this->input->post("comment", true);
            if (is_array($comment)) {
                $post_data = array_merge($post_data, $comment);
            }

            $langs = $this->input->post("langs", true);
            $validate_data = $this->Field_editor_model->validate_field($id, $field_type, $post_data, $langs);

            if (!empty($validate_data["errors"])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
                $validate_lang = $langs;
            } else {
                if ($flag == "change") {
                    $validate_data["data"]["gid"] = $data["gid"];
                    $validate_data["data"]["field_type"] = $data["field_type"];
                } else {
                    $params["where"]["section_gid"] = $section;
                    $validate_data["data"]["sorter"] = $this->Field_editor_model->get_fields_count($params) + 1;
                }

                $id = $this->Field_editor_model->save_field($id, $type, $section, $validate_data["data"],
                                                            $validate_data['lang']);

                $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_update_section_data', 'field_editor'));
                if ($flag == "add") {
                    $this->view->setRedirect(site_url() . "admin/field_editor/field_edit/" . $type . "/" . $section . "/" . $id);
                } else {
                    $this->view->setRedirect(site_url() . "admin/field_editor/fields/" . $type . "/" . $section);
                }
            }
        }
        $validate_data['data'] = array_merge($validate_data['data'], $data);

        // languages
        $this->view->assign('languages', $this->pg_language->languages);
        $this->view->assign('languages_count', count($this->pg_language->languages));
        $this->view->assign('cur_lang', $this->pg_language->current_lang_id);

        if (!empty($validate_lang)) {
            $this->view->assign('validate_lang', $validate_lang);
        }

        if ($id) {
            $this->view->assign('type_block_content', $this->_get_field_type_block($data["field_type"], $data));
        }

        $this->view->assign('data', $validate_data["data"]);
        $this->view->assign('field_type_lang', ld('field_type', 'field_editor'));
        $this->Menu_model->set_menu_active_item('admin_fields_menu', 'fields_list_item');
        $this->view->setHeader(l('admin_header_fields_list', 'field_editor'));
        $this->view->render('edit_fields');
    }

    /**
     * Block of settings by field type
     *
     * @param strign $field_type field type
     * @param array  $data       field data
     *
     * @return void
     */
    protected function _get_field_type_block($field_type, $data)
    {
        $this->view->assign('field_type', $field_type);
        $this->view->assign('data', $data);
        $this->view->assign('initial', $this->Field_editor_model->get_field_settings($field_type));

        if ($field_type == 'select' || $field_type == 'multiselect') {
            $this->view->assign('options_block',
                                $this->_get_field_select_options($data["options"],
                                                                 $data["settings_data_array"]["default_value"]));
        }

        return $this->view->fetch('edit_fields_type_block');
    }

    /**
     * Block of options for select field
     *
     * @param array  $data               field data
     * @param string $default_option_gid option guid by default
     *
     * @return string
     */
    protected function _get_field_select_options($data, $default_option_gid = 0)
    {
        $this->view->assign('reference_data', $data);
        $this->view->assign('default_option_gid', $default_option_gid);

        return $this->view->fetch('edit_fields_select_options');
    }

    /**
     * Edit options of select field by ajax
     *
     * @param integer $id_field field identifier
     *
     * @return void
     */
    public function ajax_get_field_select_options($id_field)
    {
        $field_data = $this->Field_editor_model->get_field_by_id($id_field);

        $this->view->output(
                $this->_get_field_select_options($field_data["options"],
                                                 $field_data["settings_data_array"]["default_value"]));

        $this->view->render();
    }

    /**
     * Form of select field options by ajax
     *
     * @param integer $id_field   field identifier
     * @param strign  $option_gid option guid
     *
     * @return void
     */
    public function ajax_get_select_option_form($id_field, $option_gid = '')
    {
        $field_data = $this->Field_editor_model->get_field_by_id($id_field);

        foreach ($this->pg_language->languages as $lid => $lang) {
            $lang_field_data = $this->Field_editor_model->format_field($field_data, $lid);
            $lang_data[$lid]["name"] = $lang["name"]; //$lang_field_data["name"];
            $lang_data[$lid]["value"] = ($option_gid) ? $lang_field_data['options']["option"][$option_gid] : "";
        }

        $this->view->assign('field_data', $field_data);
        $this->view->assign('lang_data', $lang_data);
        $this->view->assign('option_gid', $option_gid);
        $this->view->render('edit_fields_select_option_form');
    }

    /**
     * Save options of select field by ajax
     *
     * @param integer $id_field   field identifier
     * @param string  $option_gid option guid
     *
     * @return void
     */
    public function ajax_set_select_option($id_field, $option_gid = '')
    {
        $lang_data = $this->input->post('data', true);
        $return = $this->Field_editor_model->validate_field_option($id_field, $option_gid, $lang_data);
        if (empty($return['errors'])) {
            $this->Field_editor_model->set_field_option($id_field, $option_gid, $return['lang']);
            $return['success'] = l('success_update_option_data', 'field_editor');
            $return['is_success'] = 1;
        } else {
            $return['errors'] = implode(', ', $return['errors']);
            $return['is_error'] = 1;
        }

        $this->view->assign($return);

        $this->view->render();
    }

    /**
     * Remove option from select field by ajax
     *
     * @param integer $id_field   field identifier
     * @param string  $option_gid option guid
     *
     * @return void
     */
    public function ajax_delete_select_option($id_field, $option_gid)
    {
        $this->Field_editor_model->delete_field_option($id_field, $option_gid);
        return;
    }

    /**
     * Save sorting data of select field options by ajax
     *
     * @param integer $id_field field identifier
     *
     * @return void
     */
    public function ajax_save_select_option_sorter($id_field)
    {
        $field_data = $this->Field_editor_model->get_field_by_id($id_field);

        $sorter = $this->input->post("sorter");
        foreach ($sorter as $item_str => $sort_index) {
            $sorter_data[$sort_index] = str_replace("option_", "", $item_str);
        }

        if (empty($sorter_data)) {
            return;
        }
        ksort($sorter_data);
        $this->Field_editor_model->sorter_field_option($id_field, $sorter_data);

        return;
    }

    /**
     * Remove field data
     *
     * @param string  $type    editor type
     * @param string  $section section guid
     * @param integer $id      field identifier
     *
     * @return void
     */
    public function field_delete($type, $section, $id)
    {
        $this->Field_editor_model->initialize($type);
        if (!empty($id)) {
            $this->Field_editor_model->delete_field($id);
            $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_delete_field', 'field_editor'));
        }
        $this->view->setRedirect(site_url() . "admin/field_editor/fields/" . $type . "/" . $section);
    }

    /**
     * Resort fields
     *
     * @return boolean
     */
    public function ajax_field_sort()
    {
        $item_data = $this->input->post('sorter');
        $item_data = $item_data["parent_0"];
        if (empty($item_data)) {
            return false;
        }

        foreach ($item_data as $key => $sorter) {
            $field_id = intval(str_replace("item_", "", $key));
            if (empty($field_id)) {
                continue;
            }
            $this->Field_editor_model->set_field_sorter($field_id, $sorter);
        }

        return true;
    }

    //// forms

    /**
     * Forms management
     *
     * @param strign $type editor type
     *
     * @return void
     */
    public function forms($type = '')
    {
        if (!$this->pg_module->is_module_active("users")) {
            $this->sections();
            return false;
        }
        if (empty($type))
            $type = $this->Field_editor_model->get_default_editor_type(true);
        $this->Field_editor_model->initialize($type);
        $this->view->assign('type', $type);

        $types = $this->Field_editor_model->get_editor_types(true);
        $this->view->assign('types', $types);

        $this->load->model('field_editor/models/Field_editor_forms_model');
        $params["where"]["editor_type_gid"] = $type;
        $forms = $this->Field_editor_forms_model->get_forms_list($params);
        $this->view->assign('forms', $forms);

        $forms_count = count($forms);

        $this->load->helper("navigation");
        $page_data = get_admin_pages_data(site_url() . "admin/field_editor/forms/" . $type, $forms_count, $forms_count,
                                          1, 'briefPage');
        $this->view->assign('page_data', $page_data);

        $this->view->assign('allow_config_add', $this->allow_config_add);

        $this->Menu_model->set_menu_active_item('admin_fields_menu', 'forms_list_item');
        $this->view->setHeader(l('admin_header_forms_list', 'field_editor'));
        $this->view->render('list_forms');
    }

    /**
     * Edit form data
     *
     * @param string  $type editor type
     * @param integer $id   form identifier
     *
     * @return void
     */
    public function form_edit($type, $id = 0)
    {
        if (empty($type)) {
            $type = $this->Field_editor_model->get_default_editor_type(true);
        }

        $this->Field_editor_model->initialize($type);
        $this->view->assign('type', $type);

        $types = $this->Field_editor_model->get_editor_types(true);
        $this->view->assign('types', $types);

        $this->load->model('field_editor/models/Field_editor_forms_model');
        if (!empty($id)) {
            $data = $this->Field_editor_forms_model->get_form_by_id($id);
        } else {
            $data = array('editor_type_gid' => $type);
        }

        if ($this->input->post('btn_save')) {
            $post_data = array(
                "gid" => $this->input->post("gid", true),
                "name" => $this->input->post("name", true),
                "editor_type_gid" => $this->input->post("editor_type_gid", true),
            );

            $validate_data = $this->Field_editor_forms_model->validate_form($id, $post_data);
            if (!empty($validate_data["errors"])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
            } else {
                $new_id = $this->Field_editor_forms_model->save_form($id, $validate_data["data"]);

                $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_update_form_data', 'field_editor'));
                $this->view->setRedirect(site_url() . "admin/field_editor/forms");
            }
        }

        $this->view->assign('data', $data);
        $this->Menu_model->set_menu_active_item('admin_fields_menu', 'forms_list_item');
        $this->view->setHeader(l('admin_header_forms_list', 'field_editor'));
        $this->view->render('edit_forms');
    }

    /**
     * Remove form
     *
     * @param string  $type editor type
     * @param integer $id   form identifier
     *
     * @return void
     */
    public function form_delete($type, $id = 0)
    {
        if (empty($type)) {
            $type = $this->Field_editor_model->get_default_editor_type(true);
        }
        $this->Field_editor_model->initialize($type);

        $this->load->model('field_editor/models/Field_editor_forms_model');
        $this->Field_editor_forms_model->delete_form_by_id($id);
        $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_delete_form', 'field_editor'));
        $this->view->setRedirect(site_url() . "admin/field_editor/forms/" . $type);
    }

    /**
     * Manage form fields
     *
     * @param integer $id form identifier
     *
     * @return void
     */
    public function form_fields($id)
    {
        $this->load->model('field_editor/models/Field_editor_forms_model');
        $data = $this->Field_editor_forms_model->format_form(
                $this->Field_editor_forms_model->get_form_by_id($id)
        );
        $data["field_data_json"] = (!empty($data["field_data"])) ? json_encode($data["field_data"]) : '';

        $data["names"] = array();
        if (!empty($data["field_data"])) {
            list($section_gids, $field_gids) = $this->_get_disallowed_data($data["field_data"]);
            $this->Field_editor_model->initialize($data["editor_type_gid"]);

            if (!empty($section_gids)) {
                $raw_sections = $this->Field_editor_forms_model->get_form_sections($data["editor_type_gid"]);
                foreach ($raw_sections as $sgid => $sname) {
                    $data["names"]["section_" . $sgid] = $sname;
                }
            }
            if (!empty($field_gids)) {
                $f_params["where_in"]["gid"] = $field_gids;
                $raw_fields = $this->Field_editor_model->get_fields_list($f_params);
                foreach ($raw_fields as $r) {
                    $data["names"]["field_" . $r["gid"]] = $r["name"];
                }
            }
        }
        $data["field_names_json"] = (!empty($data["names"])) ? json_encode($data["names"]) : '';

        $this->view->assign('data', $data);
        $this->Menu_model->set_menu_active_item('admin_fields_menu', 'forms_list_item');
        $this->view->setBackLink(site_url() . "admin/field_editor/forms");
        $this->view->setHeader(l('admin_header_forms_list', 'field_editor'));
        $this->view->render('edit_form_fields');
    }

    /**
     * Save form fields by ajax
     *
     * @param integer $id form identifier
     *
     * @return void
     */
    public function ajax_save_form_fields($id)
    {
        $this->load->model('field_editor/models/Field_editor_forms_model');
        $post_data = array(
            'field_data' => $this->input->post("field_data", true),
        );
        $validate_data = $this->Field_editor_forms_model->validate_form($id, $post_data);
        $this->Field_editor_forms_model->save_form($id, $validate_data["data"]);
        $return["success"] = l('success_update_form_data', 'field_editor');

        return json_encode($return);
    }

    /**
     * Add section to form by ajax
     *
     * @param integer $id          form identifier
     * @param strign  $section_gid section guid
     *
     * @return void
     */
    public function ajax_get_add_section_form($id, $section_gid = '')
    {
        $this->load->model('field_editor/models/Field_editor_forms_model');
        $form = $this->Field_editor_forms_model->get_form_by_id($id);

        if (!empty($section_gid)) {
            $data = $this->Field_editor_forms_model->get_form_section($form["editor_type_gid"], $section_gid, 'all');
        } else {
            $data = array();
        }
        ///// languages
        $this->view->assign('languages', $this->pg_language->languages);
        $this->view->assign('cur_lang', $this->pg_language->current_lang_id);
        $this->view->assign('section_gid', $section_gid);

        $this->view->assign('data', $data);

        $this->view->output($this->view->fetch('ajax_form_add_section'));

        $this->view->render();
    }

    /**
     * Add field to form by ajax
     *
     * @param integer $id          form identifier
     * @param string  $section_gid section guid
     *
     * @return void
     */
    public function ajax_get_add_field_form($id, $section_gid = '')
    {
        $this->load->model('field_editor/models/Field_editor_forms_model');
        $data = $this->Field_editor_forms_model->get_form_by_id($id);

        list($disallowed_sections, $disallowed_fields) = $this->_get_disallowed_data($data["field_data"]);

        $this->Field_editor_model->initialize($data["editor_type_gid"]);

        if (!$section_gid) {
            $sections = $this->Field_editor_model->get_section_list();
            $this->view->assign('sections', $sections);
            $this->view->assign('form_type', 'sections');
        } else {
            if (!empty($disallowed_fields)) {
                $params["where_sql"][] = "gid NOT IN ('" . implode("', '", $disallowed_fields) . "')";
            } else {
                $params = array();
            }
            $params["where"]["section_gid"] = $section_gid;
            $fields = $this->Field_editor_model->get_fields_list($params);
            $this->view->assign('fields', $fields);
            $this->view->assign('form_type', 'fields');
        }

        $this->view->output($this->view->fetch('ajax_form_add_field'));

        $this->view->render();
    }

    /**
     * Get form section by ajax
     *
     * @param integer $id_form     form identifier
     * @param string  $section_gid section guid
     *
     * @return void
     */
    public function ajax_get_section_data($id_form, $section_gid = '')
    {
        $this->load->model('field_editor/models/Field_editor_forms_model');
        $data = $this->Field_editor_forms_model->get_form_by_id($id_form);
        $return["action"] = (!empty($section_gid)) ? 'update' : "add";

        $langs = $this->input->post("langs", true);
        if (!empty($langs)) {
            $section_gid = $this->Field_editor_forms_model->form_section_save($data["editor_type_gid"], $section_gid,
                                                                              $langs);
        }
        if (empty($section_gid)) {
            return false;
        }

        $return["data"] = array(
            "type" => 'section',
            "section" => array("gid" => $section_gid),
            "fields" => array(),
        );
        $return["names"]["section_" . $section_gid] = $this->Field_editor_forms_model->get_form_section($data["editor_type_gid"],
                                                                                                        $section_gid);

        $this->view->assign($return);

        $this->view->render();
    }

    /**
     * Get from field by ajax
     *
     * @param integer $id_form   form identifier
     * @param string  $field_gid field guid
     *
     * @return void
     */
    public function ajax_get_field_data($id_form, $field_gid)
    {
        $this->load->model('field_editor/models/Field_editor_forms_model');
        $data = $this->Field_editor_forms_model->get_form_by_id($id_form);
        list($disallowed_sections, $disallowed_fields) = $this->_get_disallowed_data($data["field_data"]);

        if (!empty($disallowed_fields) && in_array($field_gid, $disallowed_fields)) {
            return $this->view->render();
        }
        $this->Field_editor_model->initialize($data["editor_type_gid"]);
        $field = $this->Field_editor_model->get_field_by_gid($field_gid);

        $settings = array();
        $default = $this->Field_editor_forms_model->get_form_field_settings($field["field_type"]);
        if (!empty($default)) {
            foreach ($default as $pname => $pdata) {
                $settings[$pname] = $pdata["default"];
            }
        }

        $return["data"] = array(
            'type' => 'field',
            'field' => array(
                'gid' => $field["gid"],
                'section_gid' => $field["section_gid"],
                'type' => $field["field_type"],
            ),
            'settings' => $settings,
        );
        $return["names"]["field_" . $field_gid] = $field["name"];

        if (!empty($return["data"])) {
            $this->view->assign($return);
        }

        $this->view->render();
    }

    /**
     * Remove form section by ajax
     *
     * @param integer $id_form     form identfier
     * @param string  $section_gid section guid
     *
     * @return void
     */
    public function ajax_delete_form_section($id_form, $section_gid)
    {
        $this->load->model('field_editor/models/Field_editor_forms_model');
        $data = $this->Field_editor_forms_model->get_form_by_id($id_form);
        $this->Field_editor_forms_model->form_section_delete($data["editor_type_gid"], $section_gid);

        return;
    }

    /**
     * Edit view settings of form field
     *
     * @param integer $id_form   form identifier
     * @param string  $field_gid field guid
     *
     * @return void
     */
    public function ajax_get_field_settings_form($id_form, $field_gid)
    {
        $this->load->model('field_editor/models/Field_editor_forms_model');
        $data = $this->Field_editor_forms_model->get_form_by_id($id_form);

        $field = array();
        if (!empty($data["field_data"])) {
            foreach ($data["field_data"] as $item) {
                if ($item["type"] == 'section') {
                    if (!empty($item["section"]["fields"])) {
                        foreach ($item["section"]["fields"] as $f) {
                            if ($f["field"]['gid'] == $field_gid) {
                                $field = $f;
                                break;
                            }
                        }
                    }
                } elseif ($item["type"] == 'field' && $item["field"]['gid'] == $field_gid) {
                    $field = $item;
                    break;
                }
            }
        }

        if (empty($field) || !in_array($field["field"]["type"], array('select', 'text'))) {
            $this->view->render();

            return;
        }

        $this->view->assign('settings',
                            $this->Field_editor_forms_model->get_form_field_settings($field["field"]["type"]));
        $this->view->assign('field_type', $field["field"]["type"]);
        $this->view->assign('field', $field);

        $this->view->output($this->view->fetch('ajax_form_field_settings'));

        $this->view->render();
    }

    /**
     * Return fields guids of form
     *
     * @param array $field_data fields data
     *
     * @return array
     */
    protected function _get_disallowed_data($field_data)
    {
        $this->load->model('field_editor/models/Field_editor_forms_model');

        return $this->Field_editor_forms_model->get_form_field_gids($field_data);
    }

}
