<?php

/**
 * Geomap module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

/**
 * Geomap management
 *
 * @package 	PG_RealEstate
 * @subpackage 	Geomap
 *
 * @category	helpers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!function_exists('show_map')) {

    /**
     * Show map block
     *
     * @param array $params block parameters
     *
     * @return string
     */
    function show_map($params)
    {
        $ci = &get_instance();
        $ci->load->model('Geomap_model');
        if (isset($params['width'])) {
            $params['settings']['width'] = $params['width'];
        }
        if (isset($params['height'])) {
            $params['settings']['height'] = $params['height'];
        }
        if (empty($params['id_user'])) {
            $params['id_user'] = 0;
        }
        return $ci->Geomap_model->create_map(
                $params['map_gid'], $params['id_user'], get($params['object_id']), $params['gid'], 
                $params['markers'], $params['settings'], !empty($params['only_load_scripts']), 
                !empty($params['only_load_content']), isset($params['map_id']) ? $params['map_id'] : false, 
                isset($params['map_action']) ? $params['map_action'] : null
        );
    }

}

if (!function_exists('show_default_map')) {

    /**
     * Show map block for driver by default
     *
     * @param array $params block parameters
     *
     * @return string
     */
    function show_default_map($params)
    {
        $ci = &get_instance();
        $ci->load->model('Geomap_model');

        if (!isset($params['id_user'])) {
            $params['id_user'] = 0;
        }
        if (!isset($params['object_id'])) {
            $params['object_id'] = 0;
        }
        if (!isset($params['gid'])) {
            $params['gid'] = '';
        }
        if (!isset($params['markers'])) {
            $params['markers'] = array();
        }
        if (!isset($params['settings'])) {
            $params['settings'] = array();
        }

        if (isset($params["width"])) {
            $params["settings"]["width"] = $params["width"];
        }
        if (isset($params["height"])) {
            $params["settings"]["height"] = $params["height"];
        }

        $only_load_scripts = !empty($params["only_load_scripts"]) ? true : false;
        $only_load_content = !empty($params["only_load_content"]) ? true : false;

        if (!isset($params['map_id'])) {
            $params['map_id'] = false;
        }

        return $ci->Geomap_model->create_default_map(
                $params['id_user'], $params['object_id'], $params['gid'], $params['markers'], $params['settings'], $only_load_scripts, $only_load_content, $params['map_id']
        );
    }

}

if (!function_exists("show_map_view")) {

    /**
     * Show link to map view
     *
     * @param array $url map url
     *
     * @return string
     */
    function show_map_view($url)
    {
        $ci = &get_instance();
        $ci->view->assign("url", $url);

        return $ci->view->fetch("helper_show_map", "user", "geomap");
    }

}

if (!function_exists("update_default_map")) {

    /**
     * Update map block for driver by default
     *
     * @param array $params block parameters
     *
     * @return string
     */
    function update_default_map($params)
    {
        if (!isset($params['map_id'])) {
            return '';
        }
        $ci = &get_instance();
        $ci->load->model("Geomap_model");

        return $ci->Geomap_model->update_default_map($params["map_id"], $params["markers"]);
    }

}

if (!function_exists("update_map")) {

    /**
     * Update map block
     *
     * @param array $params block parameters
     *
     * @return string
     */
    function update_map($params)
    {
        if (!isset($params['map_id'])) {
            return '';
        }
        $ci = &get_instance();
        $ci->load->model("Geomap_model");

        return $ci->Geomap_model->update_map($params["map_gid"], $params["map_id"], $params["markers"]);
    }

}

if (!function_exists('geomap_amenity_select')) {

    /**
     * Choose amenities block
     *
     * amenities = array of ids
     *
     * @param string  $gid         driver guid
     * @param integer $max         amenities limit
     * @param array   $amenities   selected amenities
     * @param string  $var         variable name
     * @param string  $var_js_name variable name for javascript
     * @param string  $output      output template
     *
     * @return string
     */
    function geomap_amenity_select($gid, $max = 5, $amenities = array(), $var = 'id_amenity', $var_js_name = '', $output = "max")
    {
        $ci = &get_instance();
        $ci->load->model("geomap/models/Geomap_settings_model");

        $max = intval($max);
        if ($max < 1) {
            $max = 1;
        }

        $c_by_id = $selected_all = $selected = array();
        $selected_data = $max == 1 ? "" : 0;
        $selected_text = "";

        if (!empty($amenities)) {
            $amenities_data = $ci->pg_language->ds->get_reference(
                $ci->Geomap_settings_model->module_gid, $gid, $ci->pg_language->current_lang_id
            );

            $amenities_data = array_intersect_key($amenities_data["option"], array_flip($amenities));
            if (!empty($amenities_data)) {
                foreach ($amenities_data as $amenity_gid => $amenity_name) {
                    $selected_all[$amenity_gid] = 1;
                    $c_by_id[$amenity_gid] = array("id" => $amenity_gid, "name" => $amenity_name);
                    if ($max === 1) {
                        // get text
                        $selected_data .= $amenity_name . " ";
                    } else {
                        // get count
                        ++$selected_data;
                    }
                }
                if ($max > 1) {
                    $selected_text = implode(", ", $amenities_data);
                }
            }
        }

        $data["var"] = $var;
        $data["selected_all"] = $selected_all;
        $data["selected_all_json"] = json_encode($selected_all);
        $data["var_js_name"] = $var_js_name;
        $data["raw_data"] = $c_by_id;
        $data["raw_data_json"] = json_encode($c_by_id);
        $data["max"] = $max;
        $data["gid"] = $gid;
        $data["output"] = ($output) ? $output : 'max';
        $data["selected_text"] = $selected_text;
        $data["selected_data"] = $selected_data;
        $data["rand"] = rand(100000, 999999);

        $ci->view->assign('amenity_helper_data', $data);

        return $ci->view->fetch('helper_amenity_select', 'admin', 'geomap');
    }

}

if (!function_exists('geomap_panorama_available')) {

    /**
     * Check street view is available
     *
     * @param string $gid option guid
     *
     * @return boolean
     */
    function geomap_panorama_available($gid)
    {
        $ci = &get_instance();
        $ci->load->model('Geomap_model');
        $ci->load->model('geomap/models/Geomap_settings_model');
        $map_driver = $ci->Geomap_model->get_default_driver();
        $settings = $ci->Geomap_settings_model->get_parsed_settings($map_driver['gid'], 0, 0, $gid);

        return isset($settings['use_panorama']) && !empty($settings['use_panorama']);
    }

}

if (!function_exists('geomap_load_geocoder')) {

    /**
     * Load geocoder for driver by default
     *
     * @return string
     */
    function geomap_load_geocoder()
    {
        $ci = &get_instance();
        $ci->load->model('Geomap_model');

        return $ci->Geomap_model->create_default_geocoder();
    }

}

if (!function_exists('geomap_load_street_view')) {

    /**
     * Load street view for driver by default
     *
     * @return string
     */
    function geomap_load_street_view()
    {
        $ci = &get_instance();
        $ci->load->model('Geomap_model');

        return $ci->Geomap_model->create_default_street_view();
    }

}

if (!function_exists('geomap_map_example')) {

    /**
     * Return example map content
     *
     * @param float  $lat  latitude of locality
     * @param float  $lon  longitude of locality
     * @param string $text info about locality
     *
     * @return string
     */
    function geomap_map_example($lat, $lon, $text)
    {
        $ci = &get_instance();
        $ci->load->model('Geomap_model');

        $markers[] = array("gid" => 'test', "lat" => $lat, "lon" => $lon, "html" => $text, "dragging" => false);

        return $ci->create_default_map(0, 'example', $markers);
    }

}
