<?php

/**
 * Import module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined("BASEPATH")) {
    exit("No direct script access allowed");
}

/**
 * XML import driver model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Import
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Import_xml_model extends Model
{
    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $CI;

    /**
     * Driver configuration
     *
     * @var array
     */
    public $drivers = array(
        "gid"            => "xml",
        "link"           => "http://www.w3.org/XML/",
        "editable"       => 0,
        "is_upload"      => 1,
        "is_integration" => 0,
        "is_logging"     => 0,
        "status"         => 1,
    );

    /**
     * Allowed file extensions
     *
     * @var string
     */
    public $allowed_types = "xml|zip";

    /**
     * Import file extension
     *
     * @var string
     */
    public $extension = "xml";

    /**
     * Use two step
     *
     * @param boolean
     */
    public $two_step = false;

    /**
     * Driver settings
     *
     * @var array
     */
    public $settings = array();

    /**
     * Driver options
     *
     * @var array
     */
    public $options = array();

    /**
     * Rows limit per request
     *
     * @var integer
     */
    private $rows_per_request = 1;

    /**
     * Elements depth
     *
     * @param integer
     */
    protected static $_depth = 0;

    /**
     * Open items element
     *
     * @var boolean
     */
    protected static $_open1 = false;

    /**
     * Open item element
     *
     * @var boolean
     */
    protected static $_open2 = false;

    /**
     * Current elements data list
     *
     * @var array
     */
    protected static $_elements = array();

    /**
     * Current element data
     *
     * @var array
     */
    protected static $_element = array();

    /**
     * Column data
     *
     * @var array
     */
    protected static $_column = array();

    /**
     * Rows number
     *
     * @var integer
     */
    protected static $_rows = 0;

    /**
     * Import data from file
     *
     * @var boolean
     */
    public $need_upload = true;

    /**
     * Not available symbols
     *
     * @var array
     */
    protected $search = array('&amp;', '&apos;', '&quot;', "&gt;", "&lt;");

    /**
     * Symbols for replace
     *
     * @var array
     */
    protected $replace = array('&', "'", '"', ">", "<");

    /**
     * Constructor
     *
     * @return Import_xml_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->CI = &get_instance();
    }

    /**
     * Set element depth for repeating request
     *
     * @return void
     */
    public function set_depth()
    {
        ++self::$_depth;
        self::$_elements = array();
        self::$_open1 = true;
    }

    /**
     * Return start element
     *
     * @param object $parser xml parser object
     * @param string $name   element name
     * @param array  $attrs  element attributes
     *
     * @return void
     */
    protected function _start_element($parser, $name, $attrs)
    {
        ++self::$_depth;
        switch (self::$_depth) {
            case 1:
                if ($name == "ITEMS") {
                    self::$_elements = array();
                    self::$_open1 = true;
                }
            break;
            case 2:
                if (self::$_open1 && $name == "ITEM") {
                    self::$_element = array();
                    self::$_open2 = true;
                }
            break;
            case 3:
                if (self::$_open2) {
                    self::$_column = array("name" => $name, "value" => '');
                }
            break;
        }
    }

    /**
     * Return end element
     *
     * @param object $parser xml parser object
     * @param string $name   element name
     *
     * @return void
     */
    protected function _end_element($parser, $name)
    {
        switch (self::$_depth) {
            case 1:
                if (self::$_open1) {
                    self::$_open1 = false;
                }
            break;
            case 2:
                if (self::$_open2) {
                    self::$_open2 = false;
                    self::$_elements[] = self::$_element;
                    ++self::$_rows;
                }
            break;
            case 3:
                if (self::$_open2) {
                    self::$_element[] = self::$_column;
                }
            break;
        }
        --self::$_depth;
    }

    /**
     * Return parsed data
     *
     * @param object $parser xml parser object
     * @param string $data   parsed data
     *
     * @return void
     */
    protected function _data_element($parser, $data)
    {
        if (self::$_open2 && self::$_depth == 3) {
            self::$_column["value"] .= $data;
        }
    }

    /**
     * Analyze uploaded file
     *
     * @param array  $settings  driver settings
     * @param array  $data      data settings
     * @param array  $relations data relations
     * @param string $filename  import filename
     *
     * @return array
     */
    public function analyze($settings, $data = array(), $relations = array(), $filename = null)
    {
        $return = array("errors" => array(), "data" => array());

        $return['data']['total'] = 0;
        $return['data']['position'] = 0;
        $return['data']['relations'] = array();

        $text = '';

        $xml_parser = xml_parser_create();
        xml_parser_set_option($xml_parser, XML_OPTION_CASE_FOLDING, true);
        xml_set_element_handler($xml_parser, array("Import_xml_model", "_start_element"), array("Import_xml_model", "_end_element"));
        xml_set_character_data_handler($xml_parser, array("Import_xml_model", "_data_element"));

        $handle = fopen($filename, "rb");

        while (!feof($handle) && ($str = fgets($handle, 4096))) {
            $text = preg_replace("/<\?xml.*\?>/im", "", $text . $str);
            $pos = strrpos($text, ">");
            if ($pos === false) {
                continue;
            }
            if (!xml_parse($xml_parser, $text)) {
                $message = "XML Error: " . xml_error_string(xml_get_error_code($xml_parser)) .
                           " at line " . xml_get_current_line_number($xml_parser);
                throw new Exception($message);
            }

            $text = substr($text, $pos + 1);

            if (self::$_rows > 1) {
                continue;
            }

            foreach (self::$_element as $i => $column) {
                if (!empty($relations[$i]) && ($column["name"] == $relations[$i]['name'] || empty($column["name"]))) {
                    $return['data']['relations'][$i] = $relations[$i];
                } else {
                    $return['data']['relations'][$i] = array("name" => $column["name"], "type" => "", "link" => "");
                }
            }
        }

        fclose($handle);

        $return["data"]["total"] = self::$_rows;

        return $return;
    }

    /**
     * Parse uploaded file
     *
     * @param string  $filename    import filename
     * @param array   $settings    import settings
     * @param array   $driver      driver data
     * @param array   $relations   field relations
     * @param array   $search_data search criteria
     * @param integer $position    start position
     *
     * @return array
     */
    public function parse($filename, $settings, $driver, $relations, $search_data, $position)
    {
        $return = array("data" => array(), "position" => 0, "status" => 1, "processed" => 0);

        $xml_parser = xml_parser_create();
        xml_parser_set_option($xml_parser, XML_OPTION_CASE_FOLDING, true);
        xml_set_element_handler($xml_parser, array("Import_xml_model", "_start_element"), array("Import_xml_model", "_end_element"));
        xml_set_character_data_handler($xml_parser, array("Import_xml_model", "_data_element"));

        $handle = fopen($filename, "rb");

        if (!$handle) {
            return $return;
        }

        fseek($handle, $position);

        $text = "";
        while (!feof($handle) && ($str = fgets($handle, 4096))) {
            $text = preg_replace("/<\?xml.*\?>/uim", "", $text . $str);
            $pos = mb_strrpos($text, ">");
            if ($pos === false) {
                continue;
            }
            if (!xml_parse($xml_parser, mb_substr($text, 0, $pos + 1))) {
                $message = "XML Error: " . xml_error_string(xml_get_error_code($xml_parser)) .
                           " at line " . xml_get_current_line_number($xml_parser);
                throw new Exception($message);
            }
            $text = mb_substr($text, $pos + 1);
            if (self::$_rows < $this->rows_per_request) {
                continue;
            }
            $return["status"] = 0;
            break;
        }

        $return["position"] = ftell($handle);

        fclose($handle);

        $data = array();
        foreach (self::$_elements as $i => $element) {
            foreach ($element as $column) {
                foreach ($relations as $index => $relation) {
                    if ($column["name"] == $relation["name"]) {
                        $data[$i][$index] = str_replace($this->search, $this->replace, $column["value"]);
                        break;
                    }
                }
            }
        }

        $return["data"] = $data;
        $return["processed"] = self::$_rows;

        return $return;
    }

    /**
     * Validate selection data
     *
     * @param array  $data       selection data
     * @param string $upload_gid upload gid
     *
     * @return array
     */
    public function validate_data($data, $upload_gid = 'import_file')
    {
        $return = array("errors" => array(), "data" => $data);

        $this->CI->load->helper("upload");

        //// upload src file
        $upload_config = array(
            "allowed_types" => $this->allowed_types,
            "overwrite"     => true,
        );

        $result = validate_file($upload_gid, $upload_config);
        if ($result["error"]) {
            $return["errors"] = $result["error"];
        }

        return $return;
    }

    /**
     * Validate settings for saving to data source
     *
     * @param array $settings settings data
     *
     * @return array
     */
    public function validate_settings($settings)
    {
        $return = array('errors' => array(), 'data' => array());

        return $return;
    }

    /**
     * Connect to driver
     *
     * @param array $settings driver settings
     *
     * @return array
     */
    public function connect($settings)
    {
        $return = array("errors" => array(), "data" => array());

        return $return;
    }

    /**
     * Return rets service metadata
     *
     * @param array $settings driver settings
     *
     * @return array
     */
    public function get_settings($settings, $data = array())
    {
        $return = array();

        return $return;
    }
}
