<?php

namespace Pg\Modules\Languages\Controllers;

/**
 * Languages module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Languages api controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	Languages
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Api_Languages extends \Controller
{

    /**
     * Constructor
     *
     * @return Api_Languages
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Change current language
     *
     * @param integer $lang_id language identifier
     *
     * @return void
     */
    public function change_lang()
    {
        $lang_id = intval($this->input->post('lang_id', true));
        if (!$lang_id) {
            log_message('error', 'languages API: Empty lang id');
            $this->set_api_content('error', l('api_error_empty_lang_id', 'languages'));

            return false;
        }
        $this->session->set_userdata('lang_id', $lang_id);
        $this->set_api_content('data', array('lang_id' => $lang_id));
    }

    /**
     * Change current language by code
     *
     * @param string $lang_code language code
     *
     * @return void
     */
    public function change_language_by_code($lang_code)
    {
        $lang_id = $this->pg_language->get_lang_id_by_code($lang_code, 1);

        if (!$lang_id) {
            log_message('error', 'languages API: Invalid lang code');
            $this->set_api_content('error', l('api_error_empty_lang_id', 'languages'));

            return false;
        }

        $this->session->set_userdata('lang_id', $lang_id);
        $this->set_api_content('data', array('lang_id' => $lang_id));
    }

}
