<?php

/**
 * Listings module
 *
 * @package     PG_RealEstate
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
/**
 * Listings management
 *
 * @package     PG_RealEstate
 * @subpackage  Listings
 * @category    helpers
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!function_exists('listings_saved_searches')) {

    /**
     * Show saved searches search block
     *
     * @param array $params block parameters
     * @return string
     */
    function listings_saved_searches($params)
    {
        $ci = &get_instance();
        $ci->load->model('Listings_model');

        //get saved searches
        $use_save_search = $ci->session->userdata('auth_type') == 'user';
        if (!$use_save_search) {
            return;
        }

        $ci->load->model('listings/models/Listings_search_model');
        $search_params = array('user' => $ci->session->userdata('user_id'));
        $save_search_items_on_page = $ci->pg_module->get_module_config('listings', 'save_search_items_per_page');
        $save_search_data = $ci->Listings_search_model->getSearchesList(
            $search_params, 1, $save_search_items_on_page, array('id' => 'DESC')
        );
        $ci->view->assign('save_search_data', $save_search_data);

        return $ci->view->fetch('saved_searches_block', 'user', 'listings');
    }

}

if (!function_exists('listings_search_block')) {

    /**
     * Show search search block
     *
     * @param array $params block parameters
     * @return string
     */
    function listings_search_block($params)
    {
        $ci = &get_instance();
        $ci->load->model('Listings_model');

        //get search criteria
        $filters = isset($_SESSION['listings_search']['data']) ? $_SESSION['listings_search']['data'] : array();
        if (!empty($filters['id_country'])) {
            $ci->load->helper('countries');
            $location = array();
            $location['country'] = $filters['id_country'];
            if (!empty($filters['id_region'])) {
                $location['region'] = $filters['id_region'];
                if (!empty($filters['id_city'])) {
                    $location['city'] = $filters['id_city'];
                    if (!empty($filters['id_district'])) {
                        $location['district'] = $filters['id_district'];
                        $location = districts_output_format(array($location));
                        if (!empty($location)) {
                            $filters['location'] = $location[0];
                        }
                    } else {
                        $location = cities_output_format(array($location));
                        if (!empty($location)) {
                            $filters['location'] = $location[0];
                        }
                    }
                } else {
                    $location = regions_output_format(array($location));
                    if (!empty($location)) {
                        $filters['location'] = $location[0];
                    }
                }
            } else {
                $location = countries_output_format(array($location));
                if (!empty($location)) {
                    $filters['location'] = $location[0];
                }
            }
        }

        //get current search
        $current_search_data = $ci->Listings_model->get_search_data();
        $ci->view->assign('current_search_data', $current_search_data);

        $use_save_search = $ci->session->userdata('auth_type') == 'user';
        $ci->view->assign('use_save_search', $use_save_search);

        return $ci->view->fetch('helper_search_block', 'user', 'listings');
    }

}

if (!function_exists('listings_save_search_block')) {

    function listings_save_search_block()
    {
        $ci = &get_instance();
        $use_save_search = $ci->session->userdata('auth_type') == 'user';
        if (!$use_save_search) {
            return '';
        }
        $ci->load->model('Listings_model');

        $ci->load->model('listings/models/Listings_search_model');
        $search_params = array('user' => $ci->session->userdata('user_id'));
        $save_search_items_on_page = $ci->pg_module->get_module_config('listings', 'save_search_items_per_page');
        $save_search_data = $ci->Listings_search_model->getSearchesList(
            $search_params, 1, $save_search_items_on_page, array('id' => 'DESC')
        );

        $ci->view->assign('save_search_data', $save_search_data);

        // get current search
        $ci->view->assign('current_search_data', $ci->Listings_model->get_search_data());
        return $ci->view->fetch('helper_save_search_block', 'user', 'listings');
    }

}

if (!function_exists('listings_quick_search')) {

    /**
     * Show quick search form
     *
     * @return string
     */
    function listings_quick_search()
    {
        $ci = &get_instance();
        $ci->load->model('Listings_model');

        //get search criteria
        $filters = isset($_SESSION['listings_search']['data']) ? $_SESSION['listings_search']['data'] : array();

        //load properties
        $ci->load->model('Properties_model');
        $property_types = $ci->Properties_model->get_categories('property_types', $ci->pg_language->current_lang_id);
        $ci->view->assign('property_types', $property_types);
        $property_items = $ci->Properties_model->get_all_properties('property_types', $ci->pg_language->current_lang_id);
        $ci->view->assign('property_items', $property_items);

        $ci->load->model('listings/models/Listings_module_model');
        //load price range
        $price_range = $ci->Listings_module_model->get_price_range();
        $ci->view->assign('price_range', $price_range);

        //load square range
        $square_range = $ci->Listings_module_model->get_square_range();
        $ci->view->assign('square_range', $square_range);

        //links for search_listings_sections
        $ci->load->model('Listings_model');
        $menu_action_data = $ci->Listings_model->get_search_link_data();
        $method = !empty($menu_action_data['keyword']) ? 'search' : 'index';
        unset($menu_action_data['order']);
        unset($menu_action_data['order_direction']);
        unset($menu_action_data['page']);
        $operation_types = $ci->Listings_model->get_operation_types(true);
        $operation_types_links = array();
        foreach ($operation_types as $type) {
            $menu_action_data['operation_type'] = $type;
            $menu_action_data['operation_type_str'] = l("operation_search_$type", 'listings');
            $operation_types_links[$type] = array(
                'gid' => $type,
                'str' => $menu_action_data['operation_type_str'],
                'link' => $ci->pg_seo->create_url('listings', $method, $menu_action_data),
            );
        }
        $ci->view->assign('operation_types_links', $operation_types_links);

        //load extends forms
        $ci->load->model('listings/models/Listings_form_model');
        $property_type_gid = $ci->Listings_model->get_field_editor_type($filters);
        $quick_search_form_data = $ci->Listings_form_model->get_search_extend_form($property_type_gid, 'quick_search_form_gid', false);
        if (count($quick_search_form_data) > 0) {
            if (!isset($filters['field_editor_data']['quick_search_form'])) {
                $filters['field_editor_data']['quick_search_form'] = [];
            }
            $ci->view->assign('form_data', $quick_search_form_data);
            $quick_search_extend_form = $ci->view->fetch('helper_search_form_block', 'user', 'listings');
            $ci->view->assign('data', $filters['field_editor_data']['quick_search_form']);
            $ci->view->assign('search_form_rand', rand(100000, 999999));
            $ci->view->assign('quick_search_extend_form', $quick_search_extend_form);
        }

        if (!isset($filters['booking_date_start'])) {
            $filters['booking_date_start'] = date('Y-m-d');
        }
        if (!isset($filters['booking_date_end'])) {
            $filters['booking_date_end'] = date('Y-m-d', strtotime($filters['booking_date_start']) + 86400);
        }

        $ci->view->assign('data', $filters);

        $operation_types = $ci->Listings_model->get_operation_types(true);

        $ci->load->helper('start');
        $form_settings = array(
            'object' => $filters['type'],
            'operations' => $operation_types,
            'controls' => main_search_controls(),
        );

        if (!in_array($form_settings['object'], $operation_types)) {
            $form_settings['object'] = current($operation_types);
        }

        $ci->view->assign('search_form_settings', $form_settings);
        return $ci->view->fetch('helper_quick_search_form', 'user', 'listings');
    }

}

if (!function_exists('listings_advanced_search')) {

    /**
     * Show advanced search form
     *
     * @param array $params block parameters
     * @return string
     */
    function listings_advanced_search($params)
    {
        $ci = &get_instance();
        $ci->load->model('Listings_model');

        $filters = isset($_SESSION['listings_search']['data']) ? $_SESSION['listings_search']['data'] : array();

        //load radius data
        $radius_data = $ci->pg_language->ds->get_reference('listings', $ci->Listings_model->radius_data_gid, $ci->pg_language->current_lang_id);
        $ci->view->assign('radius_data', $radius_data);

        //load extends forms
        $ci->load->model('listings/models/Listings_form_model');
        $property_type_gid = $ci->Listings_model->get_field_editor_type($filters);
        $advanced_search_form_data = $ci->Listings_form_model->get_search_extend_form($property_type_gid, 'advanced_search_form_gid', false);
        if (count($advanced_search_form_data) > 0) {
            if (!isset($filters['field_editor_data']['advanced_search_form'])) {
                $filters['field_editor_data']['advanced_search_form'] = array();
            }
            $ci->view->assign('data', $filters['field_editor_data']['advanced_search_form']);
            $ci->view->assign('form_data', $advanced_search_form_data);
            $ci->view->assign('form_rand', rand(100000, 999999));
            $advanced_search_extend_form = $ci->view->fetch('helper_search_form_block', 'user', 'listings');
            $ci->view->assign('advanced_search_extend_form', $advanced_search_extend_form);
        }
        $ci->view->assign('advanced_search_extend_form', '');
        $ci->view->assign('data', $filters);

        return $ci->view->fetch('helper_advanced_search_form', 'user', 'listings');
    }

}

if (!function_exists('listings_main_search_form_block')) {

    function listings_main_search_form_block($property_type_gid, $data=[], $form_settings=[])
    {
        $ci = &get_instance();
        $ci->load->model('listings/models/Listings_form_model');
        $main_search_form_data = $ci->Listings_form_model->get_search_extend_form($property_type_gid, 'main_search_form_gid', false);
        if (empty($main_search_form_data)) {
            return '';
        }
        $ci->view->assign('data', $data);
        $ci->view->assign('main_search_form_data', $main_search_form_data);
    
        if (!empty($form_settings)) {
            $ci->view->assign('form_settings', $form_settings);
        }

        return trim($ci->view->fetch('helper_main_search_form_block', 'user', 'listings'));
    }

}

if (!function_exists('listings_main_search_form')) {

    /**
     * Show main search form
     *
     * @param string $type form type
     * @param boolean $show_data fill selected data
     * @param integer $width block size
     * @return string
     */
    function listings_main_search_form($type = 'line', $show_data = false, $width = 100, $object=null)
    {
        $ci = &get_instance();
        $ci->load->model('Listings_model');

        //get search criteria
        $filters = isset($_SESSION['listings_search']['data']) ? $_SESSION['listings_search']['data'] : array();
        if ($show_data) {
            if (!empty($filters['id_country'])) {
                $ci->load->helper('countries');

                $location = array();
                $location['country'] = $filters['id_country'];
                if (!empty($filters['id_region'])) {
                    $location['region'] = $filters['id_region'];
                    if (!empty($filters['id_city'])) {
                        $location['city'] = $filters['id_city'];
                        if (!empty($filters['id_district'])) {
                            $location['district'] = $filters['id_district'];
                            $location = districts_output_format(array($location));
                            if (!empty($location)) {
                                $filters['location'] = $location[0];
                            }
                        } else {
                            $location = cities_output_format(array($location));
                            if (!empty($location)) {
                                $filters['location'] = $location[0];
                            }
                        }
                    } else {
                        $location = regions_output_format(array($location));
                        if (!empty($location)) {
                            $filters['location'] = $location[0];
                        }
                    }
                } else {
                    $location = countries_output_format(array($location));
                    if (!empty($location)) {
                        $filters['location'] = $location[0];
                    }
                }
            }
        }

        //load properties
        $ci->load->model('Properties_model');
        $property_types = $ci->Properties_model->get_categories('property_types', $ci->pg_language->current_lang_id);
        $ci->view->assign('property_types', $property_types);
        $property_items = $ci->Properties_model->get_all_properties('property_types', $ci->pg_language->current_lang_id);
        $ci->view->assign('property_items', $property_items);

        //load radius data
        $radius_data = $ci->pg_language->ds->get_reference('listings', $ci->Listings_model->radius_data_gid, $ci->pg_language->current_lang_id);
        $ci->view->assign('radius_data', $radius_data);

        $page_data = array(
            'type' => $type,
            'use_advanced' => false,
        );

        $page_data['date_format'] = $ci->pg_date->get_format('date_literal', 'st');
        $page_data['datepicker_date_format'] = $ci->pg_date->get_format('date_literal', 'ui');
        $page_data['datepicker_alt_format'] = $ci->pg_date->get_format('date_numeric', 'ui');

        $ci->view->assign('block_width', $width);
        
        $form_settings = ['rand'=>rand(11111,99999)];
       
        if ($object) {
			$form_settings['object'] = $object;
        }

        if ($type != 'line' && $type != 'slider') {
            $page_data['use_advanced'] = true;
            $property_type_gid = $ci->Listings_model->get_field_editor_type($filters);
            if (empty($filters['field_editor_data']['main_search_form'])) {
                $filters['field_editor_data']['main_search_form'] = [];
            }
            $ci->view->assign('main_search_full_form', listings_main_search_form_block($property_type_gid, $filters['field_editor_data']['main_search_form'], $form_settings));
        }

        if (!isset($filters['booking_date_start'])) {
            $filters['booking_date_start'] = date('Y-m-d');
        }
        if (!isset($filters['booking_date_end'])) {
            $filters['booking_date_end'] = date('Y-m-d', strtotime($filters['booking_date_start']) + 86400);
        }

        if ($ci->pg_module->get_module_config('country_identification', 'automatic_identification')) {
            $ci->load->model('countries/models/Countries_model');
            $user_country = $ci->Countries_model->get_country_by_id($ci->session->userdata('id_country'));
            if (!empty($user_country) && empty($filters['id_country'])) {
                $filters['id_country'] = $user_country['code'];
            }
            $user_region = $ci->Countries_model->get_region($ci->session->userdata('id_region'));
            if (!empty($user_region) && empty($filters['id_city'])) {
                $filters['id_region'] = $user_region['id'];
            }
            $user_city = $ci->Countries_model->get_city($ci->session->userdata('id_city'));
            if (!empty($user_city) && empty($filters['id_city'])) {
                $filters['id_city'] = $user_city['id'];
            }
        }

        $ci->view->assign('data', $filters);
        $ci->view->assign('listings_page_data', $page_data);
        $ci->load->helper('seo');

        $search_action_data = $ci->Listings_model->get_search_link_data();

        unset($search_action_data['order']);
        unset($search_action_data['order_direction']);
        unset($search_action_data['page']);

        $search_action_link = rewrite_link('listings', $search_action_data['keyword'] ? 'search' : 'index', $search_action_data);
        $ci->view->assign('search_action_link', $search_action_link);

        $template_preview_mode = $ci->input->get('template_preview_mode');
        if ($template_preview_mode) {
            $ci->view->assign('template_preview_mode', '?template_preview_mode=1');
        }
        $ci->view->assign('datepicker_names', array('start' => 'filters[booking_date_start]' , 'end' => 'filters[booking_date_end]'));
        
		$ci->view->assign('form_settings', $form_settings);
		
        return $ci->view->fetch('helper_main_search_form', 'user', 'listings');
    }

}

if (!function_exists('listings_main_search_tabs')) {

    /**
     * Available tabs of main search form
     *
     * @return array
     */
    function listings_main_search_tabs()
    {
        $ci = &get_instance();
        $ci->load->model('Listings_model');

        if (!function_exists('listings_operation_names')) {

            function listings_operation_names($operation_type)
            {
                return l('operation_search_' . $operation_type, 'listings');
            }

        }

        $operation_types = $ci->Listings_model->get_operation_types(true);
        if (empty($operation_types)) {
            return array();
        }
        return array_combine($operation_types, array_map('listings_operation_names', $operation_types));
    }

}

if (!function_exists('listings_count')) {

    /**
     * Number of user listings in data source
     *
     * @param array $attrs block parameters
     * @return string
     */
    function listings_count($attrs = array())
    {
        $ci = &get_instance();
        if (empty($attrs['user_id'])) {
            $attrs['user_id'] = $ci->session->userdata('user_id');
        }
        if (!$attrs['user_id'])
            return false;

        $ci->load->model('Listings_model');

        $operation_types = $ci->Listings_model->get_operation_types();

        $counts = array();
        $total = array('active' => 0, 'not_active' => 0, 'total' => 0);

        foreach ($operation_types as $operation_type) {
            $counts[$operation_type]['active'] = $ci->Listings_model->get_listings_count(array('user' => $attrs['user_id'], 'type' => $operation_type, 'active' => 1));
            $counts[$operation_type]['not_active'] = $ci->Listings_model->get_listings_count(array('user' => $attrs['user_id'], 'type' => $operation_type, 'not_active' => 1));
            $counts[$operation_type]['total'] = $counts[$operation_type]['active'] + $counts[$operation_type]['not_active'];
            $total['active'] += $counts[$operation_type]['active'];
            $total['not_active'] += $counts[$operation_type]['not_active'];
            $total['total'] += $counts[$operation_type]['total'];
        }
        $ci->view->assign('listings_counts', $counts);
        $ci->view->assign('listings_total', $total);
        return $ci->view->fetch('helper_listings_count', 'user', 'listings');
    }

}

if (!function_exists('listings_searches_block')) {

    /**
     * User's saved searches
     *
     * @param array $params block parameters
     * @return string
     */
    function listings_searches_block($params = [])
    {
        $ci = &get_instance();
        if (empty($params['user_id'])) {
            $params['user_id'] = $ci->session->userdata('user_id');
        }
        if (!$params['user_id']) {
            return false;
        }

        $ci->load->model('listings/models/Listings_search_model');

        $searches_count = $ci->Listings_search_model->getSearchesCount(array('user' => $params['user_id']));
        $ci->view->assign('searches_count', $searches_count);

        return $ci->view->fetch('helper_listings_searches_count', 'user', 'listings');
    }

}

if (!function_exists('user_listings_button')) {

    /**
     * Link to user's listings page
     *
     * @param array $params block parameters
     * @return string
     */
    function user_listings_button($params)
    {
        $ci = &get_instance();
        if (!$ci->pg_module->is_module_active("users") || !isset($params['user'])) {
            return '';
        }
        if (!is_array($params['user'])) {
            $ci->load->model('Users_model');
            $ci->Users_model->set_format_settings('use_format', false);
            $params['user'] = $ci->Users_model->get_user_by_id($user);
            $ci->Users_model->set_format_settings('use_format', true);
        }
        $ci->view->assign('user_id', $params['user']['id']);
        $ci->view->assign('user_name', $params['user']['name']);

        $ci->load->model('Listings_model');
        $operation_types = $ci->Listings_model->get_operation_types(true);
        $is_listings = false;
        foreach ($operation_types as $operation_type) {
            if (!$params['user']['listings_for_' . $operation_type . '_count']) {
                continue;
            }
            $is_listings = true;
            break;
        }
        $ci->view->assign('is_listings', $is_listings);

        return $ci->view->fetch('helper_user_listings_button', 'user', 'listings');
    }

}

if (!function_exists('user_listings_block')) {

    /**
     * User's listings block
     *
     * @param array $params block parameters
     * @return string
     */
    function user_listings_block($params)
    {
        $ci = &get_instance();

        if (!isset($params['user'])) {
            return false;
        }
        if (!is_array($params['user']) && $ci->pg_module->is_module_active('users')) {
            $ci->load->model('Users_model');
            $ci->Users_model->set_format_settings('use_format', false);
            $params['user'] = $ci->Users_model->get_user_by_id($user);
            $ci->Users_model->set_format_settings('use_format', true);
        }
        $ci->view->assign('user_id', $params['user']['id']);
        $ci->view->assign('user_name', $params['user']['name']);

        $ci->load->model('Listings_model');

        $operation_types = $ci->Listings_model->get_operation_types(true);
        $listings_count = 0;
        foreach ($operation_types as $operation_type) {
            $listings_count += get($params['user']['listings_for_' . $operation_type . '_count'], 0);
        }

        if (!$listings_count) {
            return false;
        }

        if (isset($params['max_count']) && $params['max_count'] > 0) {
            $items_on_page = $params['max_count'];
        } else {
            $items_on_page = $ci->pg_module->get_module_config('start', 'index_items_per_page');
        }

        $filters['user'] = $params['user']['id'];
        $filters['active'] = 1;

        $listings = $ci->Listings_model->get_listings_list($filters, 1, $items_on_page, array('date_modified' => 'DESC'));
        $ci->view->assign('listings', $listings);
        if ($listings_count > $items_on_page) {
            $ci->view->assign('listings_count', $listings_count);
        }

        return $ci->view->fetch('helper_user_listings_block', 'user', 'listings');
    }

}

if (!function_exists('virtual_tour_block')) {

    /**
     * Listing virtual tour block (deprecated)
     *
     * @param array $params block parameters
     * @return string
     */
    function virtual_tour_block($params)
    {
        if (empty($params['data'])) {
            return false;
        }

        $ci = &get_instance();
        $ci->view->assign('virtual_tour_data', $params['data']);

        if (!empty($params['width'])) {
            $width = $params['width'];
        } else {
            $width = 620;
        }
        if (!empty($params['height'])) {
            $height = $params['height'];
        } else {
            $height = 400;
        }

        $ci->view->assign('virtual_tour_width', $width);
        $ci->view->assign('virtual_tour_height', $height);

        $rand = rand(100000, 999999);
        $ci->view->assign('virtual_tour_rand', $rand);

        return $ci->view->fetch('helper_virtual_tour_block', 'user', 'listings');
    }

}

if (!function_exists('wish_list_select')) {

    /**
     * Wish list selector
     *
     * @param array $selected selected objects
     * @param integer $max_select max number of selected
     * @param string $var_name name of variable
     * @param string $template block template
     * @param array $params block parameters
     * @return string
     */
    function wish_list_select($selected = array(), $max_select = 0, $var_name = 'id_wish_lists', $template = 'default', $params = array())
    {
        $ci = &get_instance();
        $ci->load->model('listings/models/Wish_list_model');

        if ($max_select == 1 && !empty($selected) && !is_array($selected)) {
            $selected = array($selected);
        }

        if (!empty($selected)) {
            $filters['ids'] = $selected;
            $data['selected'] = $ci->Wish_list_model->get_wish_lists_list($filters, null, null, null);
            $data['selected_str'] = implode(',', $selected);
        } else {
            $data['selected_str'] = '';
        }

        $data['max_select'] = $max_select;
        $data['var_name'] = $var_name;
        $data['template'] = $template;
        $data['params'] = $params;
        $data['rand'] = rand(100000, 999999);

        $ci->view->assign('select_data', $data);
        return $ci->view->fetch('helper_wish_list_select_' . $template, null, 'listings');
    }

}

if (!function_exists('property_type_select')) {

    /**
     * Property types selector
     *
     * @param integer $max max selected objects
     * @param array $categories categories data
     * @param string $var name of input field
     * @param integer $level level of category
     * @param string $var_js_name name of js variable
     * @param string $output output template
     * @return string
     */
    function property_type_select($max = 1, $categories = array(), $var = 'id_category', $level = 2, $var_js_name = '', $output = 'max')
    {
        $ci = & get_instance();

        $ci->load->model('Listings_model');

        $max = intval($max);
        if ($max < 1) {
            $max = 1;
        }

        $level = intval($level);
        if ($level < 1) {
            $level = 1;
        }
        if ($level > 2) {
            $level = 2;
        }

        $c_by_id = $selected_all = $selected = array();

        $data['var'] = $var ? $var : 'id_category';
        $data['selected_all'] = $selected_all;
        $data['selected_all_json'] = json_encode($selected_all);
        $data['var_js_name'] = $var_js_name;
        $data['raw_data'] = $c_by_id;
        $data['raw_data_json'] = json_encode($c_by_id);
        $data['max'] = $max;
        $data['level'] = $level;
        $data['output'] = ($output) ? $output : 'max';

        $data['selected_data'] = '';
        $data['rand'] = rand(100000, 999999);

        $ci->view->assign('property_type_helper_data', $data);
        return $ci->view->fetch('helper_property_type_select', 'user', 'listings');
    }

}

if (!function_exists('show_mortgage_calc')) {

    /**
     * Content of mortgage calculator
     *
     * @return string
     */
    function show_mortgage_calc()
    {
        $ci = &get_instance();
        $show_mortgage_calc = $ci->pg_module->get_module_config('listings', 'show_mortgage_calc');
        if (!$show_mortgage_calc && !$show_mortgage_calc) {
            return '';
        }

        return $ci->view->fetch('helper_mortgage_calc', 'user', 'listings');
    }

}

if (!function_exists('listing_price_block')) {

    /**
     * Format listing price
     *
     * @param array $params block parameters
     * @return string
     */
    function listing_price_block($params)
    {
        $ci = & get_instance();

        if (empty($params['template'])) {
            $params['template'] = '';
        }

        $ci->view->assign('data', $params['data']);
        $ci->view->assign('template', $params['template']);

        return $ci->view->fetch('helper_price_block', null, 'listings');
    }

}

if (!function_exists('post_listing_button')) {

    /**
     * Post listing button
     *
     * @return string
     */
    function post_listing_button()
    {
        $ci = & get_instance();
        if ($ci->pg_module->is_module_active("users")) {
            static $is_login_enabled = null;

            if (is_null($is_login_enabled)) {
                $ci->load->helper('users');
                $is_login_enabled = is_login_enabled();

                if ($ci->session->userdata('auth_type') != 'user') {
                    $ci->load->model('Users_model');
                    $user_types = $ci->Users_model->getUserTypes();
                    foreach ($user_types as $user_type) {
                        $enabled = $ci->pg_module->get_module_config('users', $user_type . '_register_enabled');
                        if (!$enabled) {
                            $is_login_enabled = false;
                        }
                    }
                }
            }

            if (!$is_login_enabled) {
                return '';
            }

            $ci->load->model('Listings_model');
            $is_guest = $ci->session->userdata('auth_type') != 'user';
            $ci->view->assign('is_guest', $is_guest);

            $noshow = $ci->Listings_model->is_operation_types_disabled();
            return (!$noshow) ? $ci->view->fetch('helper_post_block', 'user', 'listings') : '';
        }
    }

}

if (!function_exists('admin_home_listings_block')) {

    /**
     * Homepage statistics for admin mode
     *
     * @return string
     */
    function admin_home_listings_block()
    {
        $ci = & get_instance();

        $auth_type = $ci->session->userdata('auth_type');
        if ($auth_type != 'admin')
            return '';

        $user_type = $ci->session->userdata('user_type');

        $show = true;

        $stat_listings = array(
            'index_method' => true,
        );

        if ($user_type == 'moderator') {
            $show = false;
            $ci->load->model('Moderators_model');
            $methods_listings = $ci->Moderators_model->get_module_methods('listings');
            if ((is_array($methods_listings) && !in_array('index', $methods_listings))) {
                $show = true;
            } else {
                $permission_data = $ci->session->userdata('permission_data');
                if ((isset($permission_data['listings']['index']) && $permission_data['listings']['index'] == 1) ||
                    (isset($permission_data['moderation']['index']) && $permission_data['moderation']['index'] == 1)) {
                    $show = true;
                    $stat_listings['index_method'] = (bool) $permission_data['listings']['index'];
                }
            }
        }

        if (!$show) {
            return '';
        }

        $ci->load->model('Listings_model');
        $stat_listings['all'] = $ci->Listings_model->get_listings_count();
        $stat_listings['active'] = $ci->Listings_model->get_listings_count(array('active' => 1));
        $stat_listings['inactive'] = $ci->Listings_model->get_listings_count(array('not_active' => 1));

        $is_moderation_installed = $ci->pg_module->is_module_active('moderation');
        if ($is_moderation_installed) {
            $ci->load->model('listings/models/Listings_moderation_model');
            $stat_listings['moderate'] = $ci->Listings_moderation_model->get_listings_count();
        }

        $ci->view->assign('stat_listings', $stat_listings);
        return $ci->view->fetch('helper_admin_home_block', 'admin', 'listings');
    }

}

if (!function_exists('listings_visitors_block')) {

    /**
     * Information about listings visitors
     *
     * @return string
     */
    function listings_visitors_block()
    {
        $ci = & get_instance();

        $user_id = intval($ci->session->userdata('user_id'));

        $ci->load->model('listings/models/Listings_visitor_model');

        $visitors_count_week = $ci->Listings_visitor_model->get_visitors_count(array('only_listings' => 1, 'user' => $user_id, 'week' => 1));
        $ci->view->assign('visitors_count_week', $visitors_count_week);

        $visitors_count_month = $ci->Listings_visitor_model->get_visitors_count(array('only_listings' => 1, 'user' => $user_id, 'month' => 1));
        $ci->view->assign('visitors_count_month', $visitors_count_month);

        return $ci->view->fetch('helper_listings_visitors', 'user', 'listings');
    }

}

if (!function_exists('listings_visits_block')) {

    /**
     * Information about listings visits
     *
     * @return string
     */
    function listings_visits_block()
    {
        $ci = & get_instance();

        $user_id = intval($ci->session->userdata('user_id'));

        $ci->load->model('listings/models/Listings_visitor_model');

        $visits_count_week = $ci->Listings_visitor_model->get_visitors_count(array('only_listings' => 1, 'visitor' => $user_id, 'week' => 1));
        $ci->view->assign('visits_count_week', $visits_count_week);

        $visits_count_month = $ci->Listings_visitor_model->get_visitors_count(array('only_listings' => 1, 'visitor' => $user_id, 'month' => 1));
        $ci->view->assign('visits_count_month', $visits_count_month);

        return $ci->view->fetch('helper_listings_visits', 'user', 'listings');
    }

}

if (!function_exists('listings_saved_block')) {

    /**
     * Number of saved listings
     *
     * @return string
     */
    function listings_saved_block()
    {
        $ci = & get_instance();

        $user_id = intval($ci->session->userdata('user_id'));

        $ci->load->model('listings/models/Saved_Listings_model');

        $saved_count = $ci->Saved_Listings_model->get_saved_count(array('user' => $user_id));
        $ci->view->assign('saved_count', $saved_count);

        return $ci->view->fetch('helper_saved_listings_count', 'user', 'listings');
    }

}

if (!function_exists('similar_listings_block')) {

    /**
     * Similar listings
     *
     * @param array $listing listing data
     * @return string
     */
    function similar_listings_block($listing)
    {
        $ci = & get_instance();
        $ci->load->model('Listings_model');

        $operation_type = $ci->Listings_model->getOperationTypeById($listing['id_type']);

        $similar_items = $ci->pg_module->get_module_config('listings', 'similar_items');
        $filters = array(
            'type' => $operation_type,
            'id_country' => $listing['id_country'],
            'id_region' => $listing['id_region'],
            'id_city' => $listing['id_city'],
            'id_category' => $listing['id_category'],
            'property_type' => $listing['property_type'],
            'exclude_id' => $listing['id'],
            'active' => 1,
        );
        $similar_listings = $ci->Listings_model->get_listings_list($filters, 1, $similar_items);
        $similar_total = count($similar_listings);
        if (!$similar_total)
            return '';

        $ci->view->assign('similar_listings', $similar_listings);
        $ci->view->assign('similar_visible', 2);
        $ci->view->assign('similar_total', $similar_total);
        $ci->view->assign('similar_rand', rand(10000, 90000));

        return $ci->view->fetch('helper_similar_listings', 'user', 'listings');
    }

}

if (!function_exists('listings_pagination_block')) {

    /**
     * Listing pagination block
     *
     * @param array $listing listing data
     * @return string
     */
    function listings_pagination_block($listing)
    {
        $ci = & get_instance();
        $ci->load->model('Listings_model');

        $prev_listing = $listing;
        $next_listing = $listing;
        $current_page = 1;
        $total_pages = 1;

        $current_settings = isset($_SESSION['listings_pagination']) ? $_SESSION['listings_pagination'] : array();
        if (!isset($current_settings['order'])) {
            $current_settings['order'] = 'modified';
        }
        if (!isset($current_settings['order_direction'])) {
            $current_settings['order_direction'] = 'DESC';
        }
        if (!isset($current_settings['page'])) {
            $current_settings['page'] = 1;
        }
        if (!isset($current_settings['data'])) {
            $current_settings['data'] = array();
        }
        $items_on_page = $ci->pg_module->get_module_config('start', 'index_items_per_page');
        $operation_type = $ci->Listings_model->getOperationTypeById($listing['id_type']);
        if ($operation_type == 'sale' && $listing['sold']) {
            $operation_type = 'sold';
        }
        if (get($current_settings['data']['type']) != $operation_type) {
            $current_settings['data'] = array('type' => $operation_type);
        }
        if ($current_settings['order_direction'] != 'ASC') {
            $current_settings['order_direction'] = 'DESC';
        }

        $params = $current_settings['data'];
        $params['active'] = 1;

        $listings_count = $ci->Listings_model->get_listings_count($params);
        if ($listings_count > 1) {
            $order_array = array();
            if ($current_settings['order'] == 'default') {
                if (!empty($current_settings['data']['id_city'])) {
                    $order_array['is_lift_up_city'] = 'DESC';
                }
                if (!empty($current_settings['data']['id_region'])) {
                    $order_array['is_lift_up_region'] = 'DESC';
                }
                if (!empty($current_settings['data']['id_country'])) {
                    $order_array['is_lift_up_country'] = 'DESC';
                }
                $order_array['is_lift_up'] = $current_settings['order_direction'];
                if (!empty($current_settings['data']['keyword'])) {
                    $boolean_mode = isset($current_settings['data']['keyword_mode']);
                    $keyword = trim($current_settings['data']['keyword']);
                    $ci->load->model('Field_editor_model');
                    $property_type_gid = $ci->Listings_model->get_field_editor_type(array('type' => $current_settings['data']['type'], 'id_category' => $current_settings['data']['id_category']));
                    $ci->Field_editor_model->initialize($property_type_gid);
                    $temp_criteria = $ci->Field_editor_model->return_fulltext_criteria($keyword, $boolean_mode);
                    $field_editor_table = constant('LISTINGS_' . strtoupper($property_type_gid) . '_TABLE');
                    $fe_table_name = substr($field_editor_table, strlen(DB_PREFIX));
                    if (!empty($temp_criteria[$fe_table_name]['field'])) {
                        $order_array[str_replace(array('search_field', ' AS score'), array($field_editor_table . '.search_field', ''), $temp_criteria[$fe_table_name]['field'])] = 'DESC';
                    }
                }
                $order_array['id'] = $current_settings['order_direction'];
            } elseif ($current_settings['order'] == 'price') {
                $order_array['price_sorting'] = $current_settings['order_direction'];
            } elseif ($current_settings['order'] == 'open') {
                $order_array['date_open'] = $current_settings['order_direction'];
            } elseif ($current_settings['order'] == 'modified') {
                $order_array['date_modified'] = $current_settings['order_direction'];
                $order_array['id'] = $current_settings['order_direction'];
            } else {
                $order_array[$current_settings['order']] = $current_settings['order_direction'];
            }
            if ($current_settings['order'] == 'price' || isset($current_settings['data']['price_range'])) {
                $order_array = array_merge(array('price_negotiated' => $current_settings['order_direction'] == 'ASC' ? 'DESC' : 'ASC'), $order_array);
            }

            $listings = $ci->Listings_model->get_listings_pagination($params, $current_settings['page'], $items_on_page, $order_array);
            $pages_count = ceil($listings_count / $items_on_page);
            $count = count($listings);
            foreach ($listings as $i => $data) {
                if ($data['id'] == $listing['id']) {
                    $page = $current_settings['page'];
                    $current_page = ($current_settings['page'] - 1) * $items_on_page + $i + ($current_settings['page'] == 1 ? 1 : -1);
                    $total_pages = $listings_count;

                    if ($i > 1) {
                        $prev_listing = $ci->Listings_model->format_listing($listings[$i - 1]);
                    } elseif ($current_settings['page'] > 1) {
                        $prev_listing = $ci->Listings_model->format_listing($listings[$i - 1]);
                        $page--;
                    } elseif ($i > 0) {
                        $prev_listing = $ci->Listings_model->format_listing($listings[$i - 1]);
                    }

                    if ($i < $count - ($pages_count > 1 ? 2 : 1)) {
                        $next_listing = $ci->Listings_model->format_listing($listings[$i + 1]);
                    } elseif ($current_settings['page'] < $pages_count) {
                        $next_listing = $ci->Listings_model->format_listing($listings[$i + 1]);
                        $page++;
                    } elseif ($current_settings['page'] > 1 && $current_settings['page'] == $pages_count && $i < $count - 1) {
                        $next_listing = $ci->Listings_model->format_listing($listings[$i + 1]);
                    }

                    if ($page != $current_settings['page']) {
                        $current_settings['page'] = $page;
                        $_SESSION['listings_pagination'] = $current_settings;
                    }
                    break;
                }
            }
        }

        $ci->view->assign('prev_listing', $prev_listing);
        $ci->view->assign('next_listing', $next_listing);
        $ci->view->assign('current_page', $current_page);
        $ci->view->assign('total_pages', $total_pages);

        return $ci->view->fetch('helper_listings_pagination', 'user', 'listings');
    }

}

if (!function_exists('listing_select')) {

    /**
     * Listings objects selector
     *
     * @param array $params block parameters
     * @return string
     */
    function listing_select($params = array())
    {

        $ci = & get_instance();
        $ci->load->model('Listings_model');

        $selected = isset($params['selected']) ? $params['selected'] : array();
        $max_select = isset($params['max']) ? $params['max'] : 0;
        $var_name = isset($params['var_name']) ? $params['var_name'] : 'id_listing';
        $template = isset($params['template']) ? $params['template'] : 'default';

        if (array_key_exists('selected', $params))
            unset($params['selected']);

        if ($max_select == 1 && !empty($selected) && !is_array($selected)) {
            $selected = array($selected);
        }

        if (!empty($selected)) {
            $filters['ids'] = $selected;
            if (isset($params['id_user']))
                $filters['id_user'] = $params['id_user'];
            if (isset($params['operation_type']))
                $filters['operation_type'] = $params['operation_type'];
            $data['selected'] = $ci->Listings_model->get_listings_list($filters, null, null, array());
            $data['selected_str'] = implode(',', $selected);
        }else {
            $data['selected_str'] = '';
        }

        if (isset($params['id_user'])) {
            $data['id_user'] = $params['id_user'];
        }
        if (isset($params['operation_type'])) {
            $data['operation_type'] = $params['operation_type'];
        }
        if (isset($params['callback'])) {
            $data['callback'] = $params['callback'];
            unset($params['callback']);
        }

        $data['max_select'] = $max_select;
        $data['var_name'] = $var_name;
        $data['template'] = $template;
        $data['params'] = $params;
        $data['rand'] = rand(100000, 999999);

        $ci->view->assign('select_data', $data);
        return $ci->view->fetch('helper_listing_select_' . $template, null, 'listings');
    }

}

if (!function_exists('save_listing_block')) {

    /**
     * Move listing to saved
     *
     * @param array $params block data
     * @return string
     */
    function save_listing_block($params)
    {
        $ci = &get_instance();
        if (!$ci->pg_module->is_module_active('users')) {
            return '';
        }
        if (!isset($params['listing'])) {
            return '';
        }

        if (isset($params['view_page'])) {
            $ci->view->assign('view_page', $params['view_page']);
        }

        static $is_login_enabled = null;
        if (is_null($is_login_enabled)) {
            $ci->load->helper('users');
            $is_login_enabled = is_login_enabled();
        }
        if (!$is_login_enabled) {
            return '';
        }

        if ($ci->session->userdata('auth_type') == 'user') {
            $user_id = $ci->session->userdata('user_id');
            if ($params['listing']['id_user'] == $user_id) {
                return '';
            }
        } else {
            $ci->view->assign('is_guest', 1);
        }
        if (!isset($params['listing']['is_saved'])) {
            $params['listing']['is_saved'] = false;
        }
        $ci->view->assign('listing', $params['listing']);
        $ci->view->assign('template', $params['template']);
        $ci->view->assign('template', $params['template']);

        if (isset($params['separator'])) {
            $separator = $params['separator'];
        } else {
            $separator = '';
        }

        $ci->view->assign('separator', $separator);
        $ci->view->assign('rand', rand(100000, 999999));

        return $ci->view->fetch('helper_save_block', 'user', 'listings');
    }

}

if (!function_exists('listings_slider_form')) {

    /**
     * Listings slider with search form
     *
     * @param array $type block type
     * @param integer $count listings count
     * @param string $view view mode
     * @return string
     */
    function listings_slider_form($type = 'all', $count = 8, $view = 'default')
    {
        $ci = & get_instance();
        $ci->load->model('Listings_model');

        if (func_num_args() == 1 && is_array($type)) {
            $params = $type;
            $type = isset($params['type']) ? $params['type'] : 'all';
            $count = isset($params['count']) ? $params['count'] : 8;
            $view = isset($params['view']) ? $params['view'] : 'default';
        }

        $count = intval($count);
        if (!$count)
            $count = 8;

        $filters = array('active' => 1, 'slider_image' => 1);
        $order_by = array();

        switch ($type) {
            case 'featured':
                $filters['featured'] = 1;
                $order_by = array('RAND()' => '');
                break;
            case 'latest_added':
                $order_by = array('date_created' => 'DESC');
                break;
            case 'sale':
                $filters['type'] = 'sale';
                $order_by = array('date_created' => 'DESC');
                break;
            case 'rent':
                $filters['type'] = 'rent';
                $order_by = array('date_created' => 'DESC');
                break;
        }

        $listings = $ci->Listings_model->get_listings_list($filters, 1, $count, $order_by);
        $ci->view->assign('listings', $listings);

        $slider_auto = $ci->pg_module->get_module_config('listings', 'slider_auto');
        $slider_rotation = $ci->pg_module->get_module_config('listings', 'slider_rotation');

        $page_data = array('count' => count($listings), 'slider_auto' => $slider_auto, 'rotation' => $slider_rotation * 1000);
        $ci->view->assign('slider_form_page_data', $page_data);

        $ci->view->assign('type', $type);
        $ci->view->assign('view', $view);

        $slider_form_settings = array('rand' => rand(11111, 99999));
        $ci->view->assign('slider_form_settings', $slider_form_settings);

        $ci->load->helper('start');
        $slider_search_form = main_search_form('sale', 'slider', true);
        $ci->view->assign('slider_search_form', $slider_search_form);

        $slider_form_settings = array('rand' => rand(11111, 99999));
        $ci->view->assign('slider_form_settings', $slider_form_settings);

        return $ci->view->fetch('helper_listings_block_form_slider', 'user', 'listings');
    }

}

if (!function_exists('listings_slider')) {

    /**
     * Listings slider without search form
     *
     * @param array $type block type
     * @param integer $count listings count
     * @param string $view view mode
     * @return string
     */
    function listings_slider($type = 'all', $count = 8, $view = 'default', $user_id = null)
    {
        $ci = & get_instance();
        $ci->load->model('Listings_model');

        if (func_num_args() == 1 && is_array($type)) {
            $params = $type;
            $type = isset($params['type']) ? $params['type'] : 'all';
            $count = isset($params['count']) ? $params['count'] : 8;
            $view = isset($params['view']) ? $params['view'] : 'default';
            $user_id = isset($params['user_id']) ? $params['user_id'] : null;
        }

        $count = intval($count);
        if (!$count)
            $count = 8;

        $filters = array('active' => 1, 'slider_image' => 1);
        $order_by = array();

        if ($user_id) {
            $filters['id_user'] = $user_id;
        }

        switch ($type) {
            case 'featured':
                $filters['featured'] = 1;
                $order_by = array('RAND()' => '');
                break;
            case 'latest_added':
                $order_by = array('date_created' => 'DESC');
                break;
            case 'sale':
                $filters['type'] = 'sale';
                $order_by = array('date_created' => 'DESC');
                break;
            case 'rent':
                $filters['type'] = 'rent';
                $order_by = array('date_created' => 'DESC');
                break;
        }

        $listings = $ci->Listings_model->get_listings_list($filters, 1, $count, $order_by);
        if (empty($listings))
            return;

        $ci->view->assign('listings', $listings);

        $ci->load->model('Properties_model');
        $property_types = $ci->Properties_model->get_categories('property_types', $ci->pg_language->current_lang_id);
        $ci->view->assign('property_types', $property_types);

        $slider_auto = $ci->pg_module->get_module_config('listings', 'slider_auto');
        $slider_rotation = $ci->pg_module->get_module_config('listings', 'slider_rotation');

        $page_data = array(
            'rand' => rand(100000, 999999),
            'count' => count($listings),
            'visible' => 5,
            'slider_auto' => $slider_auto,
            'rotation' => $slider_rotation * 1000,
        );
        $ci->view->assign('listings_page_data', $page_data);

        $ci->view->assign('type', $type);
        $ci->view->assign('view', $view);

        $ci->view->assign('slider_search_form', false);

        return $ci->view->fetch('helper_listings_block_slider', 'user', 'listings');
    }

}

if (!function_exists('listings_block')) {


    function listings_block($type, $count = 8, $size = 'middle', $view = 'gallery', $width = 100, $user_id = null)
    {
        $ci = &get_instance();
        $ci->load->model('Listings_model');
        if (!$count) {
            $count = 8;
        }
        $filters = array('active' => 1);
        $order_by = array();

        if ($user_id) {
            $filters['id_user'] = $user_id;
        }

        switch ($type) {
            case 'featured':
                $filters['featured'] = 1;
                $order_by = array('RAND()' => '');
                break;
            case 'latest_added':
                $order_by = array('date_created' => 'DESC');
                break;
            case 'sale':
                $filters['type'] = 'sale';
                $order_by = array('date_created' => 'DESC');
                break;
            case 'rent':
                $filters['type'] = 'rent';
                $order_by = array('date_created' => 'DESC');
                break;
        }

        $ci->Listings_model->set_format_settings(['get_saved_listings' => true, 'get_description' => true]);
        $listings = $ci->Listings_model->get_listings_list( $filters, 1, $count, $order_by );
        $ci->Listings_model->set_format_settings(['get_saved_listings' => false, 'get_description' => false,]);

        $listings_count = count($listings);
        if ($listings_count == 0) {
            return false;
        }

        $ci->view->assign('listings', $listings);
        $ci->view->assign('type', $type);
        $ci->view->assign('photo_size', $size);
        $ci->view->assign('block_width', $width);

        $page_data = array(
            'rand' => rand(100000, 999999),
            'count' => $listings_count,
            'visible' => round(($view == 'scroller' ? 2.5 : 4) * $width / 100),
        );
        $ci->view->assign('listings_page_data', $page_data);
        $ci->load->library('mobile_detect');
        if ($ci->mobile_detect->isMobile()) {
            $ci->view->assign('is_mobile', 1);
        }
        
        return $ci->view->fetch('helper_listings_block_' . $view, 'user', 'listings');
    }

}

if (!function_exists('listings_featured')) {

    /**
     * Featured listings block
     *
     * @param integer $count listings count
     * @param strign $size photo size
     * @param string $view view mode
     * @param integer $width block width
     * @return string
     */
    function listings_featured($count = 8, $size = 'middle', $view = 'gallery', $width = 100)
    {
        if (func_num_args() == 1 && is_array($count)) {
            $params = $count;
            $count = isset($params['count']) ? $params['count'] : 8;
            $size = isset($params['size']) ? $params['size'] : 'middle';
            $view = isset($params['view']) ? $params['view'] : 'gallery';
            $width = isset($params['width']) ? $params['width'] : 100;
        }
        return listings_block('featured', $count, $size, $view, $width);
    }

}

if (!function_exists('listings_last_added')) {

    /**
     * Last added listings block
     *
     * @param integer $count listings count
     * @param strign $size photo size
     * @param string $view view mode
     * @param integer $width block width
     * @return string
     */
    function listings_last_added($count = 8, $size = 'middle', $view = 'gallery', $width = 100, $user_id = null)
    {
        if (func_num_args() == 1 && is_array($count)) {
            $params = $count;
            $count = isset($params['count']) ? $params['count'] : 8;
            $size = isset($params['size']) ? $params['size'] : 'middle';
            $view = isset($params['view']) ? $params['view'] : 'gallery';
            $width = isset($params['width']) ? $params['width'] : 100;
            $user_id = isset($params['user_id']) ? $params['user_id'] : null;
        }
        return listings_block('last_added', $count, $size, $view, $width, $user_id);
    }

}

if (!function_exists('listings_sale')) {

    /**
     * Listings for sale block
     *
     * @param integer $count listings count
     * @param strign $size photo size
     * @param string $view view mode
     * @param integer $width block width
     * @return string
     */
    function listings_sale($count = 8, $size = 'middle', $view = 'gallery', $width = 100)
    {
        if (func_num_args() == 1 && is_array($count)) {
            $params = $count;
            $count = isset($params['count']) ? $params['count'] : 8;
            $size = isset($params['size']) ? $params['size'] : 'middle';
            $view = isset($params['view']) ? $params['view'] : 'gallery';
            $width = isset($params['width']) ? $params['width'] : 100;
        }
        return listings_block('sale', $count, $size, $view, $width);
    }

}

if (!function_exists('listings_buy')) {

    /**
     * Listings for buy block
     *
     * @param integer $count listings count
     * @param strign $size photo size
     * @param string $view view mode
     * @param integer $width block width
     * @return string
     */
    function listings_buy($count = 8, $size = 'middle', $view = 'gallery', $width = 100)
    {
        if (func_num_args() == 1 && is_array($count)) {
            $params = $count;
            $count = isset($params['count']) ? $params['count'] : 8;
            $size = isset($params['size']) ? $params['size'] : 'middle';
            $view = isset($params['view']) ? $params['view'] : 'gallery';
            $width = isset($params['width']) ? $params['width'] : 100;
        }
        return listings_block('buy', $count, $size, $view, $width);
    }

}

if (!function_exists('listings_rent')) {

    /**
     * Listings for rent block
     *
     * @param integer $count listings count
     * @param strign $size photo size
     * @param string $view view mode
     * @param integer $width block width
     * @return string
     */
    function listings_rent($count = 8, $size = 'middle', $view = 'gallery', $width = 100)
    {
        if (func_num_args() == 1 && is_array($count)) {
            $params = $count;
            $count = isset($params['count']) ? $params['count'] : 8;
            $size = isset($params['size']) ? $params['size'] : 'middle';
            $view = isset($params['view']) ? $params['view'] : 'gallery';
            $width = isset($params['width']) ? $params['width'] : 100;
        }
        return listings_block('rent', $count, $size, $view, $width);
    }

}

if (!function_exists('listings_lease')) {

    /**
     * Listings for lease block
     *
     * @param integer $count listings count
     * @param strign $size photo size
     * @param string $view view mode
     * @param integer $width block width
     * @return string
     */
    function listings_lease($count = 8, $size = 'middle', $view = 'middle', $width = 100)
    {
        if (func_num_args() == 1 && is_array($count)) {
            $params = $count;
            $count = isset($params['count']) ? $params['count'] : 8;
            $size = isset($params['size']) ? $params['size'] : 'middle';
            $view = isset($params['view']) ? $params['view'] : 'gallery';
            $width = isset($params['width']) ? $params['width'] : 100;
        }
        return listings_block('lease', $count, $size, $view, $width);
    }

}

if (!function_exists('listings_categories')) {

    /**
     * Listings categories block
     *
     * @param string $type block type
     * @param integer $columns columns count
     * @param string $view view mode
     * @param integer $width block width
     * @return string
     */
    function listings_categories($type = 'all', $columns = 3, $view = 'default', $width = 100)
    {
        $ci = & get_instance();

        if (func_num_args() == 1 && is_array($type)) {
            $params = $type;
            $type = isset($params['type']) ? $params['type'] : 'all';
            $columns = isset($params['columns']) ? $params['columns'] : 3;
            $view = isset($params['view']) ? $params['view'] : 'default';
            $width = isset($params['width']) ? $params['width'] : 100;
        }

        $ci->load->model('Properties_model');
        $ci->load->model('listings/models/Listings_stat_model');

        $property_types = $ci->Properties_model->get_categories('property_types', $ci->pg_language->current_lang_id);
        $property_items = $ci->Properties_model->get_all_properties('property_types', $ci->pg_language->current_lang_id);

        $property_items_count = count($property_items, COUNT_RECURSIVE);
        $ci->view->assign('property_items_count', $property_items_count);

        $max = 0;
        $items = array();
        $stat_gids = array();

        foreach ($property_types as $property_type_gid => $property_type_name) {
            if (!isset($property_items[$property_type_gid]))
                continue;
            $item = array(
                array(
                    'name' => $property_type_name,
                    'gid' => 'property_type_' . $property_type_gid,
                    'group' => true,
                    'empty' => false,
                    'data' => array(
                        'operation_type' => $type,
                        'category' => $property_type_name,
                    ),
                ),
            );
            foreach ($property_items[$property_type_gid] as $subtype_gid => $subtype_name) {
                $item[] = array(
                    'name' => $subtype_name,
                    'gid' => 'property_type_' . $property_type_gid . '_' . $subtype_gid,
                    'data' => array(
                        'operation_type' => $type,
                        'id_category' => $property_type_gid,
                        'category' => $property_type_name,
                        'property_type' => $subtype_gid,
                        'property' => $subtype_name . '-' . $property_type_gid,
                    ),
                    'group' => false,
                    'empty' => false
                );
                $stat_gids[] = 'property_type_' . $property_type_gid . '_' . $subtype_gid;
            }
            $items[] = $item;
            $max = max($max, count($item));
            $stat_gids[] = 'property_type_' . $property_type_gid;
        }

        $stats = $ci->Listings_stat_model->get_stat_by_gids($stat_gids);
        foreach ($items as $i => $item) {
            foreach ($item as $index => $value) {
                $item[$index]['statistics'] = array('listings_active' => isset($stats[$item[$index]['gid']][$type . '_cnt']) ? $stats[$item[$index]['gid']][$type . '_cnt'] : 0);
            }
            $items[$i] = array_pad($item, $max, array('name' => '&nbsp;', 'group' => false, 'empty' => true));
        }

        $block_view = 'property_types';

        $ci->view->assign('listings_categories', $items);

        $ci->view->assign('block_view', $block_view);

        $categories_count = count($items);
        $ci->view->assign('listings_categories_count', $categories_count);

        $columns = intval($columns);
        if (!$columns) {
            $columns = 3;
        }
        $ci->view->assign('columns', $columns);

        $listings_per_column = ceil($categories_count / $columns);
        $ci->view->assign('listings_per_column', $listings_per_column);

        $ci->view->assign('type', $type);

        $ci->view->assign('block_width', $width);

        $ci->load->library('mobile_detect');
        if ($ci->mobile_detect->isMobile()) {
            $ci->view->assign('is_mobile', 1);
        }

        return $ci->view->fetch('helper_categories_search', 'user', 'listings');
    }

}

if (!function_exists('listings_regions')) {

    /**
     * Listings regions block
     *
     * @param string $type block type
     * @param integer $columns columns count
     * @param string $view view mode
     * @param integer $width block width
     * @return string
     */
    function listings_regions($type = 'all', $columns = 3, $view = 'default', $width = 100)
    {
        $ci = & get_instance();

        if (func_num_args() == 1 && is_array($type)) {
            $params = $type;
            $type = isset($params['type']) ? $params['type'] : 'all';
            $columns = isset($params['columns']) ? $params['columns'] : 3;
            $view = isset($params['view']) ? $params['view'] : 'default';
            $width = isset($params['width']) ? $params['width'] : 100;
        }

        $listings_regions = array();

        $ci->load->helper('seo');

        $ci->load->model('Countries_model');
        $ci->load->model('listings/models/Listings_stat_model');

        $stat_gids = array();

        $locations = $ci->Countries_model->get_countries(null, array(), array(), $ci->pg_language->current_lang_id);
        if (count($locations) == 1) {
            $country = current($locations);
            $country_name = $country['name'];
            $locations = $ci->Countries_model->get_regions($country['code'], null, array(), array(), $ci->pg_language->current_lang_id);
            if (count($locations) == 1) {
                $region = current($locations);
                $region_name = $region['name'];
                $locations = $ci->Countries_model->get_cities(1, 30, array(), array(), array(), $ci->pg_language->current_lang_id);
                foreach ($locations as $location) {
                    $stat_gids[] = 'city_' . $location['id'];
                }
                $gid_prefix = 'city_';
                $field_name = 'id';
                $ci->view->assign('field_name', 'id_city');
            } else {
                foreach ($locations as $location) {
                    $stat_gids[] = 'region_' . $location['id'];
                }
                $gid_prefix = 'region_';
                $field_name = 'id';
                $ci->view->assign('field_name', 'id_region');
            }
        } else {
            foreach ($locations as $location) {
                $stat_gids[] = 'country_' . strtolower($location['code']);
            }
            $gid_prefix = 'country_';
            $field_name = 'code';
            $ci->view->assign('field_name', 'id_country');
        }

        if (empty($locations)) {
            return '';
        }

        $stats = $ci->Listings_stat_model->get_stat_by_gids($stat_gids);

        usort($locations, function($a, $b) {
            return strcmp($a['name'], $b['name']);
        });
        $max = ceil(count($locations) / 3) * 3;
        for ($i = 0; $i < $max; $i++) {
            if (isset($locations[$i])) {
                if (isset($locations[$i]['country_code'])) {
                    $country = $locations[$i]['country_code'];
                    if (isset($locations[$i]['id_region'])) {
                        $region = $locations[$i]['id_region'];
                        $city_name = $locations[$i]['name'];
                    } else {
                        $region = $locations[$i]['id'];
                        $region_name = $locations[$i]['name'];
                        $city_name = 'all';
                    }
                } else {
                    $country = $locations[$i]['code'];
                    $country_name = $locations[$i]['name'];
                    $region = 0;
                    $region_name = 'all';
                    $city = 0;
                    $city_name = 'all';
                }

                $listings_regions[] = array(
                    'id' => $locations[$i][$field_name],
                    'name' => $locations[$i]['name'],
                    'data' => array(
                        'operation_type' => $type,
                        'id_country' => $country,
                        'country' => $country_name,
                        'id_region' => $region,
                        'region' => $region_name,
                        'id_city' => $city,
                        'city' => $city_name,
                    ),
                    'statistics' => array(
                        'listing_active' => isset($stats[$gid_prefix . strtolower($locations[$i][$field_name])][$type . '_cnt']) ? $stats[$gid_prefix . strtolower($locations[$i][$field_name])][$type . '_cnt'] : 0,
                    ),
                    'empty' => false,
                    'group' => false,
                );
            } else {
                $listings_regions[] = array('name' => '&nbsp;', 'empty' => true, 'group' => false);
            }
        }

        $ci->view->assign('listings_regions', $listings_regions);

        $regions_count = count($listings_regions);
        $ci->view->assign('listings_regions_count', $regions_count);

        $locations_count = count($locations);
        $ci->view->assign('listings_locations_count', $locations_count);

        $columns = !empty($params['columns']) ? intval($params['columns']) : 3;
        $ci->view->assign('columns', $columns);

        $listings_per_column = ceil($regions_count / $columns);
        $ci->view->assign('listings_per_column', $listings_per_column);
        $ci->view->assign('type', $type);
        $ci->view->assign('block_width', $width);

        $ci->load->library('mobile_detect');
        if ($ci->mobile_detect->isMobile()) {
            $ci->view->assign('is_mobile', 1);
        }

        return $ci->view->fetch('helper_regions_search', 'user', 'listings');
    }

}

if (!function_exists('listings_wish_lists')) {

    /**
     * Wish lists block
     *
     * @param integer $count wish lists count
     * @param string $view view mode
     * @param integer $width block width
     * @return string
     */
    function listings_wish_lists($count = 8, $view = 'default', $width = 100)
    {
        $ci = & get_instance();
        $ci->load->model('listings/models/Wish_list_model');

        if (func_num_args() == 1 && is_array($count)) {
            $params = $count;
            $count = isset($params['count']) ? $params['count'] : 8;
            $view = isset($params['view']) ? $params['view'] : 'default';
            $width = isset($params['width']) ? $params['width'] : 100;
        }

        $count = intval($count);
        if (!$count) {
            $count = 4;
        }
        $wish_lists = $ci->Wish_list_model->get_wish_lists_list(array('active' => 1), 1, $count, array('RAND()' => ''));
        $ci->view->assign('wish_lists', $wish_lists);

        $ci->view->assign('block_width', $width);

        $ci->load->library('mobile_detect');
        if ($ci->mobile_detect->isMobile()) {
            $ci->view->assign('is_mobile', 1);
        }

        return $ci->view->fetch('helper_wish_lists', 'user', 'listings');
    }

}

if (!function_exists('service_status')) {
    function service_status($params)
    {
        $ci = & get_instance();
        if ($params['listing']) {
            $ci->view->assign('listing', $params['listing']);
        }
        if ($params['location_filter']) {
            $ci->view->assign('location_filter', $params['location_filter']);
        }
        return $ci->view->fetch('helper_listing_service_status', 'user', 'listings');
    }
}

if (!function_exists('show_user_info')) {
    function show_user_info($params)
    {
        $ci = & get_instance();
        
        if ($params['listing']) {
            $ci->view->assign('listing', $params['listing']);
        }
        
        if (!array_key_exists('show_logo', $params)) {
            $params['show_logo'] = true;
        }
        
        $ci->view->assign('show_logo', $params['show_logo']);
        
        return $ci->view->fetch('helper_listing_show_user_info', 'user', 'listings');
    }
}


if (!function_exists('listings_search_form')) {

    /**
     * Return form for quick search of listings (deprecated)
     *
     * @param array $params form parameters
     * @return string
     */
    function listings_search_form($params)
    {
        $ci = & get_instance();
        $ci->load->model('Listings_model');
        $ci->load->model('listings/models/Listings_module_model');
        $ci->load->model('listings/models/Listings_form_model');

        //check need methods
        $ci->uri->_fetch_uri_string();
        $uri = $ci->uri->ruri_string();
        $uri = trim(substr($uri, 1), '/');
        if (empty($uri) || count(explode('/', $uri)) < 2) {
            $class = $ci->router->fetch_clauser = $output_namess(true);
            $method = $ci->router->fetch_method();
            $uri = $class . '/' . $method;
        }

        if (!strstr($uri, 'listings/index') && !strstr($uri, 'listings/search') &&
            !strstr($uri, 'listings/open_house') && !strstr($uri, 'listings/privates') &&
            !strstr($uri, 'listings/agents') && !strstr($uri, 'listings/category') &&
            !strstr($uri, 'listings/location') && !strstr($uri, 'listings/discount'))
            return '';

        //get search criteria
        $filters = isset($_SESSION['listings_search']['data']) ? $_SESSION['listings_search']['data'] : array();
        if (!empty($filters['id_country'])) {
            $ci->load->helper('countries');

            $location = array();
            $location['country'] = $filters['id_country'];
            if (!empty($filters['id_region'])) {
                $location['region'] = $filters['id_region'];
                if (!empty($filters['id_city'])) {
                    $location['city'] = $filters['id_city'];
                    if (!empty($filters['id_district'])) {
                        $location['district'] = $filters['id_district'];
                        $location = districts_output_format(array($location));
                        if (!empty($location)) {
                            $filters['location'] = $location[0];
                        }
                    } else {
                        $location = cities_output_format(array($location));
                        if (!empty($location)) {
                            $filters['location'] = $location[0];
                        }
                    }
                } else {
                    $location = regions_output_format(array($location));
                    if (!empty($location)) {
                        $filters['location'] = $location[0];
                    }
                }
            } else {
                $location = countries_output_format(array($location));
                if (!empty($location)) {
                    $filters['location'] = $location[0];
                }
            }
        }

        $property_type_gid = $ci->Listings_model->get_field_editor_type($filters);

        //load properties
        $ci->load->model('Properties_model');
        $property_types = $ci->Properties_model->get_categories('property_types', $ci->pg_language->current_lang_id);
        $ci->view->assign('property_types', $property_types);
        $property_items = $ci->Properties_model->get_all_properties('property_types', $ci->pg_language->current_lang_id);
        $ci->view->assign('property_items', $property_items);

        //load radius data
        $radius_data = $ci->pg_language->ds->get_reference('listings', $ci->Listings_model->radius_data_gid, $ci->pg_language->current_lang_id);
        $ci->view->assign('radius_data', $radius_data);

        //load price range
        $price_range = $ci->Listings_module_model->get_price_range();
        $ci->view->assign('price_range', $price_range);

        //load square range
        $square_range = $ci->Listings_module_model->get_square_range();
        $ci->view->assign('square_range', $square_range);

        //load extends forms
        $quick_search_form_data = $ci->Listings_form_model->get_search_extend_form($property_type_gid, 'quick_search_form_gid', false);
        if (count($quick_search_form_data) > 0) {
            if (isset($filters['field_editor_data']['quick_search_form'])) {
                $ci->view->assign('data', $filters['field_editor_data']['quick_search_form']);
            }
            $ci->view->assign('form_data', $quick_search_form_data);
            $ci->view->assign('form_rand', rand(100000, 999999));
            if ($ci->pg_module->is_module_active("users")) {
                $quick_search_extend_form = $ci->view->fetch('helper_search_form_block', 'user', 'listings');
                $ci->view->assign('quick_search_extend_form', $quick_search_extend_form);
            }
        }

        $advanced_search_form_data = $ci->Listings_form_model->get_search_extend_form($property_type_gid, 'advanced_search_form_gid', false);
        if (count($advanced_search_form_data) > 0) {
            if (isset($filters['field_editor_data']['advanced_search_form'])) {
                $ci->view->assign('data', $filters['field_editor_data']['advanced_search_form']);
            }
            $ci->view->assign('form_data', $advanced_search_form_data);
            $ci->view->assign('form_rand', rand(100000, 999999));
            $advanced_search_extend_form = $ci->view->fetch('helper_search_form_block', 'user', 'listings');
            $ci->view->assign('advanced_search_extend_form', $advanced_search_extend_form);
        }

        if (!isset($filters['booking_date_start'])) {
            $filters['booking_date_start'] = date('Y-m-d');
        }
        if (!isset($filters['booking_date_end'])) {
            $filters['booking_date_end'] = date('Y-m-d', strtotime($filters['booking_date_start']) + 86400);
        }

        $ci->view->assign('data', $filters);

        //get current search
        $current_search_data = $ci->Listings_model->get_search_data();
        $ci->view->assign('current_search_data', $current_search_data);

        $search_filters_block = $ci->view->fetch('search_filters_block', 'user', 'listings');
        $ci->view->assign('search_filters_block', $search_filters_block);

        //get saved searches
        if ($ci->session->userdata('auth_type') == 'user') {
            $ci->load->model('listings/models/Listings_search_model');
            $user_id = $ci->session->userdata('user_id');
            $search_params = array('user' => $user_id);
            $save_search_items_on_page = $ci->pg_module->get_module_config('listings', 'save_search_items_per_page');
            $save_search_data = $ci->Listings_search_model->getSearchesList($search_params, 1, $save_search_items_on_page, array('id' => 'DESC'));
            $ci->view->assign('save_search_data', $save_search_data);
            $saved_searches_block = $ci->view->fetch('saved_searches_block', 'user', 'listings');
            $ci->view->assign('saved_searches_block', $saved_searches_block);
        }

        $operation_types = $ci->Listings_model->get_operation_types(true);

        $form_settings = array(
            'object' => $filters['type'],
            'operations' => $operation_types,
            'rand' => rand(100000, 999999),
        );

        $form_settings['date_format'] = $ci->pg_date->get_format('date_literal', 'st');
        $form_settings['datepicker_date_format'] = $ci->pg_date->get_format('date_literal', 'ui');
        $form_settings['datepicker_alt_format'] = $ci->pg_date->get_format('date_numeric', 'ui');

        if (!in_array($form_settings['object'], $operation_types)) {
            $form_settings['object'] = current($operation_types);
        }

        $ci->view->assign('search_form_settings', $form_settings);
        $use_save_search = $ci->session->userdata('auth_type') == 'user';
        $ci->view->assign('use_save_search', $use_save_search);

        return $ci->view->fetch('helper_search_form', 'user', 'listings');
    }

}
