<?php

namespace Pg\Modules\Listings\Models;

use Pg\Modules\Services\Models\Services_users_model;

/**
 * Listings module
 *
 * @package     PG_RealEstate
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Listings Service Model
 *
 * @package     PG_RealEstate
 * @subpackage  Listings
 * @category    models
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Listings_service_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $CI;

    /**
     * Constructor
     *
     * return Listings_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->CI = & get_instance();
    }

    public function getFreeServices($user_id, $gid)
    {
        $params = array(
            'where' => array(
                'id_user' => $user_id,
                'service_gid' => $gid,
                'status' => '1',
            ),
        );

        return $this->get_services_list(null, $params);
    }

    /**
     * Return service data by identifier
     *
     * @param integer $user_service_id service identifier
     * @return array
     */
    public function get_service($user_service_id)
    {
        $this->CI->load->model("services/models/Services_users_model");
        $user_service = $this->CI->Services_users_model->get_service_by_id($user_service_id);
        return $user_service;
    }

    /**
     * Return services objects as array
     *
     * @param array $order_by sorting data
     * @param array $params filters data
     * @param array $filter_object_ids filters identifiers
     * @return array
     */
    public function get_services_list($order_by = null, $params = array(), $filter_object_ids = null)
    {
        $this->CI->load->model("services/models/Services_users_model");
        $user_services = $this->CI->Services_users_model->get_services_list($params, $order_by, $filter_object_ids);
        return $user_services;
    }

    /**
     * Save service object to data source
     *
     * @param integer $service_id service identifier
     * @param array $attrs service data
     * @return integer
     */
    public function save_service($service_id = null, $attrs = array())
    {
        $this->CI->load->model("services/models/Services_users_model");
        $user_service_id = $this->CI->Services_users_model->save_service($service_id, $attrs);
        return $user_service_id;
    }

    /**
     * Clean up post listing service by cron
     *
     * @return void
     */
    public function service_post_cron()
    {
        $this->CI->load->model('Listings_model');

        $is_users_installed = $this->CI->pg_module->is_module_active("users");
        if ($is_users_installed) {
            $this->CI->load->model('Users_model');
            $this->CI->Users_model->set_format_settings('get_safe', false);
        }

        $filters = array('status' => 1, 'cron_date_expire' => 1);
        //$this->CI->Listings_model->set_format_settings('use_format', false);
        $listings = $this->CI->Listings_model->get_listings_list($filters);
        //$this->CI->Listings_model->set_format_settings('use_format', true);

        if ($is_users_installed) {
            $this->CI->Users_model->set_format_settings('get_safe', true);
        }

        $clean = count($listings);
        if ($clean > 0) {
            $this->CI->Listings_model->update_service_date_end('date_expire', 'status');

            $this->CI->load->model('Notifications_model');
            $this->CI->load->model('listings/Listings_stat_model');
            foreach ($listings as $listing_data) {
                // send notifications
                $email = $listing_data['user']['email'];
                $lang_id = $listing_data['user']['lang_id'];
                $listing_data['listing'] = l('field_id', 'listings', $listing_data['user']['lang_id']) . ' ' . $listing_data['id'] . ', ' . $listing_data['location'];
                $listing_data['status'] = l('text_date_expire', 'listings', $listing_data['user']['lang_id']);
                $listing_data['user'] = $listing_data['user']['fname'];
                $this->CI->Notifications_model->send_notification($email, 'listing_status_updated', $listing_data, '', $lang_id);

                $listing_data['status'] = 1;

                $listing_new = $listing_data;
                $listing_new['status'] = 0;
                $this->CI->Listings_stat_model->update_module_statistics($listing_data, $listing_new);
            }
        }
        echo 'Make clean(Listings active status): ' . $clean . ' Listings was remove active status';
    }

    /**
     * Buy featured listing service
     *
     * @param array $user_service user service data
     * @param array $user_data data filled by user
     * @param float $price payment price
     * @return boolean
     */
    public function service_featured_buy($user_service, $user_data, $price = '')
    {
        $period = $user_service["service_data_array"]["data_admin_array"]["period"];

        if (!empty($user_service["service_data_array"]["data_admin_array"]["count"])) {
            $count = $user_service["service_data_array"]["data_admin_array"]["count"];
        } else {
            $count = 1;
        }

        $service_data["available_data"] = serialize(array("period" => $period, "count" => $count));
        return $this->save_service($user_service["id"], $service_data);
    }

    private function findArrayListingsIds($data)
    {
        // TODO: Refactor this whole module to use polymorphism.
        $existent_keys = array_intersect(
            array('listing_id', 'id_listing', 'listing'),
            array_keys($data)
        );
        foreach($existent_keys as $key) {
            if(is_array($data[$key]) && !empty($data[$key])) {
                return $data[$key];
            } elseif($data[$key] > 0) {
                return array($data[$key]);
            }
        }
        log_message('error', '(listings_service) Empty listing id');
        return 0;
    }

    private function helpDoraFindListingId($data)
    {
        // TODO: Refactor this whole module to use polymorphism.
        $existent_keys = array_intersect(
            array('listing_id', 'id_listing', 'listing'),
            array_keys($data)
        );
        foreach($existent_keys as $key) {
            if($data[$key] > 0) {
                return (int) $data[$key];
            }
        }
        log_message('error', '(listings_service) Empty listing id');
        return 0;
    }

    /**
     * Activate featured listing service
     *
     * @param array $user_service user service data
     * @param array $user_data data filled by user
     * @param float $price payment price
     * @return void
     */
    public function service_featured_activate($user_service, $user_data, $price = '')
    {
        $this->CI->load->model('Listings_model');

        $is_users_installed = $this->CI->pg_module->is_module_active("users");
        if ($is_users_installed) {
            $this->CI->load->model('Users_model');
            $this->CI->Users_model->set_format_settings('get_safe', false);
        }
        $this->CI->Listings_model->set_format_settings(array('get_location' => false));

        $listing_id = $this->findArrayListingsIds($user_data);
        if(!is_array($listing_id)) {
            return false;
        }

        foreach ($listing_id as $id) {
            $listing = $this->CI->Listings_model->get_listing_by_id($id);
            $this->CI->Listings_model->set_format_settings(array('get_location' => true));

            if ($is_users_installed) {
                $this->CI->Users_model->set_format_settings('get_safe', true);
            }
            if (strtotime($listing['featured_date_end']) > 86400) {
                $uts = strtotime($listing['featured_date_end']);
            } else {
                $uts = time();
            }

            $period = $uts + $user_service["available_data_array"]["period"] * 24 * 60 * 60;

            $update_data = array('is_featured' => 1, 'featured_date_end' => date('Y-m-d H:i:s', $period));
            $this->CI->Listings_model->save_listing($listing['id'], $update_data);
        }

        $user_service["available_data_array"]["count"] --;

        $save_service = array("available_data" => serialize($user_service["available_data_array"]));

        if ($user_service["available_data_array"]["count"] <= 0) {
            $save_service["status"] = 0;
        }

        $this->save_service($user_service['id'], $save_service);

        // send notification
        $email = $listing['user']['email'];
        $lang_id = $listing['user']['lang_id'];

        $this->CI->config->load('date_formats', TRUE);
        $date_format = $this->CI->pg_date->get_format('date_time_literal', 'st');

        $this->CI->load->helper('date_format');
        $date_expired = tpl_date_format($period, $date_format);

        $mail_data = array(
            'user' => $listing['user']['output_name'],
            'name' => l('service_string_name_featured', 'listings', $lang_id),
            'date' => $date_expired,
        );

        $this->CI->load->model('Notifications_model');
        $this->CI->Notifications_model->send_notification($email, 'listing_service_enabled', $mail_data, '', $lang_id);
    }

    /**
     * Validate featured listing service
     *
     * @param integer $user_id user identifier
     * @param array $data user data
     * @param array $service_data service data
     * @param float $price service price
     * @return array
     */
    public function service_featured_validate($user_id, $data, $service_data = array(), $price = '')
    {
        $return = array('errors' => array(), 'data' => $data);

        return $return;
    }

    /**
     * Validate featured listing service
     *
     * @param array $user_service user service data
     * @param array $user_data data filled by user
     * @return array
     */
    public function validate_featured_activate($user_service, $user_data)
    {
        $return = array('errors' => array(), 'data' => array());

        if (!empty($user_service)) {
            $id_user = $this->CI->session->userdata('user_id');

            if ($user_service['id_user'] != $id_user ||
                $user_service["available_data_array"]["count"] == 0 ||
                $user_service["status"] == 0) {
                $return['errors'][] = l('error_service_invalid', 'listings');
            }
        } else {
            $return['errors'][] = l('error_service_invalid', 'listings');
        }

        if (isset($user_data['listing'])) {
            $return["data"]["listing"] = intval($user_data['listing']);
            if (empty($return["data"]["listing"])) {
                $return['errors'][] = l('error_empty_listing', 'listings');
            }
        } else {
            $return['errors'][] = l('error_empty_listing', 'listings');
        }

        return $return;
    }

    /**
     * Clean up featured listing service by cron
     *
     * @return void
     */
    public function service_featured_cron()
    {
        $this->CI->load->model('Listings_model');

        $is_users_installed = $this->CI->pg_module->is_module_active("users");
        if ($is_users_installed) {
            $this->CI->load->model('Users_model');
            $this->CI->Users_model->set_format_settings('get_safe', false);
        }

        $filters = array('status' => 1, 'cron_featured_date_end' => 1);
        $listings = $this->CI->Listings_model->get_listings_list($filters);
        
        if ($is_users_installed) {
            $this->CI->Users_model->set_format_settings('get_safe', true);
        }

        $clean = count($listings);
        if ($clean == 0) {
            echo 'Make clean(Listings featured): No listings was removed from featured';
            return;
        }

        $this->CI->Listings_model->update_service_date_end('featured_date_end', 'is_featured');

        // send notification
        $this->CI->load->model('Notifications_model');
        foreach ($listings as $listing_data) {
            $email = $listing_data['user']['email'];
            $lang_id = $listing_data['user']['lang_id'];

            $mail_data = array(
                'user' => $listing_data['user']['output_name'],
                'name' => l('service_string_name_featured', 'listings', $lang_id),
            );
            $this->CI->Notifications_model->send_notification($email, 'listing_service_expired', $mail_data, '', $lang_id);
        }

        echo 'Make clean(Listings featured): ' . $clean . ' listings was removed from featured';
    }

    /**
     * Buy lift up listing service
     *
     * @param array $user_service user service data
     * @param array $user_data data filled by user
     * @param float $price payment price
     * @return boolean
     */
    public function service_lift_up_buy($user_service, $user_data, $price = '')
    {
        $period = $user_service["service_data_array"]["data_admin_array"]["period"];

        if (!empty($user_service["service_data_array"]["data_admin_array"]["count"])) {
            $count = $user_service["service_data_array"]["data_admin_array"]["count"];
        } else {
            $count = 1;
        }

        $service_data["available_data"] = serialize(array("period" => $period, "count" => $count));
        return $this->save_service($user_service["id"], $service_data);
    }

    /**
     * Activate lift up listing service
     *
     * @param array $user_service user service data
     * @param array $user_data data filled by user
     * @param float $price payment price
     * @return void
     */
    public function service_lift_up_activate($user_service, $user_data, $price = '')
    {
        $this->CI->load->model('Listings_model');

        $is_users_installed = $this->CI->pg_module->is_module_active("users");
        if ($is_users_installed) {
            $this->CI->load->model('Users_model');
            $this->CI->Users_model->set_format_settings('get_safe', false);
        }

        $listing_id = $this->findArrayListingsIds($user_data);
        if(!is_array($listing_id)) {
            return false;
        }

        foreach ($listing_id as $id) {
            $this->CI->Listings_model->set_format_settings('get_location', false);
            $listing = $this->CI->Listings_model->get_listing_by_id($id);
            $this->CI->Listings_model->set_format_settings('get_location', true);
            if ($is_users_installed) {
                $this->CI->Users_model->set_format_settings('get_safe', true);
            }
            if (strtotime($listing['lift_up_date_end']) > 86400) {
                $uts = strtotime($listing['lift_up_date_end']);
            } else {
                $uts = time();
            }

            $period = $uts + $user_service["available_data_array"]["period"] * 24 * 60 * 60;

            $update_data = array('is_lift_up' => 1, 'lift_up_date_end' => date('Y-m-d H:i:s', $period));
            $this->CI->Listings_model->save_listing($listing['id'], $update_data);
        }

        $user_service["available_data_array"]["count"] --;

        $save_service = array("available_data" => serialize($user_service["available_data_array"]));

        if ($user_service["available_data_array"]["count"] <= 0) {
            $save_service["status"] = 0;
        }

        $this->save_service($user_service['id'], $save_service);

        // send notification
        $email = $listing['user']['email'];
        $lang_id = $listing['user']['lang_id'];

        $date_format = $this->CI->pg_date->get_format('date_time_literal', 'st');

        $this->CI->load->helper('date_format');
        $date_expired = tpl_date_format($period, $date_format);

        $mail_data = array(
            'user' => $listing['user']['output_name'],
            'name' => l('service_string_name_lift_up', 'listings', $lang_id),
            'date' => $date_expired,
        );

        $this->CI->load->model('Notifications_model');
        $this->CI->Notifications_model->send_notification($email, 'listing_service_enabled', $mail_data, '', $lang_id);
    }

    /**
     * Validate lift up listing service
     *
     * @param integer $user_id user identifier
     * @param array $data user data
     * @param array $service_data service data
     * @param float $price service price
     * @return void
     */
    public function service_lift_up_validate($user_id, $data, $service_data = array(), $price = '')
    {
        $return = array('errors' => array(), 'data' => $data);
        return $return;
    }

    /**
     * Validate lift up listing service
     *
     * @param array $user_service user service data
     * @param array $user_data data filled by user
     * @return array
     */
    public function validate_lift_up_activate($user_service, $user_data)
    {
        $return = array('errors' => array(), 'data' => array());

        if (!empty($user_service)) {
            $id_user = $this->CI->session->userdata('user_id');

            if ($user_service['id_user'] != $id_user ||
                $user_service["available_data_array"]["count"] == 0 ||
                $user_service["status"] == 0) {
                $return['errors'][] = l('error_service_invalid', 'listings');
            }
        } else {
            $return['errors'][] = l('error_service_invalid', 'listings');
        }

        if (isset($user_data['listing'])) {
            $return["data"]["listing"] = intval($user_data['listing']);
            if (empty($return["data"]["listing"])) {
                $return['errors'][] = l('error_empty_listing', 'listings');
            }
        } else {
            $return['errors'][] = l('error_empty_listing', 'listings');
        }

        return $return;
    }

    /**
     * Clean up lift up listing service by cron
     *
     * @return void
     */
    public function service_lift_up_cron()
    {
        $this->CI->load->model('Listings_model');

        $is_users_installed = $this->CI->pg_module->is_module_active("users");
        if ($is_users_installed) {
            $this->CI->load->model('Users_model');
            $this->CI->Users_model->set_format_settings('get_safe', false);
        }

        $filters = array('status' => 1, 'cron_lift_up_date_end' => 1);
        $listings = $this->CI->Listings_model->get_listings_list($filters);
        
        if ($is_users_installed) {
            $this->CI->Users_model->set_format_settings('get_safe', true);
        }
        $clean = 0;
        if (!empty($listings)) {
            $clean = count($listings);
            if ($clean > 0) {
                $this->CI->Listings_model->update_service_date_end('lift_up_date_end', 'is_lift_up');

                // send notification
                $this->CI->load->model('Notifications_model');
                foreach ($listings as $listing_data) {
                    $mail_data = array(
                        'user' => $listing_data['user']['output_name'],
                        'name' => l('service_string_name_lift_up', 'listings', $listing_data['user']['lang_id']),
                    );
                    $this->CI->Notifications_model->send_notification($listing_data['user']['email'], 'listing_service_expired', $mail_data, '', $listing_data['user']['lang_id']);
                }
            }
        }
        echo 'Make clean(Listings lift up): ' . $clean . ' Listings was removed from lift up';
    }

    /**
     * Buy lift up listing in country service
     *
     * @param array $user_service user service data
     * @param array $user_data data filled by user
     * @param float $price payment price
     * @return boolean
     */
    public function service_lift_up_country_buy($user_service, $user_data, $price = '')
    {
        $period = $user_service["service_data_array"]["data_admin_array"]["period"];

        if (!empty($user_service["service_data_array"]["data_admin_array"]["count"])) {
            $count = $user_service["service_data_array"]["data_admin_array"]["count"];
        } else {
            $count = 1;
        }

        $service_data["available_data"] = serialize(array("period" => $period, "count" => $count));
        return $this->save_service($user_service["id"], $service_data);
    }

    /**
     * Activate lift up listing in country service
     *
     * @param array $user_service user service data
     * @param array $user_data user data
     * @param float $price payment price
     * @return void
     */
    public function service_lift_up_country_activate($user_service, $user_data, $price = '')
    {
        $this->CI->load->model('Listings_model');

        $is_users_installed = $this->CI->pg_module->is_module_active("users");
        if ($is_users_installed) {
            $this->CI->load->model('Users_model');
            $this->CI->Users_model->set_format_settings('get_safe', false);
        }

        $listing_id = $this->findArrayListingsIds($user_data);
        if(!is_array($listing_id)) {
            return false;
        }

        foreach ($listing_id as $id) {
            $this->CI->Listings_model->set_format_settings('get_location', false);
            $listing = $this->CI->Listings_model->get_listing_by_id($id);
            $this->CI->Listings_model->set_format_settings('get_location', true);
            if ($is_users_installed) {
                $this->CI->Users_model->set_format_settings('get_safe', true);
            }
            if (strtotime($listing['lift_up_country_date_end']) > 86400) {
                $uts = strtotime($listing['lift_up_country_date_end']);
            } else {
                $uts = time();
            }

            $period = $uts + $user_service["available_data_array"]["period"] * 24 * 60 * 60;

            $update_data = array('is_lift_up_country' => 1, 'lift_up_country_date_end' => date('Y-m-d H:i:s', $period));
            $this->CI->Listings_model->save_listing($listing['id'], $update_data);
        }

        $save_service["status"] = 0;
        $this->save_service($user_service['id'], $save_service);

        // send notification
        $email = $listing['user']['email'];
        $lang_id = $listing['user']['lang_id'];

        $date_format = $this->CI->pg_date->get_format('date_time_literal', 'st');

        $this->CI->load->helper('date_format');
        $date_expired = tpl_date_format($period, $date_format);

        $mail_data = array(
            'user' => $listing['user']['output_name'],
            'name' => l('service_string_name_lift_up', 'listings', $lang_id),
            'date' => $date_expired,
        );

        $this->CI->load->model('Notifications_model');
        $this->CI->Notifications_model->send_notification($email, 'listing_service_enabled', $mail_data, '', $lang_id);
    }

    /**
     * Validate lift up listing in country service
     *
     * @param integer $user_id user identifier
     * @param array $data user data
     * @param array $service_data service data
     * @param float $price service price
     * @return array
     */
    public function service_lift_up_country_validate($user_id, $data, $service_data = array(), $price = '')
    {
        $return = array('errors' => array(), 'data' => $data);
        return $return;
    }

    /**
     * Validate lift up listing in country service
     *
     * @param array $user_service user service data
     * @param array $user_data data filled by user
     * @return array
     */
    public function validate_lift_up_country_activate($user_service, $user_data)
    {
        $return = array('errors' => array(), 'data' => array());

        if (!empty($user_service)) {
            $id_user = $this->CI->session->userdata('user_id');

            if ($user_service['id_user'] != $id_user ||
                $user_service["available_data_array"]["count"] == 0 ||
                $user_service["status"] == 0) {
                $return['errors'][] = l('error_service_invalid', 'listings');
            }
        } else {
            $return['errors'][] = l('error_service_invalid', 'listings');
        }

        if (isset($user_data['listing'])) {
            $return["data"]["listing"] = intval($user_data['listing']);
            if (empty($return["data"]["listing"])) {
                $return['errors'][] = l('error_empty_listing', 'listings');
            }
        } else {
            $return['errors'][] = l('error_empty_listing', 'listings');
        }

        return $return;
    }

    /**
     * Clean up lift up listing service by cron
     *
     * @return void
     */
    public function service_lift_up_country_cron()
    {
        $this->CI->load->model('Listings_model');

        $is_users_installed = $this->CI->pg_module->is_module_active("users");
        if ($is_users_installed) {
            $this->CI->load->model('Users_model');
            $this->CI->Users_model->set_format_settings('get_safe', false);
        }

        $filters = array('status' => 1, 'cron_lift_up_country_date_end' => 1);
        $listings = $this->CI->Listings_model->get_listings_list($filters);
        
        if ($is_users_installed) {
            $this->CI->Users_model->set_format_settings('get_safe', true);
        }

        $clean = count($listings);
        if ($clean == 0) {
            echo 'Make clean(Listings lift up in country): 0 listings was removed from lift up in country';
        }

        $this->CI->Listings_model->update_service_date_end('lift_up_country_date_end', 'is_lift_up_country');

        // send notification
        $this->CI->load->model('Notifications_model');
        foreach ($listings as $listing_data) {
            $email = $listing_data['user']['email'];
            $lang_id = $listing_data['user']['lang_id'];

            $mail_data = array(
                'user' => $listing_data['user']['output_name'],
                'name' => l('service_string_name_lift_up', 'listings', $lang_id),
            );
            $this->CI->Notifications_model->send_notification($email, 'listing_service_expired', $mail_data, '', $lang_id);
        }

        echo 'Make clean(Listings lift up in country): ' . $clean . ' listings was removed from lift up in country';
    }

    /**
     * Buy lift up listing in region service
     *
     * @param array $user_service user service data
     * @param array $user_data user data
     * @param float $price payment price
     * @return boolean
     */
    public function service_lift_up_region_buy($user_service, $user_data, $price)
    {
        $period = $user_service["service_data_array"]["data_admin_array"]["period"];

        if (!empty($user_service["service_data_array"]["data_admin_array"]["count"])) {
            $count = $user_service["service_data_array"]["data_admin_array"]["count"];
        } else {
            $count = 1;
        }

        $service_data["available_data"] = serialize(array("period" => $period, "count" => $count));
        return $this->save_service($user_service["id"], $service_data);
    }

    /**
     * Activate lift up listing in region service
     *
     * @param array $user_service user service data
     * @param array $user_data user data
     * @param float $price payment price
     * @return void
     */
    public function service_lift_up_region_activate($user_service, $user_data, $price = '')
    {
        $this->CI->load->model('Listings_model');
        $is_users_installed = $this->CI->pg_module->is_module_active("users");
        if ($is_users_installed) {
            $this->CI->load->model('Users_model');
            $this->CI->Users_model->set_format_settings('get_safe', false);
        }
        $this->CI->Listings_model->set_format_settings('get_location', false);

        $listing_id = $this->findArrayListingsIds($user_data);
        if(!is_array($listing_id)) {
            return false;
        }

        foreach ($listing_id as $id) {
            $listing = $this->CI->Listings_model->get_listing_by_id($id);
            $this->CI->Listings_model->set_format_settings('get_location', true);
            if ($is_users_installed) {
                $this->CI->Users_model->set_format_settings('get_safe', true);
            }
            if (strtotime($listing['lift_up_region_date_end']) > 86400) {
                $uts = strtotime($listing['lift_up_region_date_end']);
            } else {
                $uts = time();
            }

            $period = $uts + $user_service["available_data_array"]["period"] * 24 * 60 * 60;

            $update_data = array('is_lift_up_region' => 1, 'lift_up_region_date_end' => date('Y-m-d H:i:s', $period));
            $this->CI->Listings_model->save_listing($listing['id'], $update_data);
        }

        $user_service["available_data_array"]["count"] --;

        $save_service = array("available_data" => serialize($user_service["available_data_array"]));

        if ($user_service["available_data_array"]["count"] <= 0) {
            $save_service["status"] = 0;
        }

        $this->save_service($user_service['id'], $save_service);

        // send notification
        $email = $listing['user']['email'];
        $lang_id = $listing['user']['lang_id'];
        $date_format = $this->CI->pg_date->get_format('date_time_literal', 'st');

        $this->CI->load->helper('date_format');
        $date_expired = tpl_date_format($period, $date_format);

        $mail_data = array(
            'user' => $listing['user']['output_name'],
            'name' => l('service_string_name_lift_up', 'listings', $lang_id),
            'date' => $date_expired,
        );

        $this->CI->load->model('Notifications_model');
        $this->CI->Notifications_model->send_notification($email, 'listing_service_enabled', $mail_data, '', $lang_id);
    }

    /**
     * Validate lift up listing in region service
     *
     * @param integer $user_id user identifier
     * @param array $data user data
     * @param array $service_data service data
     * @param float $price service price
     * @return array
     */
    public function service_lift_up_region_validate($user_id, $data, $service_data = array(), $price = '')
    {
        $return = array('errors' => array(), 'data' => $data);
        return $return;
    }

    /**
     * Validate featured listing service
     *
     * @param array $user_service user service data
     * @param array $user_data data filled by user
     * @return array
     */
    public function validate_lift_up_region_activate($user_service, $user_data)
    {
        $return = array('errors' => array(), 'data' => array());

        if (!empty($user_service)) {
            $id_user = $this->CI->session->userdata('user_id');

            if ($user_service['id_user'] != $id_user ||
                $user_service["available_data_array"]["count"] == 0 ||
                $user_service["status"] == 0) {
                $return['errors'][] = l('error_service_invalid', 'listings');
            }
        } else {
            $return['errors'][] = l('error_service_invalid', 'listings');
        }

        if (isset($user_data['listing'])) {
            $return["data"]["listing"] = intval($user_data['listing']);
            if (empty($return["data"]["listing"])) {
                $return['errors'][] = l('error_empty_listing', 'listings');
            }
        } else {
            $return['errors'][] = l('error_empty_listing', 'listings');
        }

        return $return;
    }

    /**
     * Clean up lift up listing in region service by cron
     *
     * @return void
     */
    public function service_lift_up_region_cron()
    {
        $this->CI->load->model('Listings_model');

        $is_users_installed = $this->CI->pg_module->is_module_active("users");
        if ($is_users_installed) {
            $this->CI->load->model('Users_model');
            $this->CI->Users_model->set_format_settings('get_safe', false);
        }

        $filters = array('status' => 1, 'cron_lift_up_region_date_end' => 1);
        $listings = $this->CI->Listings_model->get_listings_list($filters);
        
        if ($is_users_installed) {
            $this->CI->Users_model->set_format_settings('get_safe', true);
        }

        $clean = count($listings);
        if ($clean > 0) {
            $this->CI->Listings_model->update_service_date_end('lift_up_region_date_end', 'is_lift_up_region');

            // send notification
            $this->CI->load->model('Notifications_model');
            foreach ($listings as $listing_data) {
                $mail_data = array(
                    'user' => $listing_data['user']['output_name'],
                    'name' => l('service_string_name_lift_up', 'listings', $listing_data['user']['lang_id']),
                );
                $this->CI->Notifications_model->send_notification($listing_data['user']['email'], 'listing_service_expired', $mail_data, '', $listing_data['user']['lang_id']);
            }
        }
        echo 'Make clean(Listings lift up in region): ' . $clean . ' Listings was removed from lift up in region';
    }

    /**
     * Buy lift up listing in city service
     *
     * @param array $user_service user service data
     * @param array $user_data user data
     * @param float $price payment price
     * @return boolean
     */
    public function service_lift_up_city_buy($user_service, $user_data, $price = '')
    {
        $period = $user_service["service_data_array"]["data_admin_array"]["period"];

        if (!empty($user_service["service_data_array"]["data_admin_array"]["count"])) {
            $count = $user_service["service_data_array"]["data_admin_array"]["count"];
        } else {
            $count = 1;
        }

        $service_data["available_data"] = serialize(array("period" => $period, "count" => $count));
        return $this->save_service($user_service["id"], $service_data);
    }

    /**
     * Activate lift up listing in city service
     *
     * @param array $user_service user service
     * @param array $user_data user data
     * @param float $price payment price
     * @return void
     */
    public function service_lift_up_city_activate($user_service, $user_data, $price = '')
    {
        $this->CI->load->model('Listings_model');
        $is_users_installed = $this->CI->pg_module->is_module_active("users");
        if ($is_users_installed) {
            $this->CI->load->model('Users_model');
            $this->CI->Users_model->set_format_settings('get_safe', false);
        }

        $listing_id = $this->findArrayListingsIds($user_data);
        if(!is_array($listing_id)) {
            return false;
        }

        foreach ($listing_id as $id) {
            $this->CI->Listings_model->set_format_settings('get_location', false);
            $listing = $this->CI->Listings_model->get_listing_by_id($id);
            $this->CI->Listings_model->set_format_settings('get_location', true);
            if ($is_users_installed) {
                $this->CI->Users_model->set_format_settings('get_safe', true);
            }
            if (strtotime($listing['lift_up_city_date_end']) > 86400) {
                $uts = strtotime($listing['lift_up_city_date_end']);
            } else {
                $uts = time();
            }

            $period = $uts + $user_service['available_data_array']['period'] * 24 * 60 * 60;

            $update_data = array('is_lift_up_city' => 1, 'lift_up_city_date_end' => date('Y-m-d H:i:s', $period));
            $this->CI->Listings_model->save_listing($listing['id'], $update_data);
        }

        $user_service["available_data_array"]["count"] --;

        $save_service = array("available_data" => serialize($user_service["available_data_array"]));

        if ($user_service["available_data_array"]["count"] <= 0) {
            $save_service["status"] = 0;
        }

        $this->save_service($user_service['id'], $save_service);

        // send notification
        $email = $listing['user']['email'];
        $lang_id = $listing['user']['lang_id'];
        $date_format = $this->CI->pg_date->get_format('date_time_literal', 'st');

        $this->CI->load->helper('date_format');
        $date_expired = tpl_date_format($period, $date_format);

        $mail_data = array(
            'user' => $listing['user']['output_name'],
            'name' => l('service_string_name_lift_up', 'listings', $lang_id),
            'date' => $date_expired,
        );

        $this->CI->load->model('Notifications_model');
        $this->CI->Notifications_model->send_notification($email, 'listing_service_enabled', $mail_data, '', $lang_id);
    }

    /**
     * Validate lift up listing in city service
     *
     * @param integer $user_id user identifier
     * @param array $data user data
     * @param array $service_data service data
     * @param float $price service price
     * @return array
     */
    public function service_lift_up_city_validate($user_id, $data, $service_data = array(), $price = '')
    {
        $return = array('errors' => array(), 'data' => $data);
        return $return;
    }

    /**
     * Validate lift up listing in city service
     *
     * @param array $user_service user service data
     * @param array $user_data data filled by user
     * @return array
     */
    public function validate_lift_up_city_activate($user_service, $user_data)
    {
        $return = array('errors' => array(), 'data' => array());

        if (!empty($user_service)) {
            $id_user = $this->CI->session->userdata('user_id');

            if ($user_service['id_user'] != $id_user ||
                $user_service["available_data_array"]["count"] == 0 ||
                $user_service["status"] == 0) {
                $return['errors'][] = l('error_service_invalid', 'listings');
            }
        } else {
            $return['errors'][] = l('error_service_invalid', 'listings');
        }

        if (isset($user_data['listing'])) {
            $return["data"]["listing"] = intval($user_data['listing']);
            if (empty($return["data"]["listing"])) {
                $return['errors'][] = l('error_empty_listing', 'listings');
            }
        } else {
            $return['errors'][] = l('error_empty_listing', 'listings');
        }

        return $return;
    }

    /**
     * Clean up lift up listing in city service cron
     *
     * @return void
     */
    public function service_lift_up_city_cron()
    {
        $this->CI->load->model('Listings_model');

        $is_users_installed = $this->CI->pg_module->is_module_active("users");
        if ($is_users_installed) {
            $this->CI->load->model('Users_model');
            $this->CI->Users_model->set_format_settings('get_safe', false);
        }

        $filters = array('status' => 1, 'cron_lift_up_city_date_end' => 1);
        $listings = $this->CI->Listings_model->get_listings_list($filters);

        if ($is_users_installed) {
            $this->CI->Users_model->set_format_settings('get_safe', true);
        }

        $clean = count($listings);
        if ($clean > 0) {
            $this->CI->Listings_model->update_service_date_end('lift_up_city_date_end', 'is_lift_up_city');

            // send notification
            $this->CI->load->model('Notifications_model');
            foreach ($listings as $listing_data) {
                $mail_data = array(
                    'user' => $listing_data['user']['output_name'],
                    'name' => l('service_string_name_lift_up', 'listings', $listing_data['user']['lang_id']),
                );
                $this->CI->Notifications_model->send_notification($listing_data['user']['email'], 'listing_service_expired', $mail_data, '', $listing_data['user']['lang_id']);
            }
        }
        echo 'Make clean(Listings lift upin city): ' . $clean . ' Listings was removed from lift up in city';
    }

    /**
     * Buy highlight listing service
     *
     * @param array $user_service user service data
     * @param array $user_data user data
     * @param float $price payment price
     * @return boolean
     */
    public function service_highlight_buy($user_service, $user_data, $price = '')
    {
        $period = $user_service["service_data_array"]["data_admin_array"]["period"];

        if (!empty($user_service["service_data_array"]["data_admin_array"]["count"])) {
            $count = $user_service["service_data_array"]["data_admin_array"]["count"];
        } else {
            $count = 1;
        }

        $service_data["available_data"] = serialize(array("period" => $period, "count" => $count));
        return $this->save_service($user_service["id"], $service_data);
    }

    /**
     * Activate highlight listing service
     *
     * @param array $user_service user service data
     * @param array $user_data user data
     * @param float $price payment price
     * @return void
     */
    public function service_highlight_activate($user_service, $user_data, $price = '')
    {
        $this->CI->load->model('Listings_model');

        $is_users_installed = $this->CI->pg_module->is_module_active("users");
        if ($is_users_installed) {
            $this->CI->load->model('Users_model');
            $this->CI->Users_model->set_format_settings('get_safe', false);
        }

        $listing_id = $this->findArrayListingsIds($user_data);
        if(!is_array($listing_id)) {
            return false;
        }

        foreach ($listing_id as $id) {
            $this->CI->Listings_model->set_format_settings('get_location', false);
            $listing = $this->CI->Listings_model->get_listing_by_id($id);
            $this->CI->Listings_model->set_format_settings('get_location', true);
            if ($is_users_installed) {
                $this->CI->Users_model->set_format_settings('get_safe', true);
            }
            if (strtotime($listing['highlight_date_end']) > 86400) {
                $uts = strtotime($listing['highlight_date_end']);
            } else {
                $uts = time();
            }

            $period = $uts + $user_service['available_data_array']['period'] * 24 * 60 * 60;

            $update_data = array('is_highlight' => 1, 'highlight_date_end' => date('Y-m-d H:i:s', $period));
            $this->CI->Listings_model->save_listing($listing['id'], $update_data);
        }

        $user_service["available_data_array"]["count"] --;

        $save_service = array("available_data" => serialize($user_service["available_data_array"]));

        if ($user_service["available_data_array"]["count"] <= 0) {
            $save_service["status"] = 0;
        }

        $this->save_service($user_service['id'], $save_service);

        // send notification
        $email = $listing['user']['email'];
        $lang_id = $listing['user']['lang_id'];
        $date_format = $this->CI->pg_date->get_format('date_time_literal', 'st');

        $this->CI->load->helper('date_format');
        $date_expired = tpl_date_format($period, $date_format);

        $mail_data = array(
            'user' => $listing['user']['output_name'],
            'name' => l('service_string_name_highlight', 'listings', $lang_id),
            'date' => $date_expired,
        );

        $this->CI->load->model('Notifications_model');
        $this->CI->Notifications_model->send_notification($email, 'listing_service_enabled', $mail_data, '', $lang_id);
    }

    /**
     * Validate highlight listing service
     *
     * @param integer $user_id user identifier
     * @param array $data user data
     * @param array $service_data service data
     * @param float $price service price
     * @return array
     */
    public function service_highlight_validate($user_id, $data, $service_data = array(), $price = '')
    {
        $return = array('errors' => array(), 'data' => $data);
        return $return;
    }

    /**
     * Validate highlight listing service
     *
     * @param array $user_service user service data
     * @param array $user_data data filled by user
     * @return array
     */
    public function validate_highlight_activate($user_service, $user_data)
    {
        $return = array('errors' => array(), 'data' => array());

        if (!empty($user_service)) {
            $id_user = $this->CI->session->userdata('user_id');

            if ($user_service['id_user'] != $id_user ||
                $user_service["available_data_array"]["count"] == 0 ||
                $user_service["status"] == 0) {
                $return['errors'][] = l('error_service_invalid', 'listings');
            }
        } else {
            $return['errors'][] = l('error_service_invalid', 'listings');
        }

        if (isset($user_data['listing'])) {
            $return["data"]["listing"] = intval($user_data['listing']);
            if (empty($return["data"]["listing"])) {
                $return['errors'][] = l('error_empty_listing', 'listings');
            }
        } else {
            $return['errors'][] = l('error_empty_listing', 'listings');
        }

        return $return;
    }

    /**
     * Clean up highlight listing service by cron
     *
     * @return void
     */
    public function service_highlight_cron()
    {
        $this->CI->load->model('Listings_model');

        $is_users_installed = $this->CI->pg_module->is_module_active("users");
        if ($is_users_installed) {
            $this->CI->load->model('Users_model');
            $this->CI->Users_model->set_format_settings('get_safe', false);
        }

        $filters = array('status' => 1, 'cron_highlight_date_end' => 1);
        $listings = $this->CI->Listings_model->get_listings_list($filters);
        
        if ($is_users_installed) {
            $this->CI->Users_model->set_format_settings('get_safe', true);
        }

        $clean = count($listings);
        if ($clean > 0) {
            $this->CI->Listings_model->update_service_date_end('highlight_date_end', 'is_highlight');

            // send notification
            $this->CI->load->model('Notifications_model');
            foreach ($listings as $listing_data) {
                $mail_data = array(
                    'user' => $listing_data['user']['output_name'],
                    'name' => l('service_string_name_highlight', 'listings', $listing_data['user']['lang_id']),
                );
                $this->CI->Notifications_model->send_notification($listing_data['user']['email'], 'listing_service_expired', $mail_data, '', $listing_data['user']['lang_id']);
            }
        }
        echo 'Make clean(Listings highlight): ' . $clean . ' Listings was removed from highlight';
    }

    /**
     * Buy slide show listing service
     *
     * @param array $user_service user service data
     * @param array $user_data user data
     * @param float $price payment price
     * @return boolean
     */
    public function service_slide_show_buy($user_service, $user_data, $price = '')
    {
        $period = $user_service["service_data_array"]["data_admin_array"]["period"];

        if (!empty($user_service["service_data_array"]["data_admin_array"]["count"])) {
            $count = $user_service["service_data_array"]["data_admin_array"]["count"];
        } else {
            $count = 1;
        }

        $service_data["available_data"] = serialize(array("period" => $period, "count" => $count));
        return $this->save_service($user_service["id"], $service_data);
    }

    /**
     * Activate slide show listing service
     *
     * @param array $user_service user service data
     * @param array $user_data user data
     * @param float $price payment price
     * @return void
     */
    public function service_slide_show_activate($user_service, $user_data, $price = '')
    {
        $this->CI->load->model('Listings_model');
        $this->CI->load->model('listings/models/Listings_uploads_model');

        $is_users_installed = $this->CI->pg_module->is_module_active("users");
        if ($is_users_installed) {
            $this->CI->load->model('Users_model');
            $this->CI->Users_model->set_format_settings('get_safe', false);
        }

        $listing_id = $this->findArrayListingsIds($user_data);
        if(!is_array($listing_id)) {
            return false;
        }

        foreach ($listing_id as $id) {
            $this->CI->Listings_model->set_format_settings('get_location', false);
            $listing = $this->CI->Listings_model->get_listing_by_id($id);
            $this->CI->Listings_model->set_format_settings('get_location', true);
            if ($is_users_installed) {
                $this->CI->Users_model->set_format_settings('get_safe', true);
            }
            if (strtotime($listing['slide_show_date_end']) > 86400) {
                $uts = strtotime($listing['slide_show_date_end']);
            } else {
                $uts = time();
            }

            $period = $uts + $user_service['available_data_array']['period'] * 24 * 60 * 60;

            $update_data = array('is_slide_show' => 1, 'slide_show_date_end' => date('Y-m-d H:i:s', $period));
            $this->CI->Listings_model->save_listing($listing['id'], $update_data);

            $this->CI->Listings_uploads_model->update_logo_image($listing['id']);
        }

        $user_service["available_data_array"]["count"] --;

        $save_service = array("available_data" => serialize($user_service["available_data_array"]));

        if ($user_service["available_data_array"]["count"] <= 0) {
            $save_service["status"] = 0;
        }

        $this->save_service($user_service['id'], $save_service);

        // send notification
        $email = $listing['user']['email'];
        $lang_id = $listing['user']['lang_id'];
        $date_format = $this->CI->pg_date->get_format('date_time_literal', 'st');

        $this->CI->load->helper('date_format');
        $date_expired = tpl_date_format($period, $date_format);

        $mail_data = array(
            'user' => $listing['user']['output_name'],
            'name' => l('service_string_name_slide_show', 'listings', $lang_id),
            'date' => $date_expired,
        );

        $this->CI->load->model('Notifications_model');
        $this->CI->Notifications_model->send_notification($email, 'listing_service_enabled', $mail_data, '', $lang_id);
    }

    /**
     * Validate slide show listing service
     *
     * @param integer $user_id user identifier
     * @param array $data user data
     * @param array $service_data service data
     * @param float $price service price
     * @return void
     */
    public function service_slide_show_validate($user_id, $data, $service_data = array(), $price = '')
    {
        $return = array('errors' => array(), 'data' => $data);
        return $return;
    }

    /**
     * Validate slide show listing service
     *
     * @param array $user_service user service data
     * @param array $user_data data filled by user
     * @return array
     */
    public function validate_slide_show_activate($user_service, $user_data)
    {
        $return = array('errors' => array(), 'data' => array());

        if (!empty($user_service)) {
            $id_user = $this->CI->session->userdata('user_id');

            if ($user_service['id_user'] != $id_user ||
                $user_service["available_data_array"]["count"] == 0 ||
                $user_service["status"] == 0) {
                $return['errors'][] = l('error_service_invalid', 'listings');
            }
        } else {
            $return['errors'][] = l('error_service_invalid', 'listings');
        }

        if (isset($user_data['listing'])) {
            $return["data"]["listing"] = intval($user_data['listing']);
            if (empty($return["data"]["listing"])) {
                $return['errors'][] = l('error_empty_listing', 'listings');
            }
        } else {
            $return['errors'][] = l('error_empty_listing', 'listings');
        }

        return $return;
    }

    /**
     * Clean up slide show listing service by cron
     *
     * @return void
     */
    public function service_slide_show_cron()
    {
        $this->CI->load->model('Listings_model');

        $is_users_installed = $this->CI->pg_module->is_module_active("users");
        if ($is_users_installed) {
            $this->CI->load->model('Users_model');
            $this->CI->Users_model->set_format_settings('get_safe', false);
        }

        $filters = array('status' => 1, 'cron_slide_show_date_end' => 1);
        $listings = $this->CI->Listings_model->get_listings_list($filters);

        if ($is_users_installed) {
            $this->CI->Users_model->set_format_settings('get_safe', true);
        }

       $clean = count($results);
        if ($clean > 0) {
            $this->CI->Listings_model->update_service_date_end('slide_show_date_end', 'is_slide_show');

            // send notification
            $this->CI->load->model('Notifications_model');
            foreach ($listings as $listing_data) {
                $mail_data = array(
                    'user' => $listing_data['user']['output_name'],
                    'name' => l('service_string_name_slide_show', 'listings', $listing_data['user']['lang_id']),
                );
                $this->CI->Notifications_model->send_notification($listing_data['user']['email'], 'listing_service_expired', $mail_data, '', $listing_data['user']['lang_id']);
            }
        }
        echo 'Make clean(Listings slide_show): ' . $clean . ' Listings was removed from slide_show';
    }

}
