<?php

/**
 * Mailbox module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

/**
 * Mailbox management
 *
 * @package 	PG_RealEstate
 * @subpackage 	Mailbox
 *
 * @category	helpers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!function_exists('new_messages')) {

    /**
     * Get new massages
     *
     * @param array $attrs block parameters
     *
     * @return string
     */
    function new_messages($attrs)
    {
        $ci = &get_instance();
        if ('user' != $ci->session->userdata("auth_type")) {
            return false;
        }
        $user_id = $ci->session->userdata("user_id");
        if (!$user_id) {
            log_message('Empty user id');
            return false;
        }
        if (empty($attrs['template'])) {
            $attrs['template'] = 'header';
        }
        $ci->load->model('Mailbox_model');
        $count = $ci->Mailbox_model->get_new_messages_count($user_id);

        $ci->view->assign('messages_count', $count);
        return $ci->view->fetch('helper_new_messages_' . $attrs['template'], 'user', 'mailbox');
    }

}

if (!function_exists('button_contact')) {

    /**
     * Show contact button
     *
     * @param array $params block parameters
     *
     * @return string
     */
    function button_contact($params)
    {
        $ci = &get_instance();

        if (!isset($params['user_id'])) {
            return '';
        }

        if ($ci->session->userdata('auth_type') === 'user') {
            $user_id = $ci->session->userdata('user_id');
            if ($user_id == $params['user_id']) {
                return '';
            }
        } else {
            $ci->view->assign('is_guest', 1);
        }

        if (!isset($params['user_type'])) {
            $ci->load->model('Users_model');
            $ci->Users_model->set_format_settings('use_format', false);
            $user = $ci->Users_model->get_user_type_by_id($params['user_id']);
            $ci->Users_model->set_format_settings('use_format', true);
            if (!$user) {
                return '';
            }
            $params['user_type'] = $user['user_type'];
        }

        if (!isset($params['template'])) {
            $params['template'] = 'icon';
        }
        $ci->view->assign('template', $params['template']);
        $ci->view->assign('user_type', $params['user_type']);
        $ci->view->assign('user_id', $params['user_id']);
        return $ci->view->fetch('helper_contact', 'user', 'mailbox');
    }

}

if (!function_exists('contacts_count')) {

    /**
     * Show number of contacts
     *
     * @param array $attrs block parameters
     *
     * @return string
     */
    function contacts_count($attrs = array())
    {
        $ci = &get_instance();
        if (empty($attrs['user_id'])) {
            $attrs['user_id'] = $ci->session->userdata("user_id");
        }
        if (!$attrs['user_id']) {
            return false;
        }
        $ci->load->model('Mailbox_model');
        $where = array();
        $where['where']['id_com_user'] = $attrs['user_id'];
        $count = $ci->Mailbox_model->get_chats_count($where);
        $ci->view->assign('contacts_count', $count);
        return $ci->view->fetch('helper_contacts_count', 'user', 'mailbox');
    }

}
