<?php

/**
 * Menu module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
/**
 * Menu management
 *
 * @package 	PG_RealEstate
 * @subpackage 	Menu
 *
 * @category	helpers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!function_exists('get_admin_main_menu')) {

    /**
     * Output main menu in admin mode
     *
     * @return void
     */
    function get_admin_main_menu()
    {
        $ci = &get_instance();
        $ci->load->model('Menu_model');

        // add link to menu
        $menu_data = $ci->Menu_model->get_menu_by_gid('admin_menu');

        $user_type = $ci->session->userdata("user_type");
        if ($user_type == "admin") {
            $menu_data["check_permissions"] = false;
            $permissions = array();
        } else {
            $menu_data["check_permissions"] = true;
            $permissions = $ci->session->userdata("permission_data");
        }

        $menu_items = $ci->Menu_model->get_menu_active_items_list($menu_data["id"], $menu_data["check_permissions"], array(), 0, $permissions);

        $ci->view->assign("menu", $menu_items);
        return $ci->view->fetch("main_menu", 'admin', 'menu');
    }

}

if (!function_exists('get_admin_level1_menu')) {

    /**
     * Output menu items of first level in admin mode
     *
     * @param string $gid menu guid
     *
     * @return void
     */
    function get_admin_level1_menu($gid)
    {
        $ci = &get_instance();
        $ci->load->model('Menu_model');

        // add link to menu
        $menu_data = $ci->Menu_model->get_menu_by_gid($gid);

        $user_type = $ci->session->userdata("user_type");
        if ($user_type == "admin") {
            $menu_data["check_permissions"] = false;
            $permissions = array();
        } else {
            $menu_data["check_permissions"] = true;
            $permissions = $ci->session->userdata("permission_data");
        }

        $menu_items = $ci->Menu_model->get_menu_active_items_list(
            $menu_data["id"], $menu_data["check_permissions"], array(), 0, $permissions);

        if (empty($menu_items)) {
            return '';
        }

        $ci->view->assign("menu", $menu_items);
        return $ci->view->fetch("level1_menu", 'admin', 'menu');
    }

}

if (!function_exists('get_admin_level2_menu')) {

    /**
     * Output menu items of second level in admin mode
     *
     * @param string $gid menu guid
     *
     * @return void
     */
    function get_admin_level2_menu($gid)
    {
        $ci = &get_instance();
        $ci->load->model('Menu_model');

        // add link to menu
        $menu_data = $ci->Menu_model->get_menu_by_gid($gid);

        $user_type = $ci->session->userdata("user_type");
        if ($user_type == "admin") {
            $menu_data["check_permissions"] = false;
            $permissions = array();
        } else {
            $menu_data["check_permissions"] = true;
            $permissions = $ci->session->userdata("permission_data");
        }

        $menu_items = $ci->Menu_model->get_menu_active_items_list($menu_data["id"], $menu_data["check_permissions"], array(), 0, $permissions);

        $ci->view->assign("menu", $menu_items);
        return $ci->view->fetch("level2_menu", 'admin', 'menu');
    }

}

if (!function_exists('get_menu')) {

    /**
     * Get menu content
     *
     * @param string $gid      menu guid
     * @param string $template menu template
     *
     * @return void
     */
    function get_menu($gid, $template = '')
    {
        $ci = &get_instance();
        $ci->load->model('Menu_model');

        if (!$template) {
            $template = $gid;
        }

        // add link to menu
        $menu_data = $ci->Menu_model->get_menu_by_gid($gid);
        if(empty($menu_data)) {
            throw new \BadMethodCallException("Menu \"$gid\" does not exist");
        }
        $menu_items = $ci->Menu_model->get_menu_active_items_list($menu_data["id"], $menu_data["check_permissions"]);

        $ci->view->assign("menu", $menu_items);
        return $ci->view->fetch($template, 'user', 'menu');
    }

}

if (!function_exists('get_breadcrumbs')) {

    /**
     * Get breadcrumbs menu
     *
     * @param string $template menu template
     *
     * @return string
     */
    function get_breadcrumbs($template = '')
    {
        $ci = &get_instance();
        $ci->load->model('Menu_model');

        if (!$template) {
            $template = 'helper_breadcrumbs';
        }

        // add link to menu
        $breadcrumbs = $ci->Menu_model->get_breadcrumbs();

        $is_guest = $ci->session->userdata("auth_type") != "user";
        $ci->view->assign("is_guest", $is_guest);

        $ci->view->assign("breadcrumbs", $breadcrumbs);
        return $ci->view->fetch($template, 'user', 'menu');
    }

}

if (!function_exists("linked_install_menu")) {

    /**
     * Install menu data
     *
     * @param string $gid  menu gid
     * @param string $name menu name
     *
     * @return integer
     */
    function linked_install_menu($gid, $name = "")
    {
        $ci = &get_instance();
        $ci->load->model("Menu_model");

        $menu = $ci->Menu_model->get_menu_by_gid($gid);
        if ($menu) {
            return $menu["id"];
        }

        $menu_data = array("gid" => $gid, "name" => $name);

        return $ci->Menu_model->save_menu(null, $menu_data);
    }

}

if (!function_exists("linked_deinstall_menu")) {

    /**
     * Uninstall menu data
     *
     * @param string $gid menu gid
     *
     * @return void
     */
    function linked_deinstall_menu($gid)
    {
        $ci = &get_instance();
        $ci->load->model("Menu_model");

        $menu = $ci->Menu_model->get_menu_by_gid($gid);
        if (empty($menu)) {
            return;
        }

        $ci->Menu_model->delete_menu($menu["id"]);
    }

}

if (!function_exists("linked_install_menu_item")) {

    /**
     * Install menu item
     *
     * @param string  $gid       menu item gid
     * @param integer $menu_id   menu identifier
     * @param integer $parent_id parent item identifier
     * @param string  $link      menu item link
     * @param string  $icon      menu item icon
     * @param integer $status    activation status
     * @param integer $sorter    menu item priority
     *
     * @return integer
     */
    function linked_install_menu_item($gid, $menu_id, $parent_id = 0, $link = "/", $icon = '', $status = 1, $sorter = 1)
    {
        $ci = &get_instance();
        $ci->load->model("Menu_model");

        $item = $ci->Menu_model->get_menu_item_by_gid($gid, $menu_id);
        if ($item) {
            return $item["id"];
        }

        $item_data = array(
            "gid" => $gid,
            "menu_id" => $menu_id,
            "parent_id" => (int) $parent_id,
            "link" => $link,
            "icon" => $icon,
            "status" => $status,
            "sorter" => $sorter,
        );

        return $ci->Menu_model->save_menu_item(null, $item_data);
    }

}

if (!function_exists("linked_deinstall_menu_item")) {

    /**
     * Uninstall menu item data
     *
     * @param string $gid menu item gid
     *
     * @return void
     */
    function linked_deinstall_menu_item($gid)
    {
        $ci = &get_instance();
        $ci->load->model("Menu_model");

        $item = $ci->Menu_model->get_menu_item_by_gid($gid, $menu_id);
        if (empty($item)) {
            return;
        }

        $ci->Menu_model->delete_menu_item($item["id"]);
    }

}

if (!function_exists("linked_import_menu")) {

    /**
     * Import menu languages
     *
     * @param integer $menu_id           menu identifier
     * @param array   $lang_data         data by languages
     * @param array   $lang_tooltip_data tooltip by languages
     *
     * @return array
     */
    function linked_import_menu($menu_id, $lang_data = array(), $lang_tooltip_data = array())
    {
        $ci = &get_instance();
        $ci->load->model("Menu_model");
        return $ci->Menu_model->save_menu_item_lang(0, $menu_id, $lang_data, $lang_tooltip_data);
    }

}

if (!function_exists("linked_import_menu_items")) {

    /**
     * Import menu items languages
     *
     * @param integer $menu_id           menu identifier
     * @param integer $item_id           menu item identifier
     * @param array   $lang_data         data by languages
     * @param array   $lang_tooltip_data tooltip by languages
     *
     * @return array
     */
    function linked_import_menu_items($menu_id, $item_id, $lang_data = array(), $lang_tooltip_data = array())
    {
        $ci = &get_instance();
        $ci->load->model("Menu_model");
        return $ci->Menu_model->save_menu_item_lang($item_id, $menu_id, $lang_data, $lang_tooltip_data);
    }

}

if (!function_exists("linked_export_menu_items")) {

    /**
     * Export menu items languages
     *
     * @param integer $menu_id menu identifier
     *
     * @return array
     */
    function linked_export_menu_items($menu_id)
    {
        $ci = &get_instance();
        $ci->load->model("Menu_model");
        return $ci->Menu_model->_get_item_string_data($menu_id, $item_id, $lang_data, $lang_type);
    }

}

if (!function_exists('linked_install_set_menu')) {

    /**
     * Install menu actions
     *
     * Available type values: create, none, delete
     *
     * @param string $gid  menu guid
     * @param string $type type action
     * @param string $name menu name
     *
     * @return integer
     */
    function linked_install_set_menu($gid, $type = "create", $name = '')
    {
        $ci = &get_instance();
        $ci->load->model('Menu_model');
        if ($type == 'create') {
            $menu_data = array('gid' => $gid, 'name' => $name);
            return $ci->Menu_model->save_menu(null, $menu_data);
        } elseif ($type == 'none') {
            $menu = $ci->Menu_model->get_menu_by_gid($gid);
            return get($menu['id']);
        } elseif ($type == 'delete') {
            $menu = $ci->Menu_model->get_menu_by_gid($gid);
            if (!empty($menu)) {
                $ci->Menu_model->delete_menu($menu["id"]);
            }
            return '';
        } else {
            return 0;
        }
    }

}

if (!function_exists('linked_install_set_menu_item')) {

    /**
     * Install menu item actions
     *
     * Available type values: create, none, delete
     *
     * @param string  $gid           menu item guid
     * @param integer $menu_id       menu identifier
     * @param string  $type          type action
     * @param integer $parent_id     parent identifier
     * @param string  $link          menu item link
     * @param string  $icon          menu item icon
     * @param integer $status        activation status
     * @param integer $sorter        menu item priority
     * @param string  $indicator_gid indicator guid
     * @param integer $is_external   extermal link
     *
     * @return integer
     */
    function linked_install_set_menu_item($gid, $menu_id, $type = "create", $parent_id = 0, $link = '/', $icon = '', $status = 1, $sorter = 1, $indicator_gid = '', $is_external = 0)
    {
        $ci = &get_instance();
        $ci->load->model('Menu_model');
        if ($type == 'create') {
            $item_data = array(
                'gid' => $gid,
                'menu_id' => $menu_id,
                'parent_id' => (int) $parent_id,
                'link' => $link,
                'icon' => $icon,
                'status' => $status,
                'sorter' => $sorter,
                'indicator_gid' => (string) $indicator_gid,
                'is_external' => $is_external,
            );

            return $ci->Menu_model->save_menu_item(null, $item_data);
        } elseif ($type == 'none') {
            $item = $ci->Menu_model->get_menu_item_by_gid($gid, $menu_id);

            return $item['id'];
        } elseif ($type == 'delete') {
            $item = $ci->Menu_model->get_menu_item_by_gid($gid, $menu_id);
            if (!empty($item)) {
                $ci->Menu_model->delete_menu_item($item['id']);
            }

            return '';
        } else {
            return 0;
        }
    }

}

if (!function_exists('linked_install_set_menu_item_lang')) {

    /**
     * Install menu item languages actions
     *
     * $lang_data is lang_id => value for update AND lang_id array for export
     *
     * Available type values: update, export
     *
     * @param integer $item_id           menu item identifier
     * @param integer $menu_id           menu identifier
     * @param string  $type              type action
     * @param array   $lang_data         languages data
     * @param array   $lang_tooltip_data languages tooltips
     * @param string  $lang_type         language type
     *
     * @return integer
     */
    function linked_install_set_menu_item_lang($item_id, $menu_id, $type = "update", $lang_data = array(), $lang_tooltip_data = array(), $lang_type = "value")
    {
        $ci = &get_instance();
        $ci->load->model('Menu_model');
        if ($type == 'update' && !empty($lang_data)) {
            return $ci->Menu_model->save_menu_item_lang($item_id, $menu_id, $lang_data, $lang_tooltip_data);
        } elseif ($type == 'export') {
            $return = $ci->Menu_model->_get_item_string_data($menu_id, $item_id, $lang_data, $lang_type);

            return $return;
        }

        return 0;
    }

}

if (!function_exists('linked_install_process_menu_items')) {

    /**
     * Process menu items actions
     *
     * $process_type = 'create', 'update', 'export'
     * $lang_data is lang_id => value for update AND lang_id array for export
     *
     * @param array   $structure         menu item data
     * @param string  $process_type      process action
     * @param string  $menu_gid          menu guid
     * @param integer $parent_id         parent item identifier
     * @param array   $items             process items
     * @param string  $lang_prefix       pages prefix
     * @param array   $lang_data         languages data
     * @param array   $lang_tooltip_data languages tooltip
     *
     * @return array/void
     */
    function linked_install_process_menu_items(&$structure, $process_type, $menu_gid, $parent_id, &$items, $lang_prefix = "", $lang_data = array(), $lang_tooltip_data = array())
    {
        $menu_data = $structure[$menu_gid];
        if (empty($menu_data["id"])) {
            $menu_data["id"] = $structure[$menu_gid]["id"] = linked_install_set_menu($menu_gid, "none");
        }
        if (!$menu_data["id"]) {
            return array();
        }
        if ($process_type == "export") {
            $return = array();
        }

        if (!empty($items)) {
            foreach ($items as $item_gid => $item_data) {
                if ($process_type == "create") {
                    if (!isset($item_data["action"])) {
                        $item_data["action"] = '';
                    }
                    if (!isset($item_data["link"])) {
                        $item_data["link"] = '/';
                    }
                    if (!isset($item_data["icon"])) {
                        $item_data["icon"] = '';
                    }
                    if (!isset($item_data["is_external"])) {
                        $item_data["is_external"] = 0;
                    }
                    if (!isset($item_data["status"])) {
                        $item_data["status"] = 0;
                    }
                    if (!isset($item_data["sorter"])) {
                        $item_data["sorter"] = 0;
                    }
                    if (!isset($item_data["indicator_gid"])) {
                        $item_data["indicator_gid"] = '';
                    }
                    $items[$item_gid]["id"] = linked_install_set_menu_item($item_gid, $menu_data['id'], $item_data["action"], (int) $parent_id, $item_data["link"], $item_data['icon'], $item_data["status"], $item_data["sorter"], $item_data["indicator_gid"], $item_data["is_external"]);
                    if (!empty($items[$item_gid]["items"])) {
                        linked_install_process_menu_items($structure, $process_type, $menu_gid, $items[$item_gid]["id"], $items[$item_gid]["items"]);
                    }
                } elseif ($process_type == "update") {
                    $new_prefix = $lang_prefix . '_' . $item_gid;
                    $new_prefix_tooltip = $lang_prefix . '_' . $item_gid . "_tooltip";
                    if (isset($lang_data[$new_prefix])) {
                        $item_lang_data = $lang_data[$new_prefix];
                    } else {
                        $item_lang_data = array();
                    }
                    if (isset($lang_data[$new_prefix_tooltip])) {
                        $item_lang_tooltip_data = $lang_data[$new_prefix_tooltip];
                    } else {
                        $item_lang_tooltip_data = array();
                    }
                    if (isset($items[$item_gid]["id"])) {
                        $item_id = $items[$item_gid]["id"];
                    } else {
                        $item_id = null;
                    }
                    if (!$item_id) {
                        $items[$item_gid]["id"] = $item_id = linked_install_set_menu_item($item_gid, $menu_data["id"], "none");
                    }
                    linked_install_set_menu_item_lang($item_id, $menu_data["id"], "update", $item_lang_data, $item_lang_tooltip_data);

                    if (!empty($items[$item_gid]["items"])) {
                        linked_install_process_menu_items($structure, $process_type, $menu_gid, $item_id, $items[$item_gid]["items"], $new_prefix, $lang_data, $lang_tooltip_data);
                    }
                } elseif ($process_type == "export") {
                    $new_prefix = $lang_prefix . '_' . $item_gid;
                    $new_prefix_tooltip = $lang_prefix . '_' . $item_gid . "_tooltip";
                    $item_id = $items[$item_gid]["id"];
                    if (!$item_id) {
                        $items[$item_gid]["id"] = $item_id = linked_install_set_menu_item($item_gid, $menu_data["id"], "none");
                    }
                    if ('create' == $items[$item_gid]["action"]) {
                        $return[$new_prefix] = linked_install_set_menu_item_lang($item_id, $menu_data["id"], "export", $lang_data);
                        $return[$new_prefix_tooltip] = linked_install_set_menu_item_lang($item_id, $menu_data["id"], "export", $lang_data, array(), 'tooltip');
                    }
                    if (!empty($items[$item_gid]["items"])) {
                        $temp = linked_install_process_menu_items($structure, $process_type, $menu_gid, $item_id, $items[$item_gid]["items"], $new_prefix, $lang_data);
                        $return = array_merge($return, $temp);
                    }
                }
            }
        }
        if ($process_type == "export") {
            return $return;
        } else {
            return;
        }
    }

}

if (!function_exists('linked_install_delete_menu_items')) {

    /**
     * Uninstall menu items action
     *
     * @param string $menu_gid menu guid
     * @param array  $items    menu items
     *
     * @return void
     */
    function linked_install_delete_menu_items($menu_gid, $items)
    {
        $menu_id = linked_install_set_menu($menu_gid, "none");
        if (!$menu_id) {
            return false;
        }

        if (!empty($items)) {
            foreach ($items as $item_gid => $item_data) {
                if ($item_data["action"] == 'create') {
                    linked_install_set_menu_item($item_gid, $menu_id, "delete");
                }
                if (!empty($items[$item_gid]["items"])) {
                    linked_install_delete_menu_items($menu_gid, $items[$item_gid]["items"]);
                }
            }
        }

        return;
    }

}
