<?php

namespace Pg\Modules\Notifications\Models;

/**
 * Notifications module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Notifications install model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Notifications
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Notifications_install_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Menu configuration
     *
     * @var array
     */
    protected $menu = array(
        'admin_menu' => array(
            'action' => 'none',
            'name' => 'Admin menu',
            'items' => array(
                'other_items' => array(
                    'action' => 'none',
                    'items' => array(
                        'newsletters-items' => array(
                            'action' => 'none',
                            'items' => array(
                                'notifications_menu_item' => array('action' => 'create', 'link' => 'admin/notifications', 'status' => 1, 'sorter' => 1),
                            ),
                        ),
                    ),
                ),
            ),
        ),
        'admin_notifications_menu' => array(
            'action' => 'create',
            'name' => 'Admin mode - Content - Notifications',
            'items' => array(
                'nf_settings_item' => array('action' => 'create', 'link' => 'admin/notifications/settings', 'status' => 1),
                'nf_items' => array('action' => 'create', 'link' => 'admin/notifications', 'status' => 1),
                'nf_templates_item' => array('action' => 'create', 'link' => 'admin/notifications/templates', 'status' => 1),
                'nf_pool_item' => array('action' => 'create', 'link' => 'admin/notifications/pool', 'status' => 1,
                ),
            ),
        ),
    );

    /**
     * Moderators configuration
     *
     * @var array
     */
    protected $moderators_methods = array(
        array('module' => 'notifications', 'method' => 'index', 'is_default' => 1),
        array('module' => 'notifications', 'method' => 'settings', 'is_default' => 0),
        array('module' => 'notifications', 'method' => 'templates', 'is_default' => 0),
        array('module' => 'notifications', 'method' => 'pool', 'is_default' => 0),
    );

    /**
     * Constructor
     *
     * @return Notifications_install_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
        $this->ci->load->model('Install_model');
    }

    /**
     * Validate module settings
     *
     * @return array
     */
    public function _validate_settings_form()
    {
        $errors = array();
        $data["mail_charset"] = $this->ci->input->post('mail_charset', true);
        $data["mail_protocol"] = $this->ci->input->post('mail_protocol', true);
        $data["mail_mailpath"] = $this->ci->input->post('mail_mailpath', true);
        $data["mail_smtp_host"] = $this->ci->input->post('mail_smtp_host', true);
        $data["mail_smtp_user"] = $this->ci->input->post('mail_smtp_user', true);
        $data["mail_smtp_pass"] = $this->ci->input->post('mail_smtp_pass', true);
        $data["mail_smtp_port"] = $this->ci->input->post('mail_smtp_port', true);
        $data["mail_useragent"] = $this->ci->input->post('mail_useragent', true);
        $data["mail_from_email"] = $this->ci->input->post('mail_from_email', true);
        $data["mail_from_name"] = $this->ci->input->post('mail_from_name', true);

        $openssl_loaded = extension_loaded('openssl');
        if ($openssl_loaded) {
            $data["dkim_private_key"] = $this->ci->input->post('dkim_private_key', true);
            $data["dkim_domain_selector"] = $this->ci->input->post('dkim_domain_selector', true);
        }

        if (empty($data["mail_charset"])) {
            $errors[] = $this->ci->pg_language->get_string('notifications', 'error_charset_incorrect');
        }

        if (empty($data["mail_protocol"]) || !in_array($data["mail_protocol"], array('mail', 'sendmail', 'smtp'))) {
            $errors[] = $this->ci->pg_language->get_string('notifications', 'error_protocol_incorrect');
        }

        if (empty($data["mail_useragent"])) {
            $errors[] = $this->ci->pg_language->get_string('notifications', 'error_useragent_incorrect');
        }

        if (empty($data["mail_from_email"])) {
            $errors[] = $this->ci->pg_language->get_string('notifications', 'error_from_email_incorrect');
        }

        if (empty($data["mail_from_name"])) {
            $errors[] = $this->ci->pg_language->get_string('notifications', 'error_from_name_incorrect');
        }

        $return = array(
            "data" => $data,
            "errors" => $errors,
        );

        return $return;
    }

    /**
     * Save module settings to data source
     *
     * @param array $data settings data
     *
     * @return void
     */
    public function _save_settings_form($data)
    {
        foreach ($data as $setting => $value) {
            $this->ci->pg_module->set_module_config('notifications', $setting, $value);
        }

        return;
    }

    /**
     * Return form of module settings
     *
     * @param boolean $submit is submit action
     *
     * @return string
     */
    public function _get_settings_form($submit = false)
    {
        $data = array(
            'mail_charset' => $this->ci->pg_module->get_module_config('notifications', 'mail_charset'),
            'mail_protocol' => $this->ci->pg_module->get_module_config('notifications', 'mail_protocol'),
            'mail_mailpath' => $this->ci->pg_module->get_module_config('notifications', 'mail_mailpath'),
            'mail_smtp_host' => $this->ci->pg_module->get_module_config('notifications', 'mail_smtp_host'),
            'mail_smtp_user' => $this->ci->pg_module->get_module_config('notifications', 'mail_smtp_user'),
            'mail_smtp_pass' => $this->ci->pg_module->get_module_config('notifications', 'mail_smtp_pass'),
            'mail_smtp_port' => $this->ci->pg_module->get_module_config('notifications', 'mail_smtp_port'),
            'mail_useragent' => $this->ci->pg_module->get_module_config('notifications', 'mail_useragent'),
            'mail_from_email' => $this->ci->pg_module->get_module_config('notifications', 'mail_from_email'),
            'mail_from_name' => $this->ci->pg_module->get_module_config('notifications', 'mail_from_name'),
        );

        // Check if openssl extension is loaded. It is required for DKIM.
        $openssl_loaded = extension_loaded('openssl');
        if ($openssl_loaded) {
            $data['dkim_private_key'] = $this->ci->pg_module->get_module_config('notifications', 'dkim_private_key');
            $data['dkim_domain_selector'] = $this->ci->pg_module->get_module_config('notifications',
                                                                                    'dkim_domain_selector');
            $this->view->assign('openssl_loaded', true);
        }

        if ($submit) {
            $validate = $this->_validate_settings_form();
            if (!empty($validate["errors"])) {
                $this->ci->view->assign('settings_errors', $validate["errors"]);
                $data = $validate["data"];
            } else {
                $this->_save_settings_form($validate["data"]);

                return false;
            }
        }

        $this->ci->view->assign('protocol_lang', ld('protocol', 'notifications'));
        $this->ci->view->assign('settings_data', $data);
        $html = $this->ci->view->fetch('install_settings_form', 'admin', 'notifications');

        return $html;
    }

    /**
     * Install data of menu module
     *
     * @return void
     */
    public function install_menu()
    {
        $this->ci->load->helper('menu');
        foreach ($this->menu as $gid => $menu_data) {
            $this->menu[$gid]['id'] = linked_install_set_menu($gid, $menu_data['action'], $menu_data['name']);
            linked_install_process_menu_items($this->menu, 'create', $gid, 0, $this->menu[$gid]['items']);
        }
    }

    /**
     * Import languages of menu module
     *
     * @param array $langs_ids languages idnetifiers
     *
     * @return boolean
     */
    public function install_menu_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $langs_file = $this->ci->Install_model->language_file_read('notifications', 'menu', $langs_ids);

        if (!$langs_file) {
            log_message('info', 'Empty menu langs data');

            return false;
        }

        $this->ci->load->helper('menu');

        foreach ($this->menu as $gid => $menu_data) {
            linked_install_process_menu_items($this->menu, 'update', $gid, 0, $this->menu[$gid]['items'], $gid,
                                              $langs_file);
        }

        return true;
    }

    /**
     * Export languages of menu module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_menu_lang_export($langs_ids)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $this->ci->load->helper('menu');

        $return = array();
        foreach ($this->menu as $gid => $menu_data) {
            $temp = linked_install_process_menu_items($this->menu, 'export', $gid, 0, $this->menu[$gid]['items'], $gid,
                                                      $langs_ids);
            $return = array_merge($return, $temp);
        }

        return array('menu' => $return);
    }

    /**
     * Uninstall data of menu module
     *
     * @return void
     */
    public function deinstall_menu()
    {
        $this->ci->load->helper('menu');
        foreach ($this->menu as $gid => $menu_data) {
            if ($menu_data['action'] == 'create') {
                linked_install_set_menu($gid, 'delete');
            } else {
                linked_install_delete_menu_items($gid, $this->menu[$gid]['items']);
            }
        }
    }

    /**
     * Install data of moderators module
     *
     * @return void
     */
    public function install_moderators()
    {
        // install moderators permissions
        $this->ci->load->model('Moderators_model');

        foreach ($this->moderators_methods as $method) {
            $this->ci->Moderators_model->save_method(null, $method);
        }
    }

    /**
     * Import languages of moderators module
     *
     * @param array $langs_ids languages idetifiers
     *
     * @return void
     */
    public function install_moderators_lang_update($langs_ids = null)
    {
        $langs_file = $this->ci->Install_model->language_file_read('notifications', 'moderators', $langs_ids);

        // install moderators permissions
        $this->ci->load->model('Moderators_model');
        $params['where']['module'] = 'notifications';
        $methods = $this->ci->Moderators_model->get_methods_lang_export($params);

        foreach ($methods as $method) {
            if (!empty($langs_file[$method['method']])) {
                $this->ci->Moderators_model->save_method($method['id'], array(), $langs_file[$method['method']]);
            }
        }
    }

    /**
     * Export languages of moderators module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_moderators_lang_export($langs_ids)
    {
        $this->ci->load->model('Moderators_model');
        $params['where']['module'] = 'notifications';
        $methods = $this->ci->Moderators_model->get_methods_lang_export($params, $langs_ids);
        foreach ($methods as $method) {
            $return[$method['method']] = $method['langs'];
        }

        return array('moderators' => $return);
    }

    /**
     * Uninstall data of moderators module
     *
     * @return void
     */
    public function deinstall_moderators()
    {
        // delete moderation methods in moderators
        $this->ci->load->model('Moderators_model');
        $params['where']['module'] = 'notifications';
        $this->ci->Moderators_model->delete_methods($params);
    }

    /**
     * Install data of cronjob module
     *
     * @return void
     */
    public function install_cronjob()
    {
        $this->ci->load->model('Cronjob_model');
        $cron_data = array(
            "name" => "Notification sender",
            "module" => "notifications",
            "model" => "Sender_model",
            "method" => "cron_que_sender",
            "cron_tab" => "*/5 * * * *",
            "status" => "1",
        );
        $this->ci->Cronjob_model->save_cron(null, $cron_data);
    }

    /**
     * Uninstall data of cronjob module
     *
     * @return void
     */
    public function deinstall_cronjob()
    {
        $this->ci->load->model('Cronjob_model');
        $cron_data = array();
        $cron_data["where"]["module"] = "notifications";
        $this->ci->Cronjob_model->delete_cron_by_param($cron_data);
    }

    /**
     * Install module data
     *
     * @return void
     */
    public function _arbitrary_installing()
    {
        // add entries for lang data updates
        $lang_dm_data = array(
            'module' => 'notifications',
            'model' => 'Templates_model',
            'method_add' => 'lang_dedicate_module_callback_add',
            'method_delete' => 'lang_dedicate_module_callback_delete',
        );
        $this->ci->pg_language->add_dedicate_modules_entry($lang_dm_data);
    }

    /**
     * Uninstall module data
     *
     * @return void
     */
    public function _arbitrary_deinstalling()
    {
        /// delete entries in dedicate modules
        $lang_dm_data['where'] = array(
            'module' => 'notifications',
            'model' => 'Templates_model',
        );
        $this->ci->pg_language->delete_dedicate_modules_entry($lang_dm_data);
    }

}
