<?php

namespace Pg\Modules\Payments\Models\Systems;

use Pg\Modules\Payments\Models\Payment_driver_model;

/**
 * Payments module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

/**
 * Authorize payment system driver model
 *
 * @package PG_RealEstate
 * @subpackage Payments
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Authorize_model extends Payment_driver_model
{

    /**
     *  Payment settings
     *
     *  @return void
     */
    public $payment_data = array(
        "gid" => "authorize",
        'name' => 'Authorize.net',
        'settings_data' => 'a:2:{s:15:"transaction_key";s:16:"ab34986jg5678655";s:9:"seller_id";s:11:"authnettest";}',
        'is_test' => 1,
        'in_use' => 0,
        'tarifs_type' => 0,
        'tarifs_editable' => 0,
        'disable' => 4,
        'date_add' => '',
    );

    /**
     * System settings
     *
     * @var array
     */
    public $settings = array(
        "transaction_key" => array("type" => "text", "content" => "string", "size" => "middle"),
        "seller_id" => array("type" => "text", "content" => "string", "size" => "middle"),
    );

    /**
     * Response variables
     *
     * @var array
     */
    protected $variables = array(
        "x_response_code" => "response_code",
        "x_response_subcode" => "response_subcode",
        "x_response_reason_code" => "response_reason_code",
        "x_response_reason_text" => "response_reason_text",
        "x_trans_id" => "trans_id",
        "x_invoice_num" => "id_payment",
        "x_amount" => "amount",
        "x_currency_code" => "currency_gid",
    );

    /**
     * Available currencies
     *
     * @var array
     */
    protected $currencies = array('AUD', 'USD', 'CAD', 'EUR', 'GBP', 'NZD');

    /**
     * Checkout url
     *
     * @var string
     */
    protected $checkout_url = 'https://secure.authorize.net/gateway/transact.dll';

    /**
     * Checkout url for test mode
     *
     * @var string
     */
    protected $test_checkout_url = 'https://sandbox.authorize.net/';

    /**
     * Do request to payment system
     *
     * @param array $payment_data    payment data
     * @param array $system_settings system settings
     *
     * @return array
     */
    public function func_request($payment_data, $system_settings)
    {
        $return = array("errors" => array(), "info" => array(), "data" => $payment_data);

        $time = time();

        $send_data = array(
            "x_login" => $system_settings["settings_data"]["seller_id"],
            "x_amount" => $payment_data["amount"],
            "x_invoice_num" => $payment_data["id_payment"],
            "x_description" => $payment_data["payment_data"]["name"],
            "x_test_request" => "FALSE",
            "x_fp_sequence" => $payment_data["id_payment"],
            "x_fp_timestamp" => $time,
            "x_show_form" => "PAYMENT_FORM",
            "x_relay_response" => "TRUE",
            "x_relay_url" => site_url() . "payments/responce/authorize",
        );

        $hash_str = $system_settings["settings_data"]["seller_id"] . "^" . $payment_data["id_payment"] . "^" . $time . "^" . $payment_data["amount"] . "^";

        if (in_array($payment_data["currency_gid"], $this->currencies)) {
            $send_data["x_currency_code"] = $payment_data["currency_gid"];
            $hash_str .= $payment_data["currency_gid"];
        }

        if (phpversion() >= '5.1.2') {
            $hash_str = hash_hmac("md5", $hash_str, $system_settings["settings_data"]["transaction_key"]);
        } else {
            $fingerprint = bin2hex(mhash(MHASH_MD5, $hash_str, $system_settings["settings_data"]["transaction_key"]));
        }

        $send_data["x_fp_hash"] = $hash_str;

        if (isset($system_settings['is_test']) && $system_settings['is_test'] == 2) {
            $checkout_url = $this->test_checkout_url;
        } else {
            $checkout_url = $this->checkout_url;
        }

        $this->send_data($checkout_url, $send_data, "post");

        return $return;
    }

    /**
     * Process response from payment system
     *
     * @param array $payment_data    payment data
     * @param array $system_settings system settings
     *
     * @return array
     */
    public function func_responce($payment_data, $system_settings)
    {
        $return = array("errors" => array(), "info" => array(), "data" => array(), "type" => "exit");

        foreach ($this->variables as $payment_var => $site_var) {
            $return["data"][$site_var] = isset($payment_data[$payment_var]) ? $this->ci->input->xss_clean($payment_data[$payment_var]) : "";
        }

        //// get status
        $return["data"]["status"] = 0;
        if (isset($return["data"]['response_code'])) {
            switch ($return["data"]['response_code']) {
                case "1": $return["data"]["status"] = 1;
                    break;
                case "2":
                case "3":
                    $return["data"]["status"] = -1;
                    break;
                default: $return["data"]["status"] = 0;
                    break;
            }
        }

        return $return;
    }

    /**
     * Return formatted system settings
     *
     * @return array
     */
    public function get_settings_map()
    {
        foreach ($this->settings as $param_id => $param_data) {
            $this->settings[$param_id]["name"] = l('system_field_' . $param_id, 'payments');
        }

        return $this->settings;
    }

}
