<?php

namespace Pg\Modules\Payments\Models\Systems;

use Pg\Modules\Payments\Models\Payment_driver_model;

/**
 * Payments module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

/**
 * Offline payment driver model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Payments
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Offline_model extends Payment_driver_model
{

    /**
     *  Payment settings
     *
     *  @return void
     */
    public $payment_data = array(
        'gid' => 'offline',
        'name' => 'Offline payment',
        'settings_data' => '',
        'is_test' => 0,
        'in_use' => 0,
        'tarifs_type' => 0,
        'tarifs_editable' => 0,
        'disable' => 4,
    );

    /**
     * System settings
     *
     * @var array
     */
    public $settings = array(
    );

    /**
     * Response variables
     *
     * @var array
     */
    protected $variables = array(
    );

    /**
     * User filled fields
     *
     * @var array
     */
    public $html_fields = array(
        "comment" => array("type" => "textarea", "content" => "string", "size" => "long"),
    );

    /**
     * Info data
     *
     * @var array
     */
    public $info_fields = array(
        "comment" => array("type" => "textarea", "content" => "string", "size" => "long"),
    );

    /**
     * Do request to payment system
     *
     * @param array $payment_data    payment data
     * @param array $system_settings system settings
     *
     * @return array
     */
    public function func_request($payment_data, $system_settings)
    {
        $return = array("errors" => array(), "info" => array(), "data" => $payment_data);

        return $return;
    }

    /**
     * Process response from payment system
     *
     * @param array $payment_data    payment data
     * @param array $system_settings system settings
     *
     * @return array
     */
    public function func_responce($payment_data, $system_settings)
    {
        $return = array("errors" => array(), "info" => array(), "data" => $payment_data);

        return $return;
    }

    /**
     * Use html form
     *
     * @return boolean
     */
    public function func_html()
    {
        return true;
    }

    /**
     * Validate payment data
     *
     * @param array $payment_data    payment data
     * @param array $system_settings system settings
     *
     * @return array
     */
    public function func_validate($payment_data, $system_settings)
    {
        $return = array("errors" => array(), "data" => $payment_data);

        if (!empty($this->html_fields)) {
            foreach ($this->html_fields as $param_id => $param_data) {
                if ($param_data["content"] == "string") {
                    $return["data"][$param_id] = trim(strip_tags($payment_data[$param_id]));
                } elseif ($param_data["content"] == "int") {
                    $return["data"][$param_id] = intval($payment_data[$param_id]);
                } else {
                    $return["data"][$param_id] = $payment_data[$param_id];
                }
            }
        }

        return $return;
    }

    /**
     * Return formatted system settings
     *
     * @return array
     */
    public function get_settings_map()
    {
        foreach ($this->settings as $param_id => $param_data) {
            $this->settings[$param_id]["name"] = l('system_field_' . $param_id, 'payments');
        }

        return $this->settings;
    }

    /**
     * Return formatted user filled fields
     *
     * @return array
     */
    public function get_html_map()
    {
        foreach ($this->html_fields as $param_id => $param_data) {
            $this->html_fields[$param_id]["name"] = l('html_field_' . $param_id, 'payments');
        }

        return $this->html_fields;
    }

}
