<?php

namespace Pg\Modules\Payments\Models\Systems;

use Pg\Modules\Payments\Models\Payment_driver_model;

/**
 * Payments module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

/**
 * Robocassa payment system driver model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Payments
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Robokassa_model extends Payment_driver_model
{

    /**
     *  Payment settings
     *
     *  @return void
     */
    public $payment_data = array(
        'gid' => 'robokassa',
        'name' => 'Robokassa',
        'settings_data' => 'a:3:{s:14:"merchant_login";s:5:"login";s:14:"merchant_pass1";s:9:"password1";s:14:"merchant_pass2";s:9:"password2";}',
        'is_test' => 1,
        'in_use' => 0,
        'tarifs_type' => 0,
        'tarifs_editable' => 0,
        'disable' => 0,
        'date_add' => '',
    );

    /**
     * System settings
     *
     * @var array
     */
    public $settings = array(
        'merchant_login' => array('type' => 'text', 'content' => 'string', 'size' => 'middle'),
        'merchant_pass1' => array('type' => 'text', 'content' => 'string', 'size' => 'middle'),
        'merchant_pass2' => array('type' => 'text', 'content' => 'string', 'size' => 'middle'),
    );

    /**
     * Response variables
     *
     * @var array
     */
    protected $variables = array(
        "OutSum" => "amount",
        "InvId" => "id_payment",
        "SignatureValue" => "hash",
    );

    /**
     * Abailable languages
     *
     * @var array
     */
    protected $available_languages = array('en', 'ru');

    /**
     * Checkout url
     *
     * @var string
     */
    protected $checkout_url = 'https://auth.robokassa.ru/Merchant/Index.aspx';

    /**
     * Checkout url for test mode
     *
     * @var string
     */
    protected $test_checkout_url = 'http://test.robokassa.ru/Index.aspx';

    /**
     * Do request to payment system
     *
     * @param array $payment_data    payment data
     * @param array $system_settings system settings
     *
     * @return array
     */
    public function func_request($payment_data, $system_settings)
    {
        $return = array("errors" => array(), "info" => array(), "data" => $payment_data);

        $user_id = $this->ci->session->userdata('user_id');

        $this->ci->load->model('Users_model');
        $this->ci->Users_model->set_format_settings('get_safe', false);
        $user = $this->ci->Users_model->get_user_by_id($user_id);
        $this->ci->Users_model->set_format_settings('get_safe', true);

        $send_data = array(
            'MrchLogin' => $system_settings["settings_data"]["merchant_login"],
            'OutSum' => $payment_data["amount"],
            'InvId' => $payment_data["id_payment"],
            'Desc' => $payment_data["payment_data"]["name"],
            'SignatureValue' => md5($system_settings["settings_data"]["merchant_login"] . ':' . $payment_data["amount"] . ':' . $payment_data["id_payment"] . ':' . $system_settings["settings_data"]["merchant_pass1"]),
            'Email' => $user['email'],
        );

        $current_lang = $this->ci->pg_language->get_lang_by_id($this->ci->pg_language->current_lang_id);
        $current_lang['code'] = strtolower($current_lang['code']);
        if (in_array($current_lang['code'], $this->available_languages)) {
            $send_data['Culture'] = $current_lang['code'];
        }

        if (isset($system_settings['is_test']) && $system_settings['is_test'] == 2) {
            $checkout_url = $this->test_checkout_url;
        } else {
            $checkout_url = $this->checkout_url;
        }

        $this->send_data($checkout_url, $send_data, "post");

        return $return;
    }

    /**
     * Process response from payment system
     *
     * @param array $payment_data    payment data
     * @param array $system_settings system settings
     *
     * @return array
     */
    public function func_responce($payment_data, $system_settings)
    {
        $return = array("errors" => array(), "info" => array(), "data" => array(), "type" => "exit");

        foreach ($this->variables as $payment_var => $site_var) {
            $data[$site_var] = isset($payment_data[$payment_var]) ? $this->ci->input->xss_clean($payment_data[$payment_var]) : "";
        }

        $error = false;

        $this->ci->load->model("Payments_model");
        $site_payment_data = $this->ci->Payments_model->get_payment_by_id($data['id_payment']);
        if (floatval($site_payment_data["amount"]) != floatval($data['amount'])) {
            $error = true;
        }

        $server_side_hash = $data['amount'] . ':' . $data['id_payment'] . ':' . $system_settings["settings_data"]["merchant_pass2"];
        $server_side_hash = strtoupper(trim(md5($server_side_hash)));
        if ($server_side_hash != $data['hash']) {
            $error = true;
        }

        $return["data"] = $data;
        if ($error) {
            $return["data"]["status"] = -1;
        } else {
            $return["data"]["status"] = 1;
        }

        return $return;
    }

    /**
     * Return formatted system settings
     *
     * @return array
     */
    public function get_settings_map()
    {
        foreach ($this->settings as $param_id => $param_data) {
            $this->settings[$param_id]["name"] = l('system_field_' . $param_id, 'payments');
        }

        return $this->settings;
    }

}
