<?php

namespace Pg\Modules\Payments\Models\Systems;

use Pg\Modules\Payments\Models\Payment_driver_model;

/**
 * Payments module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

/**
 * Tcheckout payment system driver model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Payments
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Tcheckout_model extends Payment_driver_model
{

    /**
     *  Payment settings
     *
     *  @return void
     */
    public $payment_data = array(
        'gid' => 'tcheckout',
        'name' => '2Checkout',
        'settings_data' => 'a:2:{s:9:"seller_id";s:9:"2checkout";s:11:"secret_word";s:18:"2checkout@mail.com";}',
        'is_test' => 0,
        'in_use' => 0,
        'tarifs_type' => 0,
        'tarifs_editable' => 0,
        'disable' => 4,
        'date_add' => '',
    );

    /**
     * System settings
     *
     * @var array
     */
    public $settings = array(
        "seller_id" => array("type" => "text", "content" => "string", "size" => "middle"),
        "secret_word" => array("type" => "text", "content" => "string", "size" => "middle"),
    );

    /**
     * Response variables
     *
     * @var array
     */
    protected $variables = array(
        "sid" => "seller_id",
        "cart_order_id" => "id_payment",
        "order_number" => "id_order",
        "total" => "amount",
        "key" => "hash",
    );

    /**
     * Notification variables
     *
     * @var array
     */
    protected $notifications_variables = array(
        "vendor_id" => "seller_id",
        "invoice_id" => "id_payment",
        "sale_id" => "amount",
        "md5_hash" => "hash",
    );

    /**
     * Available currencies
     *
     * @var array
     */
    protected $currencies = array('ARS', 'AUD', 'BRL', 'GBP', 'CAD', 'DKK', 'EUR', 'HKD',
        'INR', 'ILS', 'JPY', 'LTL', 'MYR', 'MXN', 'NZD', 'NOK',
        'PHP', 'RON', 'RUB', 'SGD', 'ZAR', 'SEK', 'CHF', 'TRY',
        'AED', 'USD',);

    /**
     * Checkout url
     *
     * @var string
     */
    protected $checkout_url = 'https://www.2checkout.com/checkout/purchase';

    /**
     * Checkout url for test mode
     *
     * @var string
     */
    protected $test_checkout_url = '';

    /**
     * Do request to payment system
     *
     * @param array $payment_data    payment data
     * @param array $system_settings system settings
     *
     * @return void
     */
    public function func_request($payment_data, $system_settings)
    {
        $return = array("errors" => array(), "info" => array(), "data" => $payment_data);
        $send_data = array(
            "sid" => $system_settings["settings_data"]["seller_id"],
            "total" => $payment_data["amount"],
            "cart_order_id" => "CART-" . $payment_data["id_payment"],
            //"demo" => "Y",
            "x_receipt_link_url" => site_url() . "payments/responce/tcheckout",
        );

        if (in_array($payment_data["currency_gid"], $this->currencies)) {
            $send_data["currency_code"] = $payment_data["currency_gid"];
        }

        if (isset($system_settings['is_test']) && $system_settings['is_test'] == 2) {
            $checkout_url = $this->test_checkout_url;
        } else {
            $checkout_url = $this->checkout_url;
        }

        $this->send_data($checkout_url, $send_data, "get");

        return $return;
    }

    /**
     * Process response from payment system
     *
     * @param array $payment_data    payment data
     * @param array $system_settings system settings
     *
     * @return array
     */
    public function func_responce($payment_data, $system_settings)
    {
        $return = array("errors" => array(), "info" => array(), "data" => array(), "type" => "html");

        foreach ($this->variables as $payment_var => $site_var) {
            $return["data"][$site_var] = isset($payment_data[$payment_var]) ? $this->ci->input->xss_clean($payment_data[$payment_var]) : "";
        }

        if (isset($return['data']['id_payment'])) {
            $return['data']['id_payment'] = str_replace('CART-', '', $return['data']['id_payment']);
        }
        if ($system_settings["settings_data"]["seller_id"] != $data["seller_id"]) {
            $error = true;
        }

        $this->ci->load->model("Payments_model");
        $site_payment_data = $this->ci->Payments_model->get_payment_by_id($return['data']['id_payment']);
        if (floatval($site_payment_data["amount"]) != floatval($return['data']['amount'])) {
            $error = true;
        }

        if ($payment_data['demo'] == 'Y') {
            $return['data']['id_order'] = '1';
        }

        $server_side_hash = $system_settings["settings_data"]["secret_word"] .
                $return['data']['seller_id'] .
                $return['data']['id_order'] .
                strval($return['data']['amount']);
        $server_side_hash = strtoupper(md5($server_side_hash));
        if ($server_side_hash != $return['data']["hash"]) {
            $error = true;
        }

        if ($error) {
            $return["data"]["status"] = -1;
        } else {
            $return["data"]["status"] = 1;
        }

        return $return;
    }

    /**
     * Return formatted system settings
     *
     * @return array
     */
    public function get_settings_map()
    {
        foreach ($this->settings as $param_id => $param_data) {
            $this->settings[$param_id]["name"] = l('system_field_' . $param_id, 'payments');
        }

        return $this->settings;
    }

}
