<?php

/**
 * Properties module
 *
 * @package     PG_RealEstate
 *
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Properties management
 *
 * @package     PG_RealEstate
 * @subpackage  Properties
 *
 * @category    helpers
 *
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!function_exists('categories_select')) {
    /**
     * Display categories selector
     *
     * @param array $params block parameters
     *
     * @return string
     */
    function categories_select($params)
    {
        $ci = &get_instance();
        $ci->load->model("Properties_model");

        extract($params);

        if (!$categories_gid) {
            $categories_gid = current($ci->Properties_model->categories);
        }
        if (!$lang_id) {
            $lang_id = $ci->pg_language->current_lang_id;
        }
        if (!$var_name) {
            $var_name = 'id_category';
        }
        if (!$js_var_name) {
            $js_var_name = '';
        }
        $selected = strval($selected);

        $data = $ci->Properties_model->get_categories($categories_gid, $lang_id);

        if (empty($data)) {
            return false;
        }

        $select = array(
            'options'  => $data,
            'name'     => $var_name,
            'id'       => $js_var_name,
            'selected' => $selected,
        );
        $ci->view->assign('categories_helper_data', $select);

        return $ci->view->fetch('helper_categories_select', null, 'properties');
    }
}

if (!function_exists('properties_select')) {
    /**
     * Display properties selector
     *
     * categories = array of ids
     *
     * @param array $params block parameters
     *
     * @return string
     */
    function properties_select($params)
    {
        $ci = &get_instance();
        $ci->load->model("Properties_model");

        extract($params);

        if (empty($params['categories_gid'])) {
            $params['categories_gid'] = current($ci->Properties_model->categories);
        }
        if (empty($params['lang_id'])) {
            $params['lang_id'] = $ci->pg_language->current_lang_id;
        }
        if (empty($params['var_name'])) {
            $params['var_name'] = 'id_category';
        }
        if (empty($params['js_var_name'])) {
            $params['js_var_name'] = '';
        }
        if (empty($params['category_str'])) {
            $params['category_str'] = '';
        }

        $selected = strval($selected);

        $categories = $ci->Properties_model->get_categories($params['categories_gid'], $params['lang_id']);

        if (empty($params['id_category'])) {
            $params['id_category'] = null;
            $properties = $ci->Properties_model->get_all_properties($params['categories_gid'], $params['lang_id']);
            foreach ($categories as $cid => $cdata) {
                if (!empty($options)) {
                    unset($options);
                }
                foreach ($properties[$cid] as $key => $name) {
                    $options[$key] = array(
                        'id'    => $key,
                        'value' => $cid . '_' . $key,
                        'name'  => $name,
                    );
                }

                $data[$cid] = array(
                    'id'      => $cid,
                    'value'   => $cid,
                    'name'    => $cdata,
                    'options' => $options,
                );
            }
        } else {
            $properties = $ci->Properties_model->get_properties($params['id_category'], $params['lang_id']);
            foreach ($properties as $key => $name) {
                $options[$key] = array(
                'id'    => $key,
                'value' => $params['id_category'] . '_' . $key,
                'name'  => $name,
                );
            }

            $data[$params['id_category']] = array(
                'id'      => $params['id_category'],
                'value'   => $params['id_category'],
                'options' => $options,
            );
        }
        $helper_data = array(
            'data'                 => $data,
            'count'                => count($data),
            'name'                 => $params['var_name'],
            'id'                   => $params['js_var_name'],
            'cat_select_available' => (int) get($params['cat_select']),
            'selected'             => $selected,
            'id_category'          => $params['id_category'],
            'category_str'         => $params['category_str'],
        );

        $ci->view->assign('properties_helper_data', $helper_data);

        if (!empty($params['for_edit'])) {
            if ($params['for_edit']) {
                $ci->view->assign('for_edit', $params['for_edit']);
            }
        }

        return $ci->view->fetch('helper_properties_select', null, 'properties');
    }
}

if (!function_exists('category_value')) {
    /**
     * Dispaly category name
     *
     * categories = array of ids
     *
     * @param array $params block parameters
     *
     * @return string
     */
    function category_value($params)
    {
        $ci = &get_instance();
        $ci->load->model("Properties_model");

        extract($params);

        if (!isset($categories_gid) || !$categories_gid) {
            $categories_gid = current($ci->Properties_model->categories);
        }
        if (!isset($lang_id) || !$lang_id) {
            $lang_id = $ci->pg_language->current_lang_id;
        }
        if (!isset($id_category) || !$id_category) {
            $id_category = 1;
        }

        $categories = $ci->Properties_model->get_categories($categories_gid, $lang_id);

        return !empty($categories[$id_category]) ? $categories[$id_category] : [];
    }
}

if (!function_exists('property_value')) {
    /**
     * Display property name
     *
     * categories = array of ids
     *
     * @param array $params block parameters
     *
     * @return string
     */
    function property_value($params)
    {
        $ci = &get_instance();
        $ci->load->model("Properties_model");

        extract($params);

        if (empty($categories_gid)) {
            $categories_gid = current($ci->Properties_model->categories);
        }
        if (empty($lang_id)) {
            $lang_id = $ci->pg_language->current_lang_id;
        }
        if (empty($id_category)) {
            $id_category = 1;
        }
        if (empty($property_type)) {
            $property_type = 1;
        }
        if (!isset($full_output)) {
            $full_output = false;
        }

        $categories = $ci->Properties_model->get_categories($categories_gid, $lang_id);
        $properties = $ci->Properties_model->get_properties($id_category, $lang_id);

        $return = $properties[$property_type];
        if ($full_output) {
            $return = $categories[$id_category] . ', ' . $return;
        }

        return $return;
    }
}
