<?php

/**
 * Reviews module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
/**
 * Reviews management
 *
 * @package 	PG_RealEstate
 * @subpackage 	Reviews
 *
 * @category	helpers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!function_exists("send_review_block")) {

    /**
     * Show send reviews button
     *
     * @param array $data block parameters
     *
     * @return string
     */
    function send_review_block($data)
    {
        $ci = &get_instance();

        if (!isset($data["object_id"]) || !intval($data["object_id"]) ||
            !isset($data["type_gid"]) || !$data["type_gid"]) {
            return "";
        }

        static $is_login_enabled = null;
        if (is_null($is_login_enabled)) {
            $ci->load->helper('users');
            $is_login_enabled = is_login_enabled();
        }
        if (!$is_login_enabled) {
            return '';
        }

        if (isset($data["is_owner"]) && !empty($data["is_owner"])) {
            return '';
        }

        $ci->load->model("Reviews_model");
        $ci->load->model("reviews/models/Reviews_type_model");

        $type = $ci->Reviews_type_model->get_type($data["type_gid"], true);

        $ci->view->assign("type", $type);
        $ci->view->assign("object_id", $data["object_id"]);

        $data["responder_id"] = isset($data["responder_id"]) ? intval($data["responder_id"]) : 0;
        $ci->view->assign("responder_id", $data["responder_id"]);

        if (!isset($data["template"])) {
            $data["template"] = "button";
        }
        $ci->view->assign("template", $data["template"]);

        $is_guest = $ci->session->userdata("auth_type") != "user";
        $ci->view->assign("is_guest", $is_guest);

        if ($ci->session->userdata("auth_type") == 'user') {
            $user_id = $ci->session->userdata("user_id");
            $is_send = $ci->Reviews_model->is_review_from_poster($type["gid"], $user_id, $data["object_id"]);
            $ci->view->assign('is_send', $is_send);
        } else {
            $ci->view->assign('is_send', 0);
        }

        $ci->view->assign("success", get($data["success"]));

        $ci->view->assign('rand', rand(100000, 999999));

        $html = $ci->view->fetch("helper_send_review", "user", "reviews");

        return $html;
    }

}

if (!function_exists("get_rate_block")) {

    /**
     * Display object rating block
     *
     * @param array $data block parameters
     *
     * @return string
     */
    function get_rate_block($data)
    {
        $ci = &get_instance();

        $ci->load->model("reviews/models/Reviews_type_model");

        if (isset($data['type']) && !empty($data['type'])) {
            $type = $data['type'];
        } elseif (isset($data["type_gid"]) && !empty($data["type_gid"])) {
            $type = $ci->Reviews_type_model->get_type($data["type_gid"], true);
        }

        if (!$type) {
            return "";
        }
        $ci->view->assign("type", $type);

        $template = isset($data["template"]) ? $data["template"] : "extended";
        $ci->view->assign("template", $template);

        $read_only = isset($data["read_only"]) ? true : false;
        $ci->view->assign("read_only", $read_only);

        $ratings = array_keys($type["values"][$type["rate_type"]]);

        foreach ($ratings as $rating) {
            $rating_data[$rating] = isset($data["rating_data"][$rating]) ? $data["rating_data"][$rating] : 0;
            if (isset($data["rating_data_" . $rating])) {
                $rating_data[$rating] = $data["rating_data_" . $rating];
            }
        }
        $ci->view->assign("rating_data", $rating_data);

        $show_label = isset($data["show_label"]) ? true : false;
        $ci->view->assign("show_label", $show_label);

        $rand = rand(100000, 999999);
        $ci->view->assign("rand", $rand);

        $symbolType = $type["rate_type"] == "stars" ? "previous" : "equal";
        $ci->view->assign("symbolType", $symbolType);

        $html = $ci->view->fetch("helper_get_rate", null, "reviews");

        return $html;
    }

}

if (!function_exists("get_reviews_block")) {

    /**
     * Display reviews block
     *
     * @param array $data block parameters
     *
     * @return string
     */
    function get_reviews_block($data)
    {
        $ci = &get_instance();

        if (!isset($data["object_id"]) || !intval($data["object_id"]) ||
            !isset($data["type_gid"]) || !$data["type_gid"]) {
            return "";
        }

        $ci->view->assign("review_type_gid", $data["type_gid"]);
        $ci->view->assign("review_object_id", $data["object_id"]);

        $block = get_reviews_list($data["type_gid"], $data["object_id"]);
        $ci->view->assign('block', $block);

        $current_settings = isset($_SESSION["reviews_list"]) ? $_SESSION["reviews_list"] : array();
        if (!isset($current_settings["order"])) {
            $current_settings["order"] = "date_add";
        }
        $ci->view->assign('order', $current_settings["order"]);

        if (!isset($current_settings["order_direction"])) {
            $current_settings["order_direction"] = "DESC";
        }
        $ci->view->assign('order_direction', $current_settings["order_direction"]);

        if (!isset($current_settings["page"])) {
            $current_settings["page"] = 1;
        }
        $ci->view->assign('page', $current_settings["page"]);

        $html = $ci->view->fetch("helper_get_reviews", "user", "reviews");

        return $html;
    }

}

if (!function_exists("get_reviews_list")) {

    /**
     * Display reviews helper
     *
     * @param string  $type_gid        review type guid
     * @param integer $object_id       object identifier
     * @param string  $order           sorting order
     * @param string  $order_direction sorting direction
     * @param integer $page            page of results
     *
     * @return string
     */
    function get_reviews_list($type_gid = null, $object_id = null, $order = null, $order_direction = null, $page = null)
    {
        $ci = &get_instance();
        $ci->load->model('Reviews_model');

        $attrs = array();

        $current_settings = isset($_SESSION["reviews_list"]) ? $_SESSION["reviews_list"] : array();
        if (!isset($current_settings["order"])) {
            $current_settings["order"] = "date_add";
        }
        if (!isset($current_settings["order_direction"])) {
            $current_settings["order_direction"] = "DESC";
        }
        if (!isset($current_settings["page"])) {
            $current_settings["page"] = 1;
        }

        if (!$order) {
            $order = $current_settings["order"];
        }
        $ci->view->assign("order", $order);
        $current_settings["order"] = $order;

        if (!$order_direction) {
            $order_direction = $current_settings["order_direction"];
        }
        $ci->view->assign("order_direction", $order_direction);
        $current_settings["order_direction"] = $order_direction;

        $filters = array('type' => $type_gid, 'object' => $object_id);

        $reviews_count = $ci->Reviews_model->get_reviews_count_by_filters($filters);

        if (!$page) {
            $page = $current_settings["page"];
        }
        $items_on_page = $ci->pg_module->get_module_config("start", "index_items_per_page");

        $sort_data = array(
            "url" => site_url() . "reviews/get_reviews/" . $type_gid . "/" . $object_id,
            "order" => $order,
            "direction" => $order_direction,
            "links" => array(
                "poster" => l("field_reviews_poster", "reviews"),
                "date_add" => l("field_reviews_date_add", "reviews"),
            ),
        );
        $ci->view->assign("sort_data", $sort_data);

        $ci->load->helper("sort_order");
        $page = get_exists_page_number($page, $reviews_count, $items_on_page);
        $current_settings["page"] = $page;

        $_SESSION["reviews_list"] = $current_settings;

        if ($reviews_count > 0) {
            if ($order == "poster") {
                $order = "id_poster";
            }
            $reviews = $ci->Reviews_model->get_reviews_list_by_filters($filters, $page, $items_on_page, array($order => $order_direction));
            $ci->view->assign("reviews", $reviews);
        }

        $ci->load->helper("navigation");

        $url = site_url() . "reviews/get_reviews/" . $type_gid . "/" . $object_id . "/" . $order . "/" . $order_direction . "/";
        $page_data = get_user_pages_data($url, $reviews_count, $items_on_page, $page, "briefPage");
        $page_data["date_format"] = $ci->pg_date->get_format('date_time_literal', 'st');
        $ci->view->assign("page_data", $page_data);
        $ci->view->assign("object_id", $object_id);

        $ci->view->assign("user_id", intval($ci->session->userdata("user_id", true)));

        return $ci->view->fetch("reviews_list", "user", "reviews");
    }

}

if (!function_exists("get_reviews_sorter")) {

    /**
     * Return data for sorting by reviews
     *
     * @return array
     */
    function get_reviews_sorter()
    {
        return array(
            "review_sorter" => l("field_reviews_rate", "reviews"),
            "review_count" => l("field_reviews_count", "reviews"),
        );
    }

}

if (!function_exists("get_reviews_fields")) {

    /**
     * Return reviews properties
     *
     * @return array
     */
    function get_reviews_fields()
    {
        return array("review_data", "review_count", "review_sorter", "review_type");
    }

}

if (!function_exists("admin_home_reviews_block")) {

    /**
     * Statistics of reviews on admin homepage
     *
     * @return string
     */
    function admin_home_reviews_block()
    {
        $ci = &get_instance();

        $auth_type = $ci->session->userdata("auth_type");
        if ($auth_type != "admin") {
            return "";
        }

        $user_type = $ci->session->userdata("user_type");

        $show = true;

        $stat_reviews = array(
            "index_method" => true,
        );

        if ($user_type == "moderator") {
            $show = false;
            $ci->load->model("Moderators_model");
            $methods_reviews = $ci->Moderators_model->get_module_methods("reviews");
            if ((is_array($methods_reviews) && !in_array("index", $methods_reviews))) {
                $show = true;
            } else {
                $permission_data = $ci->session->userdata("permission_data");
                if ((isset($permission_data["reviews"]["index"]) && $permission_data["reviews"]["index"] == 1)) {
                    $show = true;
                    $stat_reviews["index_method"] = (bool) $permission_data["reviews"]["index"];
                }
            }
        }

        if (!$show) {
            return "";
        }

        $ci->load->model("Reviews_model");
        $ci->load->model("reviews/models/Reviews_type_model");

        $review_types = $ci->Reviews_type_model->get_types();

        if (empty($review_types)) {
            return '';
        }

        $stat_reviews['types'] = array();
        foreach ($review_types as $review_type) {
            $review_type['count'] = $ci->Reviews_model->get_reviews_count_by_filters(array('type' => $review_type['gid']));
            $stat_reviews['types'][] = $review_type;
        }

        $ci->view->assign("stat_reviews", $stat_reviews);

        return $ci->view->fetch("helper_admin_home_block", "admin", "reviews");
    }

}

if (!function_exists("reviews_visitors_block")) {

    /**
     * Display reviews visitors block
     *
     * @param array $params block parameters
     *
     * @return string
     */
    function reviews_visitors_block($params)
    {
        $ci = &get_instance();

        $user_id = intval($ci->session->userdata("user_id"));

        $ci->load->model("reviews/models/Reviews_model");
        $ci->load->model("reviews/models/Reviews_type_model");

        $types = $ci->Reviews_type_model->get_types();
        foreach ($types as $type) {
            if ($params['type'] == $type['gid']) {
                $ci->view->assign('type', $type['gid']);

                $visitors_count_week = $ci->Reviews_model->get_reviews_count_by_filters(array('type' => $type['gid'], 'responder' => $user_id, 'week' => 1));
                $ci->view->assign("visitors_count_week", $visitors_count_week);

                $visitors_count_month = $ci->Reviews_model->get_reviews_count_by_filters(array('type' => $type['gid'], 'responder' => $user_id, 'month' => 1));
                $ci->view->assign("visitors_count_month", $visitors_count_month);

                return $ci->view->fetch("helper_reviews_visitors", "user", "reviews");
            }
        }

        return '';
    }

}

if (!function_exists("reviews_visits_block")) {

    /**
     * Display reviews visits block
     *
     * @param array $params block parameters
     *
     * @return string
     */
    function reviews_visits_block($params)
    {
        $ci = &get_instance();

        $user_id = intval($ci->session->userdata("user_id"));

        $ci->load->model("reviews/models/Reviews_model");
        $ci->load->model("reviews/models/Reviews_type_model");

        $types = $ci->Reviews_type_model->get_types();

        foreach ($types as $type) {
            if ($params['type'] == $type['gid']) {
                $ci->view->assign('type', $type['gid']);

                $visits_count_week = $ci->Reviews_model->get_reviews_count_by_filters(array('type' => $type['gid'], 'poster' => $user_id, 'week' => 1));
                $ci->view->assign("visits_count_week", $visits_count_week);

                $visits_count_month = $ci->Reviews_model->get_reviews_count_by_filters(array('type' => $type['gid'], 'poster' => $user_id, 'month' => 1));
                $ci->view->assign("visits_count_month", $visits_count_month);

                return $ci->view->fetch("helper_reviews_visits", "user", "reviews");
            }
        }

        return '';
    }

}
