<?php

namespace Pg\Modules\Services\Controllers;

/**
 * Services module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Services api controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	Services
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Api_Services extends \Controller
{

    /**
     * Constructor
     *
     * @return Api_Services
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model("Services_model");
    }

    /**
     * Get service data
     *
     * @param string $service_gid service guid
     *
     * @return void
     */
    public function get($service_gid)
    {
        if (!$service_gid) {
            log_message('error', 'services API: Empty service gid');
            $this->set_api_content('errors', l('error_service_code_incorrect', 'services'));

            return false;
        }

        $service = $this->Services_model->get_service_by_gid($service_gid);
        if (!$service['status']) {
            log_message('error', 'services API: Wrong service status');
            $this->set_api_content('errors', l('error_service_code_incorrect', 'services'));

            return false;
        }

        $t = $this->Services_model->format_service(array($service));
        $service = $t[0];
        $service['template'] = $this->Services_model->format_template($service['template']);
        if (!empty($service['data_admin_array'])) {
            foreach ($service['template']['data_admin_array'] as $gid => $temp) {
                if (!empty($service['data_admin_array'][$gid])) {
                    $service['template']['data_admin_array'][$gid]['value'] = $service['data_admin_array'][$gid];
                }
            }
        }

        $data['service'] = $service;

        $this->load->model('Currencies_model');
        $base_currency = $this->Currencies_model->get_currency_default(true);

        $this->load->helper('currencies');
        $data['service']['price_output'] = currency_format(array('value' => $service['price'], 'cur_gid' => $base_currency['gid']));

        if ($service["pay_type"] == 2 || $service["pay_type"] == 3) {
            $this->load->model("payments/models/Payment_systems_model");
            $billing_systems = $this->Payment_systems_model->get_active_system_list();
            $data['billing_systems'] = $billing_systems;
        } else {
            $data['billing_systems'] = array();
        }

        $this->set_api_content('data', $data);
    }

    /**
     * Get service form
     *
     * @param string $service_gid    service guid
     * @param array  $user_form_data form data need filled
     *
     * @return void
     */
    public function get_form()
    {
        $service_gid = $this->input->post('service_gid', true);
        if (!$service_gid) {
            log_message('error', 'services API: Empty service gid');
            $this->set_api_content('errors', l('error_service_code_incorrect', 'services'));

            return false;
        }
        $user_form_data = $this->input->post('user_form_data', true);

        $user_id = $this->session->userdata('user_id');
        $this->load->model('users/models/Auth_model');
        $this->Auth_model->update_user_session_data($user_id);

        $service = $this->Services_model->get_service_by_gid($service_gid);
        if (!$service['status']) {
            log_message('error', 'services API: Wrong service status');
            $this->set_api_content('errors', l('error_service_code_incorrect', 'services'));

            return false;
        }

        $t = $this->Services_model->format_service(array($service));
        $service = $t[0];
        $service['template'] = $this->Services_model->format_template($service['template']);
        if (!empty($service['data_admin_array'])) {
            foreach ($service['template']['data_admin_array'] as $gid => $temp) {
                if (!empty($service['data_admin_array'][$gid])) {
                    $service['template']['data_admin_array'][$gid]['value'] = $service['data_admin_array'][$gid];
                }
            }
        }

        if ($service['template']['price_type'] == '2' || $service['template']['price_type'] == '3') {
            $service['price'] = $this->input->post('price', true);
        }

        if (!empty($service['template']['data_user_array'])) {
            foreach ($service['template']['data_user_array'] as $gid => $temp) {
                $value = '';
                if ($temp['type'] == 'hidden') {
                    $value = $this->input->get_post($gid, true);
                }
                if (isset($user_form_data[$gid])) {
                    $value = $user_form_data[$gid];
                }
                $service['template']['data_user_array'][$gid]['value'] = $value;
            }
        }

        // get payments types
        $service['free_activate'] = false;
        if ($service['price'] <= 0) {
            $service['free_activate'] = true;
        }
        if ($service['pay_type'] == 1 || $service['pay_type'] == 2) {
            $this->load->model('Users_payments_model');
            $service['user_account'] = $this->Users_payments_model->get_user_account($user_id);
            if ($service["user_account"] <= 0 && $service['price'] > 0) {
                $service['disable_account_pay'] = true;
            } elseif (($service['template']['price_type'] == 1 || $service['template']['price_type'] == 3) && $service['price'] > $service['user_account']) {
                $service['disable_account_pay'] = true;
            }
        }

        if ($service['pay_type'] == 2 || $service['pay_type'] == 3) {
            $this->load->model('payments/models/Payment_systems_model');
            $data['billing_systems'] = $this->Payment_systems_model->get_active_system_list();
        }
        $data['service'] = $service;
        $data['users_payments_module_installed'] = $this->pg_module->is_module_active('users_payments');
        $this->set_api_content('data', $data);
    }

    /**
     * Send service form
     *
     * Available payment type values: account, system
     *
     * System guid is filled if payment type equal system
     *
     * @param string $service_gid  service guid
     * @param array  $user_data    user data
     * @param string $payment_type payment type
     * @param string $system_gid   payment system guid
     *
     * @return void
     */
    public function send_form()
    {
        $service_gid = $this->input->post('service_gid', true);
        if (!$service_gid) {
            log_message('error', 'services API: Empty service gid');
            $this->set_api_content('errors', l('error_service_code_incorrect', 'services'));

            return false;
        }

        $service = $this->Services_model->get_service_by_gid($service_gid);
        $user_data = $this->input->post('user_data', true);

        $user_id = $this->session->userdata('user_id');
        $this->load->model('users/models/Auth_model');
        $this->Auth_model->update_user_session_data($user_id);

        $service_return = $this->Services_model->validate_service_payment($service['id'], $user_data, $service['price']);
        if (!empty($service_return['errors'])) {
            $this->set_api_content('errors', $service_return['errors']);

            return false;
        }
        $origin_return = $this->Services_model->validate_service_original_model($service['id'], $user_data, $user_id,
                                                                                $service['price']);
        if (!empty($origin_return['errors'])) {
            $this->set_api_content('errors', $origin_return['errors']);

            return false;
        }
        $payment_type = $this->input->post('payment_type');

        if ('account' === $payment_type) {
            $return = $this->Services_model->account_payment($service['id'], $user_id, $user_data, $service['price']);
            if ($return !== true) {
                $this->set_api_content('errors', $return);
            } else {
                $this->set_api_content('messages', l('success_services_apply', 'services'));
                /* $redirect = $this->session->userdata('service_redirect');
                  $this->session->set_userdata(array('service_redirect'=>''));
                  $this->load->model('users/models/Auth_model');
                  $this->Auth_model->update_user_session_data($user_id);
                  redirect($redirect); */
            }
        } elseif ('system' === $payment_type) {
            $system_gid = $this->input->post('system_gid', true);
            $data['system_gid'] = $system_gid;
            if (empty($system_gid)) {
                $this->set_api_content('messages', l('error_select_payment_system', 'services'));
            } else {
                $this->Services_model->system_payment($system_gid, $user_id, $service['id'], $user_data,
                                                      $service['price']);
                /* $redirect = $this->session->userdata('service_redirect');
                  $this->session->set_userdata(array('service_redirect'=>''));
                  $this->load->model('users/models/Auth_model');
                  $this->Auth_model->update_user_session_data($user_id);
                  redirect($redirect); */
            }
        }
        $data['service_gid'] = $service_gid;
        $data['user_data'] = $user_data;
        $data['payment_type'] = $payment_type;
        $data['service_gid'] = $service_gid;
        $this->set_api_content('data', $data);
    }

    /**
     * Pay for service by account
     *
     * @param string $service_gid service guid
     *
     * @return void
     */
    public function account_payment($service_gid)
    {
        if (!$service_gid) {
            log_message('error', 'services API: Empty service gid');
            $this->set_api_content('errors', l('error_service_code_incorrect', 'services'));

            return false;
        }

        $service = $this->Services_model->get_service_by_gid($service_gid);
        if (empty($service) || !$service['status']) {
            log_message('error', 'services API: Empty service gid');
            $this->set_api_content('errors', l('error_service_code_incorrect', 'services'));

            return false;
        }

        $service = current($this->Services_model->format_service(array($service)));
        $service["template"] = $this->Services_model->format_template($service["template"]);

        if (!empty($service["data_admin_array"])) {
            foreach ($service["template"]["data_admin_array"] as $gid => $temp) {
                if (!empty($service["data_admin_array"][$gid])) {
                    $service["template"]["data_admin_array"][$gid]["value"] = $service["data_admin_array"][$gid];
                }
            }
        }

        if ($service["template"]["price_type"] == "2" || $service["template"]["price_type"] == "3") {
            $service["price"] = $this->input->post('price', true);
        }

        $user_id = $this->session->userdata('user_id');

        $user_form_data = $this->input->post("data_user", true);

        $service_return = $this->Services_model->validate_service_payment($service["id"], $user_form_data,
                                                                          $service["price"]);
        if (!empty($service_return["errors"])) {
            $this->set_api_content('errors', $service_return["errors"]);
        } else {
            $origin_return = $this->Services_model->validate_service_original_model($service["id"], $user_form_data,
                                                                                    $user_id, $service["price"]);
            if (!empty($origin_return["errors"])) {
                $this->set_api_content('errors', $origin_return["errors"]);
            } else {
                $base_currency_gid = $this->pg_module->get_module_config("start", "base_currency");

                $this->load->model('Currencies_model');
                $base_currency = $this->Currencies_model->get_currency_default(true);

                $this->load->helper('start');
                $service["price"] = get_currency_value(array('value' => $service["price"], 'cur_gid' => $base_currency['gid'], 'to_gid' => $base_currency_gid));

                $return = $this->Services_model->account_payment($service["id"], $user_id, $user_form_data,
                                                                 $service["price"]);
                if ($return !== true) {
                    $this->set_api_content('errors', $return);
                } else {
                    $this->set_api_content('success', l('success_services_apply', 'services'));
                }
            }
        }
    }

    /**
     * Pay for service by billing system
     *
     * @param string $service_gid service guid
     *
     * @return void
     */
    public function system_payment($service_gid)
    {
        if (!$service_gid) {
            log_message('error', 'services API: Empty service gid');
            $this->set_api_content('errors', l('error_service_code_incorrect', 'services'));
            exit;
        }

        $service = $this->Services_model->get_service_by_gid($service_gid);
        if (empty($service) || !$service['status']) {
            log_message('error', 'services API: Empty service gid');
            $this->set_api_content('errors', l('error_service_code_incorrect', 'services'));
            exit;
        }

        $system_gid = $this->input->post('system_gid', true);
        if (empty($system_gid)) {
            $this->set_api_content('errors', l('error_select_payment_system', 'services'));
            exit;
        }

        $service = array_shift($this->Services_model->format_service(array($service)));
        $service["template"] = $this->Services_model->format_template($service["template"]);

        if (!empty($service["data_admin_array"])) {
            foreach ($service["template"]["data_admin_array"] as $gid => $temp) {
                if (!empty($service["data_admin_array"][$gid])) {
                    $service["template"]["data_admin_array"][$gid]["value"] = $service["data_admin_array"][$gid];
                }
            }
        }

        if ($service["template"]["price_type"] == "2" || $service["template"]["price_type"] == "3") {
            $service["price"] = $this->input->post('price', true);
        }

        $user_id = $this->session->userdata('user_id');
        $user_form_data = $this->input->post("data_user", true);
        $payment_id = 0;

        $service_return = $this->Services_model->validate_service_payment($service["id"], $user_form_data,
                                                                          $service["price"]);
        if (!empty($service_return["errors"])) {
            $this->set_api_content('errors', $service_return["errors"]);
        } else {
            $origin_return = $this->Services_model->validate_service_original_model($service["id"], $user_form_data,
                                                                                    $user_id, $service["price"]);
            if (!empty($origin_return["errors"])) {
                $this->set_api_content('errors', $origin_return["errors"]);
            } else {
                $price = $service['price'];

                $this->load->model('Currencies_model');
                $default_currency = $this->Currencies_model->get_currency_default(true);

                //$this->load->helper('payments');
                //$price = currency_value(array('value'=>$price, 'cur_gid'=>$default_currency['gid']));

                $payment_data["name"] = l('service_payment', 'services') . $service["name"];
                $payment_data["id_service"] = $service['id'];
                $payment_data["user_data"] = $user_form_data;

                $base_currency_gid = $this->pg_module->get_module_config('start', 'base_currency');

                $this->load->model('payments/models/Payment_systems_model');
                if ($this->Payment_systems_model->action_html($system_gid)) {
                    
                } elseif ($this->Payment_systems_model->action_js($system_gid)) {
                    $save_data = array(
                        "payment_type_gid" => 'services',
                        "id_user" => $user_id,
                        "amount" => $price,
                        "currency_gid" => $base_currency_gid,
                        "system_gid" => $system_gid,
                        "payment_data" => $payment_data,
                    );
                    $this->load->model('Payments_model');
                    $validate_data = $this->Payments_model->validate_payment(null, $save_data);
                    if (!empty($validate_data["errors"])) {
                        $this->set_api_content('errors', $validate_data["errors"]);
                    } else {
                        $payment_id = $this->Payments_model->save_payment(null, $validate_data['data']);
                    }
                } else {
                    $for_validate = array(
                        "payment_type_gid" => 'services',
                        "id_user" => $user_id,
                        "amount" => $price,
                        "currency_gid" => $base_currency_gid,
                        "system_gid" => $system_gid,
                        "payment_data" => $payment_data,
                    );
                    $this->load->model('Payments_model');
                    $validate_data = $this->Payments_model->validate_payment(null, $for_validate);
                    if (!empty($validate_data["errors"])) {
                        $this->set_api_content("errors", $validate_data["errors"]);
                    } else {
                        $payment_id = $this->Payments_model->save_payment(null, $validate_data["data"]);
                    }
                }
            }
        }

        $this->set_api_content('data',
                               array('price' => $service['price'], 'system_gid' => $system_gid, 'payment_id' => $payment_id));
    }

}
