<?php

namespace Pg\Modules\Spam\Controllers;

/**
 * Spam networking module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
use Pg\Libraries\View;

/**
 * Spam admin side controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	Spam
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Admin_Spam extends \Controller
{

    /**
     * Constructor
     *
     * @return Admin_Spam
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model("Menu_model");
        $this->Menu_model->set_menu_active_item("admin_menu", "feedbacks-items");
        $this->view->setHeader(l("admin_header_spam", "spam"));
    }

    /**
     * Spam alerts manangement
     *
     * @param string  $filter          filter by type
     * @param string  $order           sorting order
     * @param string  $order_direction sorting direction
     * @param integer $page            page of results
     *
     * @return void
     */
    public function index($filter = "", $order = null, $order_direction = null, $page = 1)
    {
        $this->load->model("spam/models/Spam_alert_model");
        $this->load->model("spam/models/Spam_type_model");

        $this->Menu_model->set_menu_active_item("admin_spam_menu", "spam_alerts_item");

        $spam_types = $this->Spam_type_model->get_types(1);
        $types_count = count($spam_types);
        if (!$types_count) {
            $this->view->render("alerts_list");

            return;
        }

        $current_settings = isset($_SESSION["spam_alerts_list"]) ? $_SESSION["spam_alerts_list"] : array();
        if (!isset($current_settings["filter"])) {
            $type = current($spam_types);
            $current_settings["filter"] = $type["gid"];
        }
        if (!isset($current_settings["order"])) {
            $current_settings["order"] = "date_add";
        }
        if (!isset($current_settings["order_direction"])) {
            $current_settings["order_direction"] = "DESC";
        }
        if (!$filter) {
            $filter = $current_settings["filter"];
        }
        if (!$order) {
            $order = $current_settings["order"];
        }
        if (!$order_direction) {
            $order_direction = $current_settings["order_direction"];
        }

        $current_settings["filter"] = $filter;
        $current_settings["order"] = $order;
        $current_settings["order_direction"] = $order_direction;
        $current_settings["page"] = $page;

        $this->view->assign("order", $order);
        $this->view->assign("order_direction", $order_direction);

        $alerts_count = $this->Spam_alert_model->get_alerts_count($filter);

        if (!$page) {
            $page = $current_settings["page"];
        }
        $items_on_page = $this->pg_module->get_module_config("start", "admin_items_per_page");

        $this->load->helper("sort_order");
        $page = get_exists_page_number($page, $alerts_count, $items_on_page);
        $current_settings["page"] = $page;

        $sort_links = array(
            "poster" => site_url() . "admin/spam/index/" . $filter . "/poster/" . (($order != "poster" xor $order_direction == "DESC") ? "ASC" : "DESC"),
            //"spam_status" => site_url() . "admin/spam/index/".$filter."/spam_status/".(($order != "spam_status" xor $order_direction == "DESC") ? "ASC" : "DESC"),
            "date_add" => site_url() . "admin/spam/index/" . $filter . "/date_add/" . (($order != "date_add" xor $order_direction == "DESC") ? "ASC" : "DESC"),
        );

        $this->view->assign("sort_links", $sort_links);

        $_SESSION["spam_alerts_list"] = $current_settings;

        if ($alerts_count > 0) {
            $this->Spam_alert_model->set_format_settings(array("get_reason" => true));
            if ($order == "poster") {
                $order = "id_poster";
            }
            $alerts = $this->Spam_alert_model->get_alerts_list($page, $items_on_page, array($order => $order_direction),
                                                               $filter);
            $this->view->assign("alerts", $alerts);
        }

        $this->load->helper('navigation');

        $url = site_url() . "admin/spam/index/" . $filter . "/" . $order . "/" . $order_direction . "/";
        $page_data = get_admin_pages_data($url, $alerts_count, $items_on_page, $page, "briefPage");
        $page_data["date_format"] = $this->pg_date->get_format('date_time_literal', 'st');
        $this->view->assign("page_data", $page_data);

        $this->view->assign("spam_types", $spam_types);
        $this->view->assign("spam_types_count", $types_count);

        $this->view->assign("filter", $filter);

        $this->Menu_model->set_menu_active_item("admin_spam_menu", "spam_alerts_item");
        $this->view->render("alerts_list");
    }

    /**
     * Display spam alert data
     *
     * @param integer $id alert identifier
     *
     * @return void
     */
    public function alerts_show($id)
    {
        $this->load->model("spam/models/Spam_alert_model");
        $this->Spam_alert_model->set_format_settings(array("get_link" => true, "get_object" => true, "get_reason" => true));
        $data = $this->Spam_alert_model->get_alert_by_id($id, true);
        $this->view->assign("data", $data);

        $this->Spam_alert_model->mark_alert_as_read($id);

        $this->load->helper('navigation');

        $date_format = $this->pg_date->get_format('date_time_literal', 'st');
        $this->view->assign("date_format", $date_format);
        $this->view->setHeader(l("admin_header_alerts_show", "spam"));
        $this->view->render("alerts_view");
    }

    /**
     * Remove spam alert
     *
     * @param string  $action action name
     * @param integer $ids    alerts identifiers
     *
     * @return void
     */
    public function alerts_delete($action = "without_object", $ids = null)
    {
        $errors = false;
        $messages = array();
        if (!$ids) {
            $ids = $this->input->post("ids");
        }

        if (!empty($ids)) {
            $this->load->model("spam/models/Spam_alert_model");
            foreach ((array) $ids as $id) {
                $error = $this->Spam_alert_model->delete_alert($id, ($action == "with_object"));
                if ($error) {
                    $errors = true;
                    $messages[] = $error;
                } else {
                    $messages[] = l("success_deleted_alert", "spam");
                }
            }
            if ($errors) {
                $this->system_messages->addMessage(View::MSG_ERROR, $messages);
            } else {
                $this->system_messages->addMessage(View::MSG_SUCCESS, $messages);
            }
        }
        $current_settings = $_SESSION["spam_alerts_list"];
        redirect(site_url() . "admin/spam/index");
    }

    /**
     * Ban objects linked with spam alerts
     *
     * @param integer/array $ids spam alerts identifiers
     *
     * @return void
     */
    public function alerts_ban($ids = null)
    {
        $errors = false;
        $messages = array();
        if (!$ids) {
            $ids = $this->input->post("ids");
        }
        if (!empty($ids)) {
            $this->load->model("spam/models/Spam_alert_model");
            foreach ((array) $ids as $id) {
                $data = $this->Spam_alert_model->ban_alert($id);
                if (is_array($data)) {
                    $messages[] = l("success_" . $data["spam_status"] . "_alert", "spam");
                } else {
                    $errors = true;
                    $messages[] = $data;
                }
            }
            if ($errors) {
                $this->system_messages->addMessage(View::MSG_ERROR, $messages);
            } else {
                $this->system_messages->addMessage(View::MSG_SUCCESS, $messages);
            }
        }
        redirect(site_url() . "admin/spam/index");
    }

    /**
     * Unban objects linked with spam alert
     *
     * @param integer/array $ids spam alerts identifiers
     *
     * @return void
     */
    public function alerts_unban($ids = null)
    {
        $errors = false;
        $messages = array();
        if (!$ids) {
            $ids = $this->input->post("ids");
        }
        if (!empty($ids)) {
            $this->load->model("spam/models/Spam_alert_model");
            foreach ((array) $ids as $id) {
                try {
                    $status = $this->Spam_alert_model->unban_alert($id);
                    $messages[] = l("success_" . $status . "_alert", "spam");
                } catch (Exception $e) {
                    $errors = true;
                    $messages[] = $e->getMessage();
                }
            }
            if ($errors) {
                $this->system_messages->addMessage(View::MSG_ERROR, $messages);
            } else {
                $this->system_messages->addMessage(View::MSG_SUCCESS, $messages);
            }
        }
        $current_settings = $_SESSION["spam_alerts_list"];
        redirect(site_url() . "admin/spam/index");
    }

    /**
     * Spam types management
     *
     * @return void
     */
    public function types()
    {
        $this->load->model("spam/models/Spam_type_model");

        $types = $this->Spam_type_model->get_types();
        $this->view->assign("types", $types);

        $this->Menu_model->set_menu_active_item("admin_spam_menu", "spam_types_item");
        $this->Menu_model->set_menu_active_item("admin_menu", "system-items");
        $this->view->render("types_list");
    }

    /**
     * Edit spam type data
     *
     * @param integer $type_id spam type identifier
     *
     * @return void
     */
    public function types_edit($type_id)
    {
        $this->load->model("spam/models/Spam_type_model");

        $data = $this->Spam_type_model->get_type_by_id($type_id);

        if ($this->input->post("btn_save")) {
            $post_data = $this->input->post("data", true);
            $validate_data = $this->Spam_type_model->validate_type($type_id, $post_data);
            if (!empty($validate_data["errors"])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
            } else {
                $this->Spam_type_model->save_type($type_id, $validate_data["data"]);
                $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_updated_type", "spam"));
                $url = site_url() . "admin/spam/types";
                redirect($url);
            }

            $data = array_merge($data, $post_data);
        }

        $this->view->assign("data", $data);
        $this->view->assign("form_type_lang", ld("form_type", "spam"));

        $date_format = $this->pg_date->get_format('date_time_literal', 'st');
        $this->view->assign("date_format", $date_format);

        $this->view->setHeader(l("admin_header_types_edit", "spam"));
        $this->Menu_model->set_menu_active_item("admin_spam_menu", "spam_types_item");
        $this->Menu_model->set_menu_active_item("admin_menu", "system-items");
        $this->view->render("types_edit");
    }

    /**
     * Activate/deactivate spam type
     *
     * Available status values: 1 - activate, 0 - deactivate
     *
     * @param integer $type_id spam type identifier
     * @param integer $status  active status
     *
     * @return void
     */
    public function type_activate($type_id, $status = 0)
    {
        $this->load->model("spam/models/Spam_type_model");
        if (!empty($type_id)) {
            if ($status) {
                $this->Spam_type_model->activate_type($type_id, $status);
                $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_type_activated", "spam"));
            } else {
                $this->Spam_type_model->activate_type($type_id, $status);
                $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_type_deactivated", "spam"));
            }
        }
        $url = site_url() . "admin/spam/types/";
        redirect($url);
    }

    /**
     * Turn on/off send mail for spam type
     *
     * @param integer $type_id spam type identifier
     * @param integer $status  on/off status
     *
     * @return void
     */
    public function type_send_mail($type_id, $status = 0)
    {
        $this->load->model("spam/models/Spam_type_model");
        if (!empty($type_id)) {
            if ($status) {
                $this->Spam_type_model->send_mail_type($type_id, $status);
                $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_type_send_form_on", "spam"));
            } else {
                $this->Spam_type_model->send_mail_type($type_id, $status);
                $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_type_send_form_off", "spam"));
            }
        }
        $url = site_url() . "admin/spam/types/";
        redirect($url);
    }

    /**
     * Spam reasons management
     *
     * @param integer $lang_id language identifier
     *
     * @return void
     */
    public function reasons($lang_id = null)
    {
        $this->load->model("spam/models/Spam_reason_model");

        if (!$lang_id || !array_key_exists($lang_id, $this->pg_language->languages)) {
            $lang_id = $this->pg_language->current_lang_id;
        }

        $reference = $this->pg_language->ds->get_reference($this->Spam_reason_model->module_gid,
                                                           $this->Spam_reason_model->content[0], $lang_id);

        $this->view->assign("langs", $this->pg_language->languages);
        $this->view->assign("current_lang_id", $lang_id);
        $this->view->assign("reference", $reference);
        $this->view->assign("module_gid", $this->Spam_reason_model->module_gid);

        $this->Menu_model->set_menu_active_item("admin_spam_menu", "spam_reasons_item");
        $this->Menu_model->set_menu_active_item("admin_menu", "system-items");
        $this->view->render("reasons_list");
    }

    /**
     * Edit reason data
     *
     * @param integer $lang_id    language identifier
     * @param string  $option_gid option guid
     *
     * @return void
     */
    public function reasons_edit($lang_id = null, $option_gid = null)
    {
        $this->load->model("spam/models/Spam_reason_model");

        if (!$lang_id || !array_key_exists($lang_id, $this->pg_language->languages)) {
            $lang_id = $this->pg_language->current_lang_id;
        }

        $reference = $this->pg_language->ds->get_reference($this->Spam_reason_model->module_gid,
                                                           $this->Spam_reason_model->content[0], $lang_id);
        if ($option_gid) {
            $add_flag = false;
            foreach ($this->pg_language->languages as $lid => $lang) {
                $r = $this->pg_language->ds->get_reference($this->Spam_reason_model->module_gid,
                                                           $this->Spam_reason_model->content[0], $lid);
                $lang_data[$lid] = $r["option"][$option_gid];
            }
        } else {
            $option_gid = "";
            $lang_data = array();
            $add_flag = true;
        }

        if ($this->input->post("btn_save")) {
            $lang_data = $this->input->post("lang_data", true);

            $validate_data = $this->Spam_reason_model->validate_reason($option_gid, $lang_data);
            if (!empty($validate_data["errors"])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
            } else {
                $this->Spam_reason_model->save_reason($option_gid, $validate_data["langs"]);
                if ($add_flag) {
                    $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_created_reason", "spam"));
                } else {
                    $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_updated_reason", "spam"));
                }
                $url = site_url() . "admin/spam/reasons/" . $lang_id;
                redirect($url);
            }
        }

        $this->view->assign("lang_data", $lang_data);
        $this->view->assign("langs", $this->pg_language->languages);
        $this->view->assign("current_lang_id", $lang_id);
        $this->view->assign("current_module_gid", $this->Spam_reason_model->module_gid);
        $this->view->assign("option_gid", $option_gid);
        $this->view->assign("add_flag", $add_flag);

        $this->Menu_model->set_menu_active_item("admin_spam_menu", "spam_reasons_item");
        $this->Menu_model->set_menu_active_item("admin_menu", "system-items");

        $this->view->setHeader(l("admin_header_reasons_edit", "spam"));

        $this->view->render("reasons_edit");
    }

    /**
     * Resort reasons options by ajax
     *
     * @return void
     */
    public function ajax_reasons_save_sorter()
    {
        $this->load->model("spam/models/Spam_reason_model");

        $sorter = $this->input->post("sorter");
        foreach ($sorter as $parent_str => $items_data) {
            foreach ($items_data as $item_str => $sort_index) {
                $option_gid = str_replace("item_", "", $item_str);
                $sorter_data[$sort_index] = $option_gid;
            }
        }

        if (empty($sorter_data)) {
            return;
        }
        ksort($sorter_data);
        $this->pg_language->ds->set_reference_sorter($this->Spam_reason_model->module_gid,
                                                     $this->Spam_reason_model->content[0], $sorter_data);

        return;
    }

    /**
     * Remove spam reason by ajax
     *
     * @param string $option_gid option GUID
     *
     * @return void
     */
    public function ajax_reasons_delete($option_gid)
    {
        $this->load->model("spam/models/Spam_reason_model");

        if ($option_gid) {
            foreach ($this->pg_language->languages as $lid => $lang) {
                $reference = $this->pg_language->ds->get_reference($this->Spam_reason_model->module_gid,
                                                                   $this->Spam_reason_model->content[0], $lid);
                if (isset($reference["option"][$option_gid])) {
                    unset($reference["option"][$option_gid]);
                    $this->pg_language->ds->set_module_reference($this->Spam_reason_model->module_gid,
                                                                 $this->Spam_reason_model->content[0], $reference, $lid);
                }
            }
        }

        return;
    }

    /**
     * Manage module settings
     *
     * @return void
     */
    public function settings()
    {
        $this->Menu_model->set_menu_active_item("admin_menu", "system-items");
        $this->Menu_model->set_menu_active_item("admin_spam_menu", "spam_settings_item");

        if ($this->input->post("btn_save")) {
            $post_data["send_alert_to_email"] = $this->input->post("send_alert_to_email");

            $this->load->model("spam/models/Spam_type_model");
            $validate_data = $this->Spam_type_model->validate_settings($post_data);
            if (!empty($validate_data["errors"])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
                $data = $validate_data["data"];
            } else {
                foreach ($validate_data["data"] as $setting => $value) {
                    $this->pg_module->set_module_config("spam", $setting, $value);
                }
                $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_settings_saved", "users"));
                $data = $validate_data["data"];
            }
        } else {
            $data["send_alert_to_email"] = $this->pg_module->get_module_config("spam", "send_alert_to_email");
        }

        $this->view->assign("data", $data);
        $this->view->render("settings");
    }

}
