<?php

namespace Pg\Modules\Spam\Models;

/**
 * Spam networking module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Spam Reason Model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Spam
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Spam_reason_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Datasource references
     *
     * @var array
     */
    public $content = array("spam_object");

    /**
     * Module GUID
     *
     * @var string
     */
    public $module_gid = "spam";

    /**
     * Constructor
     *
     * @return Spam_reason_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
    }

    /**
     * Return reason object from data source
     *
     * @param integer $lang_id language identifier
     *
     * @return string
     */
    public function get_reason($lang_id = null)
    {
        if (!$lang_id) {
            $lang_id = $this->ci->session->userdata("lang_id");
        }

        return $this->ci->pg_language->ds->get_reference($this->module_gid, "spam_object", $lang_id);
    }

    /**
     * Validate reason data for saving to data source
     *
     * @param string $option_gid option guid
     * @param array  $langs      data by languages
     *
     * @return array
     */
    public function validate_reason($option_gid, $langs)
    {
        $return = array("errors" => array(), 'langs' => array());

        if (!empty($langs)) {
            foreach ($this->ci->pg_language->languages as $lid => $lang_data) {
                if (!isset($langs[$lid])) {
                    $return['errors'][] = l('error_empty_reason_name', "spam");
                    break;
                } else {
                    $return["langs"][$lid] = trim(strip_tags($langs[$lid]));
                    if (empty($return["langs"][$lid])) {
                        $return['errors'][] = l('error_empty_reason_name', "spam");
                        break;
                    }
                }
            }
        }

        return $return;
    }

    /**
     * Save reason data to data source
     *
     * @param string $option_gid option guid
     * @param array  $langs_data data by languages
     *
     * @return array
     */
    public function save_reason($option_gid, $langs_data)
    {
        if (empty($option_gid)) {
            $reference = $this->ci->pg_language->ds->get_reference($this->module_gid, $this->content[0],
                                                                   $this->ci->pg_language->current_lang_id);
            if (!empty($reference["option"])) {
                $array_keys = array_keys($reference["option"]);
            } else {
                $array_keys = array(0);
            }
            $option_gid = max($array_keys) + 1;
        }

        foreach ($langs_data as $lid => $string) {
            $reference = $this->ci->pg_language->ds->get_reference($this->module_gid, $this->content[0], $lid);
            $reference["option"][$option_gid] = $string;
            $this->ci->pg_language->ds->set_module_reference($this->module_gid, $this->content[0], $reference, $lid);
        }
    }

}
