<?php

/**
 * Start module
 *
 * @package     PG_RealEstate
 *
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
/**
 * Start management
 *
 * @package     PG_RealEstate
 * @subpackage  Start
 *
 * @category    helpers
 *
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

if (!function_exists('demo_panel')) {

    /**
     * Display demo panel
     *
     * @param array $params
     *
     * @return void
     */
    function demo_panel($params)
    {
        if (!empty($_ENV['DEMO_MODE']) || !empty($_ENV['TRIAL_MODE'])) {
            $ci = &get_instance();
            $ci->view->assign('place', $params['place']);
            return $ci->view->fetch('demo_panel', null, 'start');
        }
    }

}

if (!function_exists('product_version')) {

    /**
     * Display product version
     *
     * @return void
     */
    function product_version()
    {
        if (INSTALL_MODULE_DONE) {
            $ci = &get_instance();

            if ($ci->pg_module->is_module_active('start')) {
                $current_version_code = $ci->pg_module->get_module_config('start', 'product_version_code');
                $current_version_name = $ci->pg_module->get_module_config('start', 'product_version_name');
                $formated_current_version = str_replace('_', '.', $current_version_name);

                $cache_new_version_code = $ci->pg_module->get_module_config('start', 'product_version_code_update');
                $cache_new_version_name = $ci->pg_module->get_module_config('start', 'product_version_name_update');

                $last_update = $ci->pg_module->get_module_config('start', 'product_version_last_update');
                if ((!$cache_new_version_code || $cache_new_version_code == $current_version_code) &&
                    (!$last_update || (time() - strtotime($last_update) > 24 * 60 * 60))) {
                    try {
                        $new_version = get_new_version();
                        $cache_new_version_code = intval($new_version['code']);
                        $cache_new_version_name = $new_version['name'];
                        $ci->pg_module->set_module_config('start', 'product_version_last_update', date('Y-m-d H:i:s'));
                    } catch (Exception $e) {

                    }
                }

                if ($cache_new_version_code && $current_version_code < $cache_new_version_code) {
                    $formated_new_version = str_replace('_', '.', $cache_new_version_name);
                } else {
                    $formated_new_version = '';
                }

                $html = l('system_version', 'start') . ": " . $formated_current_version . " ";
                if ($formated_new_version) {
                    $html .= str_replace('[version]', $formated_new_version, l('system_version_available', 'start'));
                }
                return $html;
            }
        }
    }

}

if (!function_exists('get_new_version')) {

    /**
     * Get last product version
     *
     * @param string $current_version current product version
     *
     * @return string
     */
    function get_new_version()
    {
        $ci = &get_instance();
        $ci->load->library('Snoopy');
        $ci->snoopy->read_timeout = 5;
        $ci->snoopy->rawheaders["Accept"] = "application/json";
        $ci->snoopy->fetch('http://www.pilotgroup.net/feeder/realestate/version.php');
        if ($ci->snoopy->status == '200') {
            $new_version = (array) json_decode($ci->snoopy->results);
            $ci->pg_module->set_module_config('start', 'product_version_code_update', $new_version['code']);
            $ci->pg_module->set_module_config('start', 'product_version_name_update', $new_version['name']);

            return $new_version;
        } else {
            throw new Exception('error');
        }
    }

}

if (!function_exists("main_search_form")) {

    /**
     * Display search form for index page
     *
     * @param string  $object    object name
     * @param string  $type      render block type
     * @param boolean $show_data filled selected data
     * @param integer $width     block size
     *
     * @return string
     */
    function main_search_form($object = '', $type = 'line', $show_data = false, $width = 100, $tabs_type = 'select')
    {
        $ci = &get_instance();
        if (func_num_args() == 1 && is_array($object)) {
            $params = $object;
            $object = isset($params['object']) ? $params['object'] : '';
            $type = isset($params['type']) ? $params['type'] : 'line';
            $show_data = isset($params['show_data']) ? $params['show_data'] : true;
            $width = isset($params['width']) ? $params['width'] : 100;
        }

        $tabs = array();
        $show_tabs = false;
        $modules = array();

        if ($type != 'line') {
            $search_controls = main_search_controls();
            $tabs = $search_controls['tabs'];
            $modules = $search_controls['modules'];
            if (count($tabs) > 0) {
                $show_tabs = true;
                if (!$object || !in_array($object, $tabs)) {
                    $object = key($tabs);
                }
            }
        }

        if ($ci->pg_module->is_module_active('listings')) {
            $ci->load->helper('listings');
            $ci->load->model('Listings_model');
            $operation_types = $ci->Listings_model->get_operation_types(true);
            if ($type == 'line' || in_array($object, $operation_types)) {
                $form_block = listings_main_search_form($type, $show_data, $width, $object);
            }
        }

        $is_idxbroker_installed = $type != 'line' && $type != 'slider' && $ci->pg_module->is_module_active('idxbroker');
        if ($object === 'idxbroker' && $is_idxbroker_installed) {
            $ci->load->helper('idxbroker');
            $form_block = idxbroker_main_search_form($type, $show_data, $width);
        }
        if (!$form_block) {
            return '';
        }
        $ci->view->assign('form_settings', array(
            'show_tabs' => $show_tabs,
            'object' => $object,
            'type' => $type,
            'tabs' => $tabs,
            'rand' => rand(100000, 999999),
        ));
        $ci->view->assign('form_block', $form_block);
        $ci->view->assign('modules', $modules);
        $ci->view->assign('show_data', $show_data);
        $ci->view->assign('tabs_type', $tabs_type);
        return $ci->view->fetch('helper_search_form', 'user', 'start');
    }

}

if (!function_exists('main_search_controls')) {

    function main_search_controls()
    {
        $ci = &get_instance();
        $return = [
            'modules' => [],
            'tabs' => [],
        ];
        if ($ci->pg_module->is_module_active('listings')) {
            $ci->load->helper('listings');
            $more_tabs = listings_main_search_tabs();
            if (!empty($more_tabs)) {
                $return['tabs'] = array_merge($return['tabs'], $more_tabs);
                $keys = array_keys($more_tabs);
                $values = array_fill(0, count($more_tabs), 'listings');
                $return['modules'] = array_merge($return['modules'], array_combine($keys, $values));
            }
        }

        if ($ci->pg_module->is_module_active('idxbroker')) {
            $ci->load->helper('idxbroker');
            $more_tabs = idxbroker_main_search_tabs();
            if (!empty($more_tabs)) {
                $return['tabs'] = array_merge($return['tabs'], $more_tabs);
                $keys = array_keys($more_tabs);
                $values = array_fill(0, count($more_tabs), 'idxbroker');
                $return['modules'] = array_merge($return['modules'], array_combine($keys, $values));
            }
        }

        return $return;
    }

}

if (!function_exists('selectbox')) {

    /**
     * Get custom drop down block
     *
     * @param array $params block parameters
     *
     * @return string
     */
    function selectbox($params)
    {
        $ci = &get_instance();
        foreach (array(
        'input',
        'id',
        'class',
        'selected',
        'default',
        'value',
        'subvalue',) as $key) {
            if (!isset($params[$key])) {
                $params[$key] = '';
            }
        }
        if (is_array($params['selected'])) {
            $params['selected'] = json_encode($params['selected']);
        }
        foreach ($params as $key => $value) {
            $ci->view->assign('sb_' . $key, $value);
        }
        if (empty($params['subvalue'])) {
            $ci->view->assign('sb_subvalue', '');
        }
        return $ci->view->fetch('helper_selectbox', 'user', 'start');
    }

}

if (!function_exists('checkbox')) {

    /**
     * Get custom checkbox block
     *
     * @param array $params block parameters
     *
     * @return string
     */
    function checkbox($params)
    {
        if (empty($params['value'])) {
            return '';
        }

        $ci = &get_instance();

        $values = array();
        $selected = (!empty($params['selected'])) ? $params['selected'] : array();
        if (!is_array($selected)) {
            $selected = array($selected);
        }

        if (!is_array($params['value'])) {
            $params['value'] = array(1 => $params['value']);
        }

        foreach ($params['value'] as $key => $value) {
            $values[$key] = array('name' => $value, 'checked' => (in_array($key, $selected)) ? 1 : 0);
        }

        $params['value'] = $values;

        $cb_count = count($params['value']);
        $ci->view->assign("cb_count", $cb_count);

        unset($params['selected']);

        foreach ($params as $key => $value) {
            $ci->view->assign("cb_" . $key, $value);
        }

        return $ci->view->fetch("helper_checkbox", 'user', 'start');
    }

}

if (!function_exists('pagination')) {

    /**
     * Get pagination block
     *
     * @param array $params block parameters
     *
     * @return string
     */
    function pagination($params, $page_type='full')
    {
        $ci = &get_instance();

        if (!isset($params['total_rows']) || $params['total_rows'] < 2) {
            return false;
        }

        $ci->view->assign('page_type', $page_type);

        foreach ($params as $key => $value) {
            if ($key == 'type' && !in_array($value, ['cute', 'full'])) {
                continue;
            }
            $ci->view->assign("page_" . $key, $value);
        }
        
        return $ci->view->fetch("helper_pagination", 'user', 'start');
    }

}

if (!function_exists('pagination_h')) {

    /**
     * Get horizontal pagination block
     *
     * @param array $params block parameters
     *
     * @return string
     */
    function pagination_h($params)
    {
        $ci = &get_instance();
        if (empty($params['container'])) {
            return '';
        }

        $pagination = array('active' => 1, 'total' => 1);

        if (!empty($params['items_total'])) {
            $items_total = $params['items_total'];
        } else {
            $items_total = 1;
        }

        if (!empty($params['items_per_page'])) {
            $items_per_page = $params['items_per_page'];
        } else {
            $items_per_page = 1;
        }

        $pagination['total'] = ceil($items_total / $items_per_page);

        $ci->view->assign('pagination', $pagination);
        $ci->view->assign('ph_container', $params['container']);
        $ci->view->assign('ph_items_per_page', $items_per_page);
        $ci->view->assign('ph_items_total', $items_total);

        return $ci->view->fetch('helper_pagination_h', 'user', 'start');
    }

}

if (!function_exists('sorter')) {

    /**
     * Get list sorter block
     *
     * @param array $params block parameters
     *
     * @return string
     */
    function sorter($params)
    {
        $ci = &get_instance();
        if (empty($params["module"])) {
            $params["module"] = "start";
        }

        $params["rand"] = rand(0, 9999);
        foreach ($params as $key => $value) {
            $ci->view->assign("sort_" . $key, $value);
        }

        return $ci->view->fetch("helper_sorter", 'user', 'start');
    }

}

if (!function_exists('available_browsers')) {

    /**
     * Check browser version is old
     *
     * @param array $params block parameters
     *
     * @return void
     */
    function available_browsers()
    {
        $ci = &get_instance();
        return $ci->view->fetch("available_browsers", 'user');
    }

}

if (!function_exists('lang_inline_editor')) {

    /**
     * Get language editor block
     *
     * @param array $params block parameters
     *
     * @return string
     */
    function lang_inline_editor($params = [])
    {
        $ci = &get_instance();
        if (isset($params['textarea']) && $params['textarea']) {
            $ci->view->assign('textarea', true);
        }
        $ci->view->assign('rand', rand(100000, 999999));

        return $ci->view->fetch("helper_lang_inline_editor_js", null, 'start');
    }

}

if (!function_exists('get_phone_format')) {

    /**
     * Get phone format
     *
     * @param string $type format type
     *
     * @return string
     */
    function get_phone_format($type = 'js')
    {
        $ci = &get_instance();
        $use_phone_format = (bool) $ci->pg_module->get_module_config("start", "use_phone_format");
        if ($use_phone_format) {
            $format = $ci->pg_module->get_module_config("start", "phone_format");
            if ($type == 'regexp') {
                $format = prepare_regexp_phone_format($format);
            }
        } else {
            $format = false;
        }

        return $format;
    }

}
if (!function_exists('prepare_regexp_phone_format')) {

    /**
     * Prepare masked phone string
     *
     * @param string $format phone format
     *
     * @return string
     */
    function prepare_regexp_phone_format($format)
    {
        $format = preg_quote($format);
        for ($i = 20; $i > 0; --$i) {
            $str = str_repeat('9', $i);
            $format = str_replace($str, '[0-__NINE__]{' . ($i == 9 ? '__NINE__' : $i) . '}', $format);
        }
        $format = "/" . str_replace('__NINE__', '9', $format) . "/iU";

        return $format;
    }

}

if (!function_exists('currency_format_output')) {

    /**
     * Return formatted currency string
     *
     * Or unformatted if payments is not installed
     *
     * int $params['cur_id'] currency id
     * string $params['cur_gid'] currency gid
     * int $params['value'] amount
     * string $params['template']&nbsp;[abbr][value|dec_part:2|dec_sep:.|gr_sep:&nbsp;]
     *
     * @param array $params block parameters
     *
     * @return string
     */
    function currency_format_output($params = array())
    {
        $ci = &get_instance();
        static $is_currencies_installed = null;
        if (is_null($is_currencies_installed)) {
            $is_currencies_installed = $ci->pg_module->is_module_active('currencies');
        }
        if ($is_currencies_installed) {
            $ci->load->helper('currencies');

            return currency_format($params);
        } else {
            $base_currency_gid = $ci->pg_module->get_module_config("start", "base_currency");

            return '<span dir="ltr">' . trim($params['value']) . '&nbsp;' . $base_currency_gid . '</span>';
        }
    }

}

if (!function_exists('currency_output')) {

    /**
     * Return unformatted currency string
     *
     * Or unformatted if payments is not installed
     *
     * int $params['cur_id'] currency id
     * string $params['cur_gid'] currency gid
     * int $params['value'] amount
     * string $params['template']&nbsp;[abbr][value|dec_part:2|dec_sep:.|gr_sep:&nbsp;]
     *
     * @param array $params block parameters
     *
     * @return string
     */
    function currency_output($params = array())
    {
        $ci = &get_instance();

        static $is_currencies_installed = null;
        if (is_null($is_currencies_installed)) {
            $is_currencies_installed = $ci->pg_module->is_module_active('currencies');
        }
        if ($is_currencies_installed) {
            $ci->load->helper('currencies');

            return currency($params);
        } else {
            if (!empty($params['editable'])) {
                $base_currency_gid = $ci->pg_module->get_module_config("start", "base_currency");
                $ci->view->assign('base_currency_gid', $base_currency_gid);

                $ci->view->assign('block_data', $params);

                return $ci->view->fetch('helper_currency_output', null, 'start');
            } else {
                $base_currency_gid = $ci->pg_module->get_module_config("start", "base_currency");

                return '<span dir="ltr">' . $params['value'] . '&nbsp;' . $base_currency_gid . '</span>';
            }
        }
    }

}

if (!function_exists('currency_format_regexp_output')) {

    /**
     * Return formatted currency string
     *
     * Or unformatted if payments is not installed
     *
     * int $params['cur_id'] currency id
     * string $params['cur_gid'] currency gid
     * int $params['value'] amount
     * string $params['template']&nbsp;[abbr][value|dec_part:2|dec_sep:.|gr_sep:&nbsp;]
     *
     * @param array $params block parameters
     *
     * @return string
     */
    function currency_format_regexp_output($params = array())
    {
        $ci = &get_instance();
        static $is_currencies_installed = null;
        if (is_null($is_currencies_installed)) {
            $is_currencies_installed = $ci->pg_module->is_module_active('currencies');
        }
        if ($is_currencies_installed) {
            $ci->load->helper('currencies');

            return currency_format_regexp($params);
        } else {
            $base_currency_gid = $ci->pg_module->get_module_config("start", "base_currency");

            return 'function(value){return value+\' ' . $base_currency_gid . '\'}';
        }
    }

}

if (!function_exists('get_currency_value')) {

    /**
     * Return value converted to current currency
     *
     * @param array $params block parameters
     *
     * @return string
     */
    function get_currency_value($params = array())
    {
        $ci = &get_instance();
        static $is_currencies_installed = null;
        if (is_null($is_currencies_installed)) {
            $is_currencies_installed = $ci->pg_module->is_module_active('currencies');
        }
        if ($is_currencies_installed) {
            $ci->load->helper('currencies');

            return currency_value($params);
        } else {
            return $params['value'];
        }
    }

}

if (!function_exists('start_statistics')) {

    /**
     * Return site statistics
     *
     * @return string
     */
    function start_statistics()
    {
        $ci = &get_instance();

        $stat = array();

        if ($ci->pg_module->is_module_active("users")) {
            $ci->load->model("Users_model");
            $user_types = $ci->Users_model->getUserTypes();
            foreach ($user_types as $user_type) {
                $where = array();
                $where["where"]["user_type"] = $user_type;
                $where["where"]["status"] = 1;
                $stat[] = array(
                    "object_name" => l($user_type, "users"),
                    "count" => $ci->Users_model->get_active_users_count($where),
                );
            }
        }

        if ($ci->pg_module->is_module_active("listings")) {
            $ci->load->model("Listings_model");
            $where = array();
            $where["status"] = 1;
            $stat[] = array(
                "object_name" => l("listings", "start"),
                "count" => $ci->Listings_model->get_listings_count($where),
                "last" => 1,
            );
        }

        if (count($stat)) {
            $ci->view->assign('stat', $stat);

            return $ci->view->fetch('stat_block', 'user', 'start');
        } else {
            return false;
        }
    }

}

if (!function_exists('show_contacts')) {

    /**
     * Display site contacts
     *
     * @return string
     */
    function show_contacts()
    {
        $ci = &get_instance();

        return $ci->pg_language->pages->get_string('start', 'site_contacts');
    }

}

if (!function_exists('start_user_info')) {

    /**
     * Display user info
     *
     * @return string
     */
    function start_user_info($params)
    {
        $ci = &get_instance();

        $is_users_installed = $ci->pg_module->is_module_active("users");
        if ($is_users_installed) {
            $ci->load->helper('users');
            return user_info($params);
        } else {
            $ci->load->model('Start_model');
            $user = $ci->Start_model->getUserDefaultOnSite();
            $ci->view->assign('user', $user);
            if (isset($params['all_listings'])) {
                $ci->view->assign('show_all_listings', $params['all_listings']);
            }
            if ($ci->session->userdata("auth_type") != 'user') {
                $ci->view->assign('show_contact_form', 1);
            }

            $map_markers = array(
                array(
                    'gid' => $user['id'],
                    "address" => $user["address"],
                    "lat" => (float) $user["lat"],
                    "lon" => (float) $user["lon"],
                    "html" => ($user['location'] ? $user['location'] : $user["output_name"]),
                    "dragging" => false,
                ),
            );
            $current_language = $ci->pg_language->get_lang_by_id($ci->pg_language->current_lang_id);
            $view_settings = array(
                "class" => "",
                "geocode_listener" => "get_user_geocode_data",
                "lang" => $current_language["code"],
            );
            $ci->view->assign("map_settings", $view_settings);
            $ci->view->assign("map_markers", $map_markers);
            $ci->view->assign('is_user_owner', 1);

            return $ci->view->fetch("helper_user_info", null, "start");
        }
    }

}

if (!function_exists('start_user_view')) {

    /**
     * Display site contacts
     *
     * @return string
     */
    function start_user_view($params)
    {
        $ci = &get_instance();
        $ci->load->helper('seo');
        if (is_array($params['data'])) {
            $data = $params['data'];
        } else {
            $data['id'] = $params['data'];
        }
        $data = array_merge($data, $params);
        $is_users_installed = $ci->pg_module->is_module_active("users");
        if ($is_users_installed) {
            if (isset($data['id']) && $data['id'] == 0) {
                if (!empty($params['section'])) {
                    $section = $params['section'];
                }
                $url = urldecode(rewrite_link('start', 'startUser'));
            } else {
                $url = urldecode(rewrite_link('users', 'view', $data));
            }
        } else {
            if (!empty($params['section'])) {
                $section = $params['section'];
            }
            $url = urldecode(rewrite_link('start', 'startUser', $section));
        }

        return $url;
    }

}

if (!function_exists('include_view_block')) {

    /**
     * Display site contacts
     *
     * @return string
     */
    function include_view_block()
    {
        $ci = &get_instance();
        if ($ci->pg_module->is_module_active('users')) {
            $module = 'users';
        } else {
            $module = 'start';
        }
        return $ci->view->fetch('view_block', 'user', $module);
    }

}

if (!function_exists("users_lang_select")) {

    /**
     * Return lanuages selector
     *
     * @return string
     */
    function users_lang_select()
    {
        $ci = & get_instance();

        $languages = $ci->pg_language->languages;
        foreach ($languages as $lid => $lang_data) {
            if (!$lang_data["status"]) {
                unset($languages[$lid]);
            }
        }
        if (count($languages) <= 1) {
            return '';
        }
        $ci->view->assign("languages", $languages);

        $current_lang = $ci->pg_language->current_lang_id;
        $ci->view->assign("current_lang", $current_lang);

        if ($ci->pg_module->is_module_active('users')) {
            $module = 'users';
        } else {
            $module = 'start';
        }
        return $ci->view->fetch("helper_lang_select", null, $module);
    }

}

if (!function_exists('is_module_users')) {

    /**
     * Check module users
     *
     * @return string
     */
    function is_module_users()
    {
        $ci = & get_instance();

        $is_users_installed = $ci->pg_module->is_module_active("users") ? 1 : 0;
        $ci->view->assign("is_users", $is_users_installed);
    }

}

if (!function_exists('view_start_user_block')) {

    /**
     * View user info
     *
     * @return string
     */
    function view_start_user_block()
    {
        $ci = & get_instance();

        $is_users_installed = $ci->pg_module->is_module_active("users");
        if ($is_users_installed) {
            return $ci->view->fetch("helper_view_user_block", null, "users");
        } else {
            $ci->load->model('Start_model');
            $user = $ci->Start_model->getUserDefaultOnSite();
            $user_content = $ci->Start_model->getViewUserInfo($user);
            $ci->view->assign("user_content", $user_content);
            return $ci->view->fetch("helper_view_user_block", null, "start");
        }
    }

}

if (!function_exists('geolocations')) {

    function geolocations()
    {
        $ci = &get_instance();
        if ($ci->pg_module->get_module_config('language_identification', 'automatic_identification') || $ci->pg_module->get_module_config('country_identification', 'automatic_identification')) {
            $ci->view->assign(
                'language_identification', $ci->pg_module->get_module_config('language_identification', 'automatic_identification')
            );
            $ci->view->assign(
                'country_identification', $ci->pg_module->get_module_config('country_identification', 'automatic_identification')
            );
            return $ci->view->fetch('helper_geolocation', 'user', 'start');
        }
    }

}

if (!function_exists('getErrorPage')) {

    function getErrorPage()
    {
        $ci = &get_instance();
        $ci->load->model('Menu_model');
        $ci->Menu_model->breadcrumbs_set_active(l('header_error', 'start'));
        $ci->view->assign('user_session_data', $ci->session->userdata);
        return $ci->view->fetch('error', 'user', 'start');
    }

}

if (!function_exists('getCalendarInput')) {
    function getCalendarInput($name, $value, array $attrs=[])
    {
        $ci = &get_instance();
        $ci->view->assign('name', $name);
        $ci->view->assign('value', $value);
        $ci->view->assign('attrs', $attrs);
        return $ci->view->fetch('helper_calendar_input', 'admin', 'start');
    }
}
