<?php

namespace Pg\Modules\Start\Models;

use Pg\Libraries\Setup;

/**
 * Start module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Start Install Model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Start
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Start_install_model extends \Model
{
    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    private $modules_data;

    /**
     * Constructor
     *
     * @return Start_install_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->modules_data = Setup::getModuleData('start', Setup::TYPE_MODULES_DATA);
        $this->ci = &get_instance();
        $this->ci->load->model('Install_model');
    }

    /**
     * Check system requirements of module
     *
     * @return array
     */
    public function _validate_requirements()
    {
        $result = array("data" => array(), "result" => true);

        //php 5.3
        $good = phpversion() >= "5.3.0";
        $result["data"][] = array(
            "name" => "PHP version >= 5.3.0 ",
            "value" => $good ? "Yes" : "No",
            "result" => $good,
        );
        $result["result"] = $result["result"] && $good;

        //json
        $good = extension_loaded("json");
        $result["data"][] = array(
            "name" => "PECL json",
            "value" => $good ? "Yes" : "No",
            "result" => $good,
        );

        $result["result"] = $result["result"] && $good;

        return $result;
    }

    /**
     * Validate module install settings
     *
     * @return array
     */
    public function _validate_settings_form($data)
    {
        $return = array("data" => array(), "errors" => array());

        if (isset($data["product_order_key"])) {
            $return["data"]["product_order_key"] = trim(strip_tags($data["product_order_key"]));
            if (empty($return["data"]["product_order_key"])) {
                $this->ci->pg_language->get_string('start', 'error_product_key_incorrect');
            }
        } else {
            $return['errors'][] = $this->ci->pg_language->get_string('start', 'error_product_key_incorrect');
        }

        return $return;
    }

    /**
     * Save module settings
     *
     * @param array $data settings data
     *
     * @return void
     */
    private function _save_settings_form($data)
    {
        foreach ($data as $setting => $value) {
            $this->ci->pg_module->set_module_config("start", $setting, $value);
        }

        return;
    }

    /**
     * Return module settings form
     *
     * @param boolean $submit use submit button
     *
     * @return string
     */
    public function _get_settings_form($submit = false)
    {
        $data = array(
            'product_order_key' => $this->ci->pg_module->get_module_config('start', 'product_order_key'),
        );

        if (empty($data['product_order_key'])) {
            if (file_exists(SITE_PHYSICAL_PATH . 'order_key.txt')) {
                $data['product_order_key'] = file_get_contents(SITE_PHYSICAL_PATH . 'order_key.txt');
                if (!empty($data['product_order_key'])) {
                    $submit = true;
                }
            }
        } elseif ($submit) {
            $data["product_order_key"] = $this->ci->input->post('product_order_key', true);
        }

        if ($submit) {
            $validate = $this->_validate_settings_form($data);
            if (!empty($validate["errors"])) {
                $this->ci->view->assign('settings_errors', $validate["errors"]);
                $data = $validate["data"];
            } else {
                $this->_save_settings_form($validate["data"]);

                return false;
            }
        }

        $this->ci->view->assign('settings_data', $data);
        $html = $this->ci->view->fetch('install_settings_form', 'admin', 'start');

        return $html;
    }

    /**
     * Install data of menu module
     *
     * @return void
     */
    public function install_menu()
    {
        $this->ci->load->helper("menu");
        foreach ($this->modules_data['menu'] as $gid => $menu_data) {
            $this->modules_data['menu'][$gid]["id"] = linked_install_set_menu($gid, $menu_data["action"], $menu_data["name"]);
            linked_install_process_menu_items($this->modules_data['menu'], "create", $gid, 0, $this->modules_data['menu'][$gid]["items"]);
        }
    }

    /**
     * Import languages of menu module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return boolean
     */
    public function install_menu_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $langs_file = $this->ci->Install_model->language_file_read("start", "menu", $langs_ids);

        if (!$langs_file) {
            log_message("info", "Empty menu langs data");

            return false;
        }

        $this->ci->load->helper("menu");

        foreach ($this->modules_data['menu'] as $gid => $menu_data) {
            linked_install_process_menu_items($this->modules_data['menu'], "update", $gid, 0, $this->modules_data['menu'][$gid]["items"], $gid, $langs_file);
        }

        return true;
    }

    /**
     * Export languages of menu module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_menu_lang_export($langs_ids)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $this->ci->load->helper("menu");

        $return = array();
        foreach ($this->modules_data['menu'] as $gid => $menu_data) {
            $temp = linked_install_process_menu_items($this->modules_data['menu'], "export", $gid, 0, $this->modules_data['menu'][$gid]["items"], $gid, $langs_ids);
            $return = array_merge($return, $temp);
        }

        return array("menu" => $return);
    }

    /**
     * Uninstall data of menu module
     *
     * @return void
     */
    public function deinstall_menu()
    {
        $this->ci->load->model('Menu_model');

        foreach ($this->modules_data['menu'] as $gid => $menu_data) {
            $menu = $this->ci->Menu_model->get_menu_by_gid($gid);
            if ($menu["id"]) {
                $this->ci->Menu_model->delete_menu($menu["id"]);
            }
        }
    }

    /**
     * Install data of banners module
     *
     * @return void
     */
    public function install_banners()
    {
        // add banners module
        $this->ci->load->model("Start_model");
        $this->ci->load->model("banners/models/Banner_group_model");
        $this->ci->load->model("banners/models/Banner_place_model");

        $this->ci->Banner_group_model->set_module("start", "Start_model", "_banner_available_pages");

        // create banner groups
        $group_attrs = array(
            "date_created" => date("Y-m-d H:i:s"),
            "date_modified" => date("Y-m-d H:i:s"),
            "price" => 1,
            "gid" => "content_groups",
            "name" => "Content pages",
        );
        $group_id = $this->ci->Banner_group_model->create_unique_group($group_attrs);
        $all_places = $this->ci->Banner_place_model->get_all_places();
        if ($all_places) {
            foreach ($all_places as $key => $value) {
                $this->ci->Banner_place_model->save_place_group($value["id"], $group_id);
            }
        }

        $group_attrs = array(
            "date_created" => date("Y-m-d H:i:s"),
            "date_modified" => date("Y-m-d H:i:s"),
            "price" => 1,
            "gid" => "users_groups",
            "name" => "Users pages",
        );
        $group_id = $this->ci->Banner_group_model->create_unique_group($group_attrs);
        $all_places = $this->ci->Banner_place_model->get_all_places();
        if ($all_places) {
            foreach ($all_places as $key => $value) {
                $this->ci->Banner_place_model->save_place_group($value["id"], $group_id);
            }
        }

        $group_attrs = array(
            "date_created" => date("Y-m-d H:i:s"),
            "date_modified" => date("Y-m-d H:i:s"),
            "price" => 1,
            "gid" => "start_groups",
            "name" => "Start pages",
        );

        $group_id = $this->ci->Banner_group_model->create_unique_group($group_attrs);
        // add pages in group
        $pages = $this->ci->Start_model->_banner_available_pages();
        if ($pages) {
            foreach ($pages as $key => $value) {
                $page_attrs = array(
                    "group_id" => $group_id,
                    "name" => $value["name"],
                    "link" => $value["link"],
                );
                $this->ci->Banner_group_model->add_page($page_attrs);
            }
        }

        // add places in group
        $all_places = $this->ci->Banner_place_model->get_all_places();
        if ($all_places) {
            foreach ($all_places as $key => $value) {
                $this->ci->Banner_place_model->save_place_group($value["id"], $group_id);
            }
        }
    }

    /**
     * Import languages of banners module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return boolean
     */
    public function install_banners_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }
        if (!is_array($langs_ids)) {
            $langs_ids = (array) $langs_ids;
        }

        $langs_file = $this->ci->Install_model->language_file_read("start", "banners", $langs_ids);
        if (!$langs_file) {
            log_message("info", "Empty banners langs data");

            return false;
        }

        $this->ci->load->model("banners/models/Banner_group_model");

        $banners_groups[] = "banners_group_content_groups";
        $banners_groups[] = "banners_group_listings_groups";
        $banners_groups[] = "banners_group_users_groups";
        $banners_groups[] = "banners_group_start_groups";

        $this->ci->Banner_group_model->update_langs($banners_groups, $langs_file, $langs_ids);

        return true;
    }

    /**
     * Export languages of banners module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_banners_lang_export($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $this->ci->load->model("banners/models/Banner_group_model");
        $banners_groups[] = "banners_group_content_groups";
        $banners_groups[] = "banners_group_listings_groups";
        $banners_groups[] = "banners_group_users_groups";
        $banners_groups[] = "banners_group_start_groups";
        $langs = $this->ci->Banner_group_model->export_langs($banners_groups, $langs_ids);

        return array("banners" => $langs);
    }

    /**
     * Uninstall data of banners module
     *
     * @return void
     */
    public function deinstall_banners()
    {
        // delete banners module
        $this->ci->load->model("banners/models/Banner_group_model");
        $this->ci->Banner_group_model->delete_module("start");
    }

    /**
     * Install data of dynamic blocks module
     *
     * @return void
     */
    public function install_dynamic_blocks()
    {
        $this->ci->load->model("Dynamic_blocks_model");

        $area_ids = array();

        foreach ((array) $this->modules_data['dynamic_blocks'] as $block_data) {
            $validate_data = $this->ci->Dynamic_blocks_model->validate_block(null, $block_data);
            if (!empty($validate_data["errors"])) {
                continue;
            }
            $id_block = $this->ci->Dynamic_blocks_model->save_block(null, $validate_data["data"]);

            if (!empty($block_data["area"])) {
                if (!isset($area_ids[$block_data["area"]["gid"]])) {
                    $area = $this->ci->Dynamic_blocks_model->get_area_by_gid($block_data["area"]["gid"]);
                    $area_ids[$block_data["area"]["gid"]] = $area["id"];
                }

                // index area
                $block_data["area"]["id_area"] = $area_ids[$block_data["area"]["gid"]];
                $block_data["area"]["id_block"] = $id_block;

                $validate_data = $this->ci->Dynamic_blocks_model->validate_area_block(null, $block_data["area"]);
                if (empty($validate_data["errors"])) {
                    $this->ci->Dynamic_blocks_model->save_area_block(null, $validate_data["data"]);
                }
            }

            if (!empty($block_data["presets"])) {
                foreach ($block_data["presets"] as $preset_data) {
                    $preset = $this->ci->Dynamic_blocks_model->get_preset_by_gid($preset_data['gid_preset']);
                    $preset_data['id_preset'] = $preset['id'];
                    $preset_data['id_block'] = $id_block;
                    $validate_data = $this->ci->Dynamic_blocks_model->validate_preset_block(null, $preset_data);
                    if (!empty($validate_data["errors"])) {
                        continue;
                    }
                    $this->ci->Dynamic_blocks_model->save_preset_block(null, $validate_data["data"]);
                }
            }
        }
    }

    /**
     * Install languages of dynamic blocks module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return void
     */
    public function install_dynamic_blocks_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }

        $langs_file = $this->ci->Install_model->language_file_read("start", "dynamic_blocks", $langs_ids);
        if (!$langs_file) {
            log_message("info", "Empty dynamic_blocks langs data");

            return false;
        }

        $this->ci->load->model("Dynamic_blocks_model");

        $data = array();

        foreach ((array) $this->modules_data['dynamic_blocks'] as $block_data) {
            $block = $this->ci->Dynamic_blocks_model->get_block_by_gid($block_data["gid"]);
            if (!$block) {
                continue;
            }
            $data[] = $block;
        }

        $this->ci->Dynamic_blocks_model->update_langs($data, $langs_file, $langs_ids);
    }

    /**
     * Export languages of dynamic blocks data
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_dynamic_blocks_lang_export($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $this->ci->load->model("Dynamic_blocks_model");
        $data = array();
        foreach ((array) $this->modules_data['dynamic_blocks'] as $block_data) {
            $block = $this->ci->Dynamic_blocks_model->get_block_by_gid($block_data["gid"]);
            if (!$block) {
                continue;
            }
            $data[] = $block;
        }
        $langs = $this->ci->Dynamic_blocks_model->export_langs($data, $langs_ids);

        return array("dynamic_blocks" => $langs);
    }

    /**
     * Unistall data of dynamic blocks module
     *
     * @return void
     */
    public function deinstall_dynamic_blocks()
    {
        $this->ci->load->model("Dynamic_blocks_model");

        foreach ((array) $this->modules_data['dynamic_blocks'] as $block_data) {
            $this->ci->Dynamic_blocks_model->delete_block_by_gid($block_data["gid"]);
        }
    }

    /**
     * Install data of users module
     *
     * @return void
     */
    public function install_users()
    {
        $this->ci->load->helper("menu");
        foreach ($this->modules_data['menu_users'] as $gid => $menu_data) {
            $this->modules_data['menu'][$gid]["id"] = linked_install_set_menu($gid, $menu_data["action"], $menu_data["name"]);
            linked_install_process_menu_items($this->modules_data['menu_users'], "create", $gid, 0, $menu_data["items"]);
        }
    }

    /**
     * Import languages of users module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return void
     */
    public function install_users_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }

        $langs_file = $this->ci->Install_model->language_file_read("start", "menu_users", $langs_ids);

        if (!$langs_file) {
            log_message("info", "Empty menu users langs data (start)");
        } else {
            $this->ci->load->helper("menu");

            foreach ($this->modules_data['menu_users'] as $gid => $menu_data) {
                linked_install_process_menu_items($this->modules_data['menu_users'], "update", $gid, 0, $menu_data["items"], $gid, $langs_file);
            }
        }
    }

    /**
     * Export languages of users module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_users_lang_export($langs_ids)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $this->ci->load->helper("menu");

        $return = array();
        foreach ($this->modules_data['menu_users'] as $gid => $menu_data) {
            $temp = linked_install_process_menu_items($this->modules_data['menu_users'], "export", $gid, 0, $menu_data["items"], $gid, $langs_ids);
            $return = array_merge($return, $temp);
        }

        return array("menu_users" => $return);
    }

    /**
     * Uninstall data of users module
     *
     * @return void
     */
    public function deinstall_users()
    {
        $this->ci->load->model('Menu_model');

        foreach ($this->modules_data['menu_users'] as $gid => $menu_data) {
            $menu = $this->ci->Menu_model->get_menu_by_gid($gid);
            if ($menu["id"]) {
                $this->ci->Menu_model->delete_menu($menu["id"]);
            }
        }
    }

    /**
     * Install data of users_private module
     *
     * @return void
     */
    public function install_users_private()
    {
        $this->ci->load->helper("menu");
        foreach ($this->modules_data['menu_users_private'] as $gid => $menu_data) {
            $this->modules_data['menu'][$gid]["id"] = linked_install_set_menu($gid, $menu_data["action"], $menu_data["name"]);
            linked_install_process_menu_items($this->modules_data['menu_users_private'], "create", $gid, 0, $menu_data["items"]);
        }
    }

    /**
     * Import languages of users_private module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return void
     */
    public function install_users_private_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }

        $langs_file = $this->ci->Install_model->language_file_read("start", "menu_users_private", $langs_ids);

        if (!$langs_file) {
            log_message("info", "Empty menu users_private langs data (start)");
        } else {
            $this->ci->load->helper("menu");

            foreach ($this->modules_data['menu_users_private'] as $gid => $menu_data) {
                linked_install_process_menu_items($this->modules_data['menu_users_private'], "update", $gid, 0, $menu_data["items"], $gid, $langs_file);
            }
        }
    }

    /**
     * Export languages of users_private module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_users_private_lang_export($langs_ids)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $this->ci->load->helper("menu");

        $return = array();
        foreach ($this->modules_data['menu_users_private'] as $gid => $menu_data) {
            $temp = linked_install_process_menu_items($this->modules_data['menu_users_private'], "export", $gid, 0, $menu_data["items"], $gid, $langs_ids);
            $return = array_merge($return, $temp);
        }

        return array("menu_users_private" => $return);
    }

    /**
     * Uninstall data of users_private module
     *
     * @return void
     */
    public function deinstall_users_private()
    {
        $this->ci->load->model('Menu_model');

        foreach ($this->modules_data['menu_users_private'] as $gid => $menu_data) {
            $menu = $this->ci->Menu_model->get_menu_by_gid($gid);
            if ($menu["id"]) {
                $this->ci->Menu_model->delete_menu($menu["id"]);
            }
        }
    }

    /**
     * Install data of reviews module
     *
     * @return void
     */
    public function install_reviews()
    {
        $feedbacks_modules = array_diff($this->modules_data['feedbacks_modules'], array("reviews"));
        foreach ($feedbacks_modules as $module_gid) {
            if ($this->ci->pg_module->is_module_installed($module_gid)) {
                return;
            }
        }

        $this->ci->load->model("Menu_model");

        $menu = $this->ci->Menu_model->get_menu_by_gid("admin_menu");
        if (!empty($menu)) {
            $menu_item = $this->ci->Menu_model->get_menu_item_by_gid("feedbacks-items", $menu["id"]);
            if (!empty($menu_item)) {
                $this->ci->Menu_model->save_menu_item($menu_item["id"], array("status" => 1));
            }
        }
    }

    /**
     * Uninstall data of reviews module
     *
     * @return void
     */
    public function deinstall_reviews()
    {
        $feedbacks_modules = array_diff($this->modules_data['feedbacks_modules'], array("reviews"));
        foreach ($feedbacks_modules as $module_gid) {
            if ($this->ci->pg_module->is_module_installed($module_gid)) {
                return;
            }
        }

        $this->ci->load->model("Menu_model");

        $menu = $this->ci->Menu_model->get_menu_by_gid("admin_menu");
        if (!empty($menu)) {
            $menu_item = $this->ci->Menu_model->get_menu_item_by_gid("feedbacks-items", $menu["id"]);
            if (!empty($menu_item)) {
                $this->ci->Menu_model->save_menu_item($menu_item["id"], array("status" => 0));
            }
        }
    }

    /**
     * Install data of spam module
     *
     * @return void
     */
    public function install_spam()
    {
        $feedbacks_modules = array_diff($this->modules_data['feedbacks_modules'], array("spam"));
        foreach ($feedbacks_modules as $module_gid) {
            if ($this->ci->pg_module->is_module_installed($module_gid)) {
                return;
            }
        }

        $this->ci->load->model("Menu_model");

        $menu = $this->ci->Menu_model->get_menu_by_gid("admin_menu");
        if (!empty($menu)) {
            $menu_item = $this->ci->Menu_model->get_menu_item_by_gid("feedbacks-items", $menu["id"]);
            if (!empty($menu_item)) {
                $this->ci->Menu_model->save_menu_item($menu_item["id"], array("status" => 1));
            }
        }
    }

    /**
     * Uninstall data of spam module
     *
     * @return void
     */
    public function deinstall_spam()
    {
        $feedbacks_modules = array_diff($this->modules_data['feedbacks_modules'], array("spam"));
        foreach ($feedbacks_modules as $module_gid) {
            if ($this->ci->pg_module->is_module_installed($module_gid)) {
                return;
            }
        }

        $this->ci->load->model("Menu_model");

        $menu = $this->ci->Menu_model->get_menu_by_gid("admin_menu");
        if (!empty($menu)) {
            $menu_item = $this->ci->Menu_model->get_menu_item_by_gid("feedbacks-items", $menu["id"]);
            if (!empty($menu_item)) {
                $this->ci->Menu_model->save_menu_item($menu_item["id"], array("status" => 0));
            }
        }
    }

    /**
     * Install data of users_deactivate module
     *
     * @return void
     */
    public function install_users_deactivate()
    {
        $feedbacks_modules = array_diff($this->modules_data['feedbacks_modules'], array("users_deactivate"));
        foreach ($feedbacks_modules as $module_gid) {
            if ($this->ci->pg_module->is_module_installed($module_gid)) {
                return;
            }
        }

        $this->ci->load->model("Menu_model");

        $menu = $this->ci->Menu_model->get_menu_by_gid("admin_menu");
        if (!empty($menu)) {
            $menu_item = $this->ci->Menu_model->get_menu_item_by_gid("feedbacks-items", $menu["id"]);
            if (!empty($menu_item)) {
                $this->ci->Menu_model->save_menu_item($menu_item["id"], array("status" => 1));
            }
        }
    }

    /**
     * Uninstall data of users_deactivate module
     *
     * @return void
     */
    public function deinstall_users_deactivate()
    {
        $feedbacks_modules = array_diff($this->modules_data['feedbacks_modules'], array("users_deactivate"));
        foreach ($feedbacks_modules as $module_gid) {
            if ($this->ci->pg_module->is_module_installed($module_gid)) {
                return;
            }
        }

        $this->ci->load->model("Menu_model");

        $menu = $this->ci->Menu_model->get_menu_by_gid("admin_menu");
        if (!empty($menu)) {
            $menu_item = $this->ci->Menu_model->get_menu_item_by_gid("feedbacks-items", $menu["id"]);
            if (!empty($menu_item)) {
                $this->ci->Menu_model->save_menu_item($menu_item["id"], array("status" => 0));
            }
        }
    }

    /**
     * Install data of export module
     *
     * @return void
     */
    public function install_export()
    {
        $export_import_modules = array_diff($this->modules_data['export_import_modules'], array("export"));
        foreach ($export_import_modules as $module_gid) {
            if ($this->ci->pg_module->is_module_installed($module_gid)) {
                return;
            }
        }

        $this->ci->load->model("Menu_model");

        $menu = $this->ci->Menu_model->get_menu_by_gid("admin_menu");
        if (!empty($menu)) {
            $menu_item = $this->ci->Menu_model->get_menu_item_by_gid("exp-import-items", $menu["id"]);
            if (!empty($menu_item)) {
                $this->ci->Menu_model->save_menu_item($menu_item["id"], array("status" => 1));
            }
        }
    }

    /**
     * Uninstall data of export module
     *
     * @return void
     */
    public function deinstall_export()
    {
        $export_import_modules = array_diff($this->modules_data['export_import_modules'], array("export"));
        foreach ($export_import_modules as $module_gid) {
            if ($this->ci->pg_module->is_module_installed($module_gid)) {
                return;
            }
        }

        $this->ci->load->model("Menu_model");

        $menu = $this->ci->Menu_model->get_menu_by_gid("admin_menu");
        if (!empty($menu)) {
            $menu_item = $this->ci->Menu_model->get_menu_item_by_gid("exp-import-items", $menu["id"]);
            if (!empty($menu_item)) {
                $this->ci->Menu_model->save_menu_item($menu_item["id"], array("status" => 0));
            }
        }
    }

    /**
     * Install data of import module
     *
     * @return void
     */
    public function install_import()
    {
        $export_import_modules = array_diff($this->modules_data['export_import_modules'], array("import"));
        foreach ($export_import_modules as $module_gid) {
            if ($this->ci->pg_module->is_module_installed($module_gid)) {
                return;
            }
        }

        $this->ci->load->model("Menu_model");

        $menu = $this->ci->Menu_model->get_menu_by_gid("admin_menu");
        if (!empty($menu)) {
            $menu_item = $this->ci->Menu_model->get_menu_item_by_gid("exp-import-items", $menu["id"]);
            if (!empty($menu_item)) {
                $this->ci->Menu_model->save_menu_item($menu_item["id"], array("status" => 1));
            }
        }
    }

    /**
     * Uninstall data of import module
     *
     * @return void
     */
    public function deinstall_import()
    {
        $export_import_modules = array_diff($this->modules_data['export_import_modules'], array("import"));
        foreach ($export_import_modules as $module_gid) {
            if ($this->ci->pg_module->is_module_installed($module_gid)) {
                return;
            }
        }

        $this->ci->load->model("Menu_model");

        $menu = $this->ci->Menu_model->get_menu_by_gid("admin_menu");
        if (!empty($menu)) {
            $menu_item = $this->ci->Menu_model->get_menu_item_by_gid("exp-import-items", $menu["id"]);
            if (!empty($menu_item)) {
                $this->ci->Menu_model->save_menu_item($menu_item["id"], array("status" => 0));
            }
        }
    }

    /**
     * Install data of moderation
     *
     * @return void
     */
    public function install_moderation()
    {
        $this->ci->load->model("Menu_model");

        $menu = $this->ci->Menu_model->get_menu_by_gid("admin_menu");
        if (!empty($menu)) {
            $menu_item = $this->ci->Menu_model->get_menu_item_by_gid("moderation-items", $menu["id"]);
            if (!empty($menu_item)) {
                $this->ci->Menu_model->save_menu_item($menu_item["id"], array("status" => 1));
            }
        }
    }

    /**
     * Uninstall data of moderation module
     *
     * @return void
     */
    public function deinstall_moderation()
    {
        $this->ci->load->model("Menu_model");

        $menu = $this->ci->Menu_model->get_menu_by_gid("admin_menu");
        if (!empty($menu)) {
            $menu_item = $this->ci->Menu_model->get_menu_item_by_gid("moderation-items", $menu["id"]);
            if (!empty($menu_item)) {
                $this->ci->Menu_model->save_menu_item($menu_item["id"], array("status" => 0));
            }
        }
    }

    /**
     * Install data of payments module
     *
     * @return void
     */
    public function install_payments()
    {
        $this->ci->load->model("Menu_model");

        $menu = $this->ci->Menu_model->get_menu_by_gid("admin_menu");
        if (!empty($menu)) {
            $menu_item = $this->ci->Menu_model->get_menu_item_by_gid("payments_menu_item", $menu["id"]);
            if (!empty($menu_item)) {
                $this->ci->Menu_model->save_menu_item($menu_item["id"], array("status" => 1));
            }
        }
    }

    /**
     * Uninstall data of payments module
     *
     * @return void
     */
    public function deinstall_payments()
    {
        $this->ci->load->model("Menu_model");

        $menu = $this->ci->Menu_model->get_menu_by_gid("admin_menu");
        if (!empty($menu)) {
            $menu_item = $this->ci->Menu_model->get_menu_item_by_gid("payments_menu_item", $menu["id"]);
            if (!empty($menu_item)) {
                $this->ci->Menu_model->save_menu_item($menu_item["id"], array("status" => 0));
            }
        }
    }

    /**
     * Install users data to uploads module
     *
     * @return void
     */
    public function install_uploads()
    {
        ///// upload config
        $this->ci->load->model("uploads/models/Uploads_config_model");

        $watermark_ids = array();

        foreach ((array) $this->modules_data['uploads'] as $upload_data) {
            $config_data = array(
                "gid" => $upload_data["gid"],
                "name" => $upload_data["name"],
                "max_height" => $upload_data["max_height"],
                "max_width" => $upload_data["max_width"],
                "max_size" => $upload_data["max_size"],
                "name_format" => $upload_data["name_format"],
                "file_formats" => serialize((array) $upload_data["file_formats"]),
                "default_img" => $upload_data["default_img"],
                "indicator_img" => $upload_data["indicator_img"],
                "date_add" => date("Y-m-d H:i:s"),
            );
            $config_id = $this->ci->Uploads_config_model->save_config(null, $config_data);

            $wm_data = $this->ci->Uploads_config_model->get_watermark_by_gid("image-wm");
            $wm_id = isset($wm_data["id"]) ? $wm_data["id"] : 0;

            foreach ((array) $upload_data["thumbs"] as $thumb_gid => $thumb_data) {
                if (isset($thumb_data["watermark"])) {
                    if (!isset($watermark_ids[$thumb_data["watermark"]])) {
                        $wm_data = $this->ci->Uploads_config_model->get_watermark_by_gid($thumb_data["watermark"]);
                        $watermark_ids[$thumb_data["watermark"]] = isset($wm_data["id"]) ? $wm_data["id"] : 0;
                    }
                    $watermark_id = $watermark_ids[$thumb_data["watermark"]];
                } else {
                    $watermark_id = 0;
                }

                $thumb_data = array(
                    "config_id" => $config_id,
                    "prefix" => $thumb_gid,
                    "width" => $thumb_data["width"],
                    "height" => $thumb_data["height"],
                    "effect" => "none",
                    "watermark_id" => $watermark_id,
                    "crop_param" => $thumb_data["crop_param"],
                    "crop_color" => $thumb_data["crop_color"],
                    "date_add" => date("Y-m-d H:i:s"),
                );
                $this->ci->Uploads_config_model->save_thumb(null, $thumb_data);
            }
        }
    }

    /**
     * Uninstall users data from uploads module
     *
     * @return void
     */
    public function deinstall_uploads()
    {
        $this->ci->load->model("uploads/models/Uploads_config_model");

        foreach ((array) $this->modules_data['uploads'] as $upload_data) {
            $config_data = $this->ci->Uploads_config_model->get_config_by_gid($upload_data["gid"]);
            if (!empty($config_data["id"])) {
                $this->ci->Uploads_config_model->delete_config($config_data["id"]);
            }
        }
    }

    public function install_cronjob()
    {
        $this->ci->load->model('Cronjob_model');
        $this->ci->Cronjob_model->save_cron(null, array(
            'name' => 'Clear temp dirs',
            'module' => 'start',
            'model' => 'Start_model',
            'method' => 'clearTempDirs',
            'cron_tab' => '0 0 * * *',
            'status' => '1',
        ));
    }

    public function deinstall_cronjob()
    {
        $this->ci->load->model('Cronjob_model');
        $this->ci->Cronjob_model->delete_cron_by_param(array('where' => array('module' => 'start',)));
    }

    /**
     * Install demo content
     *
     * @return void
     */
    private function addDemoContent()
    {
        $this->ci->load->model("Start_model");
        $demo_content = Setup::getModuleData(Start_model::MODULE_GID, Setup::TYPE_DEMO_CONTENT);
        foreach ($demo_content['start-user'] as $setting => $value) {
            $this->pg_module->set_module_config(Start_model::MODULE_GID, $setting, $value);
        }
        $demo_upload = MODULEPATH . Start_model::MODULE_GID . "/install/demo_content/uploads/";
        $upload_dir = FRONTEND_PATH . Start_model::UPLOAD_GID . DIRECTORY_SEPARATOR;
        if (!is_dir($upload_dir)) {
            mkdir($upload_dir, 0777, true);
        }
        foreach (new \DirectoryIterator($demo_upload) as $file) {
            copy($file->getRealPath(), $upload_dir . $file->getFilename());
        }
    }

    /**
     * Install module data
     *
     * @return void
     */
    public function _arbitrary_installing()
    {
        $this->addDemoContent();
    }

    /**
     * Import module languages
     *
     * @param array $langs_ids langauges identifiers
     *
     * @return void
     */
    public function _arbitrary_lang_install($langs_ids = null)
    {
        // admin_home_page
        $langs_file = $this->ci->Install_model->language_file_read("start", "admin_home_page", $langs_ids);
        if (!$langs_file) {
            log_message("info", "Empty admin home page langs data");

            return false;
        }

        foreach ($langs_file as $gid => $ldata) {
            if (!empty($ldata)) {
                $this->ci->pg_language->pages->set_string_langs("admin_home_page", $gid, $ldata, array_keys($ldata));
            }
        }

        $langs_file = $this->ci->Install_model->language_file_read("start", "arbitrary", $langs_ids);
        if (!$langs_file) {
            log_message("info", "Empty arbitrary langs data");

            return false;
        }

        $post_data = array(
            "title" => $langs_file["seo_tags_index_title"],
            "keyword" => $langs_file["seo_tags_index_keyword"],
            "description" => $langs_file["seo_tags_index_description"],
            "header" => $langs_file["seo_tags_index_header"],
            "og_title" => $langs_file["seo_tags_index_og_title"],
            "og_type" => $langs_file["seo_tags_index_og_type"],
            "og_description" => $langs_file["seo_tags_index_og_description"],
            "url_template" => "",
        );
        $this->ci->pg_seo->set_settings("user", "", "", $post_data);

        $post_data = array(
            "title" => $langs_file["seo_tags_admin_title"],
            "keyword" => $langs_file["seo_tags_admin_keyword"],
            "description" => $langs_file["seo_tags_admin_description"],
            "header" => $langs_file["seo_tags_admin_header"],
            "og_title" => $langs_file["seo_tags_admin_og_title"],
            "og_type" => $langs_file["seo_tags_admin_og_type"],
            "og_description" => $langs_file["seo_tags_admin_og_description"],
            "url_template" => "",
        );
        $this->ci->pg_seo->set_settings("admin", "", "", $post_data);

        $this->pg_language->pages->set_string_langs('start', 'site_contacts', $langs_file['site_contacts'], $langs_ids);
    }

    /**
     * Export module languages
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function _arbitrary_lang_export($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }

        $admin_home_page_return = array();

        // admin_home_page
        foreach ($langs_ids as $lang_id) {
            $mod_langs = $this->ci->pg_language->pages->return_module("admin_home_page", $lang_id);
            foreach ($mod_langs as $gid => $value) {
                $admin_home_page_return[$gid][$lang_id] = $value;
            }
        }

        // arbitrary
        $settings = $this->ci->pg_seo->get_settings("user", "", "", $langs_ids);
        $arbitrary_return["seo_tags_index_title"] = $settings["title"];
        $arbitrary_return["seo_tags_index_keyword"] = $settings["keyword"];
        $arbitrary_return["seo_tags_index_description"] = $settings["description"];
        $arbitrary_return["seo_tags_index_header"] = $settings["header"];
        $arbitrary_return["seo_tags_index_og_title"] = $settings["og_title"];
        $arbitrary_return["seo_tags_index_og_type"] = $settings["og_type"];
        $arbitrary_return["seo_tags_index_og_description"] = $settings["og_description"];

        $settings = $this->ci->pg_seo->get_settings("admin", "", "", $langs_ids);
        $arbitrary_return["seo_tags_admin_title"] = $settings["title"];
        $arbitrary_return["seo_tags_admin_keyword"] = $settings["keyword"];
        $arbitrary_return["seo_tags_admin_description"] = $settings["description"];
        $arbitrary_return["seo_tags_admin_header"] = $settings["header"];
        $arbitrary_return["seo_tags_admin_og_title"] = $settings["og_title"];
        $arbitrary_return["seo_tags_admin_og_type"] = $settings["og_type"];
        $arbitrary_return["seo_tags_admin_og_description"] = $settings["og_description"];

        foreach ($langs_ids as $lid => $lang_data) {
            $arbitrary_return['site_contacts'][$lid] = $this->pg_language->pages->get_string('start', 'site_contacts', $lid);
        }

        return array("admin_home_page" => $admin_home_page_return, "arbitrary" => $arbitrary_return);
    }

    /**
     * Uninstall module data
     *
     * @return void
     */
    public function _arbitrary_deinstalling()
    {
    }
}
