<?php

/**
 * Upload gallery module
 *
 * @package     PG_RealEstate
 *
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Upload gallery management
 *
 * @package     PG_RealEstate
 * @subpackage  Upload_gallery
 *
 * @category    helpers
 *
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */

if (!function_exists('gallery_editor_block')) {
    
    /**
     * Return gallery editor block
     *
     * @param array $params block parameters
     *
     * @return string
     */
    function gallery_editor_block($params)
    {
        $ci = &get_instance();
        $ci->view->assign('gallery_object_id', $params['object_id']);
        $ci->view->assign('gallery_data', $params['data']);
        $ci->view->assign('gallery_redirect', $params['redirect']);
        $ci->view->assign('gallery_count', $params['count']);
        $ci->view->assign('gallery_type', $params['gallery_type']);
        $ci->view->assign('gallery_thumb', $params['thumb_name']);
        $ci->view->assign('gallery_full', get($params['thumb_full']));
        $ci->view->assign('gallery_panorama', get($params['panorama']));
        $ci->view->assign('gallery_panorama_width', get($params['panorama_width']));
        $ci->view->assign('gallery_panorama_height', get($params['panorama_height']));
        $ci->view->assign('gallery_panorama_thumb', get($params['panorama_thumb_name']));

        $ci->load->model('Upload_gallery_model');
        $gallery_type_config = $ci->Upload_gallery_model->get_type_by_gid($params['gallery_type']);

        $upload_config_gid = $gallery_type_config['gid_upload_config'];
        $ci->view->assign('gallery_type_id', $gallery_type_config['id']);

        $ci->load->model('Uploads_model');
        $gallery_type_config['upload_settings'] = $ci->Uploads_model->get_config($upload_config_gid);

        $ci->load->model('listings/models/Listings_uploads_model');
        $gallery_type_config['upload_settings']['max_size_str'] = $ci->Listings_uploads_model->getSize($gallery_type_config['upload_settings']['max_size'], 'upload_gallery');


        $ci->view->assign('gallery_settings', $gallery_type_config);

        $ci->view->assign('gallery_rand', rand(100000, 999999));

        $block_setting_data = json_encode($ci->view->fetch('photo_settings_block', null, 'upload_gallery'));

        $ci->view->assign('block_setting_data', $block_setting_data);

        return $ci->view->fetch('helper_editor_block', null, 'upload_gallery');
    }

    if (!function_exists('show_panorama_block')) {
        
        /**
         * Render panorama area
         *
         * @param array $params block parameters
         *
         * @return string
         */
        function show_panorama_block($params)
        {
            $ci = &get_instance();

            if (empty($params['data']) || empty($params['thumb_name']) ||
               !isset($params['width']) || intval($params['width']) < 1 ||
               !isset($params['height']) || intval($params['height']) < 1) {
                return false;
            }

            if (!isset($params['alt'])) {
                $params['alt'] = l('text_' . $params['data']['type_data']['gid'] . '_panorama', 'upload_gallery');
            }

            if (!isset($params['panorama_thumbs_box'])) {
                $params['panorama_thumbs_box'] = '';
            }

            $ci->view->assign('panorama_data', $params['data']);
            $ci->view->assign('panorama_type', $params['data']['type_data']['gid']);
            $ci->view->assign('panorama_width', $params['width']);
            $ci->view->assign('panorama_height', $params['height']);
            $ci->view->assign('panorama_thumb', $params['thumb_name']);
            $ci->view->assign('panorama_alt', $params['alt']);
            $ci->view->assign('panorama_thumbs_box', $params['panorama_thumbs_box']);

            $rand = rand(100000, 999999);
            $ci->view->assign('panorama_rand', $rand);

            return $ci->view->fetch('helper_panorama_block', 'user', 'upload_gallery');
        }
    }

    if (!function_exists('uploader_form_block')) {
    
        /**
         * Return gallery editor block
         *
         * @param array $params block parameters
         *
         * @return string
         */
        function uploader_form_block($params = null) {
            $ci = &get_instance();
            $uploader_param = null;
            if (isset($params['video_uploader'])) {
                $uploader_param = $params['video_uploader'];
                $ci->view->assign('file_uploader', '');
                $ci->view->assign($uploader_param . '_data', $params['data']);
                $embed_code_form = json_encode($ci->view->fetch('embed_code_block', 'user', 'video_uploads'));
                $ci->view->assign('embed_code_form', $embed_code_form);
            }
            if (isset($params['file_uploader'])) {
                $uploader_param = $params['file_uploader'];
                $ci->view->assign('video_uploader', '');
                $ci->view->assign($uploader_param . '_data', $params['data']);
                $file_block = json_encode($ci->view->fetch('file_description_block', 'user', 'upload_gallery'));
                $ci->view->assign('file_description_block', $file_block);
            }
            if (isset($params[$uploader_param . '_uploader'])) {
                $ci->view->assign($uploader_param . '_uploader', $params[$uploader_param . '_uploader']);
                $block_setting_data = json_encode($ci->view->fetch($uploader_param . '_settings_block', 'user', 'upload_gallery'));
                $ci->view->assign('block_setting_data', $block_setting_data);
                $ci->view->assign($uploader_param . '_uploader', '1');
            }
            $ci->view->assign('upload_url', $params['upload_url']);
            $ci->view->assign('form_id', $params['form_id']);
            $ci->view->assign('field_name', $params['field_name']);


            return $ci->view->fetch('helper_uploader_form_block', 'user', 'upload_gallery');
        }
    }
}
