<?php

namespace Pg\Modules\Users_payments\Controllers;

/**
 * Users payments module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2016 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
use Pg\Libraries\View;

/**
 * Users payments admin side controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	Users payments
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2016 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Admin_users_payments extends \Controller
{

    /**
     * Constructor
     *
     * @return Users_payments
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Display form of adding funds to user account by ajax
     *
     * @return void
     */
    public function ajax_add_funds_form()
    {
        $this->view->render('funds_form');
    }

    /**
     * Add funds to user account by ajax
     *
     * @return void
     */
    public function ajax_add_funds()
    {
        $this->load->model('Currencies_model');
        $amount = $this->input->post('amount', true);
        $user_ids = $this->input->post('user_ids', true);

        if (empty($amount)) {
            $return['error'] = l('error_empty_amount', 'users_payments');
        } elseif (empty($user_ids)) {
            $return['error'] = l('error_empty_users_list', 'users_payments');
        } else {
            $base_currency_gid = $this->pg_module->get_module_config('start', 'base_currency');

            $this->load->model('Currencies_model');
            $default_currency = $this->Currencies_model->default_currency;

            $this->load->helper('payments');

            foreach ($user_ids as $id_user) {
                $payment_data = send_payment('account', $id_user, $amount, $default_currency['gid'], 'manual',
                                             array('name' => l('added_by_admin', 'users_payments')));

                $payment_data['data']['status'] = 1;
                receive_payment('manual', $payment_data['data']);
            }
            $return['success'] = '';
            $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_add_funds', 'users_payments'));
        }

        $this->view->assign($return);

        $this->view->render();
    }

}
