<?php

namespace Pg\Modules\Users_payments\Models;

/**
 * Users payments module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

if (!defined('USERS_TABLE')) {
    define('USERS_TABLE', DB_PREFIX . 'users');
}
if (!defined('USER_ACCOUNT_STAT_TABLE')) {
    define('USER_ACCOUNT_STAT_TABLE', DB_PREFIX . 'user_account_list');
}

/**
 * Users payments model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Users payments
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Users_payments_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Constructor
     *
     * @return Users_payments_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();

        $this->ci->load->model('Users_model');
    }

    // user account functions

    /**
     * Return user account
     *
     * @param integer $user_id user identifier
     *
     * @return float
     */
    public function get_user_account($user_id)
    {
        $result = $this->ci->db->select("account")->from(USERS_TABLE)->where("id", $user_id)->get()->result_array();
        if (empty($result)) {
            return 0;
        } else {
            return $result[0]["account"];
        }
    }

    /**
     * Update user account
     *
     * @param array   $payment_data   payment data
     * @param integer $payment_status payment_status
     *
     * @return void
     */
    public function update_user_account($payment_data, $payment_status)
    {
        if ($payment_status == 1) {
            $user_id = $payment_data["id_user"];

            $this->ci->load->model('Users_model');

            $this->ci->Users_model->set_format_settings('get_safe', false);
            $user_data = $this->ci->Users_model->get_user_by_id($user_id);
            $this->ci->Users_model->set_format_settings('get_safe', true);

            $this->ci->load->helper('start');
            $amount = get_currency_value(array('value' => $payment_data["amount"], 'cur_gid' => $payment_data['currency_gid'], 'to_gid' => 'USD'));

            $data["account"] = $user_data["account"] + $amount;

            $this->ci->Users_model->save_user($user_id, $data);

            $this->account_add_spend_entry($user_id, $amount, "add", l('account_funds_add_message', 'users_payments'));

            $mail_data = $user_data;

            $this->ci->load->model('Currencies_model');

            if (!empty($user_data['id_currency'])) {
                $currency = $this->ci->Currencies_model->get_currency_by_id($user_data['id_currency']);
                if (empty($currency)) {
                    $currency = $this->ci->Currencies_model->default_currency;
                }
            } else {
                $currency = $this->ci->Currencies_model->default_currency;
            }

            $this->ci->load->helper('start');
            $mail_data['account'] = currency_format_output(array('value' => $data["account"], 'cur_gid' => $currency['gid']));
            $mail_data['account'] = trim(strip_tags($mail_data['account']));

            $this->ci->load->model('Notifications_model');
            $this->ci->Notifications_model->send_notification($user_data["email"], 'users_update_account', $mail_data,
                                                              '', $user_data["lang_id"]);
        }

        return;
    }

    /**
     * Write off amount from user account
     *
     * @param integer $id_user user identifier
     * @param float   $amount  amount value
     * @param string  $message spend message
     *
     * @return true/string
     */
    public function write_off_user_account($id_user, $amount, $message)
    {
        $user_data = $this->ci->Users_model->get_user_by_id($id_user);
        $data["account"] = $user_data["account"] - $amount;
        if ($data["account"] < 0) {
            return l('error_money_not_sufficient', 'users_payments');
        } else {
            $this->ci->Users_model->save_user($id_user, $data);
            $this->account_add_spend_entry($id_user, $amount, "spend", $message);

            return true;
        }
    }

    /**
     * Logging user spend from account
     *
     * @param integer $id_user    user identifier
     * @param float   $price      price value
     * @param string  $price_type price type
     * @param string  $message    message text
     *
     * @return void
     */
    public function account_add_spend_entry($id_user, $price, $price_type = "add", $message = "")
    {
        if (empty($message)) {
            $message = ($price_type = "add") ? l('account_funds_add_message', 'users_payments') : l('account_funds_spend_message',
                                                                                                    'users_payments');
        }

        $data = array(
            "id_user" => $id_user,
            "date_add" => date("Y-m-d H:i:s"),
            "price_type" => $price_type,
            "price" => $price,
            "message" => $message,
        );
        $this->ci->db->insert(USER_ACCOUNT_STAT_TABLE, $data);
    }

}
