<?php

/**
 * Users services module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
/**
 * Users services management
 *
 * @package 	PG_RealEstate
 * @subpackage 	Users services
 *
 * @category	helpers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined("BASEPATH")) {
    exit("No direct script access allowed");
}

if (!function_exists("user_featured_services")) {

    /**
     * Check featured status of user
     *
     * @return string/false
     */
    function user_featured_services()
    {
        $ci = &get_instance();
        $ci->load->model("Users_model");

        $user_type = $ci->session->userdata("user_type");
        $user_id = $ci->session->userdata("user_id");
        $ci->load->model("Services_model");
        $service_template = array("featured_template");
        $services = $ci->Services_model->get_user_type_active_services($user_type, $service_template);
        //  ...and if the service is activated
        if (count($services) == 0) {
            return false;
        }

        $ci->view->assign('services', $services);

        $ci->load->model("Users_services_model");
        $is_featured = $ci->Users_services_model->is_user_featured($user_id);
        $ci->view->assign("is_user_featured", $is_featured);

        $params = array(
            'where' => array(
                'id_user' => $user_id,
                'template_gid' => 'featured_template',
                'status' => 1,
            ),
        );
        $user_services = $ci->Users_services_model->get_services_list(null, $params);
        if (!empty($user_services)) {
            $ci->view->assign('user_service', current($user_services));
        }

        $page_data["date_format"] = $ci->pg_date->get_format('date_time_literal', 'st');
        $ci->view->assign("page_data", $page_data);

        $ci->load->model('Currencies_model');
        $base_currency = $ci->Currencies_model->get_currency_default(true);
        $ci->view->assign('base_currency', $base_currency);

        return $ci->view->fetch("helper_featured", "user", "users_services");
    }

}

if (!function_exists("user_show_logo_services")) {

    /**
     * Check featured status of user
     *
     * @return string/false
     */
    function user_show_logo_services()
    {
        $ci = &get_instance();
        $ci->load->model("Users_model");

        $user_type = $ci->session->userdata("user_type");
        $user_id = $ci->session->userdata("user_id");
        $ci->load->model("Services_model");
        $service_template = array("show_logo_template");
        $services = $ci->Services_model->get_user_type_active_services($user_type, $service_template);
        //  ...and if the service is activated
        if (count($services) == 0) {
            return false;
        }
        $ci->view->assign('services', $services);

        $ci->load->model("Users_services_model");
        $is_show_logo = $ci->Users_services_model->is_user_show_logo($user_id);
        $ci->view->assign("is_show_logo", $is_show_logo);

        $params = array(
            'where' => array(
                'id_user' => $user_id,
                'template_gid' => 'show_logo_template',
                'status' => 1,
            ),
        );
        $user_services = $ci->Users_services_model->get_services_list(null, $params);
        if (!empty($user_services)) {
            $ci->view->assign('user_service', current($user_services));
        }

        $page_data["date_format"] = $ci->pg_date->get_format('date_time_literal', 'st');
        $ci->view->assign("page_data", $page_data);

        $ci->load->model('Currencies_model');
        $base_currency = $ci->Currencies_model->get_currency_default(true);
        $ci->view->assign('base_currency', $base_currency);

        return $ci->view->fetch("helper_show_logo", "user", "users_services");
    }

}
if (!function_exists('users_featured')) {

    /**
     * Return featured users block
     *
     * @param string  $user_type user type
     * @param integer $count     users count
     * @param string  $size      photo size
     * @param string  $view      view mode
     * @param integer $width     block width
     *
     * @return void
     */
    function users_featured($user_type, $count = 8, $size = 'middle', $view = 'gallery', $width = 100)
    {
        $ci = &get_instance();
        $ci->load->model('Users_model');

        if (func_num_args() == 1 && is_array($user_type)) {
            $params = $user_type;
            $user_type = isset($params['user_type']) ? $params['user_type'] : null;
            $count = isset($params['count']) ? $params['count'] : 8;
            $size = isset($params['size']) ? $params['size'] : 'middle';
            $view = isset($params['view']) ? $params['view'] : 'gallery';
            $width = isset($params['width']) ? $params['width'] : 100;
        }

        $count = intval($count);
        if (!$count) {
            $count = 8;
        }

        $user_types = $ci->Users_model->getUserTypes();

        $attrs = array();
        if ($user_type && in_array($user_type, $user_types)) {
            $attrs['where']['user_type'] = $user_type;
        }
        $attrs['where']['is_featured'] = 1;
        $users = $ci->Users_model->get_active_users_list(1, $count, array('RAND()' => ''), $attrs);
        $header_lang_gids = [null => 'users', 'agent' => 'agents', 'company' => 'companies'];

        $ci->view->assign('users', $users);
        $ci->view->assign('type', 'featured');
        $ci->view->assign('photo_size', $size);
        $ci->view->assign('block_width', $width);
        $ci->view->assign('page_header', l('header_featured_' . $header_lang_gids[$user_type], 'users_services'));
        
        return $ci->view->fetch('helper_users_block_' . $view, 'user', 'users_services');
    }

}
